@@ -1,6 +1,20 @@
 #ifndef LIBPROB_H
 #define LIBPROB_H
 
+/* area under the binomial pdf, from 0 to k, of the
+ * binomial distribution with success probability p
+ * and n trials.
+*/
+
+double bdtr (int k, int n, double p);
+
+/* area under the binomial pdf, from k+1 to n, of the
+ * binomial distribution with success probability p
+ * and n trials. 
+*/
+
+double bdtrc (int k, int n, double p);
+
 /* area under the left hand tail (from 0 to x)
  * of the Chi square probability density function with
  * v degrees of freedom.
@@ -66,6 +80,15 @@ double ndtri (double y0);
 
 double cephes_gamma (double x); /* alias for gamma() */
 
+/*
+ * Returns the base e (2.718...) logarithm of the absolute
+ * value of the gamma function of the argument.
+ * The sign (+1 or -1) of the gamma function is returned in a
+ * global (extern) variable named sgngam.
+*/
+
+double cephes_lgamma (double x); /* alias for lgam() */
+
 #endif /* LIBPROB_H */
 
 
