@@ -34,6 +34,8 @@
 #include "libset.h"
 #include "forecast.h"
 #include "cmd_private.h"
+#include "objstack.h"
+#include "gretl_xml.h"
 
 #ifdef WIN32
 # include <windows.h>
@@ -90,6 +92,7 @@ gretl_restriction_set *rset;
 static int exec_line (char *line, LOOPSET **ploop, PRN *prn); 
 static int push_input_file (FILE *fp);
 static FILE *pop_input_file (void);
+static int saved_object_action (const char *line, PRN *prn);
 
 static void usage(void)
 {
@@ -227,9 +230,9 @@ int fn_get_line (void)
     return err;
 }
 
-unsigned char gp_flags (int batch, gretlopt opt)
+gnuplot_flags gp_flags (int batch, gretlopt opt)
 {
-    unsigned char flags = 0;
+    gnuplot_flags flags = 0;
 
     if (batch) flags |= GP_BATCH;
 
@@ -267,7 +270,7 @@ static void force_language (int f)
 {
     if (f == ENGLISH) {
 	setlocale(LC_ALL, "C");
-    } else {
+    } else if (f == BASQUE) {
 # ifdef WIN32
 	setlocale(LC_ALL, "eu");
 # else
@@ -279,7 +282,7 @@ static void force_language (int f)
     if (f == ENGLISH) { 
 	SetEnvironmentVariable("LC_ALL", "C");
 	putenv("LC_ALL=C");
-    } else {
+    } else if (f == BASQUE) {
 	SetEnvironmentVariable("LC_ALL", "eu");
 	putenv("LC_ALL=eu");
     }	
@@ -311,7 +314,7 @@ static int clear_data (void)
     free_modelspec(modelspec);
     modelspec = NULL;
 
-    gretl_equation_systems_cleanup();
+    gretl_saved_objects_cleanup();
 
     reset_model_count();
 
@@ -534,8 +537,8 @@ int main (int argc, char *argv[])
 	    err = gretl_get_data(&Z, &datainfo, paths.datfile, &paths, 
 				 DATA_NONE, prn);
 	} else if (err == GRETL_XML_DATA) {
-	    err = get_xmldata(&Z, &datainfo, paths.datfile, &paths, 
-			      DATA_NONE, prn, 0);
+	    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, 
+				 DATA_NONE, prn, 0);
 	} else if (err == GRETL_CSV_DATA) {
 	    err = import_csv(&Z, &datainfo, paths.datfile, prn);
 	} else if (err == GRETL_OCTAVE) {
@@ -566,8 +569,8 @@ int main (int argc, char *argv[])
     models = malloc(2 * sizeof *models);
     if (models == NULL) noalloc("models"); 
 
-    models[0] = gretl_model_new();
-    models[1] = gretl_model_new();
+    models[0] = gretl_model_new_protected();
+    models[1] = gretl_model_new_protected();
 
     if (models[0] == NULL || models[1] == NULL) {
 	noalloc("models"); 
@@ -653,15 +656,11 @@ int main (int argc, char *argv[])
 
     /* leak check -- try explicitly freeing all memory allocated */
 
-    free_Z(Z, datainfo);
-
-    free_model(models[0]);
-    free_model(models[1]);
+    gretl_model_free_on_exit(models[0]);
+    gretl_model_free_on_exit(models[1]);
     free(models);
 
-    if (data_status) {
-	free_datainfo(datainfo);
-    }
+    destroy_dataset(Z, datainfo);
 
     if (fb != stdin && fb != NULL) {
 	fclose(fb);
@@ -749,6 +748,7 @@ static int do_autofit_plot (PRN *prn)
 static int exec_line (char *line, LOOPSET **ploop, PRN *prn) 
 {
     LOOPSET *loop = *ploop;
+    GRETL_VAR *var = NULL;
     int chk, nulldata_n, renumber;
     int dbdata = 0, do_arch = 0, do_nls = 0;
     unsigned char echo_flags;
@@ -777,6 +777,12 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	}
 	return err;
     }  
+
+    /* catch requests relating to saved objects, which are not
+       really "commands" as such */
+    chk = saved_object_action(line, prn);
+    if (chk == 1) return 0;   /* action was OK */
+    if (chk == -1) return 1;  /* action was faulty */
     
     /* are we ready for this? */
     if (!data_status && !cmd.ignore && 
@@ -785,8 +791,6 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	return 1;
     }
 
-    compress_spaces(line);
-
     /* tell libgretl if we're in batch mode */
     gretl_set_batch_mode(batch);
 
@@ -903,11 +907,13 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case RHODIFF:
     case RMPLOT: 
     case RUNS: 
+    case SDIFF:
     case SIM: 
     case SPEARMAN:
     case SQUARE: 
     case STORE:
-    case SUMMARY: 
+    case SUMMARY:
+    case TRANSPOSE:
     case VARLIST:
     case VARTEST: 
 	err = simple_commands(&cmd, line, &Z, datainfo, prn);
@@ -945,7 +951,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case OMITFROM:
 	i = atoi(cmd.param);
 	if ((err = model_test_start(cmd.ci, i, prn))) break;
-	if (i == (models[0])->ID) goto plain_add_omit;
+	if (i == models[0]->ID) goto plain_add_omit;
 	err = re_estimate(modelspec_get_command_by_id(modelspec, i), 
 			  &tmpmod, &Z, datainfo);
 	if (err) {
@@ -978,7 +984,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	clear_model(models[0]);
 	*models[0] = ar_func(cmd.list, atoi(cmd.param), &Z, 
 			     datainfo, cmd.opt, prn);
-	if ((err = (models[0])->errcode)) { 
+	if ((err = models[0]->errcode)) { 
 	    errmsg(err, prn); 
 	}
 	break;
@@ -988,10 +994,10 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	clear_model(models[1]);
 	*models[1] = arch_model(cmd.list, order, &Z, datainfo, 
 				cmd.opt, prn);
-	if ((err = (models[1])->errcode)) { 
+	if ((err = models[1]->errcode)) { 
 	    errmsg(err, prn);
 	}
-	if ((models[1])->ci == ARCH) {
+	if (models[1]->ci == ARCH) {
 	    do_arch = 1;
 	    swap_models(&models[0], &models[1]); 
 	}
@@ -1000,18 +1006,8 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     case ARMA:
 	clear_model(models[0]);
-#ifdef HAVE_X12A
-	if (cmd.opt & OPT_X) {
-	    *models[0] = arma_x12(cmd.list, (const double **) Z, datainfo,
-				  &paths, cmd.opt, prn);
-	} else {
-	    *models[0] = arma(cmd.list, (const double **) Z, datainfo, 
-			      cmd.opt, prn);
-	}
-#else
-	*models[0] = arma(cmd.list, (const double **) Z, datainfo, 
+	*models[0] = arma(cmd.list, (const double **) Z, datainfo,
 			  cmd.opt, prn);
-#endif
 	if ((err = (models[0])->errcode)) { 
 	    errmsg(err, prn); 
 	} else {	
@@ -1093,17 +1089,19 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 		errmsg(err, prn);
 	    }
 	    sys = NULL;
-	} else if (!strcmp(cmd.param, "nls")) {
+	} else if (!strcmp(cmd.param, "mle") || !strcmp(cmd.param, "nls")) {
 	    clear_model(models[0]);
-	    *models[0] = nls(&Z, datainfo, prn);
-	    if ((err = (models[0])->errcode)) {
+	    *models[0] = nls(&Z, datainfo, cmd.opt, prn);
+	    if ((err = models[0]->errcode)) {
 		errmsg(err, prn);
 	    } else {
-		do_nls = 1;
+		if (!strcmp(cmd.param, "nls")) {
+		    do_nls = 1;
+		}
 		printmodel(models[0], datainfo, cmd.opt, prn);
 	    }
 	} else if (!strcmp(cmd.param, "restrict")) {
-	    err = gretl_restriction_set_finalize(rset, prn);
+	    err = gretl_restriction_set_finalize(rset, datainfo, prn);
 	    if (err) {
 		errmsg(err, prn);
 	    }
@@ -1180,7 +1178,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case GENR:
-	err = generate(line, &Z, datainfo, models[0], cmd.opt);
+	err = generate(line, &Z, datainfo, cmd.opt);
 	if (err) { 
 	    errmsg(err, prn);
 	} else {
@@ -1197,7 +1195,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    break;
 	}
 	if ((cmd.opt & OPT_M) || (cmd.opt & OPT_Z) || (cmd.opt & OPT_S)) { 
-	    err = gnuplot(cmd.list, NULL, NULL, &Z, datainfo,
+	    err = gnuplot(cmd.list, NULL, cmd.param, &Z, datainfo,
 			  &plot_count, gp_flags(batch, cmd.opt));
 	} else {
 	    lines[0] = (cmd.opt != 0);
@@ -1295,8 +1293,8 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	} else if (chk == GRETL_BOX_DATA) {
 	    err = import_box(&Z, &datainfo, datfile, prn);
 	} else if (chk == GRETL_XML_DATA) {
-	    err = get_xmldata(&Z, &datainfo, datfile, &paths, 
-			      data_status, prn, 0);
+	    err = gretl_read_gdt(&Z, &datainfo, datfile, &paths, 
+				 data_status, prn, 0);
 	} else if (dbdata) {
 	    err = set_db_name(datfile, chk, &paths, prn);
 	} else {
@@ -1377,7 +1375,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case TSLS:
 	clear_model(models[0]);
 	if (cmd.ci == LOGIT || cmd.ci == PROBIT) {
-	    *models[0] = logit_probit(cmd.list, &Z, datainfo, cmd.ci);
+	    *models[0] = logit_probit(cmd.list, &Z, datainfo, cmd.ci, cmd.opt);
 	} else if (cmd.ci == HSK) {
 	    *models[0] = hsk_func(cmd.list, &Z, datainfo);
 	} else if (cmd.ci == LOGISTIC) {
@@ -1389,8 +1387,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	    *models[0] = poisson_model(cmd.list, &Z, datainfo,
 				       (cmd.opt & OPT_V)? prn : NULL);
 	} else if (cmd.ci == TSLS) {
-	    *models[0] = tsls_func(cmd.list, atoi(cmd.param), 
-				   &Z, datainfo, cmd.opt);
+	    *models[0] = tsls_func(cmd.list, TSLS, &Z, datainfo, cmd.opt);
 	} else if (cmd.ci == LAD) {
 	    *models[0] = lad(cmd.list, &Z, datainfo);
 	} else if (cmd.ci == GARCH) {
@@ -1407,12 +1404,13 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	}
 	break;
 
+    case MLE:
     case NLS:
-	err = nls_parse_line(line, (const double **) Z, datainfo);
+	err = nls_parse_line(cmd.ci, line, (const double **) Z, datainfo, prn);
 	if (err) {
 	    errmsg(err, prn);
 	} else {
-	    gretl_cmd_set_context(&cmd, NLS);
+	    gretl_cmd_set_context(&cmd, cmd.ci);
 	}
 	break;
 
@@ -1450,7 +1448,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	} else {
 	    *models[0] = lsq(cmd.list, &Z, datainfo, cmd.ci, cmd.opt, 0.0);
 	}
-	if ((err = (models[0])->errcode)) {
+	if ((err = models[0]->errcode)) {
 	    errmsg(err, prn);
 	} else {
 	    printmodel(models[0], datainfo, cmd.opt, prn);
@@ -1479,7 +1477,7 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case PRINTF:
-	err = do_printf(line, &Z, datainfo, models[0], prn);
+	err = do_printf(line, &Z, datainfo, prn);
 	break;
 
     case PVALUE:
@@ -1662,11 +1660,25 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     case VAR:
 	order = atoi(cmd.param);
-	err = simple_var(order, cmd.list, &Z, datainfo, !batch, 
-			 cmd.opt, prn);
-	if (err) {
-	    errmsg(err, prn);
+	var = full_VAR(order, cmd.list, &Z, datainfo, cmd.opt, prn);
+	if (var == NULL) {
+	    err = 1;
+	} else {
+	    err = maybe_stack_var(var, &cmd);
 	}
+	if (err) errmsg(err, prn);
+	break;
+
+    case VECM:
+	order = atoi(cmd.param);
+	var = vecm(order, atoi(cmd.extra), cmd.list, &Z, datainfo, 
+		   cmd.opt, prn);
+	if (var == NULL) {
+	    err = 1;
+	} else {
+	    err = maybe_stack_var(var, &cmd);
+	}
+	if (err) errmsg(err, prn);
 	break;
 
     default:
@@ -1682,13 +1694,13 @@ static int exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
     if (!err && (is_model_cmd(cmd.word) || do_nls || do_arch)
 	&& !is_quiet_model_test(cmd.ci, cmd.opt)) { 
-
 	attach_subsample_to_model(models[0], datainfo);
 #ifdef MSPEC_DEBUG
 	fprintf(stderr, "\ngretlcli: saving spec: model.ID = %d, model_count = %d\n",
 		(models[0])->ID, get_model_count());
 #endif
 	err = modelspec_save(models[0], &modelspec);
+	maybe_stack_model(models[0], &cmd, prn);
     }
 
     return err;
@@ -1724,3 +1736,5 @@ static FILE *pop_input_file (void)
 
     return ret;
 }
+
+#include "cli_object.c"
