@@ -20,6 +20,7 @@
 /* gretl.c : main for gretl */
 
 #include "gretl.h"
+#include "version.h"
 #include "ssheet.h"
 #include "console.h"
 #include "session.h"
@@ -31,6 +32,8 @@
 #include "cmdstack.h"
 #include "toolbar.h"
 #include "menustate.h"
+#include "fileselect.h"
+#include "gretl_xml.h"
 
 #include <dirent.h>
 #include <unistd.h>
@@ -104,6 +107,11 @@ float gui_scale;
 int expert = FALSE; 
 int updater = FALSE;
 int want_toolbar = TRUE;
+int winsize = TRUE;
+int main_x = -1;
+int main_y = -1;
+int mainwin_width = 520;
+int mainwin_height = 420;
 
 char dbproxy[21];
 
@@ -112,13 +120,9 @@ char calculator[MAXSTR] = "xcalc";
 char latex[MAXSTR] = "latex";
 char viewdvi[MAXSTR] = "xdvi";
 char viewps[MAXSTR] = "gv";
-
-#ifdef USE_GNOME
-char Rcommand[MAXSTR] = "R --gui=gnome";
-extern const char *version_string;
-#else
+char viewpdf[MAXSTR] = "acroread";
 char Rcommand[MAXSTR] = "xterm -e R";
-#endif
+char Browser[MAXSTR] = "mozilla";
 
 #ifdef HAVE_TRAMO
 char tramo[MAXSTR] = "tramo";
@@ -135,14 +139,14 @@ static void manual_update_query (gpointer p, guint u, GtkWidget *w)
     update_query();
 }
 
-#ifdef USE_GNOME
+#ifdef USE_GNOME_HELP
 static void gnome_help (void)
 {
     static GnomeHelpMenuEntry help_entry = { "gretl", "index.html" };
 
     gnome_help_display(NULL, &help_entry);
 }
-#endif /* USE_GNOME */
+#endif /* USE_GNOME_HELP */
 
 extern void find_var (gpointer p, guint u, GtkWidget *w); /* gui_utils.c */
 
@@ -151,6 +155,13 @@ static void varinfo_callback (gpointer p, guint u, GtkWidget *w)
     varinfo_dialog(mdata->active_var, 1);
 }
 
+#ifdef ENABLE_MAILER
+static void email_data (gpointer p, guint u, GtkWidget *w)
+{
+    send_file(paths.datfile);
+}
+#endif
+
 GtkItemFactoryEntry data_items[] = {
     /* File menu */
     { N_("/_File"), NULL, NULL, 0, "<Branch>" },
@@ -160,14 +171,14 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Open data/user file..."), NULL, open_data, OPEN_DATA, NULL },
     { N_("/File/Open data/sample file..."), NULL, display_files, TEXTBOOK_DATA, NULL },
     { N_("/File/Open data/sep1"), NULL, NULL, 0, "<Separator>" },    
-    { N_("/File/Open data/import CSV..."), NULL, open_data, OPEN_CSV, NULL },
-    { N_("/File/Open data/import ASCII..."), NULL, open_data, OPEN_ASCII, NULL },
-    { N_("/File/Open data/import Octave..."), NULL, open_data, OPEN_OCTAVE, NULL },
-    { N_("/File/Open data/import BOX..."), NULL, open_data, OPEN_BOX, NULL },
-    { N_("/File/Open data/import Gnumeric..."), NULL, open_data, 
-      OPEN_GNUMERIC, NULL },
-    { N_("/File/Open data/import Excel..."), NULL, open_data, 
-      OPEN_EXCEL, NULL },
+    { N_("/File/Open data/Import"), NULL, NULL, 0, "<Branch>" },
+    { N_("/File/Open data/Import/CSV..."), NULL, open_data, OPEN_CSV, NULL },
+    { N_("/File/Open data/Import/ASCII..."), NULL, open_data, OPEN_ASCII, NULL },
+    { N_("/File/Open data/Import/Octave..."), NULL, open_data, OPEN_OCTAVE, NULL },
+    { N_("/File/Open data/Import/Gnumeric..."), NULL, open_data, OPEN_GNUMERIC, NULL },
+    { N_("/File/Open data/Import/Excel..."), NULL, open_data, OPEN_EXCEL, NULL },
+    { N_("/File/Open data/Import/Eviews..."), NULL, open_data, OPEN_WF1, NULL },
+    { N_("/File/Open data/Import/Stata..."), NULL, open_data, OPEN_DTA, NULL },
 
     /* File, Append data */
     { N_("/File/_Append data"), NULL, NULL, 0, "<Branch>" },
@@ -175,28 +186,27 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Append data/from CSV..."), NULL, open_data, APPEND_CSV, NULL },
     { N_("/File/Append data/from ASCII..."), NULL, open_data, APPEND_ASCII, NULL },
     { N_("/File/Append data/from Octave..."), NULL, open_data, APPEND_OCTAVE, NULL },
-    { N_("/File/Append data/from Gnumeric..."), NULL, open_data, 
-      APPEND_GNUMERIC, NULL },
-    { N_("/File/Append data/from Excel..."), NULL, open_data, 
-      APPEND_EXCEL, NULL },
+    { N_("/File/Append data/from Gnumeric..."), NULL, open_data, APPEND_GNUMERIC, NULL },
+    { N_("/File/Append data/from Excel..."), NULL, open_data, APPEND_EXCEL, NULL },
+    { N_("/File/Append data/from Eviews..."), NULL, open_data, APPEND_WF1, NULL },
+    { N_("/File/Append data/from Stata..."), NULL, open_data, APPEND_DTA, NULL },
 
     /* File, Save data */
     { N_("/File/_Save data"), NULL, auto_store, 0, NULL },
     { N_("/File/Save data _as"), NULL, NULL, 0, "<Branch>" },
-    { N_("/File/Save data as/_standard format..."), NULL, file_save, 
-      SAVE_DATA_AS, NULL },
-    { N_("/File/Save data as/_gzipped..."), NULL, 
-      file_save, SAVE_GZDATA, NULL },
-    { N_("/File/Save data as/_database..."), NULL, 
-      file_save, SAVE_DBDATA, NULL },
+    { N_("/File/Save data as/_standard format..."), NULL, file_save, SAVE_DATA_AS, NULL },
+    { N_("/File/Save data as/_gzipped..."), NULL, file_save, SAVE_GZDATA, NULL },
+    { N_("/File/Save data as/_database..."), NULL, file_save, SAVE_DBDATA, NULL },
 
     /* File, Export data */
     { N_("/File/_Export data"), NULL, NULL, 0, "<Branch>" },
     { N_("/File/Export data/_CSV..."), NULL, file_save, EXPORT_CSV, NULL },
     { N_("/File/Export data/GNU _R..."), NULL, file_save, EXPORT_R, NULL },
-    { N_("/File/Export data/GNU _octave..."), NULL, file_save, 
-      EXPORT_OCTAVE, NULL },
+    { N_("/File/Export data/GNU _octave..."), NULL, file_save, EXPORT_OCTAVE, NULL },
     { N_("/File/Export data/_PcGive..."), NULL, file_save, EXPORT_DAT, NULL },
+#ifdef ENABLE_MAILER
+    { N_("/File/Send To..."), NULL, email_data, OPEN_DATA, NULL },
+#endif
     { N_("/File/C_lear data set"), NULL, verify_clear_data, 0, NULL },
     { N_("/File/sep0"), NULL, NULL, 0, "<Separator>" },
 
@@ -352,6 +362,8 @@ GtkItemFactoryEntry data_items[] = {
       add_logs_etc, DIFF, NULL },
     { N_("/Data/Add variables/log differences of selected variables"), NULL, 
       add_logs_etc, LDIFF, NULL },
+    { N_("/Data/Add variables/seasonal differences of selected variables"), NULL, 
+      add_logs_etc, SDIFF, NULL },
     { N_("/Data/Add variables/periodic dummies"), NULL, add_dummies, 0, NULL },
     { N_("/Data/Add variables/unit dummies"), NULL, add_dummies, 1, NULL },
     { N_("/Data/Add variables/panel dummies"), NULL, add_dummies, 2, NULL },
@@ -375,6 +387,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Sample/sep1"), NULL, NULL, 0, "<Separator>" },
     { N_("/Sample/Dataset structure..."), NULL, data_structure_wizard, 0, NULL },
     { N_("/Sample/Compact data..."), NULL, do_compact_data_set, 0, NULL },
+    { N_("/Sample/Expand data..."), NULL, do_expand_data_set, 0, NULL },
     { N_("/Sample/sep2"), NULL, NULL, 0, "<Separator>" },   
     { N_("/Sample/_Define, based on dummy..."), NULL, sample_range_dialog, 
       SMPLDUM, NULL },
@@ -418,7 +431,6 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Variable/Spectrum/Bartlett lag window"), NULL, do_pergm, 1, NULL }, 
     { N_("/Variable/_Augmented Dickey-Fuller test"), NULL, unit_root_test, ADF, NULL },
     { N_("/Variable/_KPSS test"), NULL, unit_root_test, KPSS, NULL },
-    { N_("/Variable/ARMA model"), NULL, arma_options_dialog, 0, NULL },
 #ifdef HAVE_X12A
     { N_("/Variable/X-12-ARIMA analysis"), NULL, do_tramo_x12a, X12A, NULL },
 #endif
@@ -457,19 +469,19 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/_Rank correlation..."), NULL, selector_callback, SPEARMAN, NULL },
     { N_("/Model/_Pooled OLS (panel)..."), NULL, model_callback, POOLED, NULL },
     { N_("/Model/Nonlinear Least Squares..."), NULL, gretl_callback, NLS, NULL },
+    { N_("/Model/Maximum likelihood..."), NULL, gretl_callback, MLE, NULL },
+    { N_("/Model/Simultaneous equations..."), NULL, gretl_callback, SYSTEM, NULL },
 #ifdef ENABLE_GMP
     { N_("/Model/High precision OLS..."), NULL, mp_ols_callback, MPOLS, NULL },
 #endif
     { N_("/_Help"), NULL, NULL, 0, "<LastBranch>" },
-    { N_("/Help/_GUI commands"), NULL, do_gui_help, 0, NULL },
-    { N_("/Help/_Script commands syntax"), NULL, do_script_help, 1, NULL },
+    { N_("/Help/_Command reference"), NULL, NULL, 0, "<Branch>" },
+    { N_("/Help/Command reference/plain text"), NULL, plain_text_cmdref, 0, NULL },
+    { N_("/Help/Command reference/PDF"), NULL, display_pdf_help, 0, NULL },
+    { N_("/Help/_User's guide"), NULL, display_pdf_help, 1, NULL },
     { N_("/Help/sep1"), NULL, NULL, 0, "<Separator>" },
-#if defined(USE_GNOME)
-    { N_("/Help/Manual in HTML"), NULL, gnome_help, 0, NULL },
+    { N_("/Help/Check for _updates"), NULL, manual_update_query, 0, NULL },
     { N_("/Help/sep2"), NULL, NULL, 0, "<Separator>" },
-#endif
-    { N_("/Help/_Check for updates"), NULL, manual_update_query, 0, NULL },
-    { N_("/Help/sep3"), NULL, NULL, 0, "<Separator>" },
     { N_("/Help/_About gretl"), NULL, about_dialog, 0, NULL }
 };
 
@@ -585,9 +597,9 @@ static void force_language (int f)
     if (f == ENGLISH) {
 	setlocale(LC_ALL, "C");
 	force_english_help();
-    } else {
-	setlocale(LC_ALL, "XXX");
-    }
+    } else if (f == BASQUE) {
+	setlocale(LC_ALL, "eu_ES");
+    } 
 }
 
 #endif /* ENABLE_NLS */
@@ -611,7 +623,7 @@ int main (int argc, char *argv[])
 
     /* Initialize gnome or GTK */
 #ifdef USE_GNOME
-    gnome_init_with_popt_table("gretl", version_string, argc, argv,
+    gnome_init_with_popt_table("gretl", GRETL_VERSION, argc, argv,
 			       options, 0, NULL);
 #else
     gtk_init(&argc, &argv);
@@ -691,9 +703,9 @@ int main (int argc, char *argv[])
     models = malloc(3 * sizeof *models);
     if (models == NULL) noalloc(_("models")); 
 
-    models[0] = gretl_model_new();
-    models[1] = gretl_model_new();
-    models[2] = gretl_model_new();
+    models[0] = gretl_model_new_protected();
+    models[1] = gretl_model_new_protected();
+    models[2] = gretl_model_new_protected();
 
     if (models[0] == NULL || models[1] == NULL || models[2] == NULL) 
 	noalloc(_("models")); 
@@ -726,8 +738,8 @@ int main (int argc, char *argv[])
 				 prn);
 	    break;
 	case GRETL_XML_DATA:
-	    err = get_xmldata(&Z, &datainfo, paths.datfile, &paths, data_status, 
-			      prn, 0);
+	    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, data_status, 
+				 prn, 0);
 	    break;
 	case GRETL_CSV_DATA:
 	    err = import_csv(&Z, &datainfo, paths.datfile, prn);
@@ -740,6 +752,8 @@ int main (int argc, char *argv[])
 	    break;
 	case GRETL_EXCEL:
 	case GRETL_GNUMERIC:
+	case GRETL_WF1:
+	case GRETL_DTA:
 	    err = get_worksheet_data(paths.datfile, ftype, 0, &gui_get_data);
 	    break;
 	case GRETL_SCRIPT:
@@ -805,6 +819,7 @@ int main (int argc, char *argv[])
 
     if (!gui_get_data) {
 	register_data(paths.datfile, trydatfile, 1);
+	maybe_display_string_table();
 	*trydatfile = 0;
     }
 
@@ -831,9 +846,9 @@ int main (int argc, char *argv[])
 
     if (Z) free_Z(Z, datainfo);
 
-    free_model(models[0]);
-    free_model(models[1]);
-    free_model(models[2]);
+    gretl_model_free_on_exit(models[0]);
+    gretl_model_free_on_exit(models[1]);
+    gretl_model_free_on_exit(models[2]);
     free(models);
 
     library_command_free();
@@ -849,8 +864,6 @@ int main (int argc, char *argv[])
     return EXIT_SUCCESS;
 }
 
-/* ........................................................... */
-
 static void augment_selection_count (gint i, gint *nsel)
 {
     *nsel += 1;
@@ -869,8 +882,6 @@ int mdata_selection_count (void)
     return selcount;
 }
 
-/* ........................................................... */
-
 gint main_varclick (GtkWidget *widget, GdkEventButton *event, 
 		    gpointer data)
 {
@@ -908,8 +919,6 @@ gint main_varclick (GtkWidget *widget, GdkEventButton *event,
     return TRUE;
 }
 
-/* ........................................................... */
-
 static void check_varmenu_state (GtkCList *list, gint i, gint j,
 				 GdkEventButton *event, gpointer p)
 {
@@ -926,8 +935,6 @@ static void check_varmenu_state (GtkCList *list, gint i, gint j,
     }
 }
 
-/* ........................................................... */
-
 static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 {
 #ifdef HAVE_FLITE
@@ -948,8 +955,6 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
     return 1;
 }
 
-/* ........................................................... */
-
 void populate_varlist (void)
 {
     char id[4];
@@ -963,7 +968,10 @@ void populate_varlist (void)
 
     j = 0;
     for (i=0; i<datainfo->v; i++) {
-	if (is_hidden_variable(i, datainfo)) continue;
+	if (is_hidden_variable(i, datainfo) ||
+	    is_standard_lag(i, datainfo)) {
+	    continue;
+	}
 	sprintf(id, "%d", i);
 	row[0] = id;
 	row[1] = datainfo->varname[i];
@@ -978,8 +986,7 @@ void populate_varlist (void)
     if (j >= 2) {
 	mdata->active_var = 1;	
 	/* both select and focus the first real variable */
-	gtk_clist_select_row 
-	    (GTK_CLIST(mdata->listbox), mdata->active_var, 1); 
+	gtk_clist_select_row(GTK_CLIST(mdata->listbox), mdata->active_var, 1); 
 	(GTK_CLIST(mdata->listbox))->focus_row = mdata->active_var;
     } else {
 	mdata->active_var = 0;
@@ -1012,8 +1019,6 @@ void populate_varlist (void)
     }
 }
 
-/* ........................................................... */
-
 void clear_varlist (GtkWidget *widget)
 {
     gtk_clist_clear(GTK_CLIST(widget));
@@ -1025,8 +1030,6 @@ void clear_varlist (GtkWidget *widget)
     }
 }
 
-/* ......................................................... */
-
 static float get_gui_scale (void)
 {
     GtkStyle *style = NULL;
@@ -1044,8 +1047,6 @@ static float get_gui_scale (void)
     return scale;
 }
 
-/* .................................................................. */
-
 static GtkWidget *list_box_create (GtkBox *box, char *titles[]) 
 {
     GtkWidget *view, *scroller;
@@ -1086,7 +1087,16 @@ static GtkWidget *list_box_create (GtkBox *box, char *titles[])
     return view;
 }
 
-/* ......................................................... */
+static gboolean 
+mainwin_config (GtkWidget *w, GdkEventConfigure *event, gpointer p)
+{
+    main_x = event->x;
+    main_y = event->y;
+    mainwin_width = event->width;
+    mainwin_height = event->height;
+
+    return FALSE;
+}
 
 static GtkWidget *make_main_window (int gui_get_data) 
 {
@@ -1097,8 +1107,6 @@ static GtkWidget *make_main_window (int gui_get_data)
 	_("Variable name"), 
 	_("Descriptive label")
     };
-    int mainwin_width = 540;
-    int mainwin_height = 420;
 
     mdata->data = NULL;  
     mdata->listbox = NULL;
@@ -1112,15 +1120,19 @@ static GtkWidget *make_main_window (int gui_get_data)
     mdata->w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 #endif
 
-    gtk_signal_connect (GTK_OBJECT (mdata->w), "delete_event",
-			GTK_SIGNAL_FUNC (exit_check), NULL);
-    gtk_signal_connect (GTK_OBJECT (mdata->w), "destroy",
-			GTK_SIGNAL_FUNC (destroy), NULL);
-
     gui_scale = get_gui_scale();
 
-    mainwin_width *= gui_scale;
-    mainwin_height *= gui_scale;
+    if (!winsize || mainwin_width <= 200 || mainwin_height <= 200) {
+	mainwin_width = 580 * gui_scale;
+	mainwin_height = 420 * gui_scale;
+    }
+
+    gtk_signal_connect(GTK_OBJECT(mdata->w), "configure_event",
+		       GTK_SIGNAL_FUNC(mainwin_config), NULL);
+    gtk_signal_connect(GTK_OBJECT(mdata->w), "delete_event",
+		       GTK_SIGNAL_FUNC(exit_check), NULL);
+    gtk_signal_connect(GTK_OBJECT(mdata->w), "destroy",
+		       GTK_SIGNAL_FUNC(destroy), NULL);
 
     gtk_window_set_title(GTK_WINDOW (mdata->w), "gretl");
     gtk_window_set_default_size(GTK_WINDOW (mdata->w), 
@@ -1184,11 +1196,15 @@ static GtkWidget *make_main_window (int gui_get_data)
 
     gtk_widget_show_all(mdata->w); 
 
+#if 0
+    if (winsize && main_x >= 0 && main_y >= 0) {
+	gtk_window_move(GTK_WINDOW(mdata->w), main_x, main_y);
+    }
+#endif
+
     return main_vbox;
 }
 
-/* ........................................................... */
-
 static void set_up_main_menu (void)
 {
     GtkAccelGroup *main_accel;
@@ -1206,8 +1222,6 @@ static void set_up_main_menu (void)
     gtk_accel_group_attach(main_accel, GTK_OBJECT (mdata->w));
 }
 
-/* ........................................................... */
-
 int restore_sample (gretlopt opt)
 {
     int err;
@@ -1227,7 +1241,6 @@ static void restore_sample_callback (gpointer p, int verbose, GtkWidget *w)
     int err = restore_sample(OPT_NONE); /* FIXME OPT_C? */
 
     if (verbose && !err) {
-	infobox(_("Full sample range restored"));
 	set_sample_label(datainfo);    
 	gretl_command_strcpy("smpl full");
 	check_and_record_command();
@@ -1311,7 +1324,7 @@ drag_data_received  (GtkWidget *widget,
     }
 
     /* ignore the wrong sort of data */
-    if (data == NULL || (dfname = data->data) == NULL || 
+    if (data == NULL || (dfname = (char *) data->data) == NULL || 
 	strlen(dfname) <= 5 || strncmp(dfname, "file:", 5))
 	return;
     
@@ -1371,12 +1384,10 @@ static void auto_store (void)
     if ((data_status & USER_DATA) && native_datafile()) {
 	do_store(paths.datfile, oflag, 1);
     } else {
-	file_selector(_("Save data file"), SAVE_DATA, NULL);
+	file_selector(_("Save data file"), SAVE_DATA, FSEL_DATA_NONE, NULL);
     }	
 }
 
-/* ........................................................... */
-
 static gint 
 compare_var_ids (GtkCList *cl, gconstpointer p1, gconstpointer p2)
 {
