@@ -29,6 +29,7 @@
 #include "textutil.h"
 #include "boxplots.h"
 #include "dlgutils.h"
+#include "fileselect.h"
 
 #ifdef OLD_GTK
 # include <gtkextra/gtkiconfilesel.h>
@@ -41,8 +42,6 @@ extern GtkWidget *active_edit_id;
 extern GtkWidget *active_edit_name;
 extern GtkWidget *active_edit_text;
 
-/* ......................................................... */
-
 static void doubleclick_action (windata_t *win)
 {
     switch (win->role) {
@@ -65,7 +64,7 @@ static void doubleclick_action (windata_t *win)
     case NATIVE_SERIES:
     case RATS_SERIES:
     case REMOTE_SERIES:
-	gui_get_series(win, 0, NULL);
+	gui_get_db_series(win, 0, NULL);
 	break;
     default:
 	break;
@@ -210,8 +209,6 @@ void unselectrow (GtkCList *clist, gint row, gint column,
 
 #endif /* old versus new gtk */
 
-/* ........................................................... */
-
 void open_data (gpointer data, guint code, GtkWidget *widget)
 {
     if (dataset_locked()) {
@@ -223,30 +220,38 @@ void open_data (gpointer data, guint code, GtkWidget *widget)
     case APPEND_DATA:
     case OPEN_ASCII:
     case APPEND_ASCII:
-	file_selector(_("Open data file"), code, NULL);
+	file_selector(_("Open data file"), code, FSEL_DATA_NONE, NULL);
 	break;
     case OPEN_CSV:
     case APPEND_CSV:
-	delimiter_dialog();
-	file_selector(_("Open CSV file"), code, NULL);
+	delimiter_dialog(NULL);
+	file_selector(_("Open CSV file"), code, FSEL_DATA_NONE, NULL);
 	break;
     case OPEN_OCTAVE:
     case APPEND_OCTAVE:
-	file_selector(_("Open Octave file"), code, NULL);
+	file_selector(_("Open Octave file"), code, FSEL_DATA_NONE, NULL);
 	break;
     case OPEN_BOX:
-	file_selector(_("Open BOX file"), code, NULL);
+	file_selector(_("Open BOX file"), code, FSEL_DATA_NONE, NULL);
 	break;
     case OPEN_GNUMERIC:
     case APPEND_GNUMERIC:
-	file_selector(_("Open Gnumeric file"), code, NULL);
+	file_selector(_("Open Gnumeric file"), code, FSEL_DATA_NONE, NULL);
 	break;
     case OPEN_EXCEL:
     case APPEND_EXCEL:
-	file_selector(_("Open Excel file"), code, NULL);
+	file_selector(_("Open Excel file"), code, FSEL_DATA_NONE, NULL);
+	break;
+    case OPEN_WF1:
+    case APPEND_WF1:
+	file_selector(_("Open Eviews workfile"), code, FSEL_DATA_NONE, NULL);
+	break;
+    case OPEN_DTA:
+    case APPEND_DTA:
+	file_selector(_("Open Stata file"), code, FSEL_DATA_NONE, NULL);
 	break;
     case OPEN_MARKERS:
-	file_selector(_("gretl: add markers"), code, NULL);
+	file_selector(_("gretl: add markers"), code, FSEL_DATA_NONE, NULL);
 	break;
     default:
 	errbox("Unrecognized data code");
@@ -254,35 +259,30 @@ void open_data (gpointer data, guint code, GtkWidget *widget)
     }
 }
 
-/* ........................................................... */
-
 void open_script (gpointer data, guint action, GtkWidget *widget)
 {
     if (action == OPEN_SCRIPT) {
-	file_selector(_("Open script file"), action, NULL);
+	file_selector(_("Open script file"), action, FSEL_DATA_NONE, NULL);
     } else if (action == OPEN_SESSION) {
-	file_selector(_("Open session file"), action, NULL);
+	file_selector(_("Open session file"), action, FSEL_DATA_NONE, NULL);
     }
 }
 
-/* ........................................................... */
-
 void file_save (gpointer data, guint file_code, GtkWidget *widget)
 {
     windata_t *vwin = (windata_t *) data;
+    gretlopt opt = OPT_NONE;
+    gpointer p = NULL;
 
     switch (file_code) {
     case SAVE_OUTPUT:
-	file_selector(_("Save output file"), SAVE_OUTPUT, vwin);
+	file_selector(_("Save output file"), SAVE_OUTPUT, FSEL_DATA_MISC, vwin);
 	break;
     case SAVE_CONSOLE:
-	file_selector(_("Save console output"), SAVE_CONSOLE, vwin);
-	break;
-    case SAVE_CMDS: 
-	file_selector(_("Save command log"), SAVE_CMDS, vwin);
+	file_selector(_("Save console output"), SAVE_CONSOLE, FSEL_DATA_MISC, vwin);
 	break;
     case SAVE_SCRIPT:
-	file_selector(_("Save command script"), SAVE_SCRIPT, vwin);
+	file_selector(_("Save command script"), SAVE_SCRIPT, FSEL_DATA_MISC, vwin);
 	break;
     case SAVE_DATA:
     case SAVE_DATA_AS:
@@ -290,39 +290,33 @@ void file_save (gpointer data, guint file_code, GtkWidget *widget)
     case SAVE_BIN1:
     case SAVE_BIN2:
     case SAVE_DBDATA:
-	data_save_selection_wrapper(file_code);
+	data_save_selection_wrapper(file_code, NULL);
 	break;
     case EXPORT_CSV:
-	delimiter_dialog();
+	delimiter_dialog(&opt);
+	p = GINT_TO_POINTER(opt);
     case EXPORT_R:
     case EXPORT_R_ALT:
     case EXPORT_OCTAVE:
     case EXPORT_DAT:
-	data_save_selection_wrapper(file_code);
+	data_save_selection_wrapper(file_code, p);
 	break;
     case SAVE_TEX:
-	file_selector(_("Save LaTeX file"), file_code, vwin->data);
-	break;
-    case SAVE_MODEL:
-	file_selector(_("Save model output"), file_code, vwin);
+	file_selector(_("Save LaTeX file"), file_code, FSEL_DATA_MISC, vwin->data);
 	break;
     case SAVE_GP_CMDS:
-	file_selector(_("Save gnuplot commands"), file_code, vwin);
+	file_selector(_("Save gnuplot commands"), file_code, FSEL_DATA_MISC, vwin);
 	break;
     default:
 	dummy_call();
     }
 }
 
-/* ........................................................... */
-
 void dummy_call (void)
 {
     errbox(_("Sorry, this item not yet implemented!"));
 }
 
-/* ........................................................... */
-
 void print_report (gpointer data, guint unused, GtkWidget *widget)
 {
     PRN *prn;
@@ -335,8 +329,6 @@ void print_report (gpointer data, guint unused, GtkWidget *widget)
 		DATA_REPORT, NULL);
 }
 
-/* ........................................................... */
-
 void edit_header (gpointer data, guint unused, GtkWidget *widget)
 {
     if (data_status & BOOK_DATA) {
@@ -347,8 +339,6 @@ void edit_header (gpointer data, guint unused, GtkWidget *widget)
     }
 }
 
-/* ........................................................... */
-
 void fit_resid_callback (gpointer data, guint code, GtkWidget *widget)
 {
     windata_t *mydata = (windata_t *) data; 
@@ -357,12 +347,14 @@ void fit_resid_callback (gpointer data, guint code, GtkWidget *widget)
     add_fit_resid(pmod, code, 0);
 }
 
-void var_resid_callback (gpointer data, guint eqnum, GtkWidget *widget)
+void VAR_resid_callback (gpointer data, guint eqnum, GtkWidget *widget)
 {
-    windata_t *mydata = (windata_t *) data; 
-    GRETL_VAR *var = (GRETL_VAR *) mydata->data;
+    add_system_resid(data, eqnum, VAR);
+}
 
-    add_var_resid(var, eqnum);
+void SYS_resid_callback (gpointer data, guint eqnum, GtkWidget *widget)
+{
+    add_system_resid(data, eqnum, SYSTEM);
 }
 
 void model_stat_callback (gpointer data, guint which, GtkWidget *widget)
@@ -375,13 +367,21 @@ void model_stat_callback (gpointer data, guint which, GtkWidget *widget)
 
 void model_callback (gpointer data, guint model_code, GtkWidget *widget) 
 {
-    selection_dialog(_("gretl: specify model"), do_model, model_code);
+    int presel = 0;
+
+    if (widget == NULL && data != NULL) {
+	/* preselected dependent variable */
+	presel = GPOINTER_TO_INT(data);
+    }
+
+    selection_dialog(_("gretl: specify model"), do_model, model_code,
+		     presel);
 }
 
 #ifdef ENABLE_GMP
 void mp_ols_callback (gpointer data, guint model_code, GtkWidget *widget)
 {
-    selection_dialog (_("gretl: specify model"), do_mp_ols, model_code);
+    selection_dialog(_("gretl: specify model"), do_mp_ols, model_code, 0);
 }
 #endif /* ENABLE_GMP */
 
@@ -391,7 +391,7 @@ void model_genr_callback (gpointer data, guint u, GtkWidget *widget)
 
     edit_dialog(_("gretl: add var"), _("Enter formula for new variable:"),
 		"", do_model_genr, mydata, 
-		MODEL_GENR, VARCLICK_INSERT_NAME, 0);   
+		MODEL_GENR, VARCLICK_INSERT_NAME);   
 }
 
 void selector_callback (gpointer data, guint action, GtkWidget *widget)
@@ -399,16 +399,18 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
     windata_t *vwin = (windata_t *) data;
     char title[64];
 
-    if (action == ADD || action == OMIT || action == COEFFSUM) {
+    if (action == ADD || action == OMIT || action == COEFFSUM ||
+	action == ELLIPSE) {
 	set_window_busy(vwin);
     }
 
     strcpy(title, "gretl: ");
 
-    if (action == COINT) {
-	selection_dialog(_("gretl: cointegration test"), do_coint, action);
-    } else if (action == COINT2) {
-	selection_dialog(_("gretl: cointegration test"), do_coint2, action);
+    if (action == COINT || action == COINT2) {
+	selection_dialog(_("gretl: cointegration test"), do_coint, action, 0);
+    } else if (action == VAR || action == VECM) {
+	selection_dialog((action == VAR)? _("gretl: VAR") : _("gretl: VECM"),
+			 do_vector_model, action, 0);
     } else if (action == GR_XY || action == GR_IMP || action == GR_DUMMY
 	       || action == SCATTERS || action == GR_3D) {
 	void (*okfunc)() = NULL;
@@ -430,13 +432,15 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
 	default:
 	    return;
 	}
-	selection_dialog(_("gretl: define graph"), okfunc, action);
+	selection_dialog(_("gretl: define graph"), okfunc, action, 0);
     } else if (action == ADD || action == OMIT) {
 	simple_selection(_("gretl: model tests"), do_add_omit, action, vwin);
     } else if (action == COEFFSUM) {
 	simple_selection(_("gretl: model tests"), do_coeff_sum, action, vwin);
+    } else if (action == ELLIPSE) {
+	simple_selection(_("gretl: model tests"), do_confidence_region, action, vwin);
     } else if (action == GR_PLOT) {
-	simple_selection(_("gretl: model tests"), do_graph_from_selector, action, vwin);
+	simple_selection(_("gretl: define graph"), do_graph_from_selector, action, vwin);
     } else if (action == SPEARMAN) {
 	strcat(title, _("rank correlation"));
 	simple_selection(title, do_spearman, action, vwin);
@@ -451,69 +455,78 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
     }
 }
 
-/* ........................................................... */
-
 void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 {
-    char title[64], query[MAXLABEL], defstr[MAXLEN];
+    const char *title = NULL;
+    const char *query = NULL;
+    const char *defstr = NULL;
     void (*okfunc)() = NULL;
     guint varclick = VARCLICK_NONE;
-    windata_t *mydata = (windata_t *) data;
-
-    *defstr = '\0';
 
     switch (action) {
     case SMPLBOOL:
-	strcpy(title, _("gretl: restrict sample"));
-	strcpy(query, _("Enter boolean condition for selecting cases:"));
+	title = N_("gretl: restrict sample");
+	query = N_("Enter boolean condition for selecting cases:");
 	okfunc = do_samplebool;
 	varclick = VARCLICK_INSERT_NAME;
 	break;
     case SETSEED:
-	strcpy(title, _("gretl: random variables"));
-	strcpy(query, _("Enter integer seed for\n"
-	       "pseudo-random number generator:"));
+	title = N_("gretl: random variables");
+	query = N_("Enter integer seed for\n"
+	       "pseudo-random number generator:");
 	okfunc = do_seed;
 	break; 
     case NULLDATA:
-	strcpy(title, _("gretl: simulation data"));
-	strcpy(query, _("Series length for simulation data set:"));
-	strcpy(defstr, "100");
+	title = N_("gretl: simulation data");
+	query = N_("Series length for simulation data set:");
+	defstr = "100";
 	okfunc = do_simdata;
 	break;         
     case GENR:
-	strcpy(title, _("gretl: add var"));
-	strcpy(query, _("Enter formula for new variable:"));
+	title = N_("gretl: add var");
+	query = N_("Enter formula for new variable:");
 	okfunc = do_genr;
 	varclick = VARCLICK_INSERT_NAME;
 	break;
     case VSETMISS:
-	strcpy(title, _("gretl: missing code"));
-	strcpy(query, _("Enter value to be read as \"missing\":"));
+	title = N_("gretl: missing code");
+	query = N_("Enter value to be read as \"missing\":");
 	okfunc = do_variable_setmiss;
 	break;
     case GSETMISS:
-	strcpy(title, _("gretl: missing code"));
-	strcpy(query, _("Enter value to be read as \"missing\":"));
+	title = N_("gretl: missing code");
+	query = N_("Enter value to be read as \"missing\":");
 	okfunc = do_global_setmiss;
 	break;
     case GR_BOX:
     case GR_NBOX:
-	strcpy(title, _("gretl: boxplots"));
-	strcpy(query, _("Specify variables to plot:"));
+	title = N_("gretl: boxplots");
+	query = N_("Specify variables to plot:");
 	okfunc = do_box_graph;
 	varclick = VARCLICK_INSERT_NAME;
-	strcpy(defstr, get_boxplots_string());
+	defstr = get_boxplots_string();
+	break;
+    case MLE:
+	title = N_("gretl: maximum likelihood");
+	query = N_("MLE: Specify function, and derivatives if possible:");
+	okfunc = do_mle_model;
+	varclick = VARCLICK_INSERT_TEXT;
 	break;	
     case NLS:
-	strcpy(title, _("gretl: nonlinear least squares"));
-	strcpy(query, _("NLS: Specify function, and derivatives if possible:"));
+	title = N_("gretl: nonlinear least squares");
+	query = N_("NLS: Specify function, and derivatives if possible:");
 	okfunc = do_nls_model;
 	varclick = VARCLICK_INSERT_TEXT;
 	break;	
+    case SYSTEM:
+	title = N_("gretl: simultaneous equations system");
+	query = N_("Specify simultaneous equations:");
+	okfunc = do_eqn_system;
+	varclick = VARCLICK_INSERT_TEXT;
+	break;
     case RESTRICT:
-	strcpy(title, _("gretl: linear restrictions"));
-	strcpy(query, _("Specify restrictions:"));
+	title = N_("gretl: linear restrictions");
+	query = N_("Specify restrictions:");
 	okfunc = do_restrict;
 	break;	
     default:
@@ -521,48 +534,20 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 	return;
     }
 
-    edit_dialog(title, query, defstr, okfunc, mydata, 
-		action, varclick, 0);   
+    edit_dialog(_(title), _(query), defstr, okfunc, data, 
+		action, varclick);   
 }
 
-/* ........................................................... */
-
-void text_copy_callback (GtkWidget *w, gpointer data)
-{
-    window_copy(data, GRETL_FORMAT_SELECTION, w);
-}
-
-void text_paste_callback (GtkWidget *w, gpointer data)
-{
-    text_paste(data, 0, w);
-}
-
-void text_replace_callback (GtkWidget *w, gpointer data)
-{
-    text_replace(data, 0, w);
-}
-
-void text_undo_callback (GtkWidget *w, gpointer data)
-{
-    text_undo(data, 0, w);
-}
-
-/* ........................................................... */
-
 void run_script_callback (GtkWidget *w, gpointer data)
 {
     do_run_script(data, SCRIPT_EXEC, w);
 }
 
-/* ........................................................... */
-
 void gp_send_callback (GtkWidget *w, gpointer data)
 {
     gp_to_gnuplot(data, 0, w);
 }
 
-/* ........................................................... */
-
 void file_save_callback (GtkWidget *w, gpointer data)
 {
     guint u = 0;
@@ -574,10 +559,8 @@ void file_save_callback (GtkWidget *w, gpointer data)
 	switch (vwin->role) {
 	case EDIT_SCRIPT:
 	case VIEW_SCRIPT:
-	    u = SAVE_SCRIPT;
-	    break;
 	case VIEW_LOG:
-	    u = SAVE_CMDS;
+	    u = SAVE_SCRIPT;
 	    break;
 	case GR_PLOT:
 	    u = SAVE_GP_CMDS;
@@ -591,8 +574,6 @@ void file_save_callback (GtkWidget *w, gpointer data)
     file_save(data, u, w);
 }
 
-/* ........................................................... */
-
 void add_random_callback (gpointer data, guint code, GtkWidget *widget) 
 {
     if (code == GENR_UNIFORM) {
@@ -601,18 +582,16 @@ void add_random_callback (gpointer data, guint code, GtkWidget *widget)
 		       "minimum and maximum values:"), 
 		     "unif 0 1",  
 		     do_random, NULL, 
-		     GENR_UNIFORM, GENR, 0);
+		     GENR_UNIFORM, GENR);
     } else if (code == GENR_NORMAL) {
 	edit_dialog (_("gretl: normal variable"), 
 		     _("Enter name, mean and standard deviation:"), 
 		     "norm 0 1", 
 		     do_random, NULL, 
-		     GENR_NORMAL, GENR, 0);
+		     GENR_NORMAL, GENR);
     }
 }
 
-/* ........................................................... */
-
 static void name_first_ssheet_var (GtkWidget *widget, dialog_t *dlg) 
 {
     const gchar *buf;
@@ -661,7 +640,7 @@ static int prep_spreadsheet (const char *dataspec)
 		 _("Enter name for new variable\n"
 		   "(max. 8 characters)"),
 		 NULL, name_first_ssheet_var, NULL, 
-		 0, 0, 0);
+		 0, 0);
 
     return 0;
 }
@@ -702,7 +681,7 @@ void newdata_callback (gpointer data, guint pd_code, GtkWidget *widget)
 	strcpy(datainfo->stobs, "1");
 	edit_dialog (_("gretl: create data set"), 
 		     _("Number of observations:"), "50",
-		     n_obs_callback, NULL, 0, 0, 0);
+		     n_obs_callback, NULL, 0, 0);
     } else {
 	gchar *obsstr = NULL;
 
@@ -738,7 +717,23 @@ void do_nistcheck (gpointer p, guint v, GtkWidget *w)
     close_plugin(handle);
 
     view_file(fname, 0, 1, 78, 400, VIEW_CODEBOOK);
-
     g_free(fname);
 }
 
+#if defined (ENABLE_MAILER) && !defined(G_OS_WIN32)
+
+void send_file (char *fullname)
+{
+    int (*email_file) (const char *, const char *);
+    void *handle;
+
+    email_file = gui_get_plugin_function("email_file", &handle);
+    if (email_file == NULL) {
+        return;
+    }
+    
+    email_file(fullname, paths.userdir);
+    close_plugin(handle);
+}
+
+#endif
