@@ -49,9 +49,9 @@ extern GdkColor gray;
 
 /* private functions */
 static GtkWidget *database_window (windata_t *vwin);
-static int populate_series_list (windata_t *vwin);
-static int populate_remote_series_list (windata_t *vwin, char *buf);
-static int rats_populate_series_list (windata_t *vwin);
+static int make_local_db_series_list (windata_t *vwin);
+static int make_remote_db_series_list (windata_t *vwin, char *buf);
+static int make_rats_db_series_list (windata_t *vwin);
 static SERIESINFO *get_series_info (windata_t *vwin, int action);
 
 enum db_data_actions {
@@ -96,7 +96,7 @@ float retrieve_float (netfloat nf)
 }
 #endif
 
-void display_db_error (windata_t *vwin, char *buf)
+void show_db_error (windata_t *vwin, char *buf)
 {
     if (*buf != '\0') {
 	size_t n = strlen(buf);
@@ -149,7 +149,7 @@ static int get_remote_db_data (windata_t *vwin, SERIESINFO *sinfo,
 #endif
 
     if (err) {
-	display_db_error(vwin, errbuf);
+	show_db_error(vwin, errbuf);
 	free(getbuf);
 	return DB_NOT_FOUND;
     } 
@@ -204,7 +204,7 @@ static void graph_dbdata (double ***dbZ, DATAINFO *dbdinfo)
 	list[0] = 1; list[1] = 1;
 	err = boxplots(list, NULL, dbZ, dbdinfo, 0);
 	if (err) {
-	   errbox(_("boxplot command failed"));
+	    errbox(_("boxplot command failed"));
 	}
 	return;
     }
@@ -252,35 +252,57 @@ init_datainfo_from_sinfo (DATAINFO *pdinfo, SERIESINFO *sinfo)
     pdinfo->v = 2;
 }
 
+static gchar *expand_warning (int mult)
+{
+    gchar *msg;
+
+    msg = g_strdup_printf(_("Do you really want to add a lower frequency series\n"
+			    "to a higher frequency dataset?\n\n"
+			    "If you say 'yes' I will expand the source data by\n"
+			    "repeating each value %d times.  In general, this is\n"
+			    "not a valid thing to do."),
+			  mult);
+    return msg;
+}
+
 static void add_dbdata (windata_t *vwin, double **dbZ, SERIESINFO *sinfo)
 {
     double *xvec = NULL;
     int n, t, start, stop, pad1 = 0, pad2 = 0;
-    int compact_method = COMPACT_AVG;
+    CompactMethod method = COMPACT_AVG;
     int overwrite = 0;
-    int err = 0;
+    int resp, err = 0;
 
     if (data_status) { 
+	/* we already have data in gretl's workspace */
 	int dbv;
 
-	/* we already have data in gretl's workspace */
 	if (check_db_import(sinfo, datainfo)) {
 	    errbox(get_gretl_errmsg());
 	    return;
 	}
 
+	if (sinfo->pd < datainfo->pd) {
+	    gchar *msg = expand_warning(datainfo->pd / sinfo->pd);
+
+	    resp = yes_no_dialog("gretl", msg, 0);
+	    g_free(msg);
+	    if (resp != GRETL_YES) {
+		return;
+	    }
+	}
+
 	/* is there already a var of this name? */
 	dbv = varindex(datainfo, sinfo->varname);
 	if (dbv < datainfo->v) {
-	    int resp = yes_no_dialog ("gretl",                      
-				      _("There is already a variable of this name\n"
-					"in the dataset.  OK to overwrite it?"), 0);
-
+	    resp = yes_no_dialog ("gretl",                      
+				  _("There is already a variable of this name\n"
+				    "in the dataset.  OK to overwrite it?"), 0);
 	    if (resp == GRETL_YES) {
 		overwrite = 1;
 		/* pick up on pre-registered compaction method? */
 		if (COMPACT_METHOD(datainfo, dbv) != COMPACT_NONE) {
-		    compact_method = COMPACT_METHOD(datainfo, dbv);
+		    method = COMPACT_METHOD(datainfo, dbv);
 		}
 	    } else {
 		return;
@@ -294,10 +316,23 @@ static void add_dbdata (windata_t *vwin, double **dbZ, SERIESINFO *sinfo)
 
 	n = datainfo->n;
 
-	if (sinfo->pd > datainfo->pd) {
+	if (sinfo->pd < datainfo->pd) {
+	    /* the frequency of the new var is lower: we can handle
+	       annual to quarterly or monthly, and quarterly to
+	       monthly
+	    */
+	    if (sinfo->pd != 1 && sinfo->pd != 4 && 
+		datainfo->pd != 4 && datainfo->pd != 12) {
+		errbox(_("Sorry, can't handle this conversion yet!"));
+		if (!overwrite) {
+		    dataset_drop_last_variables(1, &Z, datainfo);
+		}
+		return;
+	    }
+	    xvec = expand_db_series(dbZ[1], sinfo, datainfo->pd);
+	} else if (sinfo->pd > datainfo->pd) {
 	    /* the frequency of the new var is higher */
-	    if (datainfo->pd != 1 && datainfo->pd != 4 &&
-		sinfo->pd != 12) {
+	    if (datainfo->pd != 1 && datainfo->pd != 4 && sinfo->pd != 12) {
 		errbox(_("Sorry, can't handle this conversion yet!"));
 		if (!overwrite) {
 		    dataset_drop_last_variables(1, &Z, datainfo);
@@ -306,16 +341,16 @@ static void add_dbdata (windata_t *vwin, double **dbZ, SERIESINFO *sinfo)
 	    }
 
 	    data_compact_dialog(vwin->w, sinfo->pd, &datainfo->pd, NULL, 
-				&compact_method);
+				&method);
 
-	    if (compact_method == COMPACT_NONE) {
+	    if (method == COMPACT_NONE) {
 		if (!overwrite) {
 		    dataset_drop_last_variables(1, &Z, datainfo);
 		}
 		return;
 	    }
 	    xvec = compact_db_series(dbZ[1], sinfo, datainfo->pd, 
-				     compact_method);
+				     method);
 	} else {  
 	    /* series does not need compacting */
 	    xvec = mymalloc(sinfo->nobs * sizeof *xvec);
@@ -339,8 +374,6 @@ static void add_dbdata (windata_t *vwin, double **dbZ, SERIESINFO *sinfo)
 	strcpy(VARLABEL(datainfo, dbv), sinfo->descrip);
 	get_db_padding(sinfo, datainfo, &pad1, &pad2);
 
-	fprintf(stderr, "pad1 = %d, pad2 = %d\n", pad1, pad2);
-
 	if (pad1 > 0) {
 	    fprintf(stderr, "Padding at start, %d obs\n", pad1);
 	    for (t=0; t<pad1; t++) {
@@ -403,39 +436,31 @@ static void add_dbdata (windata_t *vwin, double **dbZ, SERIESINFO *sinfo)
     }
 
     register_data(NULL, NULL, 0);
-
-    if (!err) {
-	infobox(_("Series imported OK")); 
-    }
 }
 
-/* ........................................................... */
-
 static void gui_display_series (GtkWidget *w, windata_t *vwin)
 {
-    gui_get_series(vwin, DB_DISPLAY, NULL);
+    gui_get_db_series(vwin, DB_DISPLAY, NULL);
 }
 
 static void gui_graph_series (GtkWidget *w, windata_t *vwin)
 {
-    gui_get_series(vwin, DB_GRAPH, NULL);
+    gui_get_db_series(vwin, DB_GRAPH, NULL);
 }
 
 static void gui_import_series (GtkWidget *w, windata_t *vwin)
 {
-    gui_get_series(vwin, DB_IMPORT, NULL);
+    gui_get_db_series(vwin, DB_IMPORT, NULL);
 }
 
 void import_db_series (windata_t *vwin)
 {
-    gui_get_series(vwin, DB_IMPORT, NULL);
+    gui_get_db_series(vwin, DB_IMPORT, NULL);
 }
 
-/* ........................................................... */
-
-void gui_get_series (gpointer data, guint action, GtkWidget *widget)
+void gui_get_db_series (gpointer p, guint action, GtkWidget *w)
 {
-    windata_t *vwin = (windata_t *) data;
+    windata_t *vwin = (windata_t *) p;
     int err = 0, dbcode = vwin->role;
     DATAINFO *dbinfo;
     SERIESINFO *sinfo;
@@ -475,8 +500,7 @@ void gui_get_series (gpointer data, guint action, GtkWidget *widget)
 
     if (dbcode == RATS_SERIES && err == DB_MISSING_DATA) {
 	infobox(_("Warning: series has missing observations"));
-    }
-    else if (err && dbcode != REMOTE_SERIES) {
+    } else if (err && dbcode != REMOTE_SERIES) {
 	errbox(_("Couldn't access binary datafile"));
 	return;
     } 
@@ -497,8 +521,6 @@ void gui_get_series (gpointer data, guint action, GtkWidget *widget)
     free(sinfo);
 } 
 
-/* ........................................................... */
-
 static void db_view_codebook (GtkWidget *w, windata_t *vwin)
 {
     char cbname[MAXLEN];
@@ -553,9 +575,9 @@ static void set_up_db_menu (windata_t *vwin, int cb)
 {
 #ifndef OLD_GTK
     GtkItemFactoryEntry db_items[] = {
-	{ N_("/_Series/_Display"), NULL, gui_get_series, DB_DISPLAY, NULL, GNULL },
-	{ N_("/_Series/_Graph"), NULL, gui_get_series, DB_GRAPH, NULL, GNULL },
-	{ N_("/_Series/_Import"), NULL, gui_get_series, DB_IMPORT, NULL, GNULL },
+	{ N_("/_Series/_Display"), NULL, gui_get_db_series, DB_DISPLAY, NULL, GNULL },
+	{ N_("/_Series/_Graph"), NULL, gui_get_db_series, DB_GRAPH, NULL, GNULL },
+	{ N_("/_Series/_Import"), NULL, gui_get_db_series, DB_IMPORT, NULL, GNULL },
 	{ N_("/_Find"), NULL, NULL, 0, "<Branch>", GNULL },   
 	{ N_("/Find/_Find in window"), NULL, menu_find, 1, "<StockItem>", GTK_STOCK_FIND },
 	{ N_("/_Codebook"), NULL, NULL, 0, "<Branch>", GNULL },    
@@ -564,9 +586,9 @@ static void set_up_db_menu (windata_t *vwin, int cb)
     };
 #else
     GtkItemFactoryEntry db_items[] = {
-	{ N_("/_Series/_Display"), NULL, gui_get_series, DB_DISPLAY, NULL},
-	{ N_("/_Series/_Graph"), NULL, gui_get_series, DB_GRAPH, NULL },
-	{ N_("/_Series/_Import"), NULL, gui_get_series, DB_IMPORT, NULL },
+	{ N_("/_Series/_Display"), NULL, gui_get_db_series, DB_DISPLAY, NULL},
+	{ N_("/_Series/_Graph"), NULL, gui_get_db_series, DB_GRAPH, NULL },
+	{ N_("/_Series/_Import"), NULL, gui_get_db_series, DB_IMPORT, NULL },
 	{ N_("/_Find"), NULL, menu_find, 1, NULL },
 	{ N_("/_Codebook"), NULL, book_callback_wrapper, 0, NULL }
     };
@@ -623,7 +645,7 @@ static void test_db_book (const char *fname, int *cb)
     }
 }
 
-static int display_db_series_list (int action, char *fname, char *buf)
+static int make_db_series_list (int action, char *fname, char *buf)
 {
     GtkWidget *listbox, *closebutton;
     GtkWidget *main_vbox;
@@ -707,11 +729,11 @@ static int display_db_series_list (int action, char *fname, char *buf)
 #endif
 
     if (action == NATIVE_SERIES) { 
-	err = populate_series_list(vwin);
+	err = make_local_db_series_list(vwin);
     } else if (action == REMOTE_SERIES) { 
-	err = populate_remote_series_list(vwin, buf);
+	err = make_remote_db_series_list(vwin, buf);
     } else {
-	err = rats_populate_series_list(vwin);
+	err = make_rats_db_series_list(vwin);
     } 
 
     if (err) {
@@ -723,8 +745,6 @@ static int display_db_series_list (int action, char *fname, char *buf)
     return err;
 }
 
-/* ........................................................... */
-
 static int check_serinfo (char *str, char *sername)
 {
     char pdc;
@@ -748,8 +768,6 @@ static int check_serinfo (char *str, char *sername)
     return err;
 }
 
-/* ........................................................... */
-
 static char *end_trim (char *s)
 {
     size_t i, n = strlen(s);
@@ -795,7 +813,7 @@ static void db_drag_connect (windata_t *vwin)
 		     vwin);
 }
 
-static int populate_series_list (windata_t *vwin)
+static int make_local_db_series_list (windata_t *vwin)
 {
 #ifndef OLD_GTK
     GtkListStore *store;
@@ -809,6 +827,11 @@ static int populate_series_list (windata_t *vwin)
     size_t n;
     int err = 0;
 
+#if 0
+    fprintf(stderr, "make_series_list: vwin->fname = '%s', vwin->data = %p\n",
+	    vwin->fname, vwin->data);
+#endif
+
     strcpy(dbidx, vwin->fname);
     strcat(dbidx, ".idx");
 
@@ -889,7 +912,7 @@ static int populate_series_list (windata_t *vwin)
     return 0;
 }
 
-static int populate_remote_series_list (windata_t *vwin, char *buf)
+static int make_remote_db_series_list (windata_t *vwin, char *buf)
 {
 #ifndef OLD_GTK
     GtkListStore *store;
@@ -910,7 +933,7 @@ static int populate_remote_series_list (windata_t *vwin, char *buf)
     i = 0;
 #endif
 
-    bufgets(NULL, 0, buf);
+    bufgets_init(buf);
 
     while (bufgets(line1, sizeof line1, buf)) {
 
@@ -958,12 +981,10 @@ static int populate_remote_series_list (windata_t *vwin, char *buf)
     return 0;
 }
 
-/* ......................................................... */
-
 #ifndef OLD_GTK
 static void insert_and_free_db_table (db_table *tbl, GtkTreeView *view)
 #else
-static void insert_and_free_db_table (db_table *tbl, GtkCList *clist)
+     static void insert_and_free_db_table (db_table *tbl, GtkCList *clist)
 #endif     
 {
     int i;
@@ -1009,9 +1030,7 @@ static void insert_and_free_db_table (db_table *tbl, GtkCList *clist)
     free(tbl);
 }
 
-/* ......................................................... */
-
-static int rats_populate_series_list (windata_t *vwin)
+static int make_rats_db_series_list (windata_t *vwin)
 {
     FILE *fp;
     db_table *tbl;
@@ -1049,8 +1068,6 @@ static int rats_populate_series_list (windata_t *vwin)
     return 0;
 }
 
-/* ......................................................... */
-
 #ifndef OLD_GTK
 
 static GtkWidget *database_window (windata_t *vwin) 
@@ -1257,8 +1274,6 @@ static SERIESINFO *get_series_info (windata_t *vwin, int action)
     return sinfo;
 }
 
-/* ........................................................... */
-
 static int has_rats_suffix (const char *dbname)
 {
     const char *p = strrchr(dbname, '.');
@@ -1275,8 +1290,6 @@ static int has_rats_suffix (const char *dbname)
     return ret;
 }
 
-/* ........................................................... */
-
 void open_named_db_list (char *dbname)
 {
     int action = NATIVE_SERIES;
@@ -1297,7 +1310,7 @@ void open_named_db_list (char *dbname)
 	errbox(_("Couldn't open database"));
     } else {
 	fclose(fp);
-	display_db_series_list(action, dbname, NULL);
+	make_db_series_list(action, dbname, NULL);
     } 
 }
 
@@ -1336,7 +1349,7 @@ void open_db_list (GtkWidget *w, gpointer data)
     g_free(dbdir);
 #endif
 
-    display_db_series_list(action, dbfile, NULL); 
+    make_db_series_list(action, dbfile, NULL); 
 
 #ifndef KEEP_BROWSER_OPEN
     if (vwin != NULL && vwin->w != NULL && GTK_IS_WIDGET(vwin->w)) {
@@ -1363,11 +1376,11 @@ void open_named_remote_db_list (char *dbname)
     err = retrieve_remote_db_list(dbname, &getbuf, errbuf);
 
     if (err) {
-	display_db_error(NULL, errbuf);
+	show_db_error(NULL, errbuf);
     } else if (strncmp(getbuf, "Couldn't open", 13) == 0) {
 	errbox(getbuf);
     } else {
-	display_db_series_list(REMOTE_SERIES, dbname, getbuf);
+	make_db_series_list(REMOTE_SERIES, dbname, getbuf);
 	/* check for error */
     }
 
@@ -1402,10 +1415,10 @@ void open_remote_db_list (GtkWidget *w, gpointer data)
     err = retrieve_remote_db_list(fname, &getbuf, errbuf);
 
     if (err) {
-	display_db_error(vwin, errbuf);
+	show_db_error(vwin, errbuf);
     } else {
 	update_statusline(vwin, "OK");
-	display_db_series_list(REMOTE_SERIES, fname, getbuf);
+	make_db_series_list(REMOTE_SERIES, fname, getbuf);
 	/* check for error */
     }
 
@@ -1459,7 +1472,8 @@ static int ggz_extract (char *errbuf, char *ggzname)
 {
     gzFile fgz = NULL;
     FILE *fidx = NULL, *fbin = NULL, *fcbk = NULL;
-    size_t idxlen, datalen, cblen, bytesleft, bgot;
+    size_t idxlen, datalen, bytesleft, bgot;
+    size_t cblen = 0;
     char idxname[MAXLEN], binname[MAXLEN], cbname[MAXLEN];
     char gzbuf[GRETL_BUFSIZE];
 #if G_BYTE_ORDER == G_BIG_ENDIAN
@@ -1583,8 +1597,6 @@ static int ggz_extract (char *errbuf, char *ggzname)
     return err;
 }
 
-/* ........................................................... */
-
 void grab_remote_db (GtkWidget *w, gpointer data)
 {
     gchar *dbname;
@@ -1639,7 +1651,7 @@ void grab_remote_db (GtkWidget *w, gpointer data)
 #endif
 
     if (err) {
-	display_db_error(NULL, errbuf);
+	show_db_error(NULL, errbuf);
 #ifndef OLD_GTK
 	free(dbname);
 #endif
@@ -1677,11 +1689,9 @@ void grab_remote_db (GtkWidget *w, gpointer data)
     free(ggzname);
 }
 
-/* ........................................................... */
-
 static gchar *
-db_description (const char *fullname, const char *binname, 
-		const char *dbdir)
+real_get_db_description (const char *fullname, const char *binname, 
+			 const char *dbdir)
 {
     FILE *fp;
     char idxname[FILENAME_MAX];
@@ -1723,7 +1733,7 @@ db_description (const char *fullname, const char *binname,
 
 gchar *get_db_description (const char *binname)
 {
-    return db_description(binname, NULL, NULL);
+    return real_get_db_description(binname, NULL, NULL);
 }
 
 int write_db_description (const char *binname, const char *descrip)
@@ -1788,9 +1798,8 @@ int write_db_description (const char *binname, const char *descrip)
 #ifndef OLD_GTK
 
 static int
-read_idx_files_in_dir (DIR *dir, int dbtype, const char *filter, 
-		       const char *dbdir, GtkListStore *store, 
-		       GtkTreeIter *iter)
+read_db_files_in_dir (DIR *dir, int dbtype, const char *dbdir, 
+		      GtkListStore *store, GtkTreeIter *iter)
 {
     struct dirent *dirent;
     const char *fname;
@@ -1800,9 +1809,9 @@ read_idx_files_in_dir (DIR *dir, int dbtype, const char *filter,
     while ((dirent = readdir(dir)) != NULL) {
 	fname = dirent->d_name;
 	n = strlen(fname);
-	if (g_ascii_strcasecmp(fname + n - 4, filter) == 0) {
-	    if (dbtype == NATIVE_DB) {
-		descrip = db_description(NULL, fname, dbdir);
+	if (dbtype == NATIVE_DB) {
+	    if (!g_ascii_strcasecmp(fname + n - 4, ".bin")) {
+		descrip = real_get_db_description(NULL, fname, dbdir);
 		if (descrip != NULL) {
 		    gtk_list_store_append(store, iter);
 		    gtk_list_store_set(store, iter, 0, fname, 1, descrip, 
@@ -1810,12 +1819,12 @@ read_idx_files_in_dir (DIR *dir, int dbtype, const char *filter,
 		    g_free(descrip);
 		    ndb++;
 		}
-	    } else { 
-		/* RATS database */
-		gtk_list_store_append(store, iter);
-		gtk_list_store_set(store, iter, 0, fname, 1, dbdir, -1);
-		ndb++;
 	    }
+	} else if (!g_ascii_strcasecmp(fname + n - 4, ".rat")) {
+	    /* RATS database */
+	    gtk_list_store_append(store, iter);
+	    gtk_list_store_set(store, iter, 0, fname, 1, dbdir, -1);
+	    ndb++;
 	}
     }
 
@@ -1825,9 +1834,7 @@ read_idx_files_in_dir (DIR *dir, int dbtype, const char *filter,
 #else
 
 static int
-read_idx_files_in_dir (DIR *dir, const char *filter, 
-		       char *dbdir, windata_t *vwin,
-		       int ndb)
+read_db_files_in_dir (DIR *dir, char *dbdir, windata_t *vwin, int ndb)
 {
     struct dirent *dirent;
     char *fname;
@@ -1836,31 +1843,34 @@ read_idx_files_in_dir (DIR *dir, const char *filter,
     int n, i;
 
     while ((dirent = readdir(dir)) != NULL) {
+	int addrow = 0;
+
 	fname = dirent->d_name;
 	n = strlen(fname);
-	if (g_strcasecmp(fname + n - 4, filter) != 0) {
-	    continue;
-	}
-	row[0] = fname;
 	if (vwin->role == NATIVE_DB) {
-	    descrip = db_description(NULL, fname, dbdir);
-	    if (descrip != NULL) {
-		row[1] = descrip;
-		i = gtk_clist_append(GTK_CLIST(vwin->listbox), row);
-		gtk_clist_set_row_data(GTK_CLIST(vwin->listbox), i, dbdir);
-		g_free(descrip);
-		ndb++;
-	    } else {
-		continue;
+	    if (!g_strcasecmp(fname + n - 4, ".bin")) {
+		row[0] = fname;
+		descrip = real_get_db_description(NULL, fname, dbdir);
+		if (descrip != NULL) {
+		    addrow = 1;
+		    row[1] = descrip;
+		    i = gtk_clist_append(GTK_CLIST(vwin->listbox), row);
+		    gtk_clist_set_row_data(GTK_CLIST(vwin->listbox), i, dbdir);
+		    g_free(descrip);
+		    ndb++;
+		} 
 	    }
-	} else {
+	} else if (!g_strcasecmp(fname + n - 4, ".rat")) {
 	    /* RATS database */
+	    addrow = 1;
+	    row[0] = fname;
 	    row[1] = NULL;
 	    i = gtk_clist_append(GTK_CLIST(vwin->listbox), row);
 	    gtk_clist_set_row_data(GTK_CLIST(vwin->listbox), i, dbdir);
 	    ndb++;
 	}
-	if (ndb % 2) {
+
+	if (addrow && ndb % 2) {
 	    gtk_clist_set_background(GTK_CLIST(vwin->listbox), i, &gray);
 	}
     }
@@ -1876,19 +1886,13 @@ gint populate_dbfilelist (windata_t *vwin)
     GtkListStore *store;
     GtkTreeIter iter;
 #endif
-    gchar *dbdir, *filter;
-    gchar *db_filters[] = {
-	".rat",
-	".bin"
-    };
+    gchar *dbdir;
     DIR *dir;
     int ndb = 0;
 
     if (vwin->role == RATS_DB) {
-	filter = db_filters[0];
 	dbdir = paths.ratsbase;
     } else {
-	filter = db_filters[1];
 	dbdir = paths.binbase;
     }
 
@@ -1926,9 +1930,9 @@ gint populate_dbfilelist (windata_t *vwin)
     }
 
 #ifndef OLD_GTK
-    ndb = read_idx_files_in_dir(dir, vwin->role, filter, dbdir, store, &iter);
+    ndb = read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
 #else
-    ndb = read_idx_files_in_dir(dir, filter, dbdir, vwin, 0);
+    ndb = read_db_files_in_dir(dir, dbdir, vwin, 0);
 #endif
 
     closedir(dir);
@@ -1940,9 +1944,9 @@ gint populate_dbfilelist (windata_t *vwin)
 
     if (dir != NULL) {
 # ifndef OLD_GTK
-	ndb += read_idx_files_in_dir(dir, vwin->role, filter, dbdir, store, &iter);
+	ndb += read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
 # else
-	ndb = read_idx_files_in_dir(dir, filter, dbdir, vwin, ndb);
+	ndb = read_db_files_in_dir(dir, dbdir, vwin, ndb);
 # endif
 	closedir(dir);
     }
@@ -1973,7 +1977,7 @@ static void set_compact_info_from_default (int method)
 
 void do_compact_data_set (void)
 {
-    int default_method = COMPACT_AVG;
+    CompactMethod method = COMPACT_AVG;
     int err, newpd = 0, monstart = 1;
     int *pmonstart = NULL;
 
@@ -1985,25 +1989,49 @@ void do_compact_data_set (void)
 	pmonstart = &monstart;
     }
 
-    data_compact_dialog(mdata->w, datainfo->pd, &newpd, pmonstart, &default_method);
-    if (default_method == COMPACT_NONE) {
+    data_compact_dialog(mdata->w, datainfo->pd, &newpd, pmonstart, &method);
+    if (method == COMPACT_NONE) {
 	/* the user cancelled */
 	return;
     }
 
-    err = compact_data_set(&Z, datainfo, newpd, default_method, monstart);
+    err = compact_data_set(&Z, datainfo, newpd, method, monstart);
 
     if (err) {
 	gui_errmsg(err);
     } else {
 	data_status |= MODIFIED_DATA;
 	set_sample_label(datainfo);
-
 	if (datainfo->pd == 1 || datainfo->pd == 52) {
 	    flip(mdata->ifac, "/Sample/Compact data...", FALSE);
 	}
+	set_compact_info_from_default(method);
+    }
+}
 
-	set_compact_info_from_default(default_method);
+void do_expand_data_set (void)
+{
+    int err, newpd = 0;
+
+    if (maybe_restore_full_data(EXPAND)) {
+	return;
+    }
+
+    data_expand_dialog(mdata->w, datainfo->pd, &newpd);
+    if (newpd < 0) {
+	return;
+    }
+
+    err = expand_data_set(&Z, datainfo, newpd);
+
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	data_status |= MODIFIED_DATA;
+	set_sample_label(datainfo);
+	if (datainfo->pd == 12) {
+	    flip(mdata->ifac, "/Sample/Expand data...", FALSE);
+	}
     }
 }
 
