@@ -27,6 +27,7 @@
 #include "filelists.h"
 #include "webget.h"
 #include "menustate.h"
+#include "gretl_xml.h"
 
 #if !GLIB_CHECK_VERSION(2,0,0)
 # define OLD_GTK
@@ -625,7 +626,7 @@ static void display_datafile_info (GtkWidget *w, gpointer data)
     g_free(fname);
 #endif
 
-    descrip = get_xml_description(hdrname);
+    descrip = gretl_get_gdt_description(hdrname);
 
     if (descrip != NULL) {
 	prn = gretl_print_new_with_buffer(descrip);
@@ -694,8 +695,6 @@ void browser_open_ps (GtkWidget *w, gpointer data)
     view_file(scriptfile, 0, 0, 78, 370, VIEW_SCRIPT);
 } 
 
-/* ........................................................... */
-
 static void set_browser_status (windata_t *vwin, int status)
 {
     if (status == BROWSER_BUSY) {
@@ -718,8 +717,6 @@ static gpointer get_browser_ptr (int role)
     return (gpointer) &(browsers[role - TEXTBOOK_DATA]);
 }
 
-/* ........................................................... */
-
 static void get_local_status (char *fname, char *status, time_t remtime)
 {
     char fullname[MAXLEN];
@@ -753,8 +750,6 @@ static void get_local_status (char *fname, char *status, time_t remtime)
     }
 }
 
-/* ........................................................... */
-
 static int parse_db_list_line (char *line, char *fname, time_t *date)
 {
     char mon[4], hrs[9];
@@ -795,8 +790,6 @@ static int parse_db_list_line (char *line, char *fname, time_t *date)
     return 0;
 }
 
-/* ........................................................... */
-
 static gint populate_remote_db_list (windata_t *win)
 {
 #ifndef OLD_GTK
@@ -820,7 +813,7 @@ static gint populate_remote_db_list (windata_t *win)
     err = list_remote_dbs(&buf, errbuf);
 
     if (err) {
-	display_db_error(NULL, errbuf);
+	show_db_error(NULL, errbuf);
 	free(buf);
 	return err;
     }
@@ -835,7 +828,7 @@ static gint populate_remote_db_list (windata_t *win)
     gtk_clist_freeze(GTK_CLIST(win->listbox));
 #endif
 
-    bufgets(NULL, 0, buf);
+    bufgets_init(buf);
     i = 0;
 
     while (bufgets(line, sizeof line, buf)) {
@@ -919,7 +912,7 @@ static int browser_busy (guint code)
     return ret;
 }
 
-void display_files (gpointer data, guint code, GtkWidget *widget)
+void display_files (gpointer p, guint code, GtkWidget *w)
 {
     GtkWidget *filebox, *button;
     GtkWidget *main_vbox, *button_box;
@@ -968,6 +961,7 @@ void display_files (gpointer data, guint code, GtkWidget *widget)
 	gtk_window_set_title(GTK_WINDOW(vwin->w), 
 			     _("gretl: databases on server"));
 	browse_func = open_remote_db_list;
+	gtk_widget_set_usize(vwin->w, 640, 480);
 	break;
     }
 
@@ -1030,9 +1024,6 @@ void display_files (gpointer data, guint code, GtkWidget *widget)
 			 (code == REMOTE_DB)?
 			 G_CALLBACK(grab_remote_db) :
 			 G_CALLBACK(display_datafile_info), vwin);
-    }
-
-    if (code == TEXTBOOK_DATA) {
 	button = gtk_button_new_with_label(_("Find"));
 	gtk_box_pack_start(GTK_BOX (button_box), button, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked",
@@ -1210,7 +1201,7 @@ static GtkWidget *files_window (windata_t *fdata)
     full_width *= gui_scale;
     file_height *= gui_scale;
 
-    gtk_widget_set_usize (box, full_width, file_height);
+    gtk_widget_set_usize(box, full_width, file_height);
    
     scroller = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroller),
