@@ -27,9 +27,11 @@ enum {
     HELP_BUTTON
 } buttons;
 
+#ifndef BUILDING_PLUGIN
 typedef struct dialog_t_ dialog_t;
 
-/* functions follow */
+void copy_format_dialog (windata_t *vwin, int multicopy, int action);
+#endif
 
 void errbox (const char *msg);
 
@@ -43,9 +45,7 @@ gint exit_check (GtkWidget *widget, GdkEvent *event, gpointer data);
 
 void menu_exit_check (GtkWidget *w, gpointer data);
 
-void delimiter_dialog (void);
-
-void copy_format_dialog (windata_t *vwin, int multicopy);
+void delimiter_dialog (gretlopt *optp);
 
 void varinfo_dialog (int varnum, int full);
 
@@ -55,16 +55,16 @@ int select_var_from_list (const int *list, const char *query);
 
 void sample_range_dialog (gpointer p, guint u, GtkWidget *w);
 
-void arma_options_dialog (gpointer p, guint u, GtkWidget *w);
-
 void panel_structure_dialog (DATAINFO *pdinfo);
 
 void data_compact_dialog (GtkWidget *w, int spd, int *target_pd, 
-			  int *mon_start, gint *compact_method);
+			  int *mon_start, CompactMethod *method);
+
+void data_expand_dialog (GtkWidget *w, int spd, int *target_pd);
 
 int density_dialog (int vnum, double *bw);
 
-int radio_dialog (const char *title, const char **opts, 
+int radio_dialog (const char *title, const char *label, const char **opts, 
 		  int nopts, int deflt, int helpcode);
 
 int radio_dialog_with_spinner (const char *title, const char **opts, 
@@ -97,4 +97,6 @@ void data_structure_wizard (gpointer p, guint u, GtkWidget *w);
 
 void panel_restructure_dialog (gpointer data, guint u, GtkWidget *w);
 
+void lmax_dialog (double *lmax, double ymax);
+
 #endif /* DIALOGS_H */
