@@ -27,6 +27,7 @@
 #include "textbuf.h"
 #include "textutil.h"
 #include "filelists.h"
+#include "fileselect.h"
 
 #if (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 4)
 # ifndef G_OS_WIN32
@@ -51,14 +52,18 @@
                               i == OPEN_OCTAVE || \
 	                      i == OPEN_BOX || \
                               i == OPEN_GNUMERIC || \
-	                      i == OPEN_EXCEL)
+	                      i == OPEN_EXCEL || \
+                              i == OPEN_WF1 || \
+                              i == OPEN_DTA)
 
 #define APPEND_DATA_ACTION(i) (i == APPEND_DATA || \
                                i == APPEND_CSV || \
                                i == APPEND_OCTAVE || \
                                i == APPEND_GNUMERIC || \
                                i == APPEND_EXCEL || \
-                               i == APPEND_ASCII)
+                               i == APPEND_ASCII || \
+                               i == APPEND_WF1 || \
+                               i == APPEND_DTA)
 
 #define SAVE_GRAPH_ACTION(i) (i == SAVE_GNUPLOT || \
                               i == SAVE_THIS_GRAPH || \
@@ -67,11 +72,11 @@
                               i == SAVE_BOXPLOT_PS || \
                               i == SAVE_BOXPLOT_XPM)
 
-#define EXPORT_ACTION(i) (i == EXPORT_OCTAVE || \
-                          i == EXPORT_R || \
-                          i == EXPORT_R_ALT || \
-                          i == EXPORT_CSV || \
-                          i == EXPORT_DAT)
+#define EXPORT_ACTION(a,s) ((a == EXPORT_OCTAVE || \
+                             a == EXPORT_R || \
+                             a == EXPORT_R_ALT || \
+                             a == EXPORT_CSV || \
+                             a == EXPORT_DAT) && s != FSEL_DATA_PRN)
 
 #ifdef REMEMBER_DIR
 static char remember_dir[MAXLEN];
@@ -89,10 +94,8 @@ static struct extmap action_map[] = {
     { SAVE_BIN1,         ".gdt" },
     { SAVE_BIN2,         ".gdt" },
     { SAVE_DBDATA,       ".bin" },
-    { SAVE_CMDS,         ".inp" },
     { SAVE_SCRIPT,       ".inp" },
     { SAVE_CONSOLE,      ".inp" },
-    { SAVE_MODEL,        ".txt" },
     { SAVE_SESSION,      ".gretl" },
     { SAVE_GP_CMDS,      ".plt" },
     { SAVE_BOXPLOT_EPS,  ".eps" },
@@ -107,6 +110,7 @@ static struct extmap action_map[] = {
     { EXPORT_DAT,        ".dat" },
     { SAVE_OUTPUT,       ".txt" },
     { SAVE_TEX,          ".tex" },
+    { SAVE_RTF,          ".rtf" },
     { OPEN_DATA,         ".gdt" },
     { APPEND_DATA,       ".gdt" },    
     { OPEN_SCRIPT,       ".inp" },
@@ -120,30 +124,41 @@ static struct extmap action_map[] = {
     { APPEND_GNUMERIC,   ".gnumeric" },
     { OPEN_EXCEL,        ".xls" },
     { APPEND_EXCEL,      ".xls" },
+    { OPEN_WF1,          ".wf1" },
+    { APPEND_WF1,        ".wf1" },
+    { OPEN_DTA,          ".dta" },
+    { APPEND_DTA,        ".dta" },
     { FILE_OP_MAX,       NULL }
 };
 
-/* ........................................................... */
-
-static gretlopt action_to_opt (const int action)
+static gretlopt save_action_to_opt (int action, gpointer p)
 {
+    gretlopt opt = OPT_NONE;
+
     switch (action) {
-    case SAVE_GZDATA:   return OPT_Z;
-    case SAVE_BIN1:     return OPT_S;
-    case SAVE_BIN2:     return OPT_O;
-    case SAVE_DBDATA:   return OPT_D;
-    case EXPORT_OCTAVE: return OPT_M;
-    case EXPORT_R:      return OPT_R;
-    case EXPORT_R_ALT:  return OPT_A;
-    case EXPORT_CSV:    return OPT_C;
-    case EXPORT_DAT:    return OPT_G; /* PcGive */
-    default: return 0L;
+    case SAVE_GZDATA:   opt = OPT_Z; break;
+    case SAVE_BIN1:     opt = OPT_S; break;
+    case SAVE_BIN2:     opt = OPT_O; break;
+    case SAVE_DBDATA:   opt = OPT_D; break;
+    case EXPORT_OCTAVE: opt = OPT_M; break;
+    case EXPORT_R:      opt = OPT_R; break;
+    case EXPORT_R_ALT:  opt = OPT_A; break;
+    case EXPORT_CSV:    opt = OPT_C; break;
+    case EXPORT_DAT:    opt = OPT_G; break; /* PcGive */
+    default: break;
     }
+
+    if (p != NULL) {
+	opt |= GPOINTER_TO_INT(p);
+    }
+
+    return opt;
 }
 
 static const char *get_gp_ext (const char *termtype)
 {
     if (!strncmp(termtype, "postscript", 10))    return ".eps";
+    else if (!strcmp(termtype, "PDF"))           return ".pdf";
     else if (!strcmp(termtype, "fig"))           return ".fig";
     else if (!strcmp(termtype, "latex"))         return ".tex";
     else if (!strncmp(termtype, "png", 3))       return ".png";
@@ -152,8 +167,6 @@ static const char *get_gp_ext (const char *termtype)
     else return "*";
 }
 
-/* ........................................................... */
-
 static int dat_ext (const char *str, int showerr)
 {
     const char *suff;
@@ -176,8 +189,6 @@ static int dat_ext (const char *str, int showerr)
     return err;
 }
 
-/* ........................................................... */
-
 static const char *get_ext (int action, gpointer data)
 {
     const char *s = NULL;
@@ -205,8 +216,6 @@ static const char *get_ext (int action, gpointer data)
     return s;
 }
 
-/* ........................................................... */
-
 static int check_maybe_add_ext (char *fname, int action, gpointer data)
 {
     FILE *fp;
@@ -242,8 +251,6 @@ static int check_maybe_add_ext (char *fname, int action, gpointer data)
     return 0;
 }
 
-/* ........................................................... */
-
 static void script_window_update (windata_t *vwin, const char *fname)
 {
     gchar *title;
@@ -255,20 +262,22 @@ static void script_window_update (windata_t *vwin, const char *fname)
     strcpy(vwin->fname, fname);
     g_free(title);
 
+    if (vwin->role == VIEW_LOG || vwin->role == VIEW_SCRIPT) {
+	vwin->role = EDIT_SCRIPT;
+    }
+
     /* make the window editable */
 #ifndef OLD_GTK
     if (!gtk_text_view_get_editable(GTK_TEXT_VIEW(vwin->w))) {
 	file_view_set_editable(vwin);
     }
 #else
-    if (vwin->role == VIEW_SCRIPT) {
+    if (vwin->role == EDIT_SCRIPT) {
 	file_view_set_editable(vwin);
     } 
 #endif
 }
 
-/* ........................................................... */
-
 static void save_editable_content (int action, const char *fname,
 				   windata_t *vwin)
 {
@@ -307,20 +316,15 @@ static void save_editable_content (int action, const char *fname,
 #endif
 
     g_free(buf);
-
-    infobox(_("File saved OK"));
     fclose(fp);
     
     if (action == SAVE_SCRIPT) {
 	strcpy(scriptfile, fname);
 	mkfilelist(FILE_LIST_SCRIPT, scriptfile);
-	vwin->active_var = 0; /* zero out "changed" flag */
 	script_window_update(vwin, fname);
     }
 }
 
-/* ........................................................... */
-
 static void set_startdir (char *startdir, int action)
 {
 #ifdef REMEMBER_DIR
@@ -340,7 +344,20 @@ static void set_startdir (char *startdir, int action)
 #endif
 }
 
-/* ........................................................... */
+static void filesel_save_prn_buffer (PRN *prn, const char *fname)
+{
+    FILE *fp = gretl_fopen(fname, "w");
+
+    if (fp == NULL) {
+	sprintf(errtext, _("Couldn't write to %s"), fname);
+	errbox(errtext);
+    } else {
+	const char *buf = gretl_print_get_buffer(prn);
+
+	fputs(buf, fp);
+	fclose(fp);
+    }
+}
 
 static void filesel_open_script (const char *fname)
 {
@@ -359,8 +376,6 @@ static void filesel_open_script (const char *fname)
     }
 }
 
-/* ........................................................... */
-
 static void filesel_open_session (const char *fname)
 {
     int pub = !strncmp(tryscript, paths.scriptdir, strlen(paths.scriptdir));
@@ -375,8 +390,6 @@ static void filesel_open_session (const char *fname)
     }
 }
 
-/* ........................................................... */
-
 static char *suggested_savename (const char *fname)
 {
     const char *ss = strrchr(fname, SLASH);
@@ -457,7 +470,8 @@ static void remember_this_dir (const char *fname)
 #endif
 
 static void
-file_selector_process_result (const char *in_fname, int action, gpointer data)
+file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
+			      gpointer data)
 {
     char fname[FILENAME_MAX];
 
@@ -500,7 +514,7 @@ file_selector_process_result (const char *in_fname, int action, gpointer data)
 	return;
     }
 
-    /* now for the save options */
+    /* now for the save/export options */
 
     if (action > SAVE_BIN2 && action != EXPORT_DAT && dat_ext(fname, 1)) { 
 	return;
@@ -510,21 +524,25 @@ file_selector_process_result (const char *in_fname, int action, gpointer data)
 	return;
     }
 
-    if (SAVE_DATA_ACTION(action)) {
+    if (src == FSEL_DATA_PRN) {
+	if (action == SAVE_TEX) {
+	    save_latex((PRN *) data, fname);
+	} else {
+	    filesel_save_prn_buffer((PRN *) data, fname);
+	}
+    } else if (SAVE_DATA_ACTION(action)) {
 	int overwrite = 0;
 
 	if (!strcmp(fname, paths.datfile) || action == SAVE_DBDATA) {
 	    overwrite = 1;
 	}
-	do_store(fname, action_to_opt(action), overwrite);
+	do_store(fname, save_action_to_opt(action, data), overwrite);
     } else if (action == SAVE_GNUPLOT) {
 	int err = 0;
 	GPT_SPEC *plot = (GPT_SPEC *) data;
 
 	err = go_gnuplot(plot, fname);
-	if (err == 0) {
-	    infobox(_("graph saved"));
-	} else if (err == 1) {
+	if (err == 1) {
 	    errbox(_("gnuplot command failed"));
 	} else if (err == 2) {
 	    infobox(_("There were missing observations"));
@@ -537,14 +555,11 @@ file_selector_process_result (const char *in_fname, int action, gpointer data)
 	int err;
 
 	err = ps_print_plots(fname, action, data);
-	if (!err) infobox(_("boxplots saved"));
-	else errbox(_("boxplot save failed"));
+	if (err) {
+	    errbox(_("boxplot save failed"));
+	}
     } else if (action == SAVE_SESSION) {
 	save_session(fname);
-    } else if (action == SAVE_TEX) {
-	PRN *prn = (PRN *) data;
-
-	save_latex(prn, fname);
     } else if (action == SET_PATH) {
 	char *strvar = (char *) data;
 
@@ -615,10 +630,8 @@ static struct winfilter get_filter (int action, gpointer data)
 	{SAVE_BIN1,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
 	{SAVE_BIN2,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
 	{SAVE_DBDATA,  { N_("gretl database files (*.bin)"), "*.bin" }},
-	{SAVE_CMDS,    { N_("gretl command files (*.inp)"), "*.inp" }},
 	{SAVE_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
 	{SAVE_CONSOLE, { N_("gretl command files (*.inp)"), "*.inp" }},
-	{SAVE_MODEL,   { N_("text files (*.txt)"), "*.txt" }},
 	{SAVE_SESSION, { N_("session files (*.gretl)"), "*.gretl" }},
 	{SAVE_BOXPLOT_EPS, { N_("postscript files (*.eps)"), "*.eps" }},
 	{SAVE_BOXPLOT_PS,  { N_("postscript files (*.ps)"), "*.ps" }},
@@ -632,6 +645,7 @@ static struct winfilter get_filter (int action, gpointer data)
 	{EXPORT_DAT,   { N_("PcGive files (*.dat)"), "*.dat" }},
 	{SAVE_OUTPUT,  { N_("text files (*.txt)"), "*.txt" }},
 	{SAVE_TEX,     { N_("TeX files (*.tex)"), "*.tex" }},
+	{SAVE_RTF,     { N_("RTF files (*.rtf)"), "*.rtf" }},
 	{OPEN_DATA,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
 	{APPEND_DATA,  { N_("gretl data files (*.gdt)"), "*.gdt" }},
 	{OPEN_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
@@ -645,6 +659,10 @@ static struct winfilter get_filter (int action, gpointer data)
 	{APPEND_GNUMERIC, { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
 	{OPEN_EXCEL,   { N_("Excel files (*.xls)"), "*.xls" }},
 	{APPEND_EXCEL, { N_("Excel files (*.xls)"), "*.xls" }},
+	{OPEN_WF1,     { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
+	{APPEND_WF1,   { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
+	{OPEN_DTA,     { N_("Stata files (*.dta)"), "*.dta" }},
+	{APPEND_DTA,   { N_("Stata files (*.dta)"), "*.dta" }},
 	{SET_PATH,     { N_("program files (*.exe)"), "*.exe" }}
     };
 
@@ -711,9 +729,7 @@ static char *make_winfilter (int action, gpointer data)
     return start;
 }
 
-/* ........................................................... */
-
-void file_selector (const char *msg, int action, gpointer data) 
+void file_selector (const char *msg, int action, FselDataSrc src, gpointer data) 
 {
     OPENFILENAME of;
     int retval;
@@ -735,7 +751,7 @@ void file_selector (const char *msg, int action, gpointer data)
 	if (!(data_status & BOOK_DATA)) {
 	    get_base(startdir, paths.datfile, SLASH);
 	}
-    } else if (EXPORT_ACTION(action) && paths.datfile[0]) {
+    } else if (EXPORT_ACTION(action, src) && paths.datfile[0]) {
 	char *savename = suggested_exportname(paths.datfile, action);
 
 	strcpy(fname, savename);
@@ -799,7 +815,7 @@ void file_selector (const char *msg, int action, gpointer data)
     my_filename_to_utf8(fname);
 #endif
 
-    file_selector_process_result(fname, action, data);
+    file_selector_process_result(fname, action, src, data);
 }
 
 #else /* End of MS Windows file selection code, start GTK */
@@ -843,8 +859,6 @@ static void filesel_callback (GtkWidget *w, struct fsinfo_t *fsinfo)
 
 # endif /* !USE_GTK_CHOOSER */
 
-/* ........................................................... */
-
 static char *get_filter_suffix (int action, gpointer data, char *suffix)
 {
     
@@ -875,7 +889,7 @@ static GtkFileFilter *get_file_filter (int action, gpointer data)
     return filter;
 }
 
-void file_selector (const char *msg, int action, gpointer data) 
+void file_selector (const char *msg, int action, FselDataSrc src, gpointer data) 
 {
     GtkWidget *filesel;
     char startdir[MAXLEN];
@@ -899,7 +913,9 @@ void file_selector (const char *msg, int action, gpointer data)
 					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					      NULL);
-    }	
+    }
+
+    gtk_dialog_set_default_response(GTK_DIALOG(filesel), GTK_RESPONSE_ACCEPT);
 
     filter = get_file_filter(action, data);
     gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filesel), filter);
@@ -913,7 +929,7 @@ void file_selector (const char *msg, int action, gpointer data)
 	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
 					  savename);
 	g_free(savename);
-    } else if (EXPORT_ACTION(action) && paths.datfile[0]) {
+    } else if (EXPORT_ACTION(action, src) && paths.datfile[0]) {
 	char *savename = suggested_exportname(paths.datfile, action);
 
 	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
@@ -939,7 +955,7 @@ void file_selector (const char *msg, int action, gpointer data)
 
 	fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
 	gtk_widget_destroy(filesel);
-	file_selector_process_result(fname, action, data);
+	file_selector_process_result(fname, action, src, data);
 	g_free(fname);
     } else {
 	gtk_widget_destroy(filesel);
@@ -983,7 +999,7 @@ gtk_file_selection_glob_populate (GtkFileSelection *fs,
     g_free(pattern);
 }
 
-void file_selector (const char *msg, int action, gpointer data) 
+void file_selector (const char *msg, int action, FselDataSrc src, gpointer data) 
 {
     struct fsinfo_t fsinfo;
     GtkWidget *filesel;
@@ -1011,24 +1027,18 @@ void file_selector (const char *msg, int action, gpointer data)
 					savename);
 	if (!strstr(savename, startdir)) do_glob = 0;
 	g_free(savename);
-    }
-
-    else if (EXPORT_ACTION(action) && paths.datfile[0]) {
+    } else if (EXPORT_ACTION(action, src) && paths.datfile[0]) {
 	char *savename = suggested_exportname(paths.datfile, action);
 
 	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), 
 					savename);
 	if (!strstr(savename, startdir)) do_glob = 0;
 	g_free(savename);
-    }	
-
-    else if ((action == SAVE_SESSION) && *scriptfile != '\0') {
+    } else if ((action == SAVE_SESSION) && *scriptfile != '\0') {
 	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), 
 					scriptfile);
 	if (!strstr(scriptfile, startdir)) do_glob = 0;
-    }
-
-    else if (action == SET_PATH) {
+    } else if (action == SET_PATH) {
 	char *strvar = (char *) data;
 
 	if (strvar != NULL && slashpos(strvar) > 0) {
@@ -1060,15 +1070,15 @@ void file_selector (const char *msg, int action, gpointer data)
     gtk_main(); 
 
     if (*fsinfo.fname != '\0') {
-	file_selector_process_result(fsinfo.fname, action, data);
-    }
+	file_selector_process_result(fsinfo.fname, action, src, data);
+    } 
 }
 
 # endif
 
 # else /* gtk version diffs continue */
 
-void file_selector (const char *msg, int action, gpointer data) 
+void file_selector (const char *msg, int action, FselDataSrc src, gpointer data) 
 {
     struct fsinfo_t fsinfo;
     GtkWidget *filesel;
@@ -1108,9 +1118,7 @@ void file_selector (const char *msg, int action, gpointer data)
 	    gotdir = 1;
 	}
 	g_free(savename);
-    }
-
-    else if (EXPORT_ACTION(action) && paths.datfile[0]) {
+    } else if (EXPORT_ACTION(action, src) && paths.datfile[0]) {
 	char *savename = suggested_exportname(paths.datfile, action);
 	char startd[MAXLEN];
 
@@ -1121,9 +1129,7 @@ void file_selector (const char *msg, int action, gpointer data)
 	    gotdir = 1;
 	}
 	g_free(savename);
-    }	
-
-    else if (action == SET_PATH) {
+    } else if (action == SET_PATH) {
 	char *strvar = (char *) data;
 	char startd[MAXLEN];
 
@@ -1153,11 +1159,13 @@ void file_selector (const char *msg, int action, gpointer data)
     gtk_main(); 
 
     if (*fsinfo.fname != '\0') {
-	file_selector_process_result(fsinfo.fname, action, data);
-    }
+	file_selector_process_result(fsinfo.fname, action, src, data);
+    } 
 }
 
 # endif /* old gtk */
 
 #endif /* end of non-MS Windows code */
 
+
+
