@@ -20,6 +20,7 @@
 /* gretl.c : main for gretl */
 
 #include "gretl.h"
+#include "version.h"
 #include "treeutils.h"
 #include "ssheet.h"
 #include "console.h"
@@ -31,6 +32,8 @@
 #include "filelists.h"
 #include "toolbar.h"
 #include "menustate.h"
+#include "fileselect.h"
+#include "gretl_xml.h"
 
 #include <dirent.h>
 
@@ -113,26 +116,27 @@ float gui_scale;
 int expert = FALSE; 
 int updater = FALSE;
 int want_toolbar = TRUE;
+int winsize = FALSE;
+int main_x = -1;
+int main_y = -1;
+int mainwin_width = 520;
+int mainwin_height = 420;
 
 char dbproxy[21];
 
 #ifdef G_OS_WIN32
 char Rcommand[MAXSTR] = "RGui.exe";
 char calculator[MAXSTR] = "calc.exe";
-char latex[MAXSTR] = "latex.exe";
+char latex[MAXSTR] = "pdflatex.exe";
 char viewdvi[MAXSTR] = "windvi.exe";
-char viewps[MAXSTR] = "gsview.exe";
 #else
 char calculator[MAXSTR] = "xcalc";
 char latex[MAXSTR] = "latex";
 char viewdvi[MAXSTR] = "xdvi";
 char viewps[MAXSTR] = "gv";
-# ifdef USE_GNOME
-char Rcommand[MAXSTR] = "R --gui=gnome";
-extern const char *version_string;
-# else
+char viewpdf[MAXSTR] = "acroread";
 char Rcommand[MAXSTR] = "xterm -e R";
-# endif
+char Browser[MAXSTR] = "mozilla";
 #endif
 
 #ifdef HAVE_TRAMO
@@ -150,7 +154,7 @@ static void manual_update_query (gpointer p, guint u, GtkWidget *w)
     update_query();
 }
 
-#ifdef USE_GNOME
+#ifdef USE_GNOME_HELP
 static void gnome_help (void)
 {
     GError *error = NULL;
@@ -164,23 +168,6 @@ static void gnome_help (void)
 }
 #endif 
 
-#ifdef OSX_PKG
-static void osx_help (void)
-{
-    char *prefix, *syscmd;
-   
-    prefix = getenv("GTK_EXE_PREFIX");
-    if (prefix == NULL) {
-        errbox("Couldn't find the manual");
-	return;
-    }
-    
-    syscmd = g_strdup_printf("%s/bin/manual.sh", prefix);
-    system(syscmd);
-    g_free(syscmd);
-}
-#endif 
-
 #ifndef G_OS_WIN32
 static void root_check (void)
 {
@@ -204,6 +191,13 @@ static void varinfo_callback (gpointer p, guint u, GtkWidget *w)
     varinfo_dialog(mdata->active_var, 1);
 }
 
+#ifdef ENABLE_MAILER
+static void email_data (gpointer p, guint u, GtkWidget *w)
+{
+    send_file(paths.datfile);
+}
+#endif
+
 GtkItemFactoryEntry data_items[] = {
 
     /* File menu */
@@ -215,22 +209,20 @@ GtkItemFactoryEntry data_items[] = {
       "<StockItem>", GTK_STOCK_OPEN },
     { N_("/File/Open data/sample file..."), "", display_files, TEXTBOOK_DATA, 
       "<StockItem>", GTK_STOCK_OPEN },
-    { N_("/File/Open data/sep1"), NULL, NULL, 0, "<Separator>", GNULL },    
-    { N_("/File/Open data/import CSV..."), NULL, open_data, OPEN_CSV, NULL, GNULL },
-    { N_("/File/Open data/import ASCII..."), NULL, open_data, OPEN_ASCII, NULL, GNULL },
-    { N_("/File/Open data/import Octave..."), NULL, open_data, OPEN_OCTAVE, NULL, GNULL },
-    { N_("/File/Open data/import BOX..."), NULL, open_data, OPEN_BOX, NULL, GNULL },
+    { N_("/File/Open data/sep1"), NULL, NULL, 0, "<Separator>", GNULL }, 
+    { N_("/File/Open data/Import"), NULL, NULL, 0, "<Branch>", GNULL },
+    { N_("/File/Open data/Import/CSV..."), NULL, open_data, OPEN_CSV, NULL, GNULL },
+    { N_("/File/Open data/Import/ASCII..."), NULL, open_data, OPEN_ASCII, NULL, GNULL },
+    { N_("/File/Open data/Import/Octave..."), NULL, open_data, OPEN_OCTAVE, NULL, GNULL },
 #ifdef G_OS_WIN32
-    { N_("/File/Open data/import Excel..."), NULL, open_data, 
-      OPEN_EXCEL, NULL, GNULL },
-    { N_("/File/Open data/import Gnumeric..."), NULL, open_data, 
-      OPEN_GNUMERIC, NULL, GNULL },
+    { N_("/File/Open data/Import/Excel..."), NULL, open_data, OPEN_EXCEL, NULL, GNULL },
+    { N_("/File/Open data/Import/Gnumeric..."), NULL, open_data, OPEN_GNUMERIC, NULL, GNULL },
 #else
-    { N_("/File/Open data/import Gnumeric..."), NULL, open_data, 
-      OPEN_GNUMERIC, NULL, GNULL },
-    { N_("/File/Open data/import Excel..."), NULL, open_data, 
-      OPEN_EXCEL, NULL, GNULL },
+    { N_("/File/Open data/Import/Gnumeric..."), NULL, open_data, OPEN_GNUMERIC, NULL, GNULL },
+    { N_("/File/Open data/Import/Excel..."), NULL, open_data, OPEN_EXCEL, NULL, GNULL },
 #endif
+    { N_("/File/Open data/Import/Eviews..."), NULL, open_data, OPEN_WF1, NULL, GNULL },
+    { N_("/File/Open data/Import/Stata..."), NULL, open_data, OPEN_DTA, NULL, GNULL },
 
     /* File, Append data */
     { N_("/File/_Append data"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -238,10 +230,10 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Append data/from CSV..."), NULL, open_data, APPEND_CSV, NULL, GNULL },
     { N_("/File/Append data/from ASCII..."), NULL, open_data, APPEND_ASCII, NULL, GNULL },
     { N_("/File/Append data/from Octave..."), NULL, open_data, APPEND_OCTAVE, NULL, GNULL },
-    { N_("/File/Append data/from Gnumeric..."), NULL, open_data, 
-      APPEND_GNUMERIC, NULL, GNULL },
-    { N_("/File/Append data/from Excel..."), NULL, open_data, 
-      APPEND_EXCEL, NULL, GNULL },
+    { N_("/File/Append data/from Gnumeric..."), NULL, open_data, APPEND_GNUMERIC, NULL, GNULL },
+    { N_("/File/Append data/from Excel..."), NULL, open_data, APPEND_EXCEL, NULL, GNULL },
+    { N_("/File/Append data/from Eviews..."), NULL, open_data, APPEND_WF1, NULL, GNULL },
+    { N_("/File/Append data/from Stata..."), NULL, open_data, APPEND_DTA, NULL, GNULL },
 
     /* File, Save data */
     { N_("/File/_Save data"), "<control>S", auto_store, 0, "<StockItem>", GTK_STOCK_SAVE },
@@ -257,22 +249,21 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/_Export data"), NULL, NULL, 0, "<Branch>", GNULL },
     { N_("/File/Export data/_CSV..."), NULL, file_save, EXPORT_CSV, NULL, GNULL },
     { N_("/File/Export data/GNU _R..."), NULL, file_save, EXPORT_R, NULL, GNULL },
-    { N_("/File/Export data/GNU _octave..."), NULL, file_save, 
-      EXPORT_OCTAVE, NULL, GNULL },
+    { N_("/File/Export data/GNU _octave..."), NULL, file_save, EXPORT_OCTAVE, NULL, GNULL },
     { N_("/File/Export data/_PcGive..."), NULL, file_save, EXPORT_DAT, NULL, GNULL },
-    { N_("/File/C_lear data set"), NULL, verify_clear_data, 0, 
-      "<StockItem>", GTK_STOCK_CLEAR },
+#ifdef ENABLE_MAILER
+    { N_("/File/Send To..."), NULL, email_data, OPEN_DATA, "<StockItem>", GRETL_STOCK_MAIL },
+#endif
+    { N_("/File/C_lear data set"), NULL, verify_clear_data, 0, "<StockItem>", GTK_STOCK_CLEAR },
     { N_("/File/sep0"), NULL, NULL, 0, "<Separator>", GNULL },
 
     /* File, Browse databases */
     { N_("/File/_Browse databases"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/File/Browse databases/_gretl native"), NULL, display_files, 
-      NATIVE_DB, NULL, GNULL },
-    { N_("/File/Browse databases/_RATS 4"), NULL, display_files, 
-      RATS_DB, NULL, GNULL },
+    { N_("/File/Browse databases/_gretl native"), NULL, display_files, NATIVE_DB, NULL, GNULL },
+    { N_("/File/Browse databases/_RATS 4"), NULL, display_files, RATS_DB, NULL, GNULL },
     { N_("/File/Browse databases/sep1"), NULL, NULL, 0, "<Separator>", GNULL },
-    { N_("/File/Browse databases/on database _server"), NULL, display_files, 
-      REMOTE_DB, NULL, GNULL },
+    { N_("/File/Browse databases/on database _server"), NULL, display_files, REMOTE_DB, 
+      "<StockItem>", GTK_STOCK_NETWORK },
 
     /* File, Create dataset */
     { N_("/File/_Create data set"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -428,6 +419,8 @@ GtkItemFactoryEntry data_items[] = {
       add_logs_etc, DIFF, NULL, GNULL },
     { N_("/Data/Add variables/log differences of selected variables"), NULL, 
       add_logs_etc, LDIFF, NULL, GNULL },
+    { N_("/Data/Add variables/seasonal differences of selected variables"), NULL, 
+      add_logs_etc, SDIFF, NULL, GNULL },
     { N_("/Data/Add variables/periodic dummies"), NULL, add_dummies, 0, NULL, GNULL },
     { N_("/Data/Add variables/unit dummies"), NULL, add_dummies, 1, NULL, GNULL },
     { N_("/Data/Add variables/panel dummies"), NULL, add_dummies, 2, NULL, GNULL },
@@ -452,6 +445,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Sample/sep1"), NULL, NULL, 0, "<Separator>", NULL }, 
     { N_("/Sample/Dataset structure..."), NULL, data_structure_wizard, 0, NULL, GNULL },
     { N_("/Sample/Compact data..."), NULL, do_compact_data_set, 0, NULL, GNULL },
+    { N_("/Sample/Expand data..."), NULL, do_expand_data_set, 0, NULL, GNULL },
     { N_("/Sample/sep2"), NULL, NULL, 0, "<Separator>", NULL },   
     { N_("/Sample/_Define, based on dummy..."), NULL, sample_range_dialog, 
       SMPLDUM, NULL, GNULL },
@@ -495,7 +489,6 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Variable/Spectrum/Bartlett lag window"), NULL, do_pergm, 1, NULL, GNULL }, 
     { N_("/Variable/_Augmented Dickey-Fuller test"), NULL, unit_root_test, ADF, NULL, GNULL },
     { N_("/Variable/_KPSS test"), NULL, unit_root_test, KPSS, NULL, GNULL },
-    { N_("/Variable/ARMA model"), NULL, arma_options_dialog, 0, NULL, GNULL },
 #ifdef HAVE_X12A
     { N_("/Variable/X-12-ARIMA analysis"), NULL, do_tramo_x12a, X12A, NULL, GNULL },
 #endif
@@ -536,28 +529,30 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/_Rank correlation..."), NULL, selector_callback, SPEARMAN, NULL, GNULL },
     { N_("/Model/_Pooled OLS (panel)..."), NULL, model_callback, POOLED, NULL, GNULL },
     { N_("/Model/Nonlinear Least Squares..."), NULL, gretl_callback, NLS, NULL, GNULL },
+    { N_("/Model/Maximum likelihood..."), NULL, gretl_callback, MLE, NULL, GNULL },
+    { N_("/Model/Simultaneous equations..."), NULL, gretl_callback, SYSTEM, NULL, GNULL },
 #ifdef ENABLE_GMP
     { N_("/Model/High precision OLS..."), NULL, mp_ols_callback, MPOLS, NULL, GNULL },
 #endif
 
     /* Help menu */
     { N_("/_Help"), NULL, NULL, 0, "<LastBranch>", NULL },
-    { N_("/Help/_GUI commands"), NULL, do_gui_help, 0, NULL, GNULL },
-    { N_("/Help/_Script commands syntax"), NULL, do_script_help, 1, NULL, GNULL },
-    { N_("/Help/sep1"), NULL, NULL, 0, "<Separator>", NULL },
-#if defined(USE_GNOME)
-    { N_("/Help/Manual in HTML"), NULL, gnome_help, 0, NULL, GNULL },
-    { N_("/Help/sep2"), NULL, NULL, 0, "<Separator>", GNULL },
-#elif defined(G_OS_WIN32)
-    { N_("/Help/Manual in HTML"), NULL, win_help, 0, NULL, GNULL },
-    { N_("/Help/sep2"), NULL, NULL, 0, "<Separator>", GNULL },
-#elif defined(OSX_PKG)
-    { N_("/Help/Manual in PDF"), NULL, osx_help, 0, NULL, GNULL },
-    { N_("/Help/sep2"), NULL, NULL, 0, "<Separator>", GNULL },
-#endif
-    { N_("/Help/_Check for updates"), NULL, manual_update_query, 0, NULL, GNULL },
-    { N_("/Help/sep3"), NULL, NULL, 0, "<Separator>", NULL },
+    { N_("/Help/_Command reference"), NULL, NULL, 0, "<Branch>", NULL },
+    { N_("/Help/Command reference/plain text"), NULL, plain_text_cmdref, 0, "<StockItem>", 
+      GRETL_STOCK_BOOK },
+    { N_("/Help/Command reference/PDF"), NULL, display_pdf_help, 0, "<StockItem>", 
+      GRETL_STOCK_PDF },
+    { N_("/Help/_User's guide"), NULL, display_pdf_help, 1, "<StockItem>", 
+      GRETL_STOCK_PDF },
+    { N_("/Help/sep1"), NULL, NULL, 0, "<Separator>", GNULL },
+    { N_("/Help/Check for _updates"), NULL, manual_update_query, 0, "<StockItem>", 
+      GTK_STOCK_NETWORK },
+    { N_("/Help/sep2"), NULL, NULL, 0, "<Separator>", NULL },
+#if GTK_MINOR_VERSION >= 6
+    { N_("/Help/_About gretl"), NULL, about_dialog, 0, "<StockItem>", GTK_STOCK_ABOUT }
+#else
     { N_("/Help/_About gretl"), NULL, about_dialog, 0, NULL, GNULL }
+#endif
 };
 
 static void gui_usage (void)
@@ -697,12 +692,12 @@ void nls_init (void)
 static void force_language (int f)
 {
     if (f == ENGLISH) {
-	setlocale (LC_ALL, "C");
-    } else {
+	setlocale(LC_ALL, "C");
+    } else if (f == BASQUE) {
 # ifdef G_OS_WIN32
-	setlocale (LC_ALL, "eu");
+	setlocale(LC_ALL, "eu");
 #else
-	setlocale (LC_ALL, "eu_ES");
+	setlocale(LC_ALL, "eu_ES");
 # endif
     }
 
@@ -711,7 +706,7 @@ static void force_language (int f)
 	SetEnvironmentVariable("LC_ALL", "C");
 	putenv("LC_ALL=C");
 	textdomain("none");
-    } else {
+    } else if (f == BASQUE) {
 	SetEnvironmentVariable("LC_ALL", "eu");
 	putenv("LC_ALL=eu");
     }
@@ -724,6 +719,21 @@ static void force_language (int f)
 
 #endif /* ENABLE_NLS */
 
+#ifndef G_OS_WIN32
+static void record_filearg (char *targ, const char *src)
+{
+    if (*src == '.') {
+	gchar *cdir = g_get_current_dir();
+	gchar *tmp = g_build_filename(cdir, src, NULL);
+	
+	strcpy(targ, tmp);
+	g_free(cdir);
+	g_free(tmp);
+    } else {
+	strcpy(targ, src);
+    }
+}
+#endif
 
 int main (int argc, char *argv[])
 {
@@ -758,7 +768,7 @@ int main (int argc, char *argv[])
 
     /* Initialize gnome or GTK */
 #ifdef USE_GNOME
-    program = gnome_program_init ("gretl", version_string,
+    program = gnome_program_init ("gretl", GRETL_VERSION,
 				  LIBGNOMEUI_MODULE, argc, argv,
 				  GNOME_PARAM_POPT_TABLE, options,
 				  GNOME_PARAM_HUMAN_READABLE_NAME,
@@ -853,9 +863,9 @@ int main (int argc, char *argv[])
     models = malloc(3 * sizeof *models);
     if (models == NULL) noalloc(_("models")); 
 
-    models[0] = gretl_model_new();
-    models[1] = gretl_model_new();
-    models[2] = gretl_model_new();
+    models[0] = gretl_model_new_protected();
+    models[1] = gretl_model_new_protected();
+    models[2] = gretl_model_new_protected();
 
     if (models[0] == NULL || models[1] == NULL || models[2] == NULL) {
 	noalloc(_("models")); 
@@ -882,9 +892,10 @@ int main (int argc, char *argv[])
 	    exit(EXIT_FAILURE);
 	}
 #else
-	strcpy(paths.datfile, filearg);
+	record_filearg(paths.datfile, filearg);
 #endif
-	/* record the name the user supplied */
+
+	/* keep a copy of input filename */
 	strcpy(trydatfile, paths.datfile);
 
 	ftype = detect_filetype(paths.datfile, &paths, prn);
@@ -895,8 +906,8 @@ int main (int argc, char *argv[])
 				 prn);
 	    break;
 	case GRETL_XML_DATA:
-	    err = get_xmldata(&Z, &datainfo, paths.datfile, &paths, DATA_NONE, 
-			      prn, 0);
+	    err = gretl_read_gdt(&Z, &datainfo, paths.datfile, &paths, DATA_NONE, 
+				 prn, 0);
 	    break;
 	case GRETL_CSV_DATA:
 	    err = import_csv(&Z, &datainfo, paths.datfile, prn);
@@ -909,6 +920,8 @@ int main (int argc, char *argv[])
 	    break;
 	case GRETL_EXCEL:
 	case GRETL_GNUMERIC:
+	case GRETL_WF1:
+	case GRETL_DTA:
 	    err = get_worksheet_data(paths.datfile, ftype, 0, &gui_get_data);
 	    break;
 	case GRETL_SCRIPT:
@@ -950,6 +963,7 @@ int main (int argc, char *argv[])
 
     /* create the GUI */
     gretl_tooltips_init();
+    gretl_stock_icons_init();
 
     /* create main window */
     if ((mdata = mymalloc(sizeof *mdata)) == NULL)
@@ -980,6 +994,7 @@ int main (int argc, char *argv[])
 
     if (!gui_get_data) {
 	register_data(paths.datfile, trydatfile, 1);
+	maybe_display_string_table();
 	*trydatfile = 0;
     }
 
@@ -1012,9 +1027,9 @@ int main (int argc, char *argv[])
 	free_Z(Z, datainfo);
     }
 
-    free_model(models[0]);
-    free_model(models[1]);
-    free_model(models[2]);
+    gretl_model_free_on_exit(models[0]);
+    gretl_model_free_on_exit(models[1]);
+    gretl_model_free_on_exit(models[2]);
     free(models);
 
     library_command_free();
@@ -1032,8 +1047,6 @@ int main (int argc, char *argv[])
     return EXIT_SUCCESS;
 }
 
-/* ........................................................... */
-
 static void check_varmenu_state (GtkTreeSelection *select, gpointer p)
 {
     if (mdata->ifac != NULL) {
@@ -1049,8 +1062,6 @@ static void check_varmenu_state (GtkTreeSelection *select, gpointer p)
     }
 }
 
-/* ........................................................... */
-
 static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 {
 #if defined(HAVE_FLITE) || defined(G_OS_WIN32)
@@ -1086,8 +1097,6 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
     return FALSE;
 }
 
-/* ........................................................... */
-
 void populate_varlist (void)
 {
     GtkListStore *store;
@@ -1105,7 +1114,10 @@ void populate_varlist (void)
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
     for (i=0; i<datainfo->v; i++) {
-	if (is_hidden_variable(i, datainfo)) continue;
+	if (is_hidden_variable(i, datainfo) ||
+	    is_standard_lag(i, datainfo)) {
+	    continue;
+	}
 	gtk_list_store_append(store, &iter);
 	sprintf(id, "%d", i);
 	gtk_list_store_set (store, &iter, 
@@ -1150,8 +1162,6 @@ void populate_varlist (void)
     }
 }
 
-/* ......................................................... */
-
 static gint 
 compare_var_ids (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 		 gpointer p)
@@ -1205,8 +1215,6 @@ static void sort_varlist (gpointer p, guint col, GtkWidget *w)
 					  col, GTK_SORT_ASCENDING);
 }
 
-/* ......................................................... */
-
 void clear_varlist (GtkWidget *widget)
 {
     GtkListStore *store;
@@ -1215,8 +1223,6 @@ void clear_varlist (GtkWidget *widget)
     gtk_list_store_clear(store);
 }
 
-/* ......................................................... */
-
 static float get_gui_scale (void)
 {
     GtkSettings *settings;
@@ -1230,7 +1236,7 @@ static float get_gui_scale (void)
 
     if (fontname != NULL) {
 	if (sscanf(fontname, "%*s %d", &fsize) == 1) {
-	    if (fsize > 10.0) {
+	    if (fsize > 10 && fsize < 100) {
 		scale = fsize / 10.0;
 	    }
 	}
@@ -1240,7 +1246,16 @@ static float get_gui_scale (void)
     return scale;
 }
 
-/* ......................................................... */
+static gboolean 
+mainwin_config (GtkWidget *w, GdkEventConfigure *event, gpointer p)
+{
+    mainwin_width = event->width;
+    mainwin_height = event->height;
+
+    gdk_window_get_root_origin(mdata->w->window, &main_x, &main_y);
+
+    return FALSE;
+}
 
 static GtkWidget *make_main_window (int gui_get_data) 
 {
@@ -1251,13 +1266,14 @@ static GtkWidget *make_main_window (int gui_get_data)
 	_("Variable name"), 
 	_("Descriptive label")
     };
-    int mainwin_width = 580;  
-    int mainwin_height = 420; 
 
     gui_scale = get_gui_scale();
 
-    mainwin_width *= gui_scale;
-    mainwin_height *= gui_scale;
+    if (!winsize || mainwin_width <= 200 || mainwin_height <= 200) {
+	/* set default window size */
+	mainwin_width = 580 * gui_scale;
+	mainwin_height = 420 * gui_scale;
+    }
 
     mdata->data = NULL;  
     mdata->listbox = NULL;
@@ -1275,13 +1291,15 @@ static GtkWidget *make_main_window (int gui_get_data)
     set_up_windows_look();
 #endif
 
+    g_signal_connect(G_OBJECT(mdata->w), "configure_event",
+		     G_CALLBACK(mainwin_config), NULL);
     g_signal_connect(G_OBJECT(mdata->w), "delete_event",
 		     G_CALLBACK(exit_check), NULL);
     g_signal_connect(G_OBJECT(mdata->w), "destroy",
 		     G_CALLBACK(destroy), NULL);
 
-    gtk_window_set_title(GTK_WINDOW (mdata->w), "gretl");
-    gtk_window_set_default_size(GTK_WINDOW (mdata->w), 
+    gtk_window_set_title(GTK_WINDOW(mdata->w), "gretl");
+    gtk_window_set_default_size(GTK_WINDOW(mdata->w), 
 				mainwin_width, mainwin_height);
 #ifndef G_OS_WIN32
     g_signal_connect_after(G_OBJECT(mdata->w), "realize", 
@@ -1309,13 +1327,13 @@ static GtkWidget *make_main_window (int gui_get_data)
 
     g_object_set_data(G_OBJECT(mdata->w), "dlabel", dlabel);
 
-    box = gtk_vbox_new (FALSE, 0);
+    box = gtk_vbox_new(FALSE, 0);
     align = gtk_alignment_new(0, 0, 0, 0);
     gtk_box_pack_start(GTK_BOX(box), align, FALSE, FALSE, 0);
     gtk_widget_show(align);
     gtk_container_add(GTK_CONTAINER(align), dlabel);
    
-    mdata->listbox = list_box_create (mdata, GTK_BOX(box), 3, 0, titles);
+    mdata->listbox = list_box_create(mdata, GTK_BOX(box), 3, 0, titles);
     gtk_widget_show(box);
 
     gtk_drag_dest_set (mdata->listbox,
@@ -1331,7 +1349,7 @@ static GtkWidget *make_main_window (int gui_get_data)
 
     mdata->status = gtk_label_new("");
     
-    gtk_box_pack_start (GTK_BOX(main_vbox), mdata->status, FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(main_vbox), mdata->status, FALSE, TRUE, 0);
 
     /* put stuff into list box, activate menus */
     if (!gui_get_data) {
@@ -1351,30 +1369,36 @@ static GtkWidget *make_main_window (int gui_get_data)
     /* create gretl toolbar? */
     show_or_hide_toolbar(want_toolbar);
 
+    if (winsize && main_x >= 0 && main_y >= 0) {
+	gtk_window_move(GTK_WINDOW(mdata->w), main_x, main_y);
+    }
+
+#ifdef G_OS_WIN32
+    if (wimp) {
+	menu_font_option_off();
+    }
+#endif
+
     return main_vbox;
 }
 
-/* ........................................................... */
-
 static void set_up_main_menu (void)
 {
     GtkAccelGroup *accel_group;
     gint n_items = sizeof data_items / sizeof data_items[0];
 
-    accel_group = gtk_accel_group_new ();
-    mdata->ifac = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", 
-					accel_group);
-    gtk_window_add_accel_group (GTK_WINDOW (mdata->w), accel_group);
-    g_object_unref (accel_group);
+    accel_group = gtk_accel_group_new();
+    mdata->ifac = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
+				       accel_group);
+    gtk_window_add_accel_group(GTK_WINDOW(mdata->w), accel_group);
+    g_object_unref(accel_group);
 #ifdef ENABLE_NLS
     gtk_item_factory_set_translate_func(mdata->ifac, menu_translate, NULL, NULL);
 #endif    
-    gtk_item_factory_create_items (mdata->ifac, n_items, data_items, NULL);
-    mdata->mbar = gtk_item_factory_get_widget (mdata->ifac, "<main>");
+    gtk_item_factory_create_items(mdata->ifac, n_items, data_items, NULL);
+    mdata->mbar = gtk_item_factory_get_widget(mdata->ifac, "<main>");
 }
 
-/* ........................................................... */
-
 int restore_sample (gretlopt opt)
 {
     int err;
@@ -1394,7 +1418,6 @@ static void restore_sample_callback (gpointer p, int verbose, GtkWidget *w)
     int err = restore_sample(OPT_C); 
 
     if (verbose && !err) {
-	infobox(_("Full sample range restored"));
 	set_sample_label(datainfo);    
 	gretl_command_strcpy("smpl full");
 	check_and_record_command();
@@ -1406,8 +1429,6 @@ static void startRcallback (gpointer p, guint opt, GtkWidget *w)
     startR(Rcommand);
 }
 
-/* ........................................................... */
-
 #ifndef G_OS_WIN32
 
 int gretl_fork (const char *prog, const char *arg)
@@ -1473,7 +1494,7 @@ drag_data_received  (GtkWidget *widget,
     }
 
     /* ignore the wrong sort of data */
-    if (data == NULL || (dfname = data->data) == NULL || 
+    if (data == NULL || (dfname = (gchar *) data->data) == NULL || 
 	strlen(dfname) <= 5 || strncmp(dfname, "file:", 5))
 	return;
 
@@ -1540,14 +1561,12 @@ static void auto_store (void)
     if ((data_status & USER_DATA) && native_datafile()) {
 	do_store(paths.datfile, oflag, 1);
     } else {
-	file_selector(_("Save data file"), SAVE_DATA, NULL);
+	file_selector(_("Save data file"), SAVE_DATA, FSEL_DATA_NONE, NULL);
     }	
 }
 
-/* ........................................................... */
-
 static void get_selection_row (GtkTreeModel *model, GtkTreePath *path,
-			      GtkTreeIter *iter, int *row)
+			       GtkTreeIter *iter, int *row)
 {
     gint *indices = gtk_tree_path_get_indices(path);
     *row = indices[0];
