@@ -92,13 +92,20 @@ extern char *errtext;
 /* global option-related vars */
 extern int expert;
 extern int updater;
+extern int winsize;
+extern int main_x;
+extern int main_y;
+extern int mainwin_width;
+extern int mainwin_height;
 #ifdef G_OS_WIN32
 extern int wimp;
+#else
+extern char viewps[MAXSTR];
+extern char viewpdf[MAXSTR];
 #endif
 extern char calculator[MAXSTR];
 extern char latex[MAXSTR];
 extern char viewdvi[MAXSTR];
-extern char viewps[MAXSTR];
 #if defined(HAVE_AUDIO) && !defined(G_OS_WIN32)
 extern char midiplayer[MAXSTR];
 #endif
@@ -118,6 +125,7 @@ extern PangoFontDescription *fixed_font;
 
 #ifndef WIN32
 int gretl_fork (const char *prog, const char *arg);
+void set_wm_icon (GtkWidget *w, gpointer data);
 #endif
 
 int mdata_selection_count (void);
@@ -126,18 +134,7 @@ void clear_varlist (GtkWidget *widget);
 int restore_sample (gretlopt opt);
 
 /* functions defined in files other than gretl.c */
-
 void about_dialog (gpointer data);
-void file_selector (const char *msg, int action, gpointer data);
-void gui_get_series (gpointer data, guint bci_code, GtkWidget *widget);
-void import_db_series (windata_t *dbwin);
-void display_files (gpointer data, guint code, GtkWidget *widget);
-void stats_calculator (gpointer data, guint code, GtkWidget *widget);
-
-#ifndef WIN32
-void set_wm_icon (GtkWidget *w, gpointer data);
-#else
-int create_child_process (char *prog, char *env);
-#endif
+void stats_calculator (gpointer p, guint code, GtkWidget *w);
 
 #endif /* GRETL_H */
