@@ -28,6 +28,7 @@ enum extra_cmds {
     GR_NBOX,
     GR_3D,
     COMPACT,
+    EXPAND, 
     COEFFINT,
     COVAR,
     STAT_TABLE,
@@ -51,8 +52,8 @@ enum extra_cmds {
     EDIT_NOTES,
     CLI_HELP,
     GUI_HELP,
-    CLI_HELP_ENGLISH,
-    GUI_HELP_ENGLISH,
+    CLI_HELP_EN,
+    GUI_HELP_EN,
     MODELTABLE,
     GRAPHPAGE,
     CREATE_USERDIR,
@@ -64,6 +65,15 @@ enum extra_cmds {
     KERNEL_DENSITY,
     CREATE_DATASET,
     HCCME,
+    VAR_VCV,
+    VAR_IRF,
+    VAR_DECOMP,
+    IRF_BOOT,
+    HTEST, 
+    MODEL_RESTR,
+    SYS_RESTR,
+    VECM_RESTR,
+    ELLIPSE,
     GUI_CMD_MAX
 };
 
@@ -84,6 +94,10 @@ enum file_ops {
     APPEND_GNUMERIC,
     OPEN_EXCEL,
     APPEND_EXCEL,
+    OPEN_WF1,
+    APPEND_WF1,
+    OPEN_DTA,
+    APPEND_DTA,
     OPEN_SESSION,
     OPEN_MARKERS,
     END_OPEN,      /* marker for end of file open section */
@@ -100,12 +114,11 @@ enum file_ops {
     EXPORT_DAT,
     COPY_CSV,
     END_SAVE_DATA,  /* marker for end of data-saving section */
-    SAVE_CMDS,
     SAVE_TEX,
+    SAVE_RTF,
     SAVE_SCRIPT,
     SAVE_OUTPUT,
     SAVE_SESSION,
-    SAVE_MODEL,
     SAVE_GNUPLOT,
     SAVE_BOXPLOT_EPS,
     SAVE_BOXPLOT_PS,
@@ -215,6 +228,7 @@ enum dynamic_forecast_status {
 	                         c == CORR || c == FCASTERR || \
 	                         c == FCAST || c == COEFFINT || \
 	                         c == COVAR || c == VIEW_MODEL || \
-                                 c == VIEW_MODELTABLE || c == VAR)
+                                 c == VIEW_MODELTABLE || c == VAR || c == VECM || \
+                                 c == VAR_IRF || c == VAR_DECOMP)
 
 #endif /* GRETL_ENUMS_H */
