@@ -24,6 +24,8 @@
 #include "textutil.h"
 
 #include <dirent.h>
+#include <mapi.h>
+#include <shlobj.h>
 
 #define HUSH_RUNTIME_WARNINGS
 
@@ -329,6 +331,13 @@ static void try_to_get_windows_font (void)
     }
 }
 
+void menu_font_option_off (void)
+{
+    if (mdata->ifac != NULL) {
+	flip(mdata->ifac, "/File/Preferences/Menu font...", FALSE);
+    }
+}
+
 void set_up_windows_look (void)
 {
     if (wimp) { 
@@ -581,3 +590,83 @@ int win_buf_to_clipboard (const char *buf)
 
     return 0;
 }
+
+static char *fname_from_fullname (char *fullname)
+{
+    char *fname = fullname;
+    char *p;
+
+    p = strrchr(fullname, '\\');
+    if (p == NULL) {
+	p = strrchr(fullname, '/');
+    }
+
+    if (p != NULL) {
+	fname = p + 1;
+    }
+
+    return fname;
+}
+
+int send_file (char *fullname)
+{
+    HINSTANCE mapilib = NULL;
+    LPMAPISENDMAIL send_mail = NULL;
+    int err = 0;
+
+    mapilib = LoadLibrary("MAPI32.DLL");
+    if (mapilib == NULL) {
+	err = 1;
+    } else {
+	send_mail = (LPMAPISENDMAIL) GetProcAddress(mapilib, "MAPISendMail");
+	if (send_mail == NULL) {
+	    err = 1;
+	} 
+    }
+
+    if (err) {
+	errbox("Couldn't access Windows MAPI system");
+    } else {
+	char *fname = fname_from_fullname(fullname);
+	gchar *note = NULL;
+	MapiFileDesc mfd;
+	MapiMessage msg;
+	ULONG sd;
+
+	memset(&mfd, 0, sizeof mfd);
+	memset(&msg, 0, sizeof msg);
+
+	mfd.lpszPathName = fullname;
+	mfd.lpszFileName = fname;
+	mfd.nPosition = 1; /* ? */
+
+	if (strstr(fname, ".gdt") != NULL) {
+	    note = g_strdup_printf("Please find the gretl data file %s attached.\n", fname);
+	    msg.lpszSubject  = "dataset";
+	} else {
+	    note = g_strdup_printf("Please find the gretl script %s attached.\n", fname);
+	    msg.lpszSubject  = "script";
+	}
+
+	msg.lpszNoteText = note;
+	msg.nFileCount = 1;
+	msg.lpFiles = &mfd;
+
+	sd = send_mail(0L, 0, &msg, MAPI_DIALOG, 0L);
+
+	if (sd != SUCCESS_SUCCESS && sd != MAPI_E_USER_ABORT) {
+	    errbox("MAPI error sending message");
+	}
+
+	g_free(note);
+    }
+
+    if (mapilib != NULL) {
+	FreeLibrary(mapilib);
+    }
+
+    return err;
+}
+
+
+
