@@ -27,6 +27,7 @@
 enum {
     GTK_STOCK_OK,
     GTK_STOCK_CANCEL,
+    GTK_STOCK_CLEAR,
     GTK_STOCK_CLOSE,
     GTK_STOCK_APPLY,
     GTK_STOCK_HELP,
@@ -54,6 +55,7 @@ gchar *my_locale_from_utf8 (const gchar *src);
 gchar *force_locale_from_utf8 (const gchar *src);
 gchar *my_filename_to_utf8 (char *fname);
 gchar *my_locale_to_utf8 (const gchar *src);
+void gretl_stock_icons_init (void);
 
 #endif /* OLD_GTK */
 
@@ -98,6 +100,8 @@ void winstack_destroy (void);
 
 int winstack_match_data (gpointer p);
 
+GtkWidget *match_window_by_data (gpointer p);
+
 windata_t *view_buffer (PRN *prn, int hsize, int vsize, 
 			const char *title, int role,
 			gpointer data);
@@ -105,6 +109,9 @@ windata_t *view_buffer (PRN *prn, int hsize, int vsize,
 windata_t *view_file (const char *filename, int editable, int del_file, 
 		      int hsize, int vsize, int role);
 
+windata_t *
+view_help_file (const char *filename, int role, GtkItemFactoryEntry *menu_items);
+
 windata_t *edit_buffer (char **pbuf, int hsize, int vsize, 
 			char *title, int role);
 
