@@ -59,6 +59,22 @@ static GdkPixbuf *png_mono_pixbuf (const char *fname)
 
 #endif /* USE_GNOME */
 
+void rtf_print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn)
+{
+    const char *obs;
+
+    if (pdinfo->markers) { 
+	obs = pdinfo->S[t];
+    } else {
+	char tmp[OBSLEN]; 
+
+	ntodate(tmp, t, pdinfo);
+	obs = tmp;
+    }
+
+    pprintf(prn, "\\intbl \\ql %s\\cell", obs);
+}
+
 /* row format specifications for RTF "tables" */
 
 #define STATS_ROW  "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262" \
@@ -318,14 +334,16 @@ static void rtf_vmat_blank_row (int lo, int n, PRN *prn)
     pputs(prn, "\\intbl \\row\n");
 }
 
+#define FIELDS 5
+
 static void
 rtfprint_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, PRN *prn)
 {
     register int i, j;
     int n = vmat->t2 - vmat->t1 + 1;
-    int lo, nf, li2, p, k, idx, ij2;
+    int blockmax = vmat->dim / FIELDS;
+    int nf, li2, p, k, idx, ij2;
     char tmp[128];
-    enum { FIELDS = 5 };
 
     if (vmat->ci == CORR) {
 	char date1[OBSLEN], date2[OBSLEN];
@@ -347,19 +365,17 @@ rtfprint_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, PRN *prn)
 		I_("Coefficient covariance matrix"));
     }
     
-    lo = vmat->dim;
-
-    for (i=0; i<=lo/FIELDS; i++) {
+    for (i=0; i<=blockmax; i++) {
 	int pad;
 
 	nf = i * FIELDS;
-	li2 = lo - nf;
+	li2 = vmat->dim - nf;
 	p = (li2 > FIELDS) ? FIELDS : li2;
 	if (p == 0) break;
 
-	pad = (lo > FIELDS)? FIELDS - p : lo - p;
+	pad = (vmat->dim > FIELDS)? FIELDS - p : vmat->dim - p;
 
-	rtf_vmat_row(lo, prn);
+	rtf_vmat_row(vmat->dim, prn);
 
 	if (pad) rtf_table_pad(pad, prn);
 
@@ -376,7 +392,7 @@ rtfprint_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, PRN *prn)
 		rtf_table_pad(pad, prn);
 	    }
 	    for (k=0; k<p; k++) {
-		idx = ijton(j, nf+k, lo);
+		idx = ijton(j, nf+k, vmat->dim);
 		if (vmat->ci == CORR) {
 		    rtf_outxx(vmat->vec[idx], prn);
 		} else {
@@ -392,7 +408,7 @@ rtfprint_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, PRN *prn)
 	    rtf_table_pad(pad + j, prn);
 	    ij2 = nf + j;
 	    for (k=j; k<p; k++) {
-		idx = ijton(ij2, nf+k, lo);
+		idx = ijton(ij2, nf+k, vmat->dim);
 		if (vmat->ci == CORR) {
 		    rtf_outxx(vmat->vec[idx], prn);
 		} else {
@@ -401,8 +417,10 @@ rtfprint_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, PRN *prn)
 	    }
 	    pprintf(prn, "\\ql %s\\cell \\intbl \\row\n", vmat->names[ij2]);
 	}
-	
-	rtf_vmat_blank_row(lo, pad + p, prn);
+
+	if (i < blockmax) {
+	    rtf_vmat_blank_row(vmat->dim, pad + p + 1, prn);
+	}
     }
 
     pputs(prn, "}}\n");
@@ -648,10 +666,7 @@ static void rtfprint_fit_resid (const FITRESID *fr,
 	    fr->depvar, I_("fitted"), I_("residual"));
 
     for (t=0; t<fr->nobs; t++) {
-	pputs(prn, "\\qr ");
-	print_obs_marker(t + fr->t1, pdinfo, prn);
-	pputs(prn, "\\cell"); 
-
+	rtf_print_obs_marker(t + fr->t1, pdinfo, prn);
 	if (na(fr->actual[t])) {
 	    pputs(prn, "\\qc \\cell \\qc \\cell \\qc \\cell \\ql \\cell"
 		  " \\intbl \\row\n"); 
@@ -840,9 +855,7 @@ static void rtfprint_fcast_without_errs (const FITRESID *fr,
 	    I_("Obs"), fr->depvar, I_("prediction")); 
 
     for (t=fr->pre_n; t<fr->nobs; t++) {
-	pputs(prn, "\\qr ");
-	print_obs_marker(t + fr->t1, pdinfo, prn);
-	pputs(prn, "\\cell"); 
+	rtf_print_obs_marker(t + fr->t1, pdinfo, prn);
 	printfrtf(fr->actual[t], prn, 0);
 	printfrtf(fr->fitted[t], prn, 0);
     }
@@ -881,9 +894,7 @@ static void rtfprint_fcast_with_errs (const FITRESID *fr,
 	    I_("95% confidence interval"));
 
     for (t=fr->pre_n; t<fr->nobs; t++) {
-	pputs(prn, "\\qr ");
-	print_obs_marker(t + fr->t1, pdinfo, prn);
-	pputs(prn, "\\cell"); 
+	rtf_print_obs_marker(t + fr->t1, pdinfo, prn);
 	maxerr = fr->tval * fr->sderr[t];
 	printfrtf(fr->actual[t], prn, 0);
 	printfrtf(fr->fitted[t], prn, 0);
@@ -1045,7 +1056,6 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
     int tsamp = datainfo->t2 - datainfo->t1 + 1;
     char delim = datainfo->delim;
     double xx;
-    char tmp[OBSLEN];
 
     if (l0 == 0) return 1;
 
@@ -1067,11 +1077,9 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
 	}
     }	
 
-#ifdef ENABLE_NLS
     if (datainfo->decpoint != ',') {
-	setlocale(LC_NUMERIC, "C");
+	gretl_push_c_numeric_locale();
     }
-#endif
 
     /* variable names */
     pprintf(prn, "obs%c", delim);
@@ -1082,13 +1090,7 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
 
     /* actual data values */
     for (t=datainfo->t1; t<=datainfo->t2; t++) {
-	if (datainfo->S != NULL) {
-	    pprintf(prn, "%s%c", datainfo->S[t], delim);
-	} else {
-	    ntodate(tmp, t, datainfo);
-	    /* Does the "'" work correctly below? */
-	    pprintf(prn, "\"'%s\"%c", tmp, delim); 
-	}
+	csv_obs_to_prn(t, datainfo, prn);
 	for (i=1; i<=l0; i++) { 
 	    xx = (datainfo->vector[list[i]])? 
 		Z[list[i]][t] : Z[list[i]][0];
@@ -1103,9 +1105,9 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
 	}
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    if (datainfo->decpoint != ',') {
+	gretl_pop_c_numeric_locale();
+    }
 
     if (pmax != NULL) {
 	free(pmax);
@@ -1130,7 +1132,7 @@ static int real_csv_to_clipboard (const char *liststr)
 	    err = data_to_buf_as_csv(list, prn);
 	}
 	if (!err) {
-	    err = prn_to_clipboard(prn, COPY_CSV);
+	    err = prn_to_clipboard(prn, GRETL_FORMAT_CSV);
 	}
     }
 
@@ -1142,10 +1144,11 @@ static int real_csv_to_clipboard (const char *liststr)
 
 int csv_to_clipboard (void)
 {
+    gretlopt opt = OPT_NONE;
     int err = 0;
 
-    delimiter_dialog();
-    data_save_selection_wrapper(COPY_CSV);
+    delimiter_dialog(&opt);
+    data_save_selection_wrapper(COPY_CSV, GINT_TO_POINTER(opt));
 
     if (storelist != NULL && *storelist != 0) {
 	err = real_csv_to_clipboard(storelist);
@@ -1164,10 +1167,64 @@ int csv_selected_to_clipboard (void)
     liststr = main_window_selection_as_string();
 
     if (liststr != NULL) {
-	delimiter_dialog();
+	delimiter_dialog(NULL);
 	err = real_csv_to_clipboard(liststr);
 	free(liststr);
     }
 
     return err;
 }
+
+#include "series_view.h"
+#include "fileselect.h"
+
+int csv_copy_listed_vars (windata_t *vwin, int fmt, int action)
+{
+    int *list = series_view_get_list(vwin);
+    PRN *prn = NULL;
+    char delim = datainfo->delim;
+    int i, err = 0;
+
+    if (list != NULL) {
+	for (i=1; i<=list[0]; i++) {
+	    if (list[i] >= datainfo->v) {
+		gui_errmsg(E_DATA);
+		return E_DATA;
+	    }
+	}
+
+	if (fmt == GRETL_FORMAT_CSV) {
+	    delimiter_dialog(NULL);
+	} else {
+	    datainfo->delim = '\t';
+	}
+
+	if (series_view_is_sorted(vwin)) {
+	    prn = vwin_print_sorted_as_csv(vwin);
+	    if (prn == NULL) {
+		err = 1;
+	    }
+	} else {
+	    err = bufopen(&prn);
+	    if (!err) {
+		err = data_to_buf_as_csv(list, prn);
+	    }
+	}
+	if (!err) {
+	    if (action == W_COPY) {
+		err = prn_to_clipboard(prn, fmt);
+	    } else {
+		file_selector(_("Save data"), EXPORT_CSV, FSEL_DATA_PRN, prn);
+	    }
+	}
+	gretl_print_destroy(prn);
+	free(list);
+    }
+
+    if (fmt == GRETL_FORMAT_TABLE) {
+	/* restore default after forcing to TAB */
+	datainfo->delim = delim;
+    }
+
+    return err;
+}
