@@ -103,14 +103,14 @@ int get_original_n (void);
 
 void do_coint (GtkWidget *widget, gpointer p);
 
-void do_coint2 (GtkWidget *widget, gpointer p);
-
 void do_forecast (gpointer data, guint u, GtkWidget *w);
 
 void do_coeff_sum (GtkWidget *widget, gpointer p);
 
 void do_add_omit (GtkWidget *widget, gpointer p);
 
+void do_confidence_region (GtkWidget *widget, gpointer p);
+
 void do_lmtest (gpointer data, guint aux_code, GtkWidget *widget);
 
 void do_autocorr (gpointer data, guint u, GtkWidget *widget);
@@ -127,9 +127,15 @@ void do_restrict (GtkWidget *widget, dialog_t *dlg);
 
 void do_nls_model (GtkWidget *widget, dialog_t *dlg);
 
+void do_mle_model (GtkWidget *widget, dialog_t *dlg);
+
+void do_eqn_system (GtkWidget *widget, dialog_t *dlg);
+
 void do_model (GtkWidget *widget, gpointer p);
 
-void do_arma (int v, int ar, int ma, gretlopt opts);
+void do_vector_model (GtkWidget *widget, gpointer p);
+
+void do_graph_model (GPT_SPEC *spec);
 
 void do_kernel (gpointer data, guint u, GtkWidget *w);
 
@@ -201,9 +207,11 @@ void do_remove_obs (gpointer data, guint u, GtkWidget *widget);
 
 void add_logs_etc (gpointer data, guint action, GtkWidget *widget);
 
+int add_system_resid (gpointer data, int eqnum, int ci);
+
 int add_fit_resid (MODEL *pmod, int code, int undo);
 
-int add_var_resid (GRETL_VAR *var, int eqnum);
+int add_system_resid (gpointer data, int eqnum, int ci);
 
 void add_model_stat (MODEL *pmod, int which);
 
@@ -279,5 +287,7 @@ void add_pca_data (windata_t *vwin);
 
 void add_fcast_data (windata_t *vwin);
 
+void maybe_display_string_table (void);
+
 #endif /* LIBRARY_H */
 
