@@ -20,18 +20,16 @@
 #ifndef OBJECTSAVE_H
 #define OBJECTSAVE_H
 
-int maybe_save_model (const CMD *cmd, MODEL **ppmod, 
-		      DATAINFO *pdinfo, PRN *prn);
+int maybe_save_model (const CMD *cmd, MODEL *pmod, PRN *prn);
 
-int maybe_save_var (const CMD *cmd, double ***pZ, DATAINFO *pdinfo, 
-		    PRN *prn);
+int maybe_save_var (const CMD *cmd, GRETL_VAR **pvar, PRN *prn);
+
+int maybe_save_system (const CMD *cmd, gretl_equation_system *sys, PRN *prn);
 
 int maybe_save_graph (const CMD *cmd, const char *fname, int code, PRN *prn);
 
 int save_text_buffer (PRN *prn, const char *savename, PRN *errprn);
 
-int saved_object_action (const char *line, 
-			 const DATAINFO *pdinfo,
-			 PRN *prn);
+int saved_object_action (const char *line, PRN *prn);
 
 #endif /* OBJECTSAVE_H */
