@@ -23,6 +23,7 @@
 #include "textutil.h"
 #include "dlgutils.h"
 #include "menustate.h"
+#include "series_view.h"
 
 #ifdef G_OS_WIN32
 #include "gretlwin32.h"
@@ -30,75 +31,39 @@
 #include "clipboard.h"
 #endif
 
-typedef struct {
+typedef struct data_point_t data_point;
+typedef struct multi_point_t multi_point;
+typedef struct series_view_t series_view;
+
+struct data_point_t {
     char label[OBSLEN];
     double val;
-} data_point_t;    
+};
 
-typedef struct {
+struct multi_point_t {
+    int obsnum;
+    double val;
+};  
+
+struct series_view_t {
     int varnum;
     int npoints;
     int digits;
     char format;    
     GtkWidget *digit_spin;
-    data_point_t *points;
-} series_view_t;
-
-static void series_view_format_dialog (windata_t *vwin, guint a, GtkWidget *w);
-static void series_view_sort (windata_t *vwin, guint a, GtkWidget *w);
-static void series_view_graph (windata_t *vwin, guint a, GtkWidget *w);
-static void scalar_to_clipboard (windata_t *vwin, guint a, GtkWidget *w);
-
-#ifndef OLD_GTK
-GtkItemFactoryEntry series_view_items[] = {
-    { N_("/_Edit"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Edit/_Copy selection"), NULL, window_copy, GRETL_FORMAT_SELECTION, 
-      "<StockItem>", GTK_STOCK_COPY },
-    { N_("/Edit/Copy _all"), "", window_copy, GRETL_FORMAT_TXT, 
-      "<StockItem>", GTK_STOCK_COPY },
-    { N_("/Edit/_Format..."), NULL, series_view_format_dialog, 0, NULL, GNULL },
-    { N_("/_Series"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Series/_Sort"), NULL, series_view_sort, 0, NULL, GNULL },
-    { N_("/Series/_Graph"), NULL, series_view_graph, 0, NULL, GNULL },
-    { NULL, NULL, NULL, 0, NULL, GNULL }
+    data_point *points;
 };
 
-GtkItemFactoryEntry scalar_view_items[] = {
-    { N_("/_Edit"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Edit/_Format..."), NULL, series_view_format_dialog, 0, NULL, GNULL },
-    { N_("/Edit/_Copy value"), NULL, scalar_to_clipboard, 0, NULL, GNULL },
-    { NULL, NULL, NULL, 0, NULL, GNULL }
+struct multi_series_view_t {
+    int *list;
+    int sortvar;
+    int npoints;
+    multi_point *points;
 };
-#else
-GtkItemFactoryEntry series_view_items[] = {
-    { N_("/_Edit"), NULL, NULL, 0, "<Branch>" },
-    { N_("/Edit/_Copy selection"), NULL, window_copy, GRETL_FORMAT_SELECTION, NULL },
-    { N_("/Edit/Copy _all"), NULL, window_copy, GRETL_FORMAT_TXT, NULL },
-    { N_("/Edit/_Format..."), NULL, series_view_format_dialog, 0, NULL },
-    { N_("/_Series"), NULL, NULL, 0, "<Branch>" },
-    { N_("/Series/_Sort"), NULL, series_view_sort, 0, NULL },
-    { N_("/Series/_Graph"), NULL, series_view_graph, 0, NULL },
-    { NULL, NULL, NULL, 0, NULL }
-};
-
-GtkItemFactoryEntry scalar_view_items[] = {
-    { N_("/_Edit"), NULL, NULL, 0, "<Branch>" },
-    { N_("/Edit/_Format..."), NULL, series_view_format_dialog, 0, NULL },
-    { N_("/Edit/_Copy value"), NULL, scalar_to_clipboard, 0, NULL },
-    { NULL, NULL, NULL, 0, NULL }
-};
-#endif
-
-
-GtkItemFactoryEntry *get_series_view_menu_items (int code)
-{
-    if (code == VIEW_SERIES) return series_view_items;
-    else return scalar_view_items;
-}
 
 void free_series_view (gpointer p)
 {
-    series_view_t *sview = (series_view_t *) p;
+    series_view *sview = (series_view *) p;
 
     if (sview == NULL) return;
 
@@ -109,19 +74,27 @@ void free_series_view (gpointer p)
     free(sview);
 }
 
-static int series_view_allocate (series_view_t *sview)
+void free_multi_series_view (gpointer p)
+{
+    multi_series_view *mview = (multi_series_view *) p;
+
+    if (mview == NULL) return;
+
+    if (mview->list != NULL) free(mview->list);
+    if (mview->points != NULL) free(mview->points);
+
+    free(mview);
+}
+
+static int series_view_allocate (series_view *sview)
 {
     if (sview->npoints != 0) {
 	/* already allocated */
 	return 0;
-    }
-
-    else if (!datainfo->vector[sview->varnum]) {
+    } else if (!datainfo->vector[sview->varnum]) {
 	sview->npoints = 1;
 	return 0;
-    }
-
-    else {
+    } else {
 	int t, tp, T = datainfo->t2 - datainfo->t1 + 1;
 	int v = sview->varnum;
 
@@ -130,6 +103,7 @@ static int series_view_allocate (series_view_t *sview)
 	if (sview->points == NULL) {
 	    return 1;
 	} 
+
 	sview->npoints = T;
 
 	/* populate from data set */
@@ -147,14 +121,84 @@ static int series_view_allocate (series_view_t *sview)
     return 0;
 }
 
-static void series_view_print (windata_t *vwin)
+static int multi_series_view_allocate (multi_series_view *mview)
+{
+    if (mview->npoints != 0) {
+	/* already allocated */
+	return 0;
+    } else {
+	int T = datainfo->t2 - datainfo->t1 + 1;
+
+	/* allocate storage */
+	mview->points = mymalloc(T * sizeof *mview->points);
+	if (mview->points == NULL) {
+	    return 1;
+	} 
+	mview->npoints = T;
+    }
+
+    return 0;
+}
+
+static void mview_fill_points (multi_series_view *mview)
+{
+    int t, tp = 0;
+
+    for (t=datainfo->t1; t<=datainfo->t2; t++) {
+	mview->points[tp].obsnum = t;
+	mview->points[tp].val = Z[mview->sortvar][t];
+	tp++;
+    }
+}
+
+static void replace_window_text (windata_t *vwin, const char *pbuf)
 {
 #ifndef OLD_GTK
     GtkTextBuffer *tbuf;
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+    gtk_text_buffer_set_text(tbuf, pbuf, -1);
+#else
+    gtk_text_freeze(GTK_TEXT(vwin->w));
+    gtk_editable_delete_text(GTK_EDITABLE(vwin->w), 0, -1);
+    gtk_text_insert(GTK_TEXT(vwin->w), fixed_font, 
+		    NULL, NULL, pbuf, strlen(pbuf));
+    gtk_text_thaw(GTK_TEXT(vwin->w));
 #endif
+}
+
+static PRN *series_view_print_csv (windata_t *vwin)
+{
+    series_view *sview = (series_view *) vwin->data;
+    char dchar = datainfo->delim;
+    PRN *prn;
+    int t;
+
+    if (bufopen(&prn)) {
+	return NULL;
+    }
+    
+    pprintf(prn, "obs%c%s\n", dchar, datainfo->varname[sview->varnum]);
+    for (t=0; t<sview->npoints; t++) {
+	if (na(sview->points[t].val)) {
+	    pprintf(prn, "%s%cNA\n", sview->points[t].label, dchar);
+	} else {
+	    pprintf(prn, "\"%s\"%c%.8g\n", sview->points[t].label, dchar,
+		    sview->points[t].val);
+	}
+    }
+
+    return prn;
+}
+
+/* for printing sorted or reformatted data, window with a
+   single series */
+
+static void series_view_print (windata_t *vwin)
+{
+    series_view *sview = (series_view *) vwin->data;
     const char *pbuf;
     PRN *prn;
-    series_view_t *sview = (series_view_t *) vwin->data;
     int t;
 
     if (bufopen(&prn)) return;
@@ -182,58 +226,171 @@ static void series_view_print (windata_t *vwin)
 		    datainfo->varname[sview->varnum], 
 		    sview->digits, Z[sview->varnum][0]);
 	} else {
-	    pprintf(prn, "\n%*s = %13.*fg", OBSLEN - 1,
+	    pprintf(prn, "\n%*s = %13.*f", OBSLEN - 1, 
 		    datainfo->varname[sview->varnum], 
 		    sview->digits, Z[sview->varnum][0]);
 	}
     }
 
     pbuf = gretl_print_get_buffer(prn);
-
-    /* clear existing text buffer and insert data */
-#ifndef OLD_GTK
-    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
-    gtk_text_buffer_set_text(tbuf, pbuf, -1);
-#else
-    gtk_text_freeze(GTK_TEXT(vwin->w));
-    gtk_editable_delete_text(GTK_EDITABLE(vwin->w), 0, -1);
-    gtk_text_insert(GTK_TEXT(vwin->w), fixed_font, 
-		    NULL, NULL, pbuf, strlen(pbuf));
-    gtk_text_thaw(GTK_TEXT(vwin->w));
-#endif
+    replace_window_text(vwin, pbuf);
 
     gretl_print_destroy(prn);
 }
 
+static int *make_obsvec (multi_series_view *mview)
+{
+    int *ov;
+    int t;
+
+    ov = mymalloc((mview->npoints + 1) * sizeof *ov);
+    if (ov != NULL) {
+	ov[0] = mview->npoints;
+	for (t=0; t<mview->npoints; t++) {
+	    ov[t+1] = mview->points[t].obsnum;
+	}
+    }
+
+    return ov;
+}
+
+static void multi_series_view_print_sorted (windata_t *vwin)
+{
+    multi_series_view *mview = (multi_series_view *) vwin->data;
+    const char *pbuf;
+    int *obsvec;
+    PRN *prn;
+    int err = 0;
+
+    obsvec = make_obsvec(mview);
+    if (obsvec == NULL) {
+	return;
+    }
+
+    if (bufopen(&prn)) {
+	free(obsvec);
+	return;
+    }
+
+    err = print_data_sorted(mview->list, obsvec, (const double **) Z, 
+			    datainfo, prn);
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	pbuf = gretl_print_get_buffer(prn);
+	replace_window_text(vwin, pbuf);
+    }
+
+    free(obsvec);
+    gretl_print_destroy(prn);
+}
+
+int series_view_is_sorted (windata_t *vwin)
+{
+    multi_series_view *mview = (multi_series_view *) vwin->data;
+
+    return mview->sortvar != 0;
+}
+
+PRN *vwin_print_sorted_as_csv (windata_t *vwin)
+{
+    multi_series_view *mview;
+    int *obsvec;
+    PRN *prn;
+    int err = 0;
+
+    if (vwin->role == VIEW_SERIES) {
+	return series_view_print_csv(vwin);
+    }
+
+    mview = (multi_series_view *) vwin->data;
+
+    obsvec = make_obsvec(mview);
+    if (obsvec == NULL) {
+	return NULL;
+    }
+
+    if (bufopen(&prn)) {
+	free(obsvec);
+	return NULL;
+    }
+
+    gretl_print_set_format(prn, GRETL_FORMAT_CSV);
+    err = print_data_sorted(mview->list, obsvec, (const double **) Z, 
+			    datainfo, prn);
+    if (err) {
+	gui_errmsg(err);
+	gretl_print_destroy(prn);
+	prn = NULL;
+    } 
+
+    free(obsvec);
+
+    return prn;
+}
+
 static int compare_points (const void *a, const void *b)
 {
-    const data_point_t *pa = (const data_point_t *) a;
-    const data_point_t *pb = (const data_point_t *) b;
+    const data_point *pa = (const data_point *) a;
+    const data_point *pb = (const data_point *) b;
      
     return (pa->val > pb->val) - (pa->val < pb->val);
 }
 
-static void series_view_sort (windata_t *vwin, guint action, GtkWidget *w)
+static int compare_mpoints (const void *a, const void *b)
 {
-    int err;
-    series_view_t *sview = (series_view_t *) vwin->data;
-    
-    err = series_view_allocate(sview);
-
-    if (!err) {
-	/* sort the data */
-	qsort((void *) sview->points, (size_t) sview->npoints, 
-	      sizeof sview->points[0], compare_points);
-
-	/* print sorted data to buffer */
-	series_view_print(vwin);
-    }
+    const multi_point *pa = (const multi_point *) a;
+    const multi_point *pb = (const multi_point *) b;
+     
+    return (pa->val > pb->val) - (pa->val < pb->val);
 }
 
-static void 
-series_view_graph (windata_t *vwin, guint action, GtkWidget *w)
+void series_view_sort (GtkWidget *w, windata_t *vwin)
 {
-    series_view_t *sview = (series_view_t *) vwin->data;
+    series_view *sview = (series_view *) vwin->data;
+    
+    if (series_view_allocate(sview)) {
+	return;
+    }
+
+    /* sort the data */
+    qsort((void *) sview->points, (size_t) sview->npoints, 
+	  sizeof sview->points[0], compare_points);
+
+    /* print sorted data to buffer */
+    series_view_print(vwin);
+}
+
+void series_view_sort_by (GtkWidget *w, windata_t *vwin)
+{
+    multi_series_view *mview = (multi_series_view *) vwin->data;
+    int v;
+
+    if (mview == NULL || mview->list == NULL) {
+	return;
+    }
+
+    if (multi_series_view_allocate(mview)) {
+	return;
+    }
+
+    v = select_var_from_list(mview->list, _("Variable to sort by"));
+    if (v < 0) {
+	return;
+    }
+
+    mview->sortvar = v;
+    mview_fill_points(mview);
+
+    qsort((void *) mview->points, (size_t) mview->npoints, 
+	  sizeof mview->points[0], compare_mpoints);
+
+    multi_series_view_print_sorted(vwin);
+}
+
+void series_view_graph (GtkWidget *w, windata_t *vwin)
+{
+    series_view *sview = (series_view *) vwin->data;
 
     if (dataset_is_time_series(datainfo)) {
 	do_graph_var(sview->varnum);
@@ -242,10 +399,9 @@ series_view_graph (windata_t *vwin, guint action, GtkWidget *w)
     }
 }
 
-static void 
-scalar_to_clipboard (windata_t *vwin, guint action, GtkWidget *w)
+void scalar_to_clipboard (windata_t *vwin)
 {
-    series_view_t *sview = (series_view_t *) vwin->data;
+    series_view *sview = (series_view *) vwin->data;
     double val;
     gchar *buf;
 
@@ -254,7 +410,7 @@ scalar_to_clipboard (windata_t *vwin, guint action, GtkWidget *w)
     if (sview->format == 'G') {
 	buf = g_strdup_printf("%#.*g", sview->digits, val);
     } else {
-	buf = g_strdup_printf("%.*fg", sview->digits, val);
+	buf = g_strdup_printf("%.*f", sview->digits, val);
     }
 
 #ifdef G_OS_WIN32
@@ -266,9 +422,33 @@ scalar_to_clipboard (windata_t *vwin, guint action, GtkWidget *w)
     g_free(buf);
 }
 
+int *series_view_get_list (windata_t *vwin)
+{
+    int *list = NULL;
+
+    if (vwin->role == VIEW_SERIES) {
+	series_view *sview = vwin->data;
+
+	if (sview != NULL) {
+	    list = gretl_list_new(1);
+	    if (list != NULL) {
+		list[1] = sview->varnum;
+	    }
+	}
+    } else {
+	multi_series_view *mview = (multi_series_view *) vwin->data;
+
+	if (mview != NULL) {
+	    list = gretl_list_copy(mview->list);
+	}
+    }
+
+    return list;
+}
+
 void series_view_connect (windata_t *vwin, int varnum)
 {
-    series_view_t *sview;
+    series_view *sview;
 
     sview = malloc(sizeof *sview);
 
@@ -285,14 +465,52 @@ void series_view_connect (windata_t *vwin, int varnum)
     }
 }
 
+multi_series_view *multi_series_view_new (int *list)
+{
+    multi_series_view *mview = NULL;
+    int i, err = 0;
+
+    /* we do this only for a list of up to five variables,
+       with no scalars */
+
+    if (list == NULL || list[0] > 5) {
+	err = 1;
+    } else {
+	for (i=1; i<=list[0]; i++) {
+	    if (!datainfo->vector[list[i]]) {
+		err = 1;
+		break;
+	    }
+	} 
+    }
+
+    if (err) {
+	free(list);
+	return NULL;
+    }
+
+    mview = malloc(sizeof *mview);
+
+    if (mview != NULL) {
+	mview->list = list;
+	mview->sortvar = 0;
+	mview->npoints = 0;
+	mview->points = NULL;
+    } else {
+	free(list);
+    }
+
+    return mview;
+}
+
 static 
-void series_view_format_cancel (GtkWidget *w, series_view_t *sview)
+void series_view_format_cancel (GtkWidget *w, series_view *sview)
 {
     sview->digits = -1;
 }
 
 static 
-void series_view_get_figures (GtkWidget *w, series_view_t *sview)
+void series_view_get_figures (GtkWidget *w, series_view *sview)
 {
     sview->digits = gtk_spin_button_get_value_as_int
 	(GTK_SPIN_BUTTON(sview->digit_spin));
@@ -302,38 +520,28 @@ static void
 set_series_float_format (GtkWidget *w, gpointer p)
 {
     gint i;
-    series_view_t *sview = (series_view_t *) p;
+    series_view *sview = (series_view *) p;
 
     if (GTK_TOGGLE_BUTTON(w)->active) {
-#ifndef OLD_GTK
         i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "action"));
-#else
-        i = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w), "action"));
-#endif
         sview->format = i;
     }
 }
 
-static void 
-series_view_format_dialog (windata_t *vwin, guint action, GtkWidget *src)
+void series_view_format_dialog (GtkWidget *src, windata_t *vwin)
 {
     GtkWidget *w, *tmp, *label;
     GtkWidget *vbox, *hbox;
     GtkObject *adj;
     GSList *group;
-    series_view_t *sview = (series_view_t *) vwin->data;
+    series_view *sview = (series_view *) vwin->data;
 
     if (series_view_allocate(sview)) return;
 
     w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW(w), _("gretl: data format"));
-#ifndef OLD_GTK
     g_signal_connect(G_OBJECT(w), "destroy",  
 		     G_CALLBACK(gtk_main_quit), NULL);
-#else
-    gtk_signal_connect(GTK_OBJECT(w), "destroy",  
-		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-#endif
 
     vbox = gtk_vbox_new (FALSE, 5);
     gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
@@ -348,13 +556,8 @@ series_view_format_dialog (windata_t *vwin, guint action, GtkWidget *src)
     tmp = gtk_label_new(_("Show"));
     adj = gtk_adjustment_new(sview->digits, 1, 10, 1, 1, 1);
     sview->digit_spin = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-#ifndef OLD_GTK
     g_signal_connect (adj, "value_changed",
 		      G_CALLBACK (series_view_get_figures), sview);
-#else
-    gtk_signal_connect (adj, "value_changed",
-			GTK_SIGNAL_FUNC (series_view_get_figures), sview);
-#endif
     gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
     gtk_box_pack_start (GTK_BOX (hbox), sview->digit_spin, FALSE, FALSE, 5);
 
@@ -363,34 +566,18 @@ series_view_format_dialog (windata_t *vwin, guint action, GtkWidget *src)
     gtk_box_pack_start (GTK_BOX(vbox), tmp, TRUE, TRUE, 0);
     if (sview->format == 'G')
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), TRUE);
-#ifndef OLD_GTK
     g_signal_connect(G_OBJECT(tmp), "clicked",
                      G_CALLBACK(set_series_float_format), sview);
-    g_object_set_data(G_OBJECT(tmp), "action", 
-                      GINT_TO_POINTER('G'));
-#else
-    gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-		       GTK_SIGNAL_FUNC(set_series_float_format), sview);
-    gtk_object_set_data(GTK_OBJECT(tmp), "action", 
-			GINT_TO_POINTER('G'));
-#endif
+    g_object_set_data(G_OBJECT(tmp), "action", GINT_TO_POINTER('G'));
 
     group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (tmp));
     tmp = gtk_radio_button_new_with_label(group, _("decimal places"));
     gtk_box_pack_start (GTK_BOX(vbox), tmp, TRUE, TRUE, 0);
     if (sview->format == 'f')
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), TRUE);
-#ifndef OLD_GTK
     g_signal_connect(G_OBJECT(tmp), "clicked",
                      G_CALLBACK(set_series_float_format), sview);
-    g_object_set_data(G_OBJECT(tmp), "action", 
-                      GINT_TO_POINTER('f')); 
-#else
-    gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-		       GTK_SIGNAL_FUNC(set_series_float_format), sview);
-    gtk_object_set_data(GTK_OBJECT(tmp), "action", 
-			GINT_TO_POINTER('f'));    
-#endif   
+    g_object_set_data(G_OBJECT(tmp), "action", GINT_TO_POINTER('f')); 
 
     /* control buttons */
     hbox = gtk_hbox_new (TRUE, 5);
@@ -431,8 +618,7 @@ series_view_format_dialog (windata_t *vwin, guint action, GtkWidget *src)
     gtk_widget_show_all(w);
 
 #ifdef OLD_GTK
-    gtk_window_set_transient_for(GTK_WINDOW(w),
-				 GTK_WINDOW(vwin->dialog));
+    gtk_window_set_transient_for(GTK_WINDOW(w), GTK_WINDOW(vwin->dialog));
 #endif
 
     gretl_set_window_modal(w);
@@ -441,7 +627,8 @@ series_view_format_dialog (windata_t *vwin, guint action, GtkWidget *src)
 
     if (sview->digits > 0) {
 	series_view_print(vwin);
-    } else { /* canceled */
+    } else { 
+	/* canceled */
 	sview->digits = 6;
     }
 }
