@@ -22,6 +22,8 @@
 #ifndef SESSION_H
 #define SESSION_H
 
+#include "system.h"
+
 enum {
     SCHEDULE_FOR_DELETION,
     REALLY_DELETE_ALL,
@@ -39,6 +41,24 @@ enum {
     ADD_OBJECT_FAIL
 };
 
+typedef enum {
+    OBJ_UNKNOWN,
+    OBJ_DATASET,
+    OBJ_INFO,
+    OBJ_STATS,
+    OBJ_CORR,
+    OBJ_SCRIPT,
+    OBJ_NOTES,
+    OBJ_MODTAB,
+    OBJ_GPAGE,
+    OBJ_MODEL,
+    OBJ_GRAPH,
+    OBJ_PLOT,
+    OBJ_VAR,
+    OBJ_SYS,
+    OBJ_TEXT
+} SessionObjType;
+
 int session_is_saved (void);
 
 void set_session_saved (int val);
@@ -56,21 +76,35 @@ void remember_model (gpointer data, guint close, GtkWidget *widget);
 
 void remember_var (gpointer data, guint close, GtkWidget *widget);
 
-int model_already_saved (const MODEL *pmod);
+void remember_sys (gpointer data, guint close, GtkWidget *widget);
+
+int model_already_saved (const char *modname);
 
 int try_add_model_to_session (MODEL *pmod);
 
 int try_add_var_to_session (GRETL_VAR *var);
 
-void *get_session_object_by_name (const char *name, char *which);
+int try_add_system_to_session (gretl_equation_system *sys);
 
-void delete_model_from_session (MODEL *pmod);
+int display_saved_model (const char *modname);
 
-void delete_var_from_session (GRETL_VAR *var);
+int display_saved_equation_system (const char *sysname);
 
-void delete_text_from_session (const char *tname);
+int display_saved_VAR (const char *varname);
 
-void display_text_by_name (const char *tname);
+void session_VAR_do_irf (const char *varname, const char *line);
+
+int delete_model_from_session (const char *modname);
+
+int delete_system_from_session (const char *sysname);
+
+int delete_VAR_from_session (const char *varname);
+
+void *get_session_object_by_name (const char *name, int *which);
+
+void delete_text_from_session (void *p);
+
+void display_saved_text (void *p);
 
 int session_changed (int set);
 
