@@ -24,8 +24,10 @@
 #include "webget.h"
 #include "toolbar.h"
 #include "dlgutils.h"
+#include "fileselect.h"
 
 #include "libset.h"
+#include "version.h"
 
 #include <unistd.h>
 #include <sys/types.h>
@@ -51,12 +53,14 @@
 # endif
 #endif
 
-extern const char *version_string;
-
 #if !defined(G_OS_WIN32) && !defined(USE_GNOME)
 char rcfile[MAXLEN];
 #endif
 
+#ifndef G_OS_WIN32
+extern char Browser[MAXSTR];
+#endif
+
 extern int want_toolbar;
 extern char Rcommand[MAXSTR];
 extern char dbproxy[21];
@@ -109,6 +113,7 @@ GdkFont *fixed_font;
 static int usecwd;
 static int olddat;
 static int useqr;
+static int manpref;
 char gpcolors[32];
 static char datapage[24];
 static char scriptpage[24];
@@ -130,24 +135,28 @@ static int lcnumeric = 1;
 char midiplayer[MAXSTR];
 #endif
 
-enum {
-    ROOTSET = 1 << 0,
-    USERSET = 1 << 1,
-    BOOLSET = 1 << 2,
-    LISTSET = 1 << 3,
-    INVISET = 1 << 4,
-    FIXSET  = 1 << 5  /* setting fixed by admin (Windows network use) */
-};
+typedef enum {
+    ROOTSET  = 1 << 0,
+    USERSET  = 1 << 1,
+    BOOLSET  = 1 << 2,
+    INTSET   = 1 << 3,
+    LISTSET  = 1 << 4,
+    RADIOSET = 1 << 5,
+    INVISET  = 1 << 6,
+    FIXSET   = 1 << 7  /* setting fixed by admin (Windows network use) */
+} rcflags;
 
 typedef struct {
     char *key;         /* config file variable name */
     char *description; /* How the field will show up in the options dialog */
     char *link;        /* in case of radio button pair, alternate string */
     void *var;         /* pointer to variable */
-    char type;         /* ROOTSET user string
+    rcflags flags;     /* ROOTSET user string
 			  USERSET root string
 			  BOOLSET boolean (user)
+                          INTSET integer (user)
 			  LISTSET user string, from fixed menu
+                          RADIOSET user int, from fixed menu
 			  INVISET "invisible" (user) string 
 		       */
     int len;           /* storage size for string variable (also see Note) */
@@ -155,11 +164,15 @@ typedef struct {
     GtkWidget *widget;
 } RCVAR;
 
-/* Note: actually "len" above is overloaded: if an rc_var is of type
-   BOOLSET and not part of a radio group, then a non-zero value for
-   len will link the var's toggle button with the sensitivity of the
-   preceding rc_var's entry field.  For example, the "use_proxy" button
-   controls the sensitivity of the "dbproxy" entry widget. */
+/* Note: actually "len" above is overloaded: (1) if an rc_var is of
+   type BOOLSET and not part of a radio group, then a non-zero value
+   for len will link the var's toggle button with the sensitivity of
+   the preceding rc_var's entry field.  For example, the "use_proxy"
+   button controls the sensitivity of the "dbproxy" entry widget. (2)
+   if an rc_var is of type RADIOSET, len is used to represent the
+   number of radio (i.e. mutually incompatible) options for the
+   variable.
+*/
 
 RCVAR rc_vars[] = {
     { "gretldir", N_("Main gretl directory"), NULL, paths.gretldir, 
@@ -172,13 +185,20 @@ RCVAR rc_vars[] = {
       BOOLSET, 0, 1, NULL },
     { "toolbar", N_("Show gretl toolbar"), NULL, &want_toolbar, 
       BOOLSET, 0, 1, NULL },
+#ifndef G_OS_WIN32
+    { "winsize", N_("Remember main window size"), NULL, &winsize, 
+      BOOLSET, 0, 1, NULL },
+#endif
 #ifdef ENABLE_NLS
     { "lcnumeric", N_("Use locale setting for decimal point"), NULL, &lcnumeric, 
       BOOLSET, 0, 1, NULL },
 #endif
 #ifdef G_OS_WIN32
     { "wimp", N_("Emulate Windows look"), NULL, &wimp, 
-      BOOLSET, 0, 1, NULL },    
+      BOOLSET, 0, 1, NULL },
+#else
+    { "browser", N_("Web browser"), NULL, Browser, 
+      ROOTSET, MAXSTR, 3, NULL },
 #endif
     { "gnuplot", N_("Command to launch gnuplot"), NULL, paths.gnuplot, 
       ROOTSET, MAXLEN, 3, NULL },
@@ -191,6 +211,8 @@ RCVAR rc_vars[] = {
 #ifndef G_OS_WIN32
     { "viewps", N_("Command to view postscript files"), NULL, viewps, 
       ROOTSET, MAXSTR, 3, NULL },
+    { "viewpdf", N_("Command to view PDF files"), NULL, viewpdf, 
+      ROOTSET, MAXSTR, 3, NULL },
 #endif
 #if defined(HAVE_AUDIO) && !defined(G_OS_WIN32)
     { "midiplayer", N_("Program to play MIDI files"), NULL, midiplayer, 
@@ -245,6 +267,14 @@ RCVAR rc_vars[] = {
       INVISET, 32, 0, NULL },
     { "Gp_colors", N_("Gnuplot colors"), NULL, gpcolors, 
       INVISET, sizeof gpcolors, 0, NULL },
+    { "main_width", "main window width", NULL, &mainwin_width, 
+      INVISET | INTSET, 0, 0, NULL },
+    { "main_height", "main window height", NULL, &mainwin_height, 
+      INVISET | INTSET, 0, 0, NULL },
+    { "main_x", "main window x position", NULL, &main_x, 
+      INVISET | INTSET, 0, 0, NULL },
+    { "main_y", "main window y position", NULL, &main_y, 
+      INVISET | INTSET, 0, 0, NULL },
     { "HC_by_default", N_("Use robust covariance matrix by default"), NULL,
       &hc_by_default, BOOLSET, 0, 5, NULL },
     { "HC_xsect", N_("For cross-sectional data"), NULL, hc_xsect, 
@@ -253,6 +283,8 @@ RCVAR rc_vars[] = {
       LISTSET, 5, 5, NULL },
     { "HC_garch", N_("For GARCH estimation"), NULL, hc_garch, 
       LISTSET, 5, 5, NULL },
+    { "manpref", N_("PDF manual preference"), NULL, &manpref, 
+      RADIOSET | INTSET, 4, 6, NULL },
     { NULL, NULL, NULL, NULL, 0, 0, 0, NULL }
 };
 
@@ -268,7 +300,10 @@ int using_hc_by_default (void)
     return hc_by_default;
 }
 
-/* ........................................................... */
+int get_manpref (void)
+{
+    return manpref;
+}
 
 void set_datapage (const char *str)
 {
@@ -290,8 +325,6 @@ const char *get_scriptpage (void)
     return scriptpage;
 }
 
-/* ........................................................... */
-
 static gretlopt set_paths_opt = OPT_X;
 
 void force_english_help (void)
@@ -300,8 +333,6 @@ void force_english_help (void)
     set_paths(&paths, set_paths_opt);
 }
 
-/* ........................................................... */
-
 void set_fixed_font (void)
 {
 #ifndef OLD_GTK
@@ -314,8 +345,6 @@ void set_fixed_font (void)
 #endif
 }
 
-/* ........................................................... */
-
 #if !defined(USE_GNOME) && !defined(OLD_GTK)
 
 const char *get_app_fontname (void)
@@ -356,8 +385,6 @@ void set_app_font (const char *fontname)
 
 #endif
 
-/* .................................................................. */
-
 static void slash_terminate (char *path)
 {
     if (path == NULL || *path == '\0') return;
@@ -384,8 +411,6 @@ void get_default_dir (char *s, int action)
     slash_terminate(s);
 }
 
-/* ........................................................... */
-
 #if defined(HAVE_TRAMO) || defined(HAVE_X12A)
 
 # ifdef HAVE_TRAMO
@@ -446,6 +471,7 @@ static int my_mkdir (const char *dirname)
 	    err = 1;
 	}
     }
+
     return err;
 }
 
@@ -631,8 +657,6 @@ int check_for_prog (const char *prog)
 
 #endif
 
-/* ........................................................... */
-
 #if !defined(G_OS_WIN32) && !defined(USE_GNOME)
 void set_rcfile (void) 
 {
@@ -658,8 +682,6 @@ void set_rcfile (void)
 }
 #endif
 
-/* .................................................................. */
-
 void options_dialog (gpointer p, guint page, GtkWidget *w) 
 {
     GtkWidget *dialog;
@@ -690,6 +712,7 @@ void options_dialog (gpointer p, guint page, GtkWidget *w)
     make_prefs_tab(notebook, 3);
     make_prefs_tab(notebook, 4);
     make_prefs_tab(notebook, 5);
+    make_prefs_tab(notebook, 6);
 
     /* OK button */
     button = standard_button(GTK_STOCK_OK);
@@ -730,8 +753,6 @@ void options_dialog (gpointer p, guint page, GtkWidget *w)
     gtk_widget_show(dialog);
 }
 
-/* .................................................................. */
-
 static void flip_sensitive (GtkWidget *w, gpointer data)
 {
     GtkWidget *entry = GTK_WIDGET(data);
@@ -756,7 +777,7 @@ void filesel_set_path_callback (const char *setting, char *strvar)
 
 static void browse_button_callback (GtkWidget *w, RCVAR *rc)
 {
-    file_selector(_(rc->description), SET_PATH, rc->var);
+    file_selector(_(rc->description), SET_PATH, FSEL_DATA_MISC, rc->var);
 }
 
 static GtkWidget *make_path_browse_button (RCVAR *rc)
@@ -776,8 +797,6 @@ static gboolean takes_effect_on_restart (void)
     return FALSE;
 }
 
-/* .................................................................. */
-
 static GList *get_settings_list (void *var)
 {
     char *hc_strs[] = {
@@ -786,28 +805,36 @@ static GList *get_settings_list (void *var)
     char *garch_strs[] = {
 	"QML", "BW"
     };
+    const char *man_strs[] = {
+        N_("English (US letter paper)"),
+        N_("English (A4 paper)"),
+        N_("Italian"),
+	N_("Spanish"),
+    };
     GList *list = NULL;
-    char *strvar = (char *) var;
     int i, n;
 
-    if (strvar == hc_xsect || strvar == hc_tseri) {
+    if (var == hc_xsect || var == hc_tseri) {
 	n = sizeof hc_strs / sizeof hc_strs[0];
-	if (strvar == hc_xsect) n--;
+	if (var == hc_xsect) n--;
 	for (i=0; i<n; i++) {
 	    list = g_list_append(list, hc_strs[i]);
 	}
-    } else if (strvar == hc_garch) {
+    } else if (var == hc_garch) {
 	n = sizeof garch_strs / sizeof garch_strs[0];
 	for (i=0; i<n; i++) {
 	    list = g_list_append(list, garch_strs[i]);
 	}
+    } else if (var == &manpref) {
+	n = sizeof man_strs / sizeof man_strs[0];
+	for (i=0; i<n; i++) {
+	    list = g_list_append(list, _(man_strs[i]));
+	}
     }	
 
     return list;
 }
 
-/* .................................................................. */
-
 static void get_table_sizes (int page, int *b_count, int *s_count)
 {
     int i;
@@ -819,11 +846,21 @@ static void get_table_sizes (int page, int *b_count, int *s_count)
 	if (rc_vars[i].tab != page) {
 	    continue;
 	}
-	if (rc_vars[i].type & BOOLSET) {
+	if (rc_vars[i].flags & BOOLSET) {
 	    *b_count += 1;
-	} else if (!(rc_vars[i].type & INVISET)) {
+	} else if (rc_vars[i].flags & RADIOSET) {
+	    *b_count += 1;
+	} else if (!(rc_vars[i].flags & INVISET)) {
 	    *s_count += 1;
-	}
+	} 
+    }
+}
+
+static void radio_change_value (GtkWidget *w, int *v)
+{
+    if (GTK_TOGGLE_BUTTON(w)->active) {
+	gint i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "action"));
+	*v = i;
     }
 }
 
@@ -850,6 +887,8 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	w = gtk_label_new(_("File Open/Save"));
     } else if (tab == 5) {
 	w = gtk_label_new(_("HCCME"));
+    } else if (tab == 6) {
+	w = gtk_label_new(_("Manuals"));
     }
     
     gtk_widget_show(w);
@@ -881,9 +920,9 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    continue;
 	}
 
-	if ((rc->type & BOOLSET) && rc->link == NULL) { 
+	if ((rc->flags & BOOLSET) && rc->link == NULL) { 
 	    /* simple boolean variable (check box) */
-	    int rcval = *(int *)(rc->var);
+	    int rcval = *(int *) (rc->var);
 
 	    rc->widget = gtk_check_button_new_with_label(_(rc->description));
 	    gtk_table_attach_defaults(GTK_TABLE (b_table), rc->widget, 
@@ -903,7 +942,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    }
 
 	    /* special case: link between toggle and preceding entry */
-	    if (rc->len && !(rc->type & FIXSET)) {
+	    if (rc->len && !(rc->flags & FIXSET)) {
 		gtk_widget_set_sensitive(rc_vars[i-1].widget,
 					 GTK_TOGGLE_BUTTON(rc->widget)->active);
 		g_signal_connect(G_OBJECT(rc->widget), "clicked",
@@ -920,16 +959,16 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 		gtk_table_resize(GTK_TABLE(b_table), b_len + 1, 2);
 	    }
 
-	    if (rc->type & FIXSET) {
+	    if (rc->flags & FIXSET) {
 		gtk_widget_set_sensitive(rc->widget, FALSE);
 		if (rc->len) {
 		    gtk_widget_set_sensitive(rc_vars[i-1].widget, FALSE);
 		}
 	    }
 
-	} else if (rc->type & BOOLSET) { 
+	} else if (rc->flags & BOOLSET) { 
 	    /* radio-button dichotomy */
-	    int rcval = *(int *)(rc->var);
+	    int rcval = *(int *) (rc->var);
 	    GtkWidget *button;
 	    GSList *group = NULL;
 
@@ -977,11 +1016,11 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    }
 	    gtk_widget_show(rc->widget);
 
-	    if (rc->type & FIXSET) {
+	    if (rc->flags & FIXSET) {
 		gtk_widget_set_sensitive(button, FALSE);
 		gtk_widget_set_sensitive(rc->widget, FALSE);
 	    }
-	} else if (rc->type & LISTSET) {
+	} else if (rc->flags & LISTSET) {
 	    char *strvar = (char *) rc->var;
 	    GList *list;
 
@@ -1009,7 +1048,40 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(rc->widget)->entry), 
 				      FALSE);
 	    gtk_widget_show(rc->widget);
-	} else if (!(rc->type & INVISET)) { 
+	} else if (rc->flags & RADIOSET) {
+	    int i, rcval = *(int *) (rc->var);
+	    GtkWidget *b;
+	    GSList *group = NULL;
+	    GList *list, *mylist;
+
+	    b_len++;
+	    b = gtk_label_new(_(rc->description));
+	    gtk_table_attach_defaults(GTK_TABLE(b_table), b, 
+				      b_col, b_col + 1, 
+				      b_len - 1, b_len);
+	    gtk_widget_show(b);
+
+	    mylist = list = get_settings_list(rc->var);
+
+	    for (i=0; i<rc->len; i++) {
+		b_len++;
+		gtk_table_resize(GTK_TABLE(b_table), b_len, 2);
+		b = gtk_radio_button_new_with_label(group, mylist->data);
+		gtk_table_attach_defaults(GTK_TABLE(b_table), b, 
+					  b_col, b_col + 1, 
+					  b_len - 1, b_len);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), i == rcval);
+		g_object_set_data(G_OBJECT(b), "action", 
+				  GINT_TO_POINTER(i));
+		g_signal_connect(G_OBJECT(b), "clicked",
+				 G_CALLBACK(radio_change_value),
+				 rc->var);
+		gtk_widget_show(b);
+		group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b));
+		mylist = g_list_next(mylist);
+	    }
+	    g_list_free(list);
+	} else if (!(rc->flags & INVISET)) { 
 	    /* visible string variable */
 	    char *strvar = (char *) rc->var;
 
@@ -1036,7 +1108,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 		gtk_widget_show(w);
 	    }
 
-	    if (rc->type & FIXSET) {
+	    if (rc->flags & FIXSET) {
 		gtk_widget_set_sensitive(rc->widget, FALSE);
 		gtk_widget_set_sensitive(w, FALSE);
 	    }
@@ -1063,8 +1135,6 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
     }
 }
 
-/* .................................................................. */
-
 #ifdef ENABLE_NLS
 static void set_lcnumeric (void)
 {
@@ -1078,20 +1148,17 @@ static void set_lcnumeric (void)
 	    if (set == NULL) {
 		set = setlocale(LC_NUMERIC, "es");
 	    }
-	}
-	else if (lang != NULL && !strcmp(lang, "fr")) {
+	} else if (lang != NULL && !strcmp(lang, "fr")) {
 	    set = setlocale(LC_NUMERIC, "French");
 	    if (set == NULL) {
 		set = setlocale(LC_NUMERIC, "fr");
 	    }	    
-	}
-	else if (lang != NULL && !strcmp(lang, "it")) {
+	} else if (lang != NULL && !strcmp(lang, "it")) {
 	    set = setlocale(LC_NUMERIC, "Italian");
 	    if (set == NULL) {
 		set = setlocale(LC_NUMERIC, "it");
 	    }
-	}
-	else if (lang != NULL && !strcmp(lang, "pl")) {
+	} else if (lang != NULL && !strcmp(lang, "pl")) {
 	    set = setlocale(LC_NUMERIC, "Polish");
 	    if (set == NULL) {
 		set = setlocale(LC_NUMERIC, "pl");
@@ -1115,8 +1182,6 @@ static void set_lcnumeric (void)
 }
 #endif
 
-/* .................................................................. */
-
 static void set_gp_colors (void)
 {
     char cstr[4][8];
@@ -1130,8 +1195,6 @@ static void set_gp_colors (void)
     }
 }
 
-/* .................................................................. */
-
 static void apply_changes (GtkWidget *widget, gpointer data) 
 {
     const gchar *str;
@@ -1140,20 +1203,20 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 
     for (i=0; rc_vars[i].key != NULL; i++) {
 	if (rc_vars[i].widget != NULL) {
-	    if (rc_vars[i].type == BOOLSET) {
+	    if (rc_vars[i].flags & BOOLSET) {
 		if (GTK_TOGGLE_BUTTON(rc_vars[i].widget)->active) {
-		    *(int *)(rc_vars[i].var) = TRUE;
+		    *(int *) (rc_vars[i].var) = TRUE;
 		} else {
-		    *(int *)(rc_vars[i].var) = FALSE;
+		    *(int *) (rc_vars[i].var) = FALSE;
 		}
-	    } else if (rc_vars[i].type == USERSET || rc_vars[i].type == ROOTSET) {
+	    } else if (rc_vars[i].flags & USERSET || rc_vars[i].flags & ROOTSET) {
 		str = gtk_entry_get_text(GTK_ENTRY(rc_vars[i].widget));
 		if (str != NULL && *str != '\0') { 
 		    strvar = (char *) rc_vars[i].var;
 		    *strvar = '\0';
 		    strncat(strvar, str, rc_vars[i].len - 1);
 		}
-	    } else if (rc_vars[i].type == LISTSET) {
+	    } else if (rc_vars[i].flags & LISTSET) {
 		GtkWidget *entry = GTK_COMBO(rc_vars[i].widget)->entry;
 
 		str = gtk_entry_get_text(GTK_ENTRY(entry));
@@ -1183,24 +1246,37 @@ static void apply_changes (GtkWidget *widget, gpointer data)
     proxy_init(dbproxy);
 }
 
-/* .................................................................. */
-
 #ifndef GNOME2
 
 static void str_to_boolvar (char *s, void *b)
 {
+    int *bvar = (int *) b;
+
+    if (s == NULL) return;
+
     if (strcmp(s, "true") == 0 || strcmp(s, "1") == 0) {
-	*(int *)b = TRUE;
+	*bvar = TRUE;
     } else {
-	*(int *)b = FALSE;
+	*bvar = FALSE;
     }	
 }
 
+static void str_to_int (char *s, void *b)
+{
+    int *ivar = (int *) b;
+
+    if (s == NULL) return;
+
+    if (sscanf(s, "%d", ivar) != 1) {
+	*ivar = 0;
+    }
+}
+
 #endif
 
 static void boolvar_to_str (void *b, char *s)
 {
-    if (*(int *)b) {
+    if (*(int *) b) {
 	strcpy(s, "true");
     } else {
 	strcpy(s, "false");
@@ -1239,6 +1315,7 @@ void write_rc (void)
     GConfClient *client;   
     char key[MAXSTR];
     gboolean bval;
+    int ival;
     char *strvar;
     int i;
 
@@ -1246,9 +1323,12 @@ void write_rc (void)
 
     for (i=0; rc_vars[i].key != NULL; i++) {
 	sprintf(key, "/apps/gretl/%s", rc_vars[i].key);
-	if (rc_vars[i].type == BOOLSET) {
+	if (rc_vars[i].flags & BOOLSET) {
 	    bval = *(gboolean *) rc_vars[i].var;
 	    gconf_client_set_bool(client, key, bval, NULL);
+	} else if (rc_vars[i].flags & INTSET) {
+	    ival = *(int *) rc_vars[i].var;
+	    gconf_client_set_int(client, key, ival, NULL);
 	} else {
 	    strvar = (char *) rc_vars[i].var;
 	    gconf_client_set_string(client, key, strvar, NULL);
@@ -1267,7 +1347,9 @@ static void read_rc (void)
     GConfClient *client;
     GError *error = NULL;
     GSList *flist = NULL;
-    gchar *value;
+    gboolean bval;
+    int ival;
+    gchar *strval;
     char key[MAXSTR];
     const char *file_sections[] = {
 	"recent_data_files",
@@ -1280,28 +1362,35 @@ static void read_rc (void)
 
     for (i=0; rc_vars[i].key != NULL; i++) {
 	sprintf(key, "/apps/gretl/%s", rc_vars[i].key);
-	if (rc_vars[i].type == BOOLSET) {
-	    gboolean val;
-
-	    val = gconf_client_get_bool(client, key, &error);
+	if (rc_vars[i].flags & BOOLSET) {
+	    bval = gconf_client_get_bool(client, key, &error);
 	    if (error) {
 		fprintf(stderr, "Error reading %s\n", rc_vars[i].key);
 		g_clear_error(&error);
 	    } else {
-		*(int *) rc_vars[i].var = val;
+		*(int *) rc_vars[i].var = bval;
 	    }
-	} else {
-	    value = gconf_client_get_string(client, key, &error);
-
+	} else if (rc_vars[i].flags & INTSET) {
+	    ival = gconf_client_get_int(client, key, &error);
 	    if (error) {
 		fprintf(stderr, "Error reading %s\n", rc_vars[i].key);
 		g_clear_error(&error);
-	    } else if (value != NULL) {
-		char *strvar = (char *) rc_vars[i].var;
+	    } else {
+		*(int *) rc_vars[i].var = ival;
+	    }	    
+	} else {
+	    strval = gconf_client_get_string(client, key, &error);
+	    if (error) {
+		fprintf(stderr, "Error reading %s\n", rc_vars[i].key);
+		g_clear_error(&error);
+	    } else if (strval != NULL) {
+		if (*strval != '\0') {
+		    char *strvar = (char *) rc_vars[i].var;
 
-		*strvar = '\0';
-		strncat(strvar, value, rc_vars[i].len - 1);
-		g_free(value);
+		    *strvar = '\0';
+		    strncat(strvar, strval, rc_vars[i].len - 1);
+		}
+		g_free(strval);
 	    }
 	}
     }
@@ -1335,15 +1424,18 @@ static void read_rc (void)
 void write_rc (void) 
 {
     char key[MAXSTR];
-    char cval[6];
+    char cval[8];
     char *strvar;
     int i;
 
     for (i=0; rc_vars[i].key != NULL; i++) {
 	sprintf(key, "/gretl/%s/%s", rc_vars[i].description, rc_vars[i].key);
-	if (rc_vars[i].type == BOOLSET) {
+	if (rc_vars[i].flags & BOOLSET) {
 	    boolvar_to_str(rc_vars[i].var, cval);
 	    gnome_config_set_string(key, cval);
+	} else if (rc_vars[i].flags & INTSET) {
+	    sprintf(cval, "%d", *(int *) rc_vars[i].var);
+	    gnome_config_set_string(key, cval);
 	} else {
 	    strvar = (char *) rc_vars[i].var;
 	    gnome_config_set_string(key, strvar);
@@ -1373,9 +1465,11 @@ static void read_rc (void)
 	sprintf(gpath, "/gretl/%s/%s", rc_vars[i].description, 
 		rc_vars[i].key);
 	value = gnome_config_get_string(gpath);
-	if (value != NULL) {
-	    if (rc_vars[i].type == BOOLSET) {
+	if (value != NULL && *value != '\0') {
+	    if (rc_vars[i].flags & BOOLSET) {
 		str_to_boolvar(value, rc_vars[i].var);
+	    } else if (rc_vars[i].flags & INTSET) {
+		str_to_int(value, rc_vars[i].var);
 	    } else {
 		strvar = (char *) rc_vars[i].var;
 		*strvar = '\0';
@@ -1410,31 +1504,38 @@ static void read_rc (void)
 void write_rc (void) 
 {
     char bval[6];
-    char *strvar;
+    char ival[16];
+    const char *strval;
     int i = 0;
 
     for (i=0; rc_vars[i].key != NULL; i++) {
 
-	if (rc_vars[i].type & FIXSET) continue;
+	if (rc_vars[i].flags & FIXSET) continue;
 
-	if (rc_vars[i].type == BOOLSET) {
+	if (rc_vars[i].flags & BOOLSET) {
 	    boolvar_to_str(rc_vars[i].var, bval);
 	    write_reg_val(HKEY_CURRENT_USER, 
 			  "gretl", 
 			  rc_vars[i].key, 
 			  bval);
-	} else if (rc_vars[i].type == ROOTSET) {
-	    strvar = (char *) rc_vars[i].var;
+	} else if (rc_vars[i].flags & INTSET) {
+	    sprintf(ival, "%d", *(int *) rc_vars[i].var);
+	    write_reg_val(HKEY_CURRENT_USER, 
+			  "gretl", 
+			  rc_vars[i].key, 
+			  ival);	    
+	} else if (rc_vars[i].flags & ROOTSET) {
+	    strval = (char *) rc_vars[i].var;
 	    write_reg_val(HKEY_CLASSES_ROOT, 
 			  get_reg_base(rc_vars[i].key),
 			  rc_vars[i].key, 
-			  strvar);
+			  strval);
 	} else {
-	    strvar = (char *) rc_vars[i].var;
+	    strval = (char *) rc_vars[i].var;
 	    write_reg_val(HKEY_CURRENT_USER, 
 			  get_reg_base(rc_vars[i].key),
 			  rc_vars[i].key, 
-			  strvar);
+			  strval);
 	}
     }
 
@@ -1461,16 +1562,18 @@ static int get_network_settings (void)
 	    char *p = line;
 
 	    while (isspace(*p)) p++;
-
 	    if (*p == '#') continue;
+
 	    if (sscanf(p, "%31s", key) == 1) {
 		strcpy(linevar, p + strlen(key) + 3); 
 		chopstr(linevar); 
 		gotvar = 0;
 		for (j=0; rc_vars[j].key != NULL; j++) {
 		    if (!strcmp(key, rc_vars[j].key)) {
-			if (rc_vars[j].type == BOOLSET) {
+			if (rc_vars[j].flags & BOOLSET) {
 			    str_to_boolvar(linevar, rc_vars[j].var);
+			} else if (rc_vars[j].flags & INTSET) {
+			    str_to_int(linevar, rc_vars[j].var);
 			} else {
 			    if (!strcmp(key, "gretldir") && 
 				tolower(linevar[0]) != calldrive) {
@@ -1482,7 +1585,7 @@ static int get_network_settings (void)
 				strncat(strvar, linevar, rc_vars[j].len - 1);
 			    }
 			}
-			rc_vars[j].type |= FIXSET;
+			rc_vars[j].flags |= FIXSET;
 			gotvar = gotini = 1;
 		    }
 		    if (gotvar) break;
@@ -1512,7 +1615,7 @@ void read_rc (void)
 	for (i=0; rc_vars[i].key != NULL; i++) {
 	    if (rc_vars[i].var == tramodir ||
 		rc_vars[i].var == paths.x12adir) {
-		rc_vars[i].type |= FIXSET;
+		rc_vars[i].flags |= FIXSET;
 	    }
 	}
     } 
@@ -1520,11 +1623,13 @@ void read_rc (void)
     for (i=0; rc_vars[i].key != NULL; i++) {
 	int err = 0;
 
-	if (rc_vars[i].type & FIXSET) {
+	if (rc_vars[i].flags & FIXSET) {
 	    continue;
 	}
 
-	if (rc_vars[i].type == ROOTSET) {
+	*value = '\0';
+
+	if (rc_vars[i].flags & ROOTSET) {
 	    err = read_reg_val (HKEY_CLASSES_ROOT, 
 				get_reg_base(rc_vars[i].key),
 				rc_vars[i].key, 
@@ -1537,8 +1642,10 @@ void read_rc (void)
 	}
 	    
 	if (!err && *value != '\0') {
-	    if (rc_vars[i].type == BOOLSET) {
+	    if (rc_vars[i].flags & BOOLSET) {
 		str_to_boolvar(value, rc_vars[i].var);
+	    } else if (rc_vars[i].flags & INTSET) {
+		str_to_int(value, rc_vars[i].var);
 	    } else {
 		strvar = (char *) rc_vars[i].var;
 		*strvar = '\0';
@@ -1586,9 +1693,11 @@ void write_rc (void)
 
     for (i=0; rc_vars[i].var != NULL; i++) {
 	fprintf(rc, "# %s\n", rc_vars[i].description);
-	if (rc_vars[i].type == BOOLSET) {
+	if (rc_vars[i].flags & BOOLSET) {
 	    boolvar_to_str(rc_vars[i].var, val);
 	    fprintf(rc, "%s = %s\n", rc_vars[i].key, val);
+	} else if (rc_vars[i].flags & INTSET) {
+	    fprintf(rc, "%s = %d\n", rc_vars[i].key, *(int *) rc_vars[i].var);
 	} else {
 	    strvar = (char *) rc_vars[i].var;
 	    fprintf(rc, "%s = %s\n", rc_vars[i].key, strvar);
@@ -1635,8 +1744,10 @@ static void read_rc (void)
 	    chopstr(linevar); 
 	    for (j=0; rc_vars[j].key != NULL; j++) {
 		if (!strcmp(key, rc_vars[j].key)) {
-		    if (rc_vars[j].type == BOOLSET) {
+		    if (rc_vars[j].flags & BOOLSET) {
 			str_to_boolvar(linevar, rc_vars[j].var);
+		    } else if (rc_vars[j].flags & INTSET) {
+			str_to_int(linevar, rc_vars[j].var);
 		    } else {
 			strvar = (char *) rc_vars[j].var;
 			*strvar = '\0';
@@ -2309,7 +2420,7 @@ void first_time_set_user_dir (void)
 		   "Please enter a directory for gretl user files."),
                  paths.userdir, 
                  real_set_userdir, NULL, 
-                 CREATE_USERDIR, 0, 0);
+                 CREATE_USERDIR, 0);
 }
 
 #endif /* G_OS_WIN32 */
@@ -2339,7 +2450,7 @@ void dump_rc (void)
 	return;
     }
 
-    fprintf(fp, "gretl version %s\n", version_string);
+    fprintf(fp, "gretl version %s\n", GRETL_VERSION);
     fprintf(fp, "built with GTK %d.%d.%d\n", GTK_MAJOR_VERSION, GTK_MINOR_VERSION,
 	    GTK_MICRO_VERSION);
 
@@ -2352,9 +2463,11 @@ void dump_rc (void)
 
     for (i=0; rc_vars[i].var != NULL; i++) {
 	fprintf(fp, "# %s\n", rc_vars[i].description);
-	if (rc_vars[i].type == BOOLSET) {
+	if (rc_vars[i].flags & BOOLSET) {
 	    boolvar_to_str(rc_vars[i].var, val);
 	    fprintf(fp, "%s = %s\n", rc_vars[i].key, val);
+	} else if (rc_vars[i].flags & INTSET) {
+	    fprintf(fp, "%s = %d\n", rc_vars[i].key, *(int *) rc_vars[i].var);
 	} else {
 	    fprintf(fp, "%s = %s\n", rc_vars[i].key, (char *) rc_vars[i].var);
 	}
@@ -2368,3 +2481,4 @@ void dump_rc (void)
     fclose(fp);
     free(dumper);
 }
+
