@@ -18,12 +18,13 @@
  */
 
 #include "gretl.h"
-#include "var.h"
+#include "varprint.h"
 #include "textutil.h"
 #include "textbuf.h"
 #include "guiprint.h"
 #include "model_table.h"
 #include "clipboard.h"
+#include "fileselect.h"
 
 #ifdef OLD_GTK
 # include "menustate.h"
@@ -390,15 +391,9 @@ void text_replace (windata_t *mydata, guint u, GtkWidget *widget)
 
 #endif /* old versus new gtk */
 
-enum {
-    W_PREVIEW,
-    W_COPY,
-    W_SAVE
-};
-
-static int special_text_handler (int cmd, guint fmt, int what, 
-				 gpointer data)
+static int special_text_handler (windata_t *vwin, guint fmt, int what)
 {
+    int cmd = vwin->role;
     PRN *prn = NULL;
     int err = 0;
 
@@ -409,27 +404,27 @@ static int special_text_handler (int cmd, guint fmt, int what,
     gretl_print_set_format(prn, fmt);
 
     if (cmd == SUMMARY || cmd == VAR_SUMMARY) {
-	Summary *summ = (Summary *) data;
+	Summary *summ = (Summary *) vwin->data;
 
 	special_print_summary(summ, datainfo, prn);
     } else if (cmd == CORR || cmd == COVAR) {
-	VMatrix *corr = (VMatrix *) data;
+	VMatrix *corr = (VMatrix *) vwin->data;
 
 	special_print_vmatrix(corr, datainfo, prn);
     } else if (cmd == FCAST) {
-	FITRESID *fr = (FITRESID *) data;
+	FITRESID *fr = (FITRESID *) vwin->data;
 
 	special_print_fit_resid(fr, datainfo, prn);
     } else if (cmd == FCASTERR) {
-	FITRESID *fr = (FITRESID *) data;
+	FITRESID *fr = (FITRESID *) vwin->data;
 
 	special_print_forecast(fr, datainfo, prn);
     } else if (cmd == COEFFINT) {
-	CoeffIntervals *cf = (CoeffIntervals *) data;
+	CoeffIntervals *cf = (CoeffIntervals *) vwin->data;
 
 	special_print_confints(cf, prn);
     } else if (cmd == VIEW_MODEL) { 
-	MODEL *pmod = (MODEL *) data;
+	MODEL *pmod = (MODEL *) vwin->data;
 
 	if (pmod->errcode) { 
 	    errbox("Couldn't format model");
@@ -443,13 +438,28 @@ static int special_text_handler (int cmd, guint fmt, int what,
 	}
     } else if (cmd == MPOLS) {
 	/* this is not actually ready -- see modelprint.c */
-	mp_results *mpvals = (mp_results *) data;
+	mp_results *mpvals = (mp_results *) vwin->data;
 
 	print_mpols_results(mpvals, datainfo, prn);
-    } else if (cmd == VAR) {
-	GRETL_VAR *var = (GRETL_VAR *) data;
+    } else if (cmd == VAR || cmd == VECM) {
+	GRETL_VAR *var = (GRETL_VAR *) vwin->data;
 
-	gretl_var_print(var, datainfo, prn);
+	gretl_VAR_print(var, datainfo, OPT_NONE, prn);
+    } else if (cmd == VAR_IRF || cmd == VAR_DECOMP) {
+	windata_t *parent = vwin->gretl_parent;
+
+	if (parent == NULL) {
+	    errbox("Couldn't find the VAR");
+	} else {
+	    GRETL_VAR *var = (GRETL_VAR *) parent->data;
+	    int h = vwin->active_var; /* here records preferred horizon */
+
+	    if (cmd == VAR_IRF) {
+		gretl_VAR_print_all_impulse_responses(var, datainfo, h, prn);
+	    } else {
+		gretl_VAR_print_all_fcast_decomps(var, datainfo, h, prn);
+	    }
+	}
     } else if (cmd == VIEW_MODELTABLE) {
 	err = special_print_model_table(prn);
     } 
@@ -461,7 +471,9 @@ static int special_text_handler (int cmd, guint fmt, int what,
 	} else if (what == W_COPY) {
 	    prn_to_clipboard(prn, fmt);
 	} else if (what == W_SAVE) {
-	    file_selector(_("Save LaTeX file"), SAVE_TEX, prn);
+	    int action = (fmt == GRETL_FORMAT_TEX)? SAVE_TEX : SAVE_RTF;
+
+	    file_selector(_("Save"), action, FSEL_DATA_PRN, prn);
 	}
     }
 
@@ -477,10 +489,10 @@ void window_tex_callback (GtkWidget *w, windata_t *vwin)
 	N_("Copy"),
 	N_("Save")
     };
-    int opt = radio_dialog("gretl: LaTeX", opts, 3, 0, 0);
+    int opt = radio_dialog("gretl: LaTeX", NULL, opts, 3, 0, 0);
 
     if (opt >= 0) {
-	special_text_handler(vwin->role, GRETL_FORMAT_TEX, opt, vwin->data);
+	special_text_handler(vwin, GRETL_FORMAT_TEX, opt);
     }
 }
 
@@ -488,21 +500,81 @@ void model_tex_view (gpointer data, guint fmt, GtkWidget *w)
 {
     windata_t *vwin = (windata_t *) data;
 
-    special_text_handler(vwin->role, fmt, W_PREVIEW, vwin->data);
+    special_text_handler(vwin, fmt, W_PREVIEW);
 }
 
 void model_tex_save (gpointer data, guint fmt, GtkWidget *w)
 {
     windata_t *vwin = (windata_t *) data;
 
-    special_text_handler(vwin->role, fmt, W_SAVE, vwin->data);
+    special_text_handler(vwin, fmt, W_SAVE);
 }
 
 void var_tex_callback (gpointer data, guint opt, GtkWidget *w)
 {
     windata_t *vwin = (windata_t *) data;
 
-    special_text_handler(vwin->role, GRETL_FORMAT_TEX, opt, vwin->data);
+    special_text_handler(vwin, GRETL_FORMAT_TEX, opt);
+}
+
+#ifdef OLD_GTK
+
+static gchar *text_window_get_copy_buf (windata_t *vwin, int select)
+{
+    GtkEditable *ed = GTK_EDITABLE(vwin->w);
+    gchar *cpybuf = NULL;
+
+    if (!select) {
+	cpybuf = gtk_editable_get_chars(ed, 0, -1);
+    } else if (ed->has_selection) {
+	cpybuf = gtk_editable_get_chars(ed, 
+					ed->selection_start_pos,
+					ed->selection_end_pos);
+    } 
+
+    return cpybuf;
+}
+
+#else
+
+static gchar *text_window_get_copy_buf (windata_t *vwin, int select)
+{
+    GtkTextBuffer *textbuf = 
+	gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+    gchar *cpybuf = NULL;
+
+    if (!select) {
+	cpybuf = textview_get_text(GTK_TEXT_VIEW(vwin->w)); 
+    } else if (gtk_text_buffer_get_selection_bounds(textbuf, NULL, NULL)) {
+	GtkTextIter selstart, selend;
+
+	gtk_text_buffer_get_selection_bounds(textbuf, &selstart, &selend);
+	cpybuf = gtk_text_buffer_get_text(textbuf, &selstart, &selend, FALSE);
+    } 
+
+    return cpybuf;
+}
+
+#endif
+
+static gchar *maybe_amend_buffer (gchar *inbuf, int fmt)
+{
+    gchar *outbuf = inbuf;
+
+#ifndef OLD_GTK
+    outbuf = my_locale_from_utf8(inbuf);
+    free(inbuf);
+    inbuf = outbuf;
+#endif
+
+    /* FIXME win32: saving as text?? */
+
+    if (fmt == GRETL_FORMAT_RTF_TXT) {
+	outbuf = dosify_buffer(inbuf, fmt);
+	free(inbuf);
+    }
+
+    return outbuf;
 }
 
 /* copying text from gretl windows */
@@ -510,81 +582,54 @@ void var_tex_callback (gpointer data, guint opt, GtkWidget *w)
 #define SPECIAL_FORMAT(f) ((f & GRETL_FORMAT_TEX) || \
                            (f & GRETL_FORMAT_RTF)) 
 
+static void window_copy_or_save (windata_t *vwin, guint fmt, int action) 
+{
+    gchar *cpybuf = NULL;
+
+    if (MULTI_FORMAT_ENABLED(vwin->role) && SPECIAL_FORMAT(fmt)) {
+	special_text_handler(vwin, fmt, action);
+    } else if (fmt == GRETL_FORMAT_CSV || fmt == GRETL_FORMAT_TABLE) {
+	csv_copy_listed_vars(vwin, fmt, action);
+    } else if (fmt == GRETL_FORMAT_TXT || fmt == GRETL_FORMAT_RTF_TXT) {
+	cpybuf = text_window_get_copy_buf(vwin, 0);
+    } else if (fmt == GRETL_FORMAT_SELECTION) {
+	cpybuf = text_window_get_copy_buf(vwin, 1);
+	fmt = GRETL_FORMAT_TXT;
+    }
+
+    if (cpybuf != NULL) {
+	PRN *textprn;
+
+	if (action == W_SAVE) {
+	    cpybuf = maybe_amend_buffer(cpybuf, fmt);
+	    if (cpybuf == NULL) {
+		return;
+	    }
+	}
+
+	textprn = gretl_print_new_with_buffer(cpybuf);
+	if (action == W_COPY) {
+	    prn_to_clipboard(textprn, fmt);
+	} else {
+	    int fcode = (fmt == GRETL_FORMAT_RTF_TXT)? 
+		SAVE_RTF : SAVE_OUTPUT;
+
+	    file_selector(_("Save"), fcode, FSEL_DATA_PRN, textprn);
+	}
+	gretl_print_destroy(textprn);
+    }	
+}
+
 void window_copy (gpointer data, guint fmt, GtkWidget *w) 
 {
     windata_t *vwin = (windata_t *) data;
-    gchar *msg = NULL;
-    int err = 0;
 
-    /* copying from window with special stuff enabled */
-    if (MULTI_FORMAT_ENABLED(vwin->role) && SPECIAL_FORMAT(fmt)) {
-	err = special_text_handler(vwin->role, fmt, W_COPY, vwin->data);
-	if (err) {
-	    return;
-	}
-    }
+    window_copy_or_save(vwin, fmt, W_COPY);
+}
 
-    /* copying plain text from window */
-#ifndef OLD_GTK
-    else if (fmt == GRETL_FORMAT_TXT || 
-	     fmt == GRETL_FORMAT_RTF_TXT || 
-	     fmt == GRETL_FORMAT_SELECTION) {
-	GtkTextBuffer *textbuf = 
-	    gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
-	PRN *textprn;
-	int myfmt = fmt;
-
-	if (myfmt == GRETL_FORMAT_SELECTION) {
-	    myfmt = GRETL_FORMAT_TXT;
-	}
-
-	if (gtk_text_buffer_get_selection_bounds(textbuf, NULL, NULL)) {
-	    /* there is a selection in place */
-	    GtkTextIter selstart, selend;
-	    gchar *selbuf;
-
-	    gtk_text_buffer_get_selection_bounds(textbuf, &selstart, &selend);
-	    selbuf = gtk_text_buffer_get_text(textbuf, &selstart, &selend, FALSE);
-	    textprn = gretl_print_new_with_buffer(selbuf);
-	    prn_to_clipboard(textprn, myfmt);
-	    gretl_print_destroy(textprn);
-	    if (w != NULL) {
-		infobox(_("Copied selection to clipboard"));
-	    }
-	    return;
-	} else {
-	    /* no selection: copy everything */
-	    char *buf = textview_get_text(GTK_TEXT_VIEW(vwin->w)); 
-
-	    textprn = gretl_print_new_with_buffer(buf);
-	    prn_to_clipboard(textprn, myfmt);
-	    gretl_print_destroy(textprn);
-	}
-    }
-#else
-    else if (fmt == GRETL_FORMAT_TXT) {
-	char *buf;
-	PRN *textprn;
-
-	buf = gtk_editable_get_chars(GTK_EDITABLE(vwin->w), 0, -1);
-	textprn = gretl_print_new_with_buffer(buf);
-	prn_to_clipboard(textprn, 0);
-	gretl_print_destroy(textprn);
-    } else { /* COPY_SELECTION */
-	gtk_editable_copy_clipboard(GTK_EDITABLE(vwin->w));
-	return;
-    }
-#endif
-
-    if (w != NULL) {
-	msg = g_strdup_printf(_("Copied contents of window as %s"),
-			      (fmt & GRETL_FORMAT_TEX)? "LaTeX" :
-			      (fmt & GRETL_FORMAT_RTF)? "RTF" : 
-			      (fmt & GRETL_FORMAT_RTF_TXT)? "RTF" :
-			      _("plain text"));
-	infobox(msg);
-	g_free(msg);
-    }
+void window_save (windata_t *vwin, guint fmt) 
+{
+    window_copy_or_save(vwin, fmt, W_SAVE);
 }
 
 /* native printing from gretl windows */
