@@ -121,14 +121,14 @@ static double wald_test (const int *list, MODEL *pmod, int form)
 
     if (!err) {
 #if WDEBUG
-	gretl_matrix_print(C, "Wald VCV matrix", NULL);
-	gretl_matrix_print(b, "Wald coeff vector", NULL);
+	gretl_matrix_print(C, "Wald VCV matrix");
+	gretl_matrix_print(b, "Wald coeff vector");
 #endif
 	err = gretl_invert_symmetric_matrix(C);
     }
 
     if (!err) {
-	w = gretl_scalar_b_prime_X_b(b, C, &err);
+	w = gretl_scalar_b_X_b(b, GRETL_MOD_TRANSPOSE, C, &err);
     }
 
     if (form == F_FORM && !err) {
@@ -221,8 +221,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
     stat_ok = !na(cmp->F) || !na(cmp->chisq);
 
     if (stat_ok && (opt & OPT_S)) {
-	test = new_test_on_model(new, (cmp->cmd == OMIT)? 
-				 GRETL_TEST_OMIT : GRETL_TEST_ADD);
+	test = model_test_new((cmp->cmd == OMIT)? GRETL_TEST_OMIT : GRETL_TEST_ADD);
     }
 
     if (verbosity > 1) {
@@ -278,6 +277,7 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 
     if (test != NULL) {
 	add_diffvars_to_test(test, diffvars, pdinfo);
+	maybe_add_test_to_model(new, test);
     }
 
     if (verbosity > 1) {
@@ -345,21 +345,7 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int add,
     return cmp;
 }
 
-static int get_tsls_pos (const int *list)
-{
-    int i, pos = 0;
-    
-    for (i=2; i<=list[0]; i++) {
-	if (list[i] == LISTSEP) {
-	    pos = i;
-	    break;
-	}
-    }
-
-    return pos;
-}
-
-/* reconstitute full varlist for WLS, POISSON, AR and TSLS models */
+/* reconstitute full varlist for WLS, POISSON, AR and models */
 
 static int *
 full_model_list (const MODEL *pmod, const int *inlist, int *ppos)
@@ -367,8 +353,7 @@ full_model_list (const MODEL *pmod, const int *inlist, int *ppos)
     int i, len, pos = 0;
     int *flist = NULL;
 
-    if (pmod->ci != WLS && pmod->ci != POISSON && 
-	pmod->ci != AR && pmod->ci != TSLS) {
+    if (pmod->ci != WLS && pmod->ci != POISSON && pmod->ci != AR) {
 	return NULL;
     }
 
@@ -376,9 +361,6 @@ full_model_list (const MODEL *pmod, const int *inlist, int *ppos)
 	len = inlist[0] + 2;
     } else if (pmod->ci == POISSON) {
 	len = inlist[0] + 3;
-    } else if (pmod->ci == TSLS) {
-	pos = get_tsls_pos(pmod->list);
-	len = inlist[0] + 2 + pmod->list[0] - pos; 
     } else {
 	pos = pmod->arinfo->arlist[0] + 1;
 	len = pos + inlist[0] + 2;
@@ -404,17 +386,6 @@ full_model_list (const MODEL *pmod, const int *inlist, int *ppos)
 	}
 	flist[flist[0] - 1] = LISTSEP;
 	flist[flist[0]] = offvar;
-    } else if (pmod->ci == TSLS) {
-	int j = 1;
-
-	flist[0] = len - 1;
-	for (i=1; i<=inlist[0]; i++) {
-	    flist[j++] = inlist[i];
-	}
-	for (i=pos; i<=pmod->list[0]; i++) {
-	    flist[j++] = pmod->list[i];
-	}
-	pos = get_tsls_pos(flist);
     } else if (pmod->ci == AR) {
 	flist[0] = len - 2;
 	for (i=1; i<pos; i++) {
@@ -442,7 +413,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
     MODEL rep;
     double rho = 0.0;
     int *list = *plist;
-    int pos, mc = get_model_count();
+    int pos = 0, mc = get_model_count();
     int repci = orig->ci;
 
     gretl_model_init(&rep);
@@ -459,7 +430,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	/* panel model with per-unit weights */
 	lsqopt |= OPT_W;
 	repci = POOLED;
-    } else if (orig->ci == WLS || orig->ci == AR || orig->ci == TSLS ||
+    } else if (orig->ci == WLS || orig->ci == AR || 
 	       (orig->ci == POISSON && gretl_model_get_int(orig, "offset_var"))) {
 	int *full_list = full_model_list(orig, list, &pos);
 
@@ -485,7 +456,12 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	break;
     case LOGIT:
     case PROBIT:
-	rep = logit_probit(list, pZ, pdinfo, orig->ci);
+	if (gretl_model_get_int(orig, "robust")) {
+	    lsqopt = OPT_R;
+	} else {
+	    lsqopt = OPT_NONE;
+	}
+	rep = logit_probit(list, pZ, pdinfo, orig->ci, lsqopt);
 	break;
     case TOBIT:
 	rep = tobit_model(list, pZ, pdinfo, NULL);
@@ -497,7 +473,7 @@ static MODEL replicate_estimator (MODEL *orig, int **plist,
 	rep = poisson_model(list, pZ, pdinfo, NULL);
 	break;
     case TSLS:
-	rep = tsls_func(list, pos, pZ, pdinfo, lsqopt);
+	rep = tsls_func(list, TSLS, pZ, pdinfo, lsqopt);
 	break;
     case LOGISTIC: 
 	{
@@ -587,13 +563,14 @@ real_nonlinearity_test (MODEL *pmod, int *list,
 	if (opt & OPT_S) {
 	    ModelTest *test;
 
-	    test = new_test_on_model(pmod, (aux_code == AUX_SQ)?
-				     GRETL_TEST_SQUARES : GRETL_TEST_LOGS);
+	    test = model_test_new((aux_code == AUX_SQ)?
+				  GRETL_TEST_SQUARES : GRETL_TEST_LOGS);
 	    if (test != NULL) {
 		model_test_set_teststat(test, GRETL_STAT_TR2);
 		model_test_set_dfn(test, df);
 		model_test_set_value(test, trsq);
 		model_test_set_pvalue(test, chisq(trsq, df));
+		maybe_add_test_to_model(pmod, test);
 	    }
 	}
 
@@ -681,6 +658,23 @@ int nonlinearity_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
+static void remove_special_flags (gretlopt *popt)
+{
+    gretlopt opt = *popt;
+
+    if (opt & OPT_S) {
+	opt &= ~OPT_S;
+    }
+    if (opt & OPT_T) {
+	opt &= ~OPT_T;
+    }    
+    if (opt & OPT_B) {
+	opt &= ~OPT_B;
+    } 
+
+    *popt = opt;
+}
+
 /**
  * add_test:
  * @addvars: list of variables to add to original model.
@@ -690,7 +684,10 @@ int nonlinearity_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
  * @pdinfo: information on the data set.
  * @opt: can contain %OPT_Q (quiet) to suppress printing
  * of the new model, %OPT_O to print covariance matrix,
- * %OPT_I for silent operation.
+ * %OPT_I for silent operation.  Additional options that
+ * are applicable only for 2SLS models: %OPT_B (add to
+ * both list of regressors and list of instruments), %OPT_T
+ * (add to list of instruments only).
  * @prn: gretl printing struct.
  *
  * Re-estimate a given model after adding the specified
@@ -703,11 +700,11 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
 	      double ***pZ, DATAINFO *pdinfo, 
 	      gretlopt opt, PRN *prn)
 {
+    gretlopt est_opt = opt;
     int smpl_t1 = pdinfo->t1;
     int smpl_t2 = pdinfo->t2;
     int *tmplist = NULL;
     const int orig_nvar = pdinfo->v; 
-    int save_test = 0;
     int err = 0;
 
     if (orig == NULL || orig->list == NULL) {
@@ -725,7 +722,12 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
     }
 
     /* create augmented regression list */
-    tmplist = gretl_list_add(orig->list, addvars, &err);
+    if (orig->ci == TSLS) {
+	tmplist = tsls_list_add(orig->list, addvars, opt, &err);
+    } else {
+	tmplist = gretl_list_add(orig->list, addvars, &err);
+    }
+
     if (err) {
 	return err;
     }
@@ -734,18 +736,15 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
        when the original model was estimated */
     impose_model_smpl(orig, pdinfo);
 
-    if (opt & OPT_S) {
-	/* don't pass OPT_S to replicate_estimator() */
-	save_test = 1;
-	opt &= ~OPT_S;
-    }
+    /* don't pass special opts to replicate_estimator() */
+    remove_special_flags(&est_opt);
 
     /* Run augmented regression, matching the original estimation
        method; use OPT_Z to suppress the elimination of perfectly
        collinear variables.
     */
     *new = replicate_estimator(orig, &tmplist, pZ, pdinfo, 
-			       (opt | OPT_Z), prn);
+			       (est_opt | OPT_Z), prn);
 
     if (new->errcode) {
 	pprintf(prn, "%s\n", gretl_errmsg);
@@ -757,7 +756,7 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
 	new->aux = AUX_ADD;
 
 	if (print_add_omit_model(new, opt)) {
-	    printmodel(new, pdinfo, opt, prn);
+	    printmodel(new, pdinfo, est_opt, prn);
 	}
 
 	if (new->nobs == orig->nobs) {
@@ -767,10 +766,6 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
 	    addlist = gretl_list_diff_new(new->list, orig->list);
 	    cmp = add_or_omit_compare(orig, new, 1, addlist);
 
-	    if (save_test) {
-		opt |= OPT_S;
-	    }
-
 	    gretl_make_compare(&cmp, addlist, orig, pdinfo, opt, prn);
 	    free(addlist);
 	}
@@ -797,11 +792,16 @@ int add_test (const int *addvars, MODEL *orig, MODEL *new,
  * @pdinfo: information on the data set.
  * @opt: can contain %OPT_Q (quiet) to suppress printing
  * of the new model, %OPT_O to print covariance matrix,
- * %OPT_I for silent operation.
+ * %OPT_I for silent operation.  Additional options that
+ * are applicable only for 2SLS models: %OPT_B (omit from
+ * both list of regressors and list of instruments), %OPT_T
+ * (omit from list of instruments only).
  * @prn: gretl printing struct.
  *
- * Re-estimate a given model after removing a list of 
- * specified variables.
+ * Re-estimate a given model after removing the variables
+ * specified in @omitvars, or if @omitvars is %NULL and
+ * @orig was not estimated using two-stage least squares, after
+ * removing the last independent variable in @orig.
  * 
  * Returns: 0 on successful completion, error code on error.
  */
@@ -810,12 +810,12 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 	       double ***pZ, DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn)
 {
+    gretlopt est_opt = opt;
     int smpl_t1 = pdinfo->t1;
     int smpl_t2 = pdinfo->t2;
     int omitlast = 0;
     int *tmplist = NULL;
     int maxlag = 0;
-    int save_test = 0;
     int err = 0;
 
     if (orig == NULL || orig->list == NULL) {
@@ -832,11 +832,17 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
     }
 
     if (omitvars == NULL || omitvars[0] == 0) {
-	omitlast = 1;
+	if (orig->ci == TSLS) {
+	    return E_PARSE;
+	} else {
+	    omitlast = 1;
+	}
     }
 
     /* create list for test model */
-    if (omitlast) {
+    if (orig->ci == TSLS) {
+	tmplist = tsls_list_omit(orig->list, omitvars, opt, &err);	
+    } else if (omitlast) {
 	/* special: just drop the last variable */
 	tmplist = gretl_list_omit_last(orig->list, &err);
     } else {
@@ -847,6 +853,10 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 	return err;
     }
 
+    if (tmplist[0] == 1) {
+	return E_NOVARS;
+    }
+
     /* impose as sample range the estimation range of the 
        original model */
     pdinfo->t1 = orig->t1;
@@ -868,13 +878,11 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 	pdinfo->t1 -= 1;
     }
 
-    if (opt & OPT_S) {
-	/* don't pass OPT_S to replicate estimator */
-	save_test = 1;
-	opt &= ~OPT_S;
-    }
+    /* extract option flags that should not be passed to estimator
+       functions */
+    remove_special_flags(&est_opt);
 
-    *new = replicate_estimator(orig, &tmplist, pZ, pdinfo, opt, prn);
+    *new = replicate_estimator(orig, &tmplist, pZ, pdinfo, est_opt, prn);
 
     if (new->errcode) {
 	pprintf(prn, "%s\n", gretl_errmsg);
@@ -887,7 +895,7 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 	}
 
 	if (print_add_omit_model(orig, opt)) {
-	    printmodel(new, pdinfo, opt, prn); 
+	    printmodel(new, pdinfo, est_opt, prn); 
 	}	
 
 	if (new->nobs == orig->nobs && !omitlast) {
@@ -897,10 +905,6 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *new,
 	    omitlist = gretl_list_diff_new(orig->list, new->list);
 	    cmp = add_or_omit_compare(orig, new, 0, omitlist);
 
-	    if (save_test) {
-		opt |= OPT_S;
-	    }
-
 	    gretl_make_compare(&cmp, omitlist, orig, pdinfo, opt, prn); 
 	    free(omitlist);
 	}
@@ -1045,15 +1049,15 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		2, aux.dfd, RF, pval);
 
 	if (opt & OPT_S) {
-	    ModelTest *test;
+	    ModelTest *test = model_test_new(GRETL_TEST_RESET);
 
-	    test = new_test_on_model(pmod, GRETL_TEST_RESET);
 	    if (test != NULL) {
 		model_test_set_teststat(test, GRETL_STAT_RESET);
 		model_test_set_dfn(test, 2);
 		model_test_set_dfd(test, aux.dfd);
 		model_test_set_value(test, RF);
 		model_test_set_pvalue(test, pval);
+		maybe_add_test_to_model(pmod, test);
 	    }	    
 	}
 
@@ -1067,132 +1071,14 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
-/* Below: apparatus for generating standard errors that are robust in 
-   face of general serial correlation (see Wooldridge, Introductory
-   Econometrics, chapter 12)
-*/
-
-static double get_vhat (double *ahat, int g, int t1, int t2)
-{
-    int t, h;
-    double weight, a_cross_sum;
-    double vhat;
-
-    vhat = 0.0;
-
-    for (t=t1; t<=t2; t++) {
-	vhat += ahat[t] * ahat[t];
-    }
-
-    for (h=1; h<=g; h++) {
-	weight = 1.0 - (double) h / (g + 1);
-	a_cross_sum = 0.0;
-	for (t=h+t1; t<=t2; t++) {
-	    a_cross_sum += ahat[t] * ahat[t-h];
-	}
-	vhat += 2.0 * weight * a_cross_sum;
-    }
-
-    return vhat;
-}
-
-static int autocorr_standard_errors (MODEL *pmod, double ***pZ, 
-				     DATAINFO *pdinfo, PRN *prn)
-{
-    int *auxlist = NULL;
-    double *ahat = NULL;
-    double *robust = NULL;
-    double *tmp;
-    int i, j, g;
-    int aux = AUX_NONE, order = 0;
-    MODEL auxmod;
-
-    auxlist = malloc(pmod->list[0] * sizeof *auxlist);
-    ahat = malloc(pdinfo->n * sizeof *ahat);
-    robust = malloc(pmod->ncoeff * sizeof *robust);
-
-    if (auxlist == NULL || ahat == NULL || robust == NULL) {
-	free(auxlist);
-	free(ahat);
-	free(robust);
-	return E_ALLOC;
-    }
-
-    g = get_hac_lag(pmod->nobs);
-
-    auxlist[0] = pmod->list[0] - 1;
-
-    gretl_model_init(&auxmod);
-
-    /* loop across the indep vars in the original model */
-    for (i=2; i<=pmod->list[0]; i++) {
-	double vhat = 0;
-	double sderr;
-	int k, t;
-
-	/* set the given indep var as the dependent */
-	auxlist[1] = pmod->list[i];
-
-	k = 2;
-	for (j=2; j<=pmod->list[0]; j++) {
-	    /* add other indep vars as regressors */
-	    if (pmod->list[j] == auxlist[1]) continue;
-	    auxlist[k++] = pmod->list[j];
-	}
-
-	auxmod = lsq(auxlist, pZ, pdinfo, OLS, OPT_A, 0.0);
-
-	if (auxmod.errcode) {
-	    fprintf(stderr, "Error estimating auxiliary model, code=%d\n", 
-		    auxmod.errcode);
-	    pmod->sderr[i-2] = NADBL;
-	} else {
-	    /* compute robust standard error */
-	    for (t=pmod->t1; t<=pmod->t2; t++) {
-		ahat[t] = pmod->uhat[t] * auxmod.uhat[t];
-	    }
-	    vhat = get_vhat(ahat, g, pmod->t1, pmod->t2);
-	    sderr = pmod->sderr[i-2] / pmod->sigma;
-	    sderr = sderr * sderr;
-	    sderr *= sqrt(vhat);
-	    robust[i-2] = sderr;
-	}
-
-	clear_model(&auxmod);
-    }
-
-    /* save original model data */
-    tmp = pmod->sderr;
-    aux = pmod->aux;
-    order = pmod->order;
-
-    /* adjust data for SC-robust version */
-    pmod->sderr = robust;
-    pmod->aux = AUX_SCR;
-    pmod->order = g;
-
-    /* print original model, showing robust std errors */
-    printmodel(pmod, pdinfo, OPT_NONE, prn);  
-
-    /* reset the original model data */
-    pmod->sderr = tmp;
-    pmod->aux = aux;
-    pmod->order = order;
-
-    free(auxlist);
-    free(ahat);
-    free(robust);
-	
-    return 0;
-}
-
 /**
  * autocorr_test:
  * @pmod: pointer to model to be tested.
  * @order: lag order for test.
  * @pZ: pointer to data matrix.
  * @pdinfo: information on the data set.
- * @opt: if flags include %OPT_S, save test results to model.
+ * @opt: if flags include %OPT_S, save test results to model;
+ * if %OPT_Q, be less verbose.
  * @prn: gretl printing struct.
  *
  * Tests the given model for autocorrelation of order equal to
@@ -1287,7 +1173,7 @@ int autocorr_test (MODEL *pmod, int order,
 
     if (!err) {
 	newlist[1] = v;
-	/*  printlist(newlist); */
+	/* printlist(newlist); */
 	aux = lsq(newlist, pZ, pdinfo, OLS, OPT_A, 0.0);
 	err = aux.errcode;
 	if (err) {
@@ -1298,34 +1184,31 @@ int autocorr_test (MODEL *pmod, int order,
     if (!err) {
 	aux.aux = AUX_AR;
 	aux.order = order;
-	printmodel(&aux, pdinfo, OPT_NONE, prn);
 	trsq = aux.rsq * aux.nobs;
 	LMF = (aux.rsq/(1.0 - aux.rsq)) * 
 	    (aux.nobs - pmod->ncoeff - order)/order; 
-
-	pprintf(prn, "\n%s: LMF = %f,\n", _("Test statistic"), LMF);
-
 	pval = fdist(LMF, order, aux.nobs - pmod->ncoeff - order);
 
-	pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
-		order, aux.nobs - pmod->ncoeff - order, LMF, pval);
-
-	pprintf(prn, "\n%s: TR^2 = %f,\n", 
-		_("Alternative statistic"), trsq);
-	pprintf(prn, "%s = P(%s(%d) > %g) = %.3g\n\n", 	_("with p-value"), 
-		_("Chi-square"), order, trsq, chisq(trsq, order));
-
-	/* add Ljung-Box Q' */
-	if (ljung_box(v, order, (const double **) *pZ, pdinfo, &lb) == 0) {
-	    pprintf(prn, "Ljung-Box Q' = %g %s = P(%s(%d) > %g) = %.3g\n", 
-		    lb, _("with p-value"), _("Chi-square"), order,
-		    lb, chisq(lb, order));
+	if (pmod->aux != AUX_VAR) {
+	    printmodel(&aux, pdinfo, OPT_NONE, prn);
+	    pprintf(prn, "\n%s: LMF = %f,\n", _("Test statistic"), LMF);
+	    pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
+		    order, aux.nobs - pmod->ncoeff - order, LMF, pval);
+	    pprintf(prn, "\n%s: TR^2 = %f,\n", 
+		    _("Alternative statistic"), trsq);
+	    pprintf(prn, "%s = P(%s(%d) > %g) = %.3g\n\n", 	_("with p-value"), 
+		    _("Chi-square"), order, trsq, chisq(trsq, order));
+	    if (ljung_box(v, order, (const double **) *pZ, pdinfo, &lb) == 0) {
+		pprintf(prn, "Ljung-Box Q' = %g %s = P(%s(%d) > %g) = %.3g\n", 
+			lb, _("with p-value"), _("Chi-square"), order,
+			lb, chisq(lb, order));
+	    }
+	    record_test_result(LMF, pval, "autocorrelation");
 	}
 
 	if (opt & OPT_S) {
-	    ModelTest *test;
+	    ModelTest *test = model_test_new(GRETL_TEST_AUTOCORR);
 
-	    test = new_test_on_model(pmod, GRETL_TEST_AUTOCORR);
 	    if (test != NULL) {
 		model_test_set_teststat(test, GRETL_STAT_LMF);
 		model_test_set_dfn(test, order);
@@ -1333,22 +1216,15 @@ int autocorr_test (MODEL *pmod, int order,
 		model_test_set_order(test, order);
 		model_test_set_value(test, LMF);
 		model_test_set_pvalue(test, pval);
+		maybe_add_test_to_model(pmod, test);
 	    }	    
 	}
-
-	record_test_result(LMF, pval, "autocorrelation");
     }
 
     free(newlist);
     dataset_drop_last_variables(pdinfo->v - v, pZ, pdinfo); 
     clear_model(&aux); 
 
-    if (!(opt & OPT_Q)) {
-	if (pval < 0.05 && !gretl_model_get_int(pmod, "robust")) {
-	    autocorr_standard_errors(pmod, pZ, pdinfo, prn);
-	}
-    }
-
     /* reset sample as it was */
     pdinfo->t1 = smpl_t1;
     pdinfo->t2 = smpl_t2;
@@ -1471,9 +1347,8 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 		    newvars, chow_mod.dfd, F, pval);
 
 	    if (opt & OPT_S) {
-		ModelTest *test;
+		ModelTest *test = model_test_new(GRETL_TEST_CHOW);
 
-		test = new_test_on_model(pmod, GRETL_TEST_CHOW);
 		if (test != NULL) {
 		    model_test_set_teststat(test, GRETL_STAT_F);
 		    model_test_set_param(test, chowdate);
@@ -1481,6 +1356,7 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 		    model_test_set_dfd(test, chow_mod.dfd);
 		    model_test_set_value(test, F);
 		    model_test_set_pvalue(test, pval);
+		    maybe_add_test_to_model(pmod, test);
 		}	  
 	    }
 
@@ -1499,10 +1375,9 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
     return err;
 }
 
-/* ........................................................... */
+/* compute v'Mv, for symmetric M */
 
 static double vprime_M_v (double *v, double *M, int n)
-     /* compute v'Mv, for symmetric M */
 {
     int i, j, jmin, jmax, k;
     double xx, val = 0.0;
@@ -1555,6 +1430,7 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     const int T = pmod->t2 - pmod->t1 + 1;
     const int K = pmod->ncoeff;
     MODEL cum_mod;
+    gretlopt cmodopt = OPT_C | OPT_A;
     char cumdate[OBSLEN];
     double xx, yy, sigma, hct, wbar = 0.0;
     double *cresid = NULL, *W = NULL, *xvec = NULL;
@@ -1585,24 +1461,29 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 
     if (!err) {
 	for (j=0; j<n_est && !err; j++) {
-	    cum_mod = lsq(pmod->list, pZ, pdinfo, OLS, (OPT_C | OPT_A), 0.0);
+	    cum_mod = lsq(pmod->list, pZ, pdinfo, OLS, cmodopt, 0.0);
 	    err = cum_mod.errcode;
 	    if (err) {
 		errmsg(err, prn);
 	    } else {
+#if 0
+		printmodel(&cum_mod, pdinfo, OPT_NONE, prn);
+#endif
 		t = pdinfo->t2 + 1;
 		yy = 0.0;
-		for (i=0; i<K; i++) {
+		for (i=0; i<cum_mod.ncoeff; i++) {
 		    xno = cum_mod.list[i+2];
 		    xvec[i] = (*pZ)[xno][t];
-		    yy += cum_mod.coeff[i] * (*pZ)[xno][t];
+		    yy += cum_mod.coeff[i] * xvec[i];
 		}
 		cresid[j] = (*pZ)[yno][t] - yy;
 		cum_mod.ci = CUSUM;
 		makevcv(&cum_mod);
-		xx = vprime_M_v(xvec, cum_mod.vcv, K);
+		xx = vprime_M_v(xvec, cum_mod.vcv, cum_mod.ncoeff);
 		cresid[j] /= sqrt(1.0 + xx);
-		/*  printf("w[%d] = %g\n", t, cresid[j]); */
+#if 0
+		printf("cresid[%d] = %g\n", j, cresid[j]);
+#endif
 		wbar += cresid[j];
 		clear_model(&cum_mod);
 		pdinfo->t2 += 1;
@@ -1631,8 +1512,11 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	xx = 0.948 * sqrt((double) (T-K));
 	yy = 2.0 * xx / (T-K);
 
-	pputs(prn, _("Cumulated sum of scaled residuals\n"
-		"('*' indicates a value outside of 95%% confidence band):\n\n"));
+	pputs(prn, _("Cumulated sum of scaled residuals"));
+	pputc(prn, '\n');
+	pputs(prn, /* xgettext:no-c-format */
+	      _("('*' indicates a value outside of 95% confidence band)"));
+	pputs(prn, "\n\n");
     
 	for (j=0; j<n_est; j++) {
 	    W[j] = 0.0;
@@ -1653,38 +1537,37 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		T-K-1, hct, pval);
 
 	if (opt & OPT_S) {
-	    ModelTest *test;
+	    ModelTest *test = model_test_new(GRETL_TEST_CUSUM);
 
-	    test = new_test_on_model(pmod, GRETL_TEST_CUSUM);
 	    if (test != NULL) {
 		model_test_set_teststat(test, GRETL_STAT_HARVEY_COLLIER);
 		model_test_set_dfn(test, T-K-1);
 		model_test_set_value(test, hct);
 		model_test_set_pvalue(test, pval);
+		maybe_add_test_to_model(pmod, test);
 	    }
 	}
 
 	record_test_result(hct, pval, "Harvey-Collier");
 
-#ifdef ENABLE_NLS
-        setlocale(LC_NUMERIC, "C");
-#endif
+	gretl_push_c_numeric_locale();
+
 	/* plot with 95% confidence bands, if not batch mode */
 	if (!gretl_in_batch_mode() &&
 	    gnuplot_init(PLOT_CUSUM, &fq) == 0) {
-	    fputs("# CUSUM test\n", fq);
-	    fprintf(fq, "set xlabel \"%s\"\n", I_("Observation"));
+	    fprintf(fq, "set xlabel '%s'\n", I_("Observation"));
 	    fputs("set xzeroaxis\n", fq);
-	    fprintf(fq, "set title \"%s\"\n",
+	    fprintf(fq, "set title '%s'\n",
 		    /* xgettext:no-c-format */
 		    I_("CUSUM plot with 95% confidence band"));
 	    fputs("set nokey\n", fq);
-	    fprintf(fq, "plot %f+%f*x w l 1, \\\n", xx - K*yy, yy);
-	    fprintf(fq, "%f-%f*x w l 1, \\\n", -xx + K*yy, yy);
-	    fputs("'-' using 1:2 w lp\n", fq);
+	    fprintf(fq, "plot \\\n%g+%g*x title '' w lines lt 2, \\\n", xx - K*yy, yy);
+	    fprintf(fq, "%g-%g*x title '' w lines lt 2, \\\n", -xx + K*yy, yy);
+	    fputs("'-' using 1:2 w linespoints lt 1\n", fq);
+
 	    for (j=0; j<n_est; j++) { 
 		t = pmod->t1 + K + j;
-		fprintf(fq, "%d %f\n", t, W[j]);
+		fprintf(fq, "%d %g\n", t, W[j]);
 	    }
 	    fputs("e\n", fq);
 
@@ -1693,9 +1576,7 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	    err = gnuplot_make_graph();
 	}
 
-#ifdef ENABLE_NLS
-        setlocale(LC_NUMERIC, "");
-#endif
+	gretl_pop_c_numeric_locale();
     }
 
     /* restore sample */
