@@ -20,16 +20,13 @@
 #ifndef COMPAT_H
 #define COMPAT_H
 
-void graphyzx (const int *list, const double *zy1, const double *zy2, 
-	       const double *zx, int n, const char *yname, 
+void graphyzx (const int *list, const double *y1, const double *y2, 
+	       const double *x, int n, const char *yname, 
 	       const char *xname, const DATAINFO *pdinfo, 
 	       gretlopt oflag, PRN *prn);
 
-int ascii_plot (const int *list, const double **Z, const DATAINFO *pdinfo, 
-		gretlopt oflag, PRN *prn);
-
 int ascii_graph (const int *list, const double **Z, const DATAINFO *pdinfo, 
-		 gretlopt oflag, PRN *prn);
+		 gretlopt opt, PRN *prn);
 
 int rhodiff (char *param, const int *list, double ***pZ, DATAINFO *pdinfo);
 
