@@ -44,6 +44,8 @@ typedef enum {
     GRETL_OCTAVE,         /* GNU octave ascii data file */
     GRETL_GNUMERIC,       /* gnumeric workbook data */
     GRETL_EXCEL,          /* MS Excel spreadsheet data */
+    GRETL_WF1,            /* Eviews workfile data */
+    GRETL_DTA,            /* Stata .dta data */
     GRETL_SCRIPT,         /* file containing gretl commands */
     GRETL_NATIVE_DB,      /* gretl database */
     GRETL_RATS_DB,        /* RATS 4.0 database */
@@ -73,6 +75,8 @@ typedef enum {
 
 #define DBNA  -999.0 /* missing value code for gretl databases */
 
+#define GRETL_SCALAR_DIGITS 12
+
 /* functions follow */
 
 int dateton (const char *date, const DATAINFO *pdinfo);
@@ -124,11 +128,6 @@ GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn);
 
 int check_varname (const char *varname);
 
-int get_xmldata (double ***pZ, DATAINFO **ppdinfo, char *fname,
-		 PATHS *ppaths, int data_status, PRN *prn, int gui); 
-
-char *get_xml_description (const char *fname);
-
 int check_atof (const char *numstr);
 
 int transpose_data (double ***pZ, DATAINFO *pdinfo);
