@@ -30,14 +30,14 @@
 
 void free_Z (double **Z, DATAINFO *pdinfo)
 {
-    int i;
+    if (Z != NULL && pdinfo != NULL) {
+	int i;
 
-    if (Z == NULL || pdinfo == NULL) return;
-
-    for (i=0; i<pdinfo->v; i++) {
-	free(Z[i]);
+	for (i=0; i<pdinfo->v; i++) {
+	    free(Z[i]);
+	}
+	free(Z);
     }
-    free(Z);
 }
 
 /**
@@ -126,11 +126,11 @@ void clear_datainfo (DATAINFO *pdinfo, int code)
 	    free(pdinfo->varinfo);
 	    pdinfo->varinfo = NULL;
 	}
-	if (pdinfo->descrip) {
+	if (pdinfo->descrip != NULL) {
 	    free(pdinfo->descrip);
 	    pdinfo->descrip = NULL;
 	}
-	if (pdinfo->vector) {
+	if (pdinfo->vector != NULL) {
 	    free(pdinfo->vector);
 	    pdinfo->vector = NULL;
 	}
@@ -142,6 +142,43 @@ void clear_datainfo (DATAINFO *pdinfo, int code)
     } 
 }
 
+/**
+ * destroy_dataset:
+ * @Z: data array.
+ * @pdinfo: dataset information struct.
+ *
+ * Frees all resources associated with @Z and @pdinfo.
+ */
+
+void destroy_dataset (double **Z, DATAINFO *pdinfo)
+{
+    free_Z(Z, pdinfo);
+
+    if (pdinfo != NULL) {
+	clear_datainfo(pdinfo, CLEAR_FULL); 
+	free(pdinfo);
+    }
+}
+
+/**
+ * copy_dataset_obs_info:
+ * @targ: target dataset information struct.
+ * @src: source dataset information struct.
+ *
+ * Sets the "date" or observations information in @targ to that
+ * found in @src.
+ */
+
+void copy_dataset_obs_info (DATAINFO *targ, const DATAINFO *src)
+{
+    strcpy(targ->stobs, src->stobs);
+    strcpy(targ->endobs, src->endobs);
+    targ->sd0 = src->sd0;
+    targ->pd = src->pd;
+    targ->structure = src->structure;
+    targ->decpoint = src->decpoint;
+}
+
 /**
  * dataset_obs_info_default:
  * @pdinfo: dataset information struct.
@@ -440,8 +477,14 @@ int allocate_Z (double ***pZ, const DATAINFO *pdinfo)
     }
 
     if (!err) {
-	for (t=0; t<pdinfo->n; t++) {
-	    Z[0][t] = 1.0; 
+	for (i=0; i<pdinfo->v; i++) {
+	    for (t=0; t<pdinfo->n; t++) {
+		if (i == 0) {
+		    Z[i][t] = 1.0; 
+		} else {
+		    Z[i][t] = NADBL;
+		}
+	    }
 	}
     }
 
@@ -459,12 +502,14 @@ int allocate_Z (double ***pZ, const DATAINFO *pdinfo)
  * Initializes the data matrix pointed to by @pZ (adding the constant in
  * position 0) and the data information struct @pdinfo.
  * 
- * Returns: 0 on successful completion, 1 on error.
+ * Returns: 0 on successful completion, non-zero on error.
  */
 
 int start_new_Z (double ***pZ, DATAINFO *pdinfo, int resample)
 {
-    if (allocate_Z(pZ, pdinfo)) return 1;
+    if (allocate_Z(pZ, pdinfo)) {
+	return E_ALLOC;
+    }
 
     pdinfo->t1 = 0; 
     pdinfo->t2 = pdinfo->n - 1;
@@ -473,7 +518,9 @@ int start_new_Z (double ***pZ, DATAINFO *pdinfo, int resample)
 	pdinfo->varname = NULL;
 	pdinfo->varinfo = NULL;
     } else if (dataset_allocate_varnames(pdinfo)) {
-	return 1;
+	free_Z(*pZ, pdinfo);
+	*pZ = NULL;
+	return E_ALLOC;
     }
 
     pdinfo->S = NULL;
@@ -515,24 +562,68 @@ static int reallocate_markers (DATAINFO *pdinfo, int n)
     return 0;
 }
 
-static int real_periodic_dummy (const double *x, int n,
-				int *pd, int *offset,
-				int *trail)
+/* Allow for the possibility of centered seasonal dummies: usually
+   xon = 1 and xoff = 0, but in the centered case xon = 1 - 1/pd 
+   and xoff = -1/pd.
+*/
+
+static int get_xon_xoff (const double *x, int n, int pd, double *xon, double *xoff)
 {
-    int onebak = 0;
+    double cfac = 1.0 / pd;
+    double xc = 1.0 - cfac, yc = -cfac;
+    double x0 = 999, y0 = 999;
+    int t, ret = 1;
+
+    for (t=0; t<n && ret; t++) {
+	if (x[t] == 1.0) {
+	    if (x0 == 999) x0 = 1.0;
+	    else if (x[t] != x0) ret = 0;
+	} else if (x[t] == 0.0) {
+	    if (y0 == 999) y0 = 0.0;
+	    else if (x[t] != y0) ret = 0;
+	} else if (x[t] == xc) {
+	    if (x0 == 999) x0 = xc;
+	    else if (x[t] != x0) ret = 0;
+	} else if (x[t] == yc) {
+	    if (y0 == 999) y0 = yc;
+	    else if (x[t] != y0) ret = 0;
+	} else {
+	    ret = 0;
+	}
+    }
+
+    if (ret) {
+	*xon = x0;
+	*xoff = y0;
+    }
+
+    return ret;
+}
+
+static int real_periodic_dummy (const double *x, int n,
+				int *pd, int *offset, 
+				double *pxon, double *pxoff)
+{
+    double xon = 1.0, xoff = 0.0;
+    int onbak = 0;
     int gap = 0;
+    int trail = 0;
     int t, m = n - 1, ret = 1;
 
+    if (!get_xon_xoff(x, n, *pd, &xon, &xoff)) {
+	return 0;
+    }
+
     *pd = -1;
     *offset = -1;
-    *trail = 0;
+    trail = 0;
 
-    /* find number of trailing zeros */
+    /* find number of trailing "off" values */
     for (t=n-1; t>0; t--) {
-	if (x[t] == 0.0) {
-	    *trail += 1;
+	if (x[t] == xoff) {
+	    trail++;
 	} else {
-	    if (x[t] == 1.0) {
+	    if (x[t] == xon) {
 		m = t;
 	    } else {
 		ret = 0;
@@ -543,11 +634,11 @@ static int real_periodic_dummy (const double *x, int n,
 
     /* check for dummyhood and periodicity */
     for (t=0; t<=m && ret; t++) {
-	if (x[t] == 0.0) {
-	    onebak = 0;
+	if (x[t] == xoff) {
+	    onbak = 0;
 	    gap++;
-	} else if (x[t] == 1.0) {
-	    if (onebak) {
+	} else if (x[t] == xon) {
+	    if (onbak) {
 		ret = 0;
 	    } else if (*offset < 0) {
 		*offset = gap;
@@ -558,47 +649,42 @@ static int real_periodic_dummy (const double *x, int n,
 		}
 	    } else if (gap != *pd - 1) {
 		ret = 0;
-	    } else if (gap < *trail) {
+	    } else if (gap < trail) {
 		ret = 0;
 	    }
 	    gap = 0;
-	    onebak = 1;
+	    onbak = 1;
 	} else {
 	    ret = 0;
 	    break;
 	}
     }
 
+    if (ret && pxon != NULL && pxoff != NULL) {
+	*pxon = xon;
+	*pxoff = xoff;
+    }
+
     return ret;
 }
 
 /**
  * is_periodic_dummy:
  * @x: array to examine.
- * @n: number of elements in array.
+ * @pdinfo: pointer to dataset information struct.
  *
  * Returns: 1 if @x is a periodic dummy variable,
  * 0 otherwise.
  */
 
-int is_periodic_dummy (const double *x, int n)
+int is_periodic_dummy (const double *x, const DATAINFO *pdinfo)
 {
-    int pd, offset, trail;
+    int offset, pd = pdinfo->pd;
 
-    return real_periodic_dummy(x, n, &pd, &offset, &trail);
+    return real_periodic_dummy(x, pdinfo->n, &pd, &offset, NULL, NULL);
 }
 
-/**
- * is_trend_variable:
- * @x: array to examine.
- * @n: number of elements in array.
- *
- * Returns: 1 if @x is a simple linear trend variable, with each
- * observation equal to the preceding observation plus 1, 
- * 0 otherwise.
- */
-
-int is_trend_variable (const double *x, int n)
+static int is_linear_trend (const double *x, int n)
 {
     int t, ret = 1;
     
@@ -612,33 +698,76 @@ int is_trend_variable (const double *x, int n)
     return ret;
 }
 
+static int is_quadratic_trend (const double *x, int n)
+{
+    double t2;
+    int t, ret = 1;
+    
+    for (t=0; t<n; t++) {
+	t2 = (t + 1) * (t + 1);
+	if (x[t] != t2) {
+	    ret = 0;
+	    break;
+	}
+    }
+
+    return ret;
+}
+
+/**
+ * is_trend_variable:
+ * @x: array to examine.
+ * @n: number of elements in array.
+ *
+ * Returns: 1 if @x is a simple linear trend variable, with each
+ * observation equal to the preceding observation plus 1, or
+ * if @x is a quadratic trend starting at 1 for the first 
+ * observation in the data set, and 0 otherwise.
+ */
+
+int is_trend_variable (const double *x, int n)
+{
+    int ret = 0;
+
+    if (is_linear_trend(x, n)) {
+	ret = 1;
+    } else if (is_quadratic_trend(x, n)) {
+	ret = 1;
+    }
+
+    return ret;
+}
+
 static void 
 maybe_extend_trends (double **Z, const DATAINFO *pdinfo, int oldn)
 {
     int i, t;
 
     for (i=1; i<pdinfo->v; i++) {
-	if (is_trend_variable(Z[i], oldn)) {
+	if (is_linear_trend(Z[i], oldn)) {
 	    for (t=oldn; t<pdinfo->n; t++) {
 		Z[i][t] = Z[i][t-1] + 1.0;
 	    }
-	}
+	} else if (is_quadratic_trend(Z[i], oldn)) {
+	    for (t=oldn; t<pdinfo->n; t++) {
+		Z[i][t] = (t + 1) * (t + 1);
+	    }
+	}	    
     }
 }
 
 static void 
 maybe_extend_dummies (double **Z, const DATAINFO *pdinfo, int oldn)
 {
-    int pd, offset, trail;
+    int pd = pdinfo->pd;
+    double xon = 1.0, xoff = 0.0;
+    int offset;
     int i, t;
 
-    for (i=0; i<pdinfo->v; i++) {
-	if (real_periodic_dummy(Z[i], oldn, &pd, &offset, &trail)) {
-	    int s = pd - offset;
-
+    for (i=1; i<pdinfo->v; i++) {
+	if (real_periodic_dummy(Z[i], oldn, &pd, &offset, &xon, &xoff)) {
 	    for (t=oldn; t<pdinfo->n; t++) {
-		Z[i][t] = (s % pd)? 0.0 : 1.0;
-		s++;
+		Z[i][t] = ((t - offset) % pd)? xoff : xon;
 	    }
 	}
     }
@@ -651,8 +780,10 @@ maybe_extend_dummies (double **Z, const DATAINFO *pdinfo, int oldn)
  * @pdinfo: dataset information.
  *
  * Extends all series in the dataset by the specified number of
- * extra observations, and initializes all the added values to
- * the missing value code, #NADBL.
+ * extra observations.  The added values are initialized to
+ * the missing value code, #NADBL, with the exception of
+ * simple deterministic variables such as a time trend and
+ * periodic dummy variables, which are extrapolated.
  *
  * Returns: 0 on success, %E_ALLOC on error.
  */
