@@ -36,6 +36,17 @@ typedef enum {
 #define dataset_is_time_series(p) ((p)->structure == TIME_SERIES || \
 				   (p)->structure == SPECIAL_TIME_SERIES)
 
+/**
+ * dataset_is_seasonal:
+ * @p: pointer to data information struct.
+ *
+ * Attempt to determine whether a data set contains seasonal time series
+ * data (1) or not (0).
+ */
+#define dataset_is_seasonal(p) (((p)->structure == TIME_SERIES || \
+                                (p)->structure == SPECIAL_TIME_SERIES) && \
+                                (p)->pd > 1)
+
 /**
  * custom_time_series:
  * @p: pointer to data information struct.
@@ -128,7 +139,18 @@ typedef enum {
 #define calendar_data(p) (p->structure == TIME_SERIES && \
                           (p->pd == 5 || p->pd == 6 || p->pd == 7 \
                            || p->pd == 52) && p->sd0 > 10000.0) 
-                          
+
+
+/**
+ * quarterly_or_monthly:
+ * @p: pointer to data information struct.
+ *
+ * Attempt to determine whether a data set is a quarterly
+ * or monthly time series (1), or something else (0).
+ */
+#define quarterly_or_monthly(p) (p->structure == TIME_SERIES && \
+                                 (p->pd == 4 || p->pd == 12))
+
 /**
  * dataset_is_panel:
  * @p: pointer to data information struct.
@@ -149,6 +171,8 @@ DATAINFO *create_new_dataset (double ***pZ, /* data matrix */
 			      int markers   /* case markers or not? */
 			      );
 
+void destroy_dataset (double **Z, DATAINFO *pdinfo);
+
 void clear_datainfo (DATAINFO *pdinfo, int code);
 
 int allocate_Z (double ***pZ, const DATAINFO *pdinfo);
@@ -161,6 +185,8 @@ void dataset_destroy_obs_markers (DATAINFO *pdinfo);
 
 void dataset_obs_info_default (DATAINFO *pdinfo);
 
+void copy_dataset_obs_info (DATAINFO *targ, const DATAINFO *src);
+
 void set_sorted_markers (DATAINFO *pdinfo, int v, char **S);
 
 void dataset_set_regular_markers (DATAINFO *pdinfo);
@@ -169,7 +195,7 @@ int start_new_Z (double ***pZ, DATAINFO *pdinfo, int resample);
 
 int is_trend_variable (const double *x, int n);
 
-int is_periodic_dummy (const double *x, int n);
+int is_periodic_dummy (const double *x, const DATAINFO *pdinfo);
 
 int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo);
 
