@@ -25,7 +25,7 @@
 # include <glib.h>
 #endif
 
-#undef DB_DEBUG
+#define DB_DEBUG 0
 
 #define RECNUM long
 #define NAMELENGTH 16
@@ -88,9 +88,8 @@ static int db_type;
 
 static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo, 
 			    double ***pZ, DATAINFO *pdinfo,
-			    int compact_method, int dbv);
+			    CompactMethod method, int dbv);
 
-/* ........................................................... */
 
 int get_native_db_data (const char *dbbase, SERIESINFO *sinfo, 
 			double **Z)
@@ -320,13 +319,14 @@ static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
 			   int n, int offset)
 {
     int startfrac = 0;
-    char pdstr[3]; 
+    char pdstr[4] = {0}; 
     int err = 0;
 
-    if (dinfo_sanity_check(dinfo)) return 1;
+    if (dinfo_sanity_check(dinfo)) {
+	return 1;
+    }
 
     sprintf(sinfo->stobs, "%d", dinfo->year); 
-    *pdstr = 0;
 
     if (dinfo->info == 4) {
 	sprintf(pdstr, ".%d", dinfo->month);
@@ -356,6 +356,7 @@ static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
     if (*pdstr) {
 	strcat(sinfo->stobs, pdstr);
     }
+
     get_endobs(sinfo->endobs, dinfo->year, startfrac, dinfo->info, n);
 
     sinfo->pd = dinfo->info;
@@ -367,6 +368,10 @@ static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
 
     sinfo->offset = offset;
 
+#if DB_DEBUG
+    fprintf(stderr, "'%s': set sinfo->offset = %d\n", varname, (int) offset);
+#endif
+
     return err;
 }
 
@@ -437,6 +442,7 @@ static RECNUM read_rats_directory (FILE *fp, db_table_row *row,
     fread(&rdir.first_data, sizeof(RECNUM), 1, fp);
     fread(rdir.series_name, 16, 1, fp);  
     rdir.series_name[8] = '\0';
+
     chopstr(rdir.series_name);
 
     if (series_name != NULL && strcmp(series_name, rdir.series_name)) {
@@ -445,7 +451,9 @@ static RECNUM read_rats_directory (FILE *fp, db_table_row *row,
     }
 
     /* Now the dateinfo: we can't read this in one go either :-( */
-    fseek(fp, 12, SEEK_CUR); /* skip long, short, long, short */
+
+    /* skip long, short, long, short */
+    fseek(fp, 12, SEEK_CUR);
     fread(&dinfo.info, sizeof(long), 1, fp);
     fread(&dinfo.digits, sizeof(short), 1, fp);
     fread(&dinfo.year, sizeof(short), 1, fp);
@@ -466,6 +474,11 @@ static RECNUM read_rats_directory (FILE *fp, db_table_row *row,
     rdir.comments[1][79] = '\0';
     chopstr(rdir.comments[1]);
 
+#if DB_DEBUG
+    fprintf(stderr, "read_rats_directory: sinfo = %p, row = %p\n", 
+	    (void *) sinfo, (void *) row);
+#endif
+
     if (sinfo != NULL) {
 	err = dinfo_to_sinfo(&dinfo, sinfo, rdir.series_name, rdir.comments[0],
 			     rdir.datapoints, rdir.first_data);
@@ -476,6 +489,11 @@ static RECNUM read_rats_directory (FILE *fp, db_table_row *row,
 	err = 1;
     }
 
+#if DB_DEBUG
+    fprintf(stderr, "read_rats_directory: err = %d, rdir.forward_point = %d\n",
+	    err, (int) rdir.forward_point);
+#endif
+
     if (err) {
 	return RATS_PARSE_ERROR;
     } else {
@@ -619,8 +637,6 @@ db_table *read_rats_db (FILE *fp)
     return tbl;
 }
 
-/* ........................................................... */
-
 static int find_rats_dir_by_number (FILE *fp, int first_dir, 
 				    int series_number)
 {
@@ -719,7 +735,9 @@ int get_rats_data_by_series_number (const char *fname,
     offset = get_rats_series_offset_by_number(fp, series_number);
     if (offset < 0) return DB_NOT_FOUND;
 
-    fprintf(stderr, "get_rats_data_by_series_number: offset=%d\n", offset);
+#if DB_DEBUG
+    fprintf(stderr, "get_rats_data_by_series_number: offset = %d\n", offset);
+#endif
     
     fseek(fp, (offset - 1) * 256 + 12, SEEK_SET); 
     fread(&first_data, sizeof(RECNUM), 1, fp);
@@ -759,8 +777,12 @@ get_rats_series_offset_by_name (FILE *fp,
     while (forward) {
 	fseek(fp, (forward - 1) * 256L, SEEK_SET);
 	forward = read_rats_directory(fp, NULL, series_name, sinfo);
-	if (forward == RATS_PARSE_ERROR) sinfo->offset = -1;
-	if (sinfo->offset != 0) break;
+	if (forward == RATS_PARSE_ERROR) {
+	    sinfo->offset = -1;
+	}
+	if (sinfo->offset != 0) {
+	    break;
+	}
     }
 
     return sinfo->offset;
@@ -775,15 +797,23 @@ get_rats_series_info_by_name (const char *series_name,
     int ret = DB_OK;
 
     fp = gretl_fopen(db_name, "rb");
-    if (fp == NULL) return DB_NOT_FOUND;
+    if (fp == NULL) {
+	return DB_NOT_FOUND;
+    }
+
+#if DB_DEBUG
+    fprintf(stderr, "Opened %s\n", db_name);
+#endif
     
     offset = get_rats_series_offset_by_name(fp, series_name, sinfo);
     fclose(fp);
 
-    if (offset <= 0) return DB_NOT_FOUND;
+    if (offset <= 0) {
+	return DB_NOT_FOUND;
+    }
 
-#ifdef DB_DEBUG
-    fprintf(stderr, "get_rats_series_info_by_name: offset=%d\n", offset);
+#if DB_DEBUG
+    fprintf(stderr, "get_rats_series_info_by_name: offset = %d\n", offset);
     fprintf(stderr, " pd = %d, nobs = %d\n", sinfo->pd, sinfo->nobs);
 #endif
 
@@ -812,8 +842,8 @@ get_rats_data_by_offset (const char *fname,
 
 /* For importation of database series */
 
-static double *get_compacted_xt (const double *src,
-				 int n, int method, int compfac,
+static double *get_compacted_xt (const double *src, int n, 
+				 CompactMethod method, int compfac,
 				 int skip)
 {
     int p, t;
@@ -860,7 +890,7 @@ static double *get_compacted_xt (const double *src,
 */
 
 double *compact_db_series (const double *src, SERIESINFO *sinfo,
-			   int target_pd, int method)
+			   int target_pd, CompactMethod method)
 {
     int p0, y0, endskip, goodobs;
     int skip = 0, compfac = sinfo->pd / target_pd;
@@ -898,7 +928,7 @@ double *compact_db_series (const double *src, SERIESINFO *sinfo,
     goodobs = (sinfo->nobs - skip - endskip) / compfac;
     sinfo->nobs = goodobs;
 
-#ifdef DB_DEBUG
+#if DB_DEBUG
     fprintf(stderr, "startskip = %d\n", skip);
     fprintf(stderr, "endskip = %d\n", endskip);
     fprintf(stderr, "goodobs = %d\n", goodobs);
@@ -913,7 +943,60 @@ double *compact_db_series (const double *src, SERIESINFO *sinfo,
     return x;
 }
 
-/* ........................................................... */
+/* Expand a single series from a database, for importation 
+   into a working dataset of higher frequency.  At present
+   this is permitted only for the cases:
+
+     annual    -> quarterly
+     annual    -> monthly
+     quarterly -> monthly
+*/
+
+double *expand_db_series (const double *src, SERIESINFO *sinfo,
+			  int target_pd)
+{
+    char stobs[12] = {0};
+    int oldn = sinfo->nobs;
+    int mult, newn;
+    double *x = NULL;
+    int j, s, t;
+
+    mult = target_pd / sinfo->pd;
+    newn = mult * sinfo->nobs;
+
+    x = malloc(newn * sizeof *x);
+    if (x == NULL) {
+	return NULL;
+    }    
+
+    s = 0;
+    for (t=0; t<oldn; t++) {
+	for (j=0; j<mult; j++) {
+	    x[s++] = src[t];
+	}
+    }
+
+    if (sinfo->pd == 1) {
+	strcpy(stobs, sinfo->stobs);
+	if (target_pd == 4) {
+	    strcat(stobs, ":1");
+	} else {
+	    strcat(stobs, ":01");
+	}
+    } else {
+	int yr, qtr, mo;
+
+	sscanf(sinfo->stobs, "%d.%d", &yr, &qtr);
+	mo = (qtr - 1) * 3 + 1;
+	sprintf(stobs, "%d:%02d", yr, mo);
+    }
+
+    strcpy(sinfo->stobs, stobs);
+    sinfo->pd = target_pd;
+    sinfo->nobs = newn;
+
+    return x;
+}
 
 int set_db_name (const char *fname, int filetype, const PATHS *ppaths, 
 		 PRN *prn)
@@ -985,7 +1068,7 @@ int db_set_sample (const char *line, DATAINFO *pdinfo)
     pdinfo->n = t2 - t1 + 1;
     strcpy(pdinfo->endobs, stop);
 
-#ifdef DB_DEBUG
+#if DB_DEBUG
     fprintf(stderr, "db_set_sample: t1=%d, t2=%d, stobs='%s', endobs='%s' "
 	    "sd0 = %g, n = %d\n", 
 	    pdinfo->t1, pdinfo->t2, 
@@ -1016,7 +1099,7 @@ get_word_and_advance (const char *s, char *word, size_t maxlen)
 }
 
 static const char *
-get_compact_method_and_advance (const char *s, int *method)
+get_compact_method_and_advance (const char *s, CompactMethod *method)
 {
     const char *p;
 
@@ -1084,41 +1167,46 @@ int db_get_series (const char *line, double ***pZ, DATAINFO *pdinfo,
 		   PRN *prn)
 {
     char series[16];
-    int comp_method;
+    CompactMethod method;
     SERIESINFO sinfo;
     double **dbZ;
     int err = 0;
 
-#ifdef DB_DEBUG
+#if DB_DEBUG
     fprintf(stderr, "db_get_series: line='%s', pZ=%p, pdinfo=%p\n", 
 	    line, (void *) pZ, (void *) pdinfo);
+    fprintf(stderr, "db_name = '%s'\n", db_name);
 #endif
 
-    if (*db_name == 0) {
+    if (*db_name == '\0') {
 	strcpy(gretl_errmsg, _("No database has been opened"));
 	return 1;
     }   
 
-    line = get_compact_method_and_advance(line, &comp_method);
+    line = get_compact_method_and_advance(line, &method);
 
     /* now loop over variable names given on the line */
 
     while ((line = get_word_and_advance(line, series, 8))) {
-	int v, this_var_method = comp_method; 
+	int v, this_var_method = method; 
 
 	/* see if the series is already in the dataset */
 	v = varindex(pdinfo, series);
-#ifdef DB_DEBUG
+#if DB_DEBUG
 	fprintf(stderr, "db_get_series: pdinfo->v = %d, v = %d\n", pdinfo->v, v);
 #endif
-	if (v < pdinfo->v && comp_method == COMPACT_NONE) {
+	if (v < pdinfo->v && method == COMPACT_NONE) {
 	    this_var_method = COMPACT_METHOD(pdinfo, v);
 	}
 
+#if DB_DEBUG
+	fprintf(stderr, "this_var_method = %d\n", this_var_method);
+#endif
+
 	/* find the series information in the database */
 	if (db_type == GRETL_RATS_DB) {
 	    err = get_rats_series_info_by_name(series, &sinfo);
-	} else {	
+	} else {
 	    err = get_native_series_info(series, &sinfo);
 	} 
 
@@ -1160,34 +1248,43 @@ void get_db_padding (SERIESINFO *sinfo, DATAINFO *pdinfo,
 		     int *pad1, int *pad2)
 {
     *pad1 = dateton(sinfo->stobs, pdinfo); 
+    fprintf(stderr, "pad1 = dateton(%s) = %d\n", sinfo->stobs, *pad1);
+
     *pad2 = pdinfo->n - sinfo->nobs - *pad1;
+    fprintf(stderr, "pad2 = pdinfo->n - sinfo->nobs - pad1 = %d - %d - %d = %d\n", 
+	    pdinfo->n, sinfo->nobs, *pad1, *pad2);
 } 
 
 int check_db_import (SERIESINFO *sinfo, DATAINFO *pdinfo)
 {
     double sd0, sdn_new, sdn_old;
+    int err = 0;
 
     if (sinfo->pd < pdinfo->pd) {
-	strcpy(gretl_errmsg, _("You can't add a lower frequency series to a\nhigher "
-	       "frequency working data set."));
-	return 1;
+	if (sinfo->pd != 1 && sinfo->pd != 4 && 
+	    pdinfo->pd != 4 && pdinfo->pd != 12) {
+	    strcpy(gretl_errmsg, _("Sorry, can't handle this conversion yet!"));
+	    err = 1;
+	} 
     }
 
-    sd0 = get_date_x(sinfo->pd, sinfo->stobs);
-    sdn_new = get_date_x(sinfo->pd, sinfo->endobs);
-    sdn_old = get_date_x(pdinfo->pd, pdinfo->endobs);
-    if (sd0 > sdn_old || sdn_new < pdinfo->sd0) {
-	strcpy(gretl_errmsg, _("Observation range does not overlap\nwith the working "
-	       "data set"));
-	return 1;
+    if (!err) {
+	sd0 = get_date_x(sinfo->pd, sinfo->stobs);
+	sdn_new = get_date_x(sinfo->pd, sinfo->endobs);
+	sdn_old = get_date_x(pdinfo->pd, pdinfo->endobs);
+	if (sd0 > sdn_old || sdn_new < pdinfo->sd0) {
+	    strcpy(gretl_errmsg, _("Observation range does not overlap\nwith the working "
+				   "data set"));
+	    err = 1;
+	}
     }
-
-    return 0;
+    
+    return err;
 }
 
 static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo, 
 			    double ***pZ, DATAINFO *pdinfo,
-			    int compact_method, int dbv)
+			    CompactMethod method, int dbv)
 {
     double *xvec = NULL;
     int n, t, start, stop, pad1 = 0, pad2 = 0;
@@ -1207,7 +1304,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	}
     }
 
-#ifdef DB_DEBUG
+#if DB_DEBUG
     fprintf(stderr, "Z=%p\n", (void *) *pZ);
     fprintf(stderr, "pdinfo->n = %d, pdinfo->v = %d, pdinfo->varname = %p\n",
 	    pdinfo->n, pdinfo->v, (void *) pdinfo->varname);
@@ -1230,7 +1327,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	    }
 	    return 1;
 	}
-	if (compact_method == COMPACT_NONE) {
+	if (method == COMPACT_NONE) {
 	    sprintf(gretl_errmsg, _("%s: you must specify a compaction method"), 
 		    sinfo->varname);
 	    if (new) {
@@ -1238,7 +1335,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	    }
 	    return 1;
 	}
-	xvec = compact_db_series(dbZ[1], sinfo, pdinfo->pd, compact_method);
+	xvec = compact_db_series(dbZ[1], sinfo, pdinfo->pd, method);
     } else {  
 	/* series does not need compacting */
 	xvec = malloc(sinfo->nobs * sizeof *xvec);
@@ -1260,11 +1357,11 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
     /* common stuff for adding a var */
     strcpy(pdinfo->varname[dbv], sinfo->varname);
     strcpy(VARLABEL(pdinfo, dbv), sinfo->descrip);
-    COMPACT_METHOD(pdinfo, dbv) = compact_method;
+    COMPACT_METHOD(pdinfo, dbv) = method;
     get_db_padding(sinfo, pdinfo, &pad1, &pad2);
 
     if (pad1 > 0) {
-#ifdef DB_DEBUG
+#if DB_DEBUG
 	fprintf(stderr, "Padding at start, %d obs\n", pad1);
 #endif
 	for (t=0; t<pad1; t++) {
@@ -1274,7 +1371,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
     } else start = 0;
 
     if (pad2 > 0) {
-#ifdef DB_DEBUG
+#if DB_DEBUG
 	fprintf(stderr, "Padding at end, %d obs\n", pad2);
 #endif
 	for (t=n-1; t>=n-1-pad2; t--) {
@@ -1284,7 +1381,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
     } else stop = n;
 
     /* fill in actual data values */
-#ifdef DB_DEBUG
+#if DB_DEBUG
     fprintf(stderr, "Filling in values from %d to %d\n", start, stop - 1);
 #endif
     for (t=start; t<stop; t++) {
@@ -1303,14 +1400,14 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 
 static double *compact_series (const double *src, int i, int n, int oldn,
 			       int startskip, int min_startskip, int compfac,
-			       int method)
+			       CompactMethod method)
 {
     int t, idx;
     int lead = startskip - min_startskip;
     int to_weekly = (compfac >= 5 && compfac <= 7);
     double *x;
 
-#ifdef DB_DEBUG
+#if DB_DEBUG
     printf("compact_series: startskip=%d, min_startskip=%d, compfac=%d "
 	   "lead=%d\n", startskip, min_startskip, compfac, lead);
 #endif
@@ -1397,7 +1494,7 @@ static double *extend_series (const double *z, int n)
 static double *
 daily_series_to_monthly (const double *src, DATAINFO *pdinfo, int i, 
 			 int nm, int yr, int mon, int offset, 
-			 int any_eop, int method)
+			 int any_eop, CompactMethod method)
 {
     double *x;
     const double *z;
@@ -1516,7 +1613,7 @@ daily_series_to_monthly (const double *src, DATAINFO *pdinfo, int i,
 
 static void 
 get_startskip_etc (int compfac, int startmin, int endmin, 
-		   int oldn, int method, 
+		   int oldn, CompactMethod method, 
 		   int *startskip, int *newn) 
 {
     int ss = compfac - (startmin % compfac) + 1;
@@ -1549,7 +1646,7 @@ get_startskip_etc (int compfac, int startmin, int endmin,
 /* specific to compaction of daily time series */
 
 static void 
-get_daily_compact_params (int default_method, 
+get_daily_compact_params (CompactMethod default_method, 
 			  int *any_eop, int *any_sop,
 			  int *all_same,
 			  const DATAINFO *pdinfo)
@@ -1561,7 +1658,7 @@ get_daily_compact_params (int default_method,
     *any_sop = (default_method == COMPACT_SOP)? 1 : 0;
 
     for (i=1; i<pdinfo->v; i++) {
-	int method = COMPACT_METHOD(pdinfo, i);
+	CompactMethod method = COMPACT_METHOD(pdinfo, i);
 
 	if (method != default_method && method != COMPACT_NONE) {
 	    *all_same = 0;
@@ -1591,12 +1688,13 @@ get_daily_compact_params (int default_method,
 
 static void 
 get_global_compact_params (int compfac, int startmin, int endmin,
-			   int default_method, 
+			   CompactMethod default_method, 
 			   int *min_startskip, int *max_n,
 			   int *any_eop, int *all_same,
 			   DATAINFO *pdinfo)
 {
-    int i, startskip, n, method;
+    CompactMethod method;
+    int i, startskip, n;
     int n_not_eop = 0;
 
     for (i=0; i<pdinfo->v; i++) {
@@ -1669,7 +1767,8 @@ static int get_daily_offset (const DATAINFO *pdinfo,
    observations that can be constructed by compaction
 */
 
-static int get_n_ok_months (const DATAINFO *pdinfo, int default_method,
+static int get_n_ok_months (const DATAINFO *pdinfo, 
+			    CompactMethod default_method,
 			    int *startyr, int *startmon,
 			    int *endyr, int *endmon,
 			    int *offset, int *p_any_eop)
@@ -1734,10 +1833,214 @@ static int get_n_ok_months (const DATAINFO *pdinfo, int default_method,
     return nm;
 }
 
-/* driver for converting full dataset from daily to monthly */
+#define WEEKLY_DEBUG 0
 
-static int dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
-			       int default_method)
+static int 
+weeks_to_months_exec (double **mZ, const double **Z, const DATAINFO *pdinfo, int mn)
+{ 
+    char obsstr[OBSLEN];
+    int *den;
+    int yr, mon, day;
+    int monbak = 0;
+    int i, s, t = 0;
+    int err = 0;
+
+    den = malloc(pdinfo->v * sizeof *den);
+    if (den == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=1; i<pdinfo->v; i++) {
+	/* initialize all series, first obs */
+	if (pdinfo->vector[i]) {
+	    mZ[i][0] = 0.0;
+	    den[i] = 0;
+	}
+    }    
+
+    for (s=0; s<pdinfo->n; s++) {
+	/* loop across the weekly obs in this month */
+#if WEEKLY_DEBUG
+	fprintf(stderr, "\n** weekly to monthly monthly loop, s = %d\n", s);
+#endif
+	ntodate_full(obsstr, s, pdinfo);
+	sscanf(obsstr, "%d/%d/%d", &yr, &mon, &day);
+#if WEEKLY_DEBUG
+	fprintf(stderr, " month = %d\n", mon);
+#endif
+
+	if (monbak > 0 && mon != monbak) {
+	    /* new month: finalize the previous one */
+	    for (i=1; i<pdinfo->v; i++) {
+		if (pdinfo->vector[i]) {
+#if WEEKLY_DEBUG
+		    fprintf(stderr, " finalizing monthly obs %d, var %d, den = %d\n", 
+			    t, i, den[i]);
+#endif
+		    if (den[i] > 0) {
+			mZ[i][t] /= (double) den[i];
+		    } else {
+			mZ[i][t] = NADBL;
+		    }
+		}
+	    }
+	    /* and start another? */
+	    if (s < pdinfo->n - 1) {
+		t++;
+		for (i=1; i<pdinfo->v; i++) {
+		    /* initialize all series, current obs */
+		    if (pdinfo->vector[i]) {
+			mZ[i][t] = 0.0;
+			den[i] = 0;
+		    }
+		}  		
+	    }
+	} 
+
+	/* cumulate non-missing weekly observations */
+	for (i=1; i<pdinfo->v; i++) {
+	    if (pdinfo->vector[i]) {
+		if (!na(Z[i][s])) {
+		    mZ[i][t] += Z[i][s];
+		    den[i] += 1;
+		}
+		if (mon == monbak && s == pdinfo->n - 1) {
+#if WEEKLY_DEBUG
+		    fprintf(stderr, " finalizing monthly obs %d, var %d, den = %d\n", 
+			    t, i, den[i]);
+#endif
+		    /* reached the end: ship out last obs */
+		    if (den[i] > 0) {
+			mZ[i][t] /= (double) den[i];
+		    } else {
+			mZ[i][t] = NADBL;
+		    }
+		}		    
+	    }
+	}
+
+	monbak = mon;
+    }
+
+    free(den);
+
+    return err;
+}
+
+static int 
+weeks_to_months_check (const DATAINFO *pdinfo, int *startyr, int *endyr,
+		       int *startmon, int *endmon)
+{ 
+    char obsstr[OBSLEN];
+    int yr, mon, day;
+    int wcount = 0, mcount = 0;
+    int monbak = 0;
+    int t, err = 0;
+
+    for (t=0; t<pdinfo->n; t++) {
+	ntodate_full(obsstr, t, pdinfo);
+	if (sscanf(obsstr, "%d/%d/%d", &yr, &mon, &day) != 3) {
+	    err = 1;
+	    break;
+	}
+	if (monbak == 0) {
+	    /* first obs */
+	    fprintf(stderr, "starting month = '%d'\n", mon);
+	    *startyr = yr;
+	    *startmon = mon;
+	    mcount++;
+	    wcount = 1;	
+	} else if (mon != monbak) {
+	    /* got a new month: report on previous one */
+#if WEEKLY_DEBUG
+	    fprintf(stderr, "month %d ('%d'), weekly obs = %d\n", 
+		    mcount, monbak, wcount);
+#endif
+	    mcount++;
+	    wcount = 1;
+	} else {
+	    /* continuation of current month */
+	    wcount++;
+	}
+	monbak = mon;
+    }
+
+    /* flush the last observation */
+#if WEEKLY_DEBUG
+    fprintf(stderr, "month %d ('%d'), weekly obs = %d\n", 
+	    mcount, monbak, wcount);
+#endif
+    *endyr = yr;
+    *endmon = mon;
+
+    return mcount;
+}
+
+/* for now, averaging is the only compaction option in this case */
+
+static int weekly_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
+				      CompactMethod default_method)
+{
+    double **mZ = NULL;
+    DATAINFO minfo;
+    int startyr = 1, endyr;
+    int startmon = 1, endmon;
+    double *x;
+    int nseries = 0;
+    int i, err = 0;
+
+    minfo.n = weeks_to_months_check(pdinfo, &startyr, &endyr, &startmon, &endmon);
+    fprintf(stderr, "Weekly data: found %d months\n", minfo.n);
+    if (minfo.n <= 0) {
+	return E_DATA;
+    }
+
+    minfo.v = pdinfo->v;
+    err = allocate_Z(&mZ, &minfo);
+    if (err) {
+	return err;
+    }
+
+    /* handle scalars */
+    for (i=1; i<pdinfo->v && !err; i++) {
+	if (!pdinfo->vector[i]) {
+	    x = realloc(mZ[i], sizeof *x);
+	    if (x == NULL) {
+		err = E_ALLOC;
+	    } else {
+		mZ[i] = x;
+		mZ[i][0] = (*pZ)[i][0];
+	    }
+	} else {
+	    nseries++;
+	}
+    }
+
+    /* compact series */
+    if (!err && nseries > 0) {
+	err = weeks_to_months_exec(mZ, (const double **) *pZ, pdinfo, minfo.n);
+    }
+
+    if (err) {
+	free_Z(mZ, &minfo);
+    } else {
+	free_Z(*pZ, pdinfo);
+	*pZ = mZ;
+
+	pdinfo->n = minfo.n;
+	pdinfo->pd = 12;
+	sprintf(pdinfo->stobs, "%04d:%02d", startyr, startmon);
+	sprintf(pdinfo->endobs, "%04d:%02d", endyr, endmon);
+	pdinfo->sd0 = get_date_x(pdinfo->pd, pdinfo->stobs);
+	pdinfo->t1 = 0;
+	pdinfo->t2 = pdinfo->n - 1;
+    }
+    
+    return err;
+}
+
+static int daily_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
+				     CompactMethod default_method)
 {
     int nm, startyr, startmon, endyr, endmon;
     int offset, any_eop;
@@ -1751,7 +2054,7 @@ static int dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
 	err = 1;
     } else {
 	for (i=0; i<pdinfo->v && !err; i++) {
-	    int method;
+	    CompactMethod method;
 	    double *x;
 
 	    if (i > 0 && !pdinfo->vector[i]) {
@@ -1863,8 +2166,21 @@ int maybe_expand_daily_data (double ***pZ, DATAINFO *pdinfo)
     return err;
 }
 
+/**
+ * compact_data_set:
+ * @pZ: pointer to data array.
+ * @pdinfo: data information struct.
+ * @newpd: target data frequency.
+ * @default_method: code for the default compaction method.
+ * @monstart: FIXME add explanation.
+ * 
+ * Compact the data set from higher to lower frequency.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
 int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
-		      int default_method, int monstart)
+		      CompactMethod default_method, int monstart)
 {
     int newn, oldn = pdinfo->n, oldpd = pdinfo->pd;
     int compfac;
@@ -1877,6 +2193,10 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
 
     *gretl_errmsg = '\0';
 
+    if (oldpd == 52) {
+	return weekly_dataset_to_monthly(pZ, pdinfo, default_method);
+    }
+
     if (dated_daily_data(pdinfo)) {
 	/* allow for the possibility that the daily dataset
 	   contains "hidden" or suppressed missing observations
@@ -1888,7 +2208,7 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
 
     if (newpd == 12 && oldpd >= 5 && oldpd <= 7) {
 	/* daily to monthly: special */
-	return dataset_to_monthly(pZ, pdinfo, default_method);
+	return daily_dataset_to_monthly(pZ, pdinfo, default_method);
     } else if (oldpd >= 5 && oldpd <= 7) {
 	/* daily to weekly */
 	compfac = oldpd;
@@ -1963,7 +2283,7 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
     /* compact the individual data series */
     for (i=0; i<pdinfo->v && err == 0; i++) {
 	if (pdinfo->vector[i]) {
-	    int this_method = default_method;
+	    CompactMethod this_method = default_method;
 	    int startskip = min_startskip;
 	    double *x;
 
@@ -1996,3 +2316,92 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
 
     return err;
 }
+
+/**
+ * expand_data_set:
+ * @pZ: pointer to data array.
+ * @pdinfo: data information struct.
+ * @newpd: target data frequency
+ * 
+ * Expand the data set from lower to higher frequency: an "expert"
+ * option.  This is supported at present only for expansion from
+ * annual to quarterly or monthly, or from quarterly to monthly.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
+{
+    char stobs[12];
+    int oldn = pdinfo->n;
+    int mult, newn, addobs;
+    double *x = NULL;
+    int i, j, s, t;
+    int err = 0;
+
+    if (pdinfo->pd != 1 && pdinfo->pd != 4) {
+	return E_PDWRONG;
+    } else if (pdinfo->pd == 1 && newpd != 4 && newpd != 12) {
+	return E_DATA;
+    } else if (pdinfo->pd == 4 && newpd != 12) {
+	return E_DATA;
+    }
+
+    x = malloc(oldn * sizeof *x);
+    if (x == NULL) {
+	return E_ALLOC;
+    }
+
+    mult = newpd / pdinfo->pd;
+    newn = mult * pdinfo->n;
+    addobs = newn - oldn;
+
+    err = dataset_add_observations(addobs, pZ, pdinfo);
+    if (err) {
+	goto bailout;
+    }
+
+    for (i=1; i<pdinfo->v; i++) {
+	if (!pdinfo->vector[i]) {
+	    continue;
+	}
+	for (t=0; t<oldn; t++) {
+	    x[t] = (*pZ)[i][t];
+	}
+	s = 0;
+	for (t=0; t<oldn; t++) {
+	    for (j=0; j<mult; j++) {
+		(*pZ)[i][s++] = x[t];
+	    }
+	}
+    }
+
+     if (pdinfo->pd == 1) {
+	strcpy(stobs, pdinfo->stobs);
+	if (newpd == 4) {
+	    strcat(stobs, ":1");
+	} else {
+	    strcat(stobs, ":01");
+	}
+    } else {
+	int yr, qtr, mo;
+
+	sscanf(pdinfo->stobs, "%d:%d", &yr, &qtr);
+	mo = (qtr - 1) * 3 + 1;
+	sprintf(stobs, "%d:%02d", yr, mo);
+    }
+
+    strcpy(pdinfo->stobs, stobs);
+    pdinfo->pd = newpd;
+    pdinfo->sd0 = get_date_x(pdinfo->pd, pdinfo->stobs);
+
+    if (pdinfo->markers) {
+	dataset_destroy_obs_markers(pdinfo);
+    }
+
+ bailout:
+    
+    free(x);
+
+    return err;
+}
