@@ -124,7 +124,7 @@ int gretl_minmax (int t1, int t2, const double *x,
  *
  * Returns: the arithmetic mean of the series @x from obs
  * @t1 to obs @t2, skipping any missing values, or #NADBL 
- * on failure.
+ * in case there are no valid observations.
  */
 
 double gretl_mean (int t1, int t2, const double *x)
@@ -146,6 +146,10 @@ double gretl_mean (int t1, int t2, const double *x)
 	}
     }
 
+    if (n == 0) {
+	return NADBL;
+    }
+
     xbar = sum / n;
     sum = 0.0;
 
@@ -158,6 +162,99 @@ double gretl_mean (int t1, int t2, const double *x)
     return xbar + sum / n;
 }
 
+/**
+ * eval_ytest:
+ * @y: reference value.
+ * @op: operator.
+ * @test: test value.
+ *
+ * Returns: 1 if the expression @y @yop @test (for example
+ * "y = 2" or "y <= 45") evaluates as true, else 0.
+ */
+
+int eval_ytest (double y, GretlOp op, double test)
+{
+    int ret = 0;
+
+    switch (op) {
+    case OP_EQ:
+	ret = (y == test);
+	break;
+    case OP_GT:
+	ret = (y > test);
+	break;
+    case OP_LT:
+	ret = (y < test);
+	break;
+    case OP_NEQ:
+	ret = (y != test);
+	break;
+    case OP_GTE:
+	ret = (y <= test);
+	break;
+    case OP_LTE:
+	ret = (y <= test);
+	break;
+    default:
+	break;
+    }
+
+    return ret;
+}
+
+/**
+ * gretl_restricted_mean:
+ * @t1: starting observation.
+ * @t2: ending observation.
+ * @x: data series.
+ * @y: criterion series.
+ * @yop: criterion operator.
+ * @yval: criterion value.
+ *
+ * Returns: the arithmetic mean of the series @x in the
+ * range @t1 to @t2 (inclusive), but including only
+ * observations where the criterion variable @y bears the
+ * relationship @yop to the value @yval -- or #NADBL in case 
+ * there are no observations that satisfy the restriction.
+ */
+
+double gretl_restricted_mean (int t1, int t2, const double *x,
+			      const double *y, GretlOp yop, 
+			      double yval)
+{
+    int n;
+    register int t;
+    double xbar, sum = 0.0;
+
+    n = t2 - t1 + 1;
+    if (n <= 0) {
+	return NADBL;
+    }
+
+    for (t=t1; t<=t2; t++) {
+	if (!na(x[t]) && eval_ytest(y[t], yop, yval)) {
+	    sum += x[t];
+	} else {
+	    n--;
+	}
+    }
+
+    if (n == 0) {
+	return NADBL;
+    }
+
+    xbar = sum / n;
+    sum = 0.0;
+
+    for (t=t1; t<=t2; t++) {
+	if (!na(x[t]) && eval_ytest(y[t], yop, yval)) {
+	    sum += (x[t] - xbar); 
+	}
+    }
+
+    return xbar + sum / n;
+}
+
 /**
  * gretl_median:
  * @t1: starting observation.
@@ -276,6 +373,54 @@ double gretl_variance (int t1, int t2, const double *x)
     return (sumsq >= 0)? sumsq : NADBL;
 }
 
+/**
+ * gretl_restricted_variance:
+ * @t1: starting observation.
+ * @t2: ending observation.
+ * @x: data series.
+ * @y: criterion series.
+ * @yop: criterion operator. 
+ * @yval: criterion value.
+ *
+ * Returns: the variance of the series @x from obs
+ * @t1 to obs @t2, skipping any missing values and
+ * observations where the series @y does not bear the
+ * relationship @yop to the value @yval, or #NADBL on 
+ * failure.
+ */
+
+double gretl_restricted_variance (int t1, int t2, const double *x,
+				  const double *y, GretlOp yop, 
+				  double yval)
+{
+    int t, n = t2 - t1 + 1;
+    double sumsq, xx, xbar;
+
+    if (n == 0) {
+	return NADBL;
+    }
+
+    xbar = gretl_restricted_mean(t1, t2, x, y, yop, yval);
+    if (na(xbar)) {
+	return NADBL;
+    }
+
+    sumsq = 0.0;
+
+    for (t=t1; t<=t2; t++) {
+	if (!na(x[t]) && eval_ytest(y[t], yop, yval)) {
+	    xx = x[t] - xbar;
+	    sumsq += xx * xx;
+	} else {
+	    n--;
+	}
+    }
+
+    sumsq = (n > 1)? sumsq / (n - 1) : 0.0;
+
+    return (sumsq >= 0)? sumsq : NADBL;
+}
+
 /**
  * gretl_stddev:
  * @t1: starting observation.
@@ -294,6 +439,30 @@ double gretl_stddev (int t1, int t2, const double *x)
     return (na(xx))? xx : sqrt(xx);
 }
 
+/**
+ * gretl_restricted_stddev:
+ * @t1: starting observation.
+ * @t2: ending observation.
+ * @x: data series.
+ * @y: criterion series.
+ * @yop: criterion operator.
+ * @yval: criterion value.
+ *
+ * Returns: the standard deviation of the series @x from obs
+ * @t1 to obs @t2, skipping any missing values and observations
+ * where the series @y does not bear the relationship @yop to
+ * the value @yval, or #NADBL on failure.
+ */
+
+double gretl_restricted_stddev (int t1, int t2, const double *x,
+				const double *y, GretlOp yop, 
+				double yval)
+{
+    double xx = gretl_restricted_variance(t1, t2, x, y, yop, yval);
+
+    return (na(xx))? xx : sqrt(xx);
+}
+
 /**
  * gretl_covar:
  * @t1: starting observation.
@@ -594,7 +763,20 @@ static FreqDist *freq_new (void)
     return freq;
 }
 
-static double rb1_to_z1 (double rb1, double n)
+/**
+ * dh_root_b1_to_z1:
+ * @rb1: square root b1, skewness.
+ * @n: number of observations.
+ *
+ * Performs the transformation from skewness, root b1, to the 
+ * normal score, z1, as set out in Doornik and Hansen, "An Omnibus 
+ * Test for Normality", 1994.  The transformation is originally 
+ * due to D'Agostino (Biometrika, 1970).
+ *
+ * Returns: the z1 value.
+ */
+
+double dh_root_b1_to_z1 (double rb1, double n)
 {
     double b, w2, d, y, z1;
 
@@ -612,7 +794,20 @@ static double rb1_to_z1 (double rb1, double n)
     return z1;
 }
 
-static double b2_to_z2 (double b1, double b2, double n)
+/**
+ * dh_b2_to_z2:
+ * @b1: skewness.
+ * @b2: kurtosis.
+ * @n: number of observations.
+ *
+ * Performs the transformation from kurtosis, b2, to the 
+ * normal score, z2, as set out in Doornik and Hansen, "An Omnibus 
+ * Test for Normality", 1994.
+ *
+ * Returns: the z2 value.
+ */
+
+double dh_b2_to_z2 (double b1, double b2, double n)
 {
     double d, a, c, k, alpha, chi, z2;
     double n2 = n * n;
@@ -635,22 +830,225 @@ static double b2_to_z2 (double b1, double b2, double n)
     return z2;
 }
 
-/* Bowman-Shenton as modified by Doornik & Hansen */
+/**
+ * doornik_chisq:
+ * @skew: skewness.
+ * @xkurt: excess kurtosis.
+ * @n: number of observations.
+ *
+ * Calculates the Chi-square test for normality as set out by
+ * Doornik and Hansen, "An Omnibus Test for Normality", 1994.
+ * This is a modified version of the test proposed by Bowman and
+ * Shenton (Biometrika, 1975).
+ *
+ * Returns: the Chi-square value, which has 2 degrees of freedom.
+ */
 
-double doornik_chisq (double skew, double kurt, int n)
+double doornik_chisq (double skew, double xkurt, int n)
 {
     double rb1, b1, b2, z1, z2;
 
     rb1 = skew;
     b1 = skew * skew;
-    b2 = kurt + 3.0; /* convert from "excess" to regular */
+    b2 = xkurt + 3.0; /* Note: convert from "excess" to regular */
 
-    z1 = rb1_to_z1 (rb1, (double) n);
-    z2 = b2_to_z2 (b1, b2, (double) n);
+    z1 = dh_root_b1_to_z1(rb1, (double) n);
+    z2 = dh_b2_to_z2(b1, b2, (double) n);
 
     return z1*z1 + z2*z2;
 }
 
+static int
+get_moments (const gretl_matrix *M, int row, double *skew, double *kurt)
+{
+    int j, n = gretl_matrix_cols(M);
+    double xi, xbar, dev, var;
+    double s = 0.0;
+    double s2 = 0.0;
+    double s3 = 0.0;
+    double s4 = 0.0;
+    int err = 0;
+    
+    for (j=0; j<n; j++) {
+	s += gretl_matrix_get(M, row, j);
+    }
+
+    xbar = s / n;
+
+    for (j=0; j<n; j++) {
+	xi = gretl_matrix_get(M, row, j);
+	dev = xi - xbar;
+	s2 += dev * dev;
+	s3 += pow(dev, 3);
+	s4 += pow(dev, 4);
+    }
+
+    var = s2 / n;
+
+    if (var > 0.0) {
+	/* if variance is effectively zero, these should be undef'd */
+	*skew = (s3 / n) / pow(s2 / n, 1.5);
+	*kurt = ((s4 / n) / pow(s2 / n, 2));
+    } else {
+	*skew = *kurt = NADBL;
+	err = 1;
+    }
+
+    return err;
+}
+
+int 
+gretl_system_normality_test (const gretl_matrix *E, const gretl_matrix *Sigma, 
+			     PRN *prn)
+{
+    gretl_matrix *S = NULL;
+    gretl_matrix *V = NULL;
+    gretl_matrix *C = NULL;
+    gretl_matrix *X = NULL;
+    gretl_matrix *R = NULL;
+    gretl_matrix *tmp = NULL;
+
+    /* convenience pointers: do not free! */
+    gretl_matrix *H;
+    gretl_vector *Z1;
+    gretl_vector *Z2;
+
+    double *evals = NULL;
+    double x, skew, kurt;
+    double X2 = NADBL;
+    int n, p;
+    int i, j;
+    int err = 0;
+
+    if (E == NULL || Sigma == NULL) {
+	err = 1;
+	goto bailout;
+    }
+
+    p = gretl_matrix_cols(E);
+    n = gretl_matrix_rows(E);
+
+    S = gretl_matrix_copy(Sigma);
+    V = gretl_vector_alloc(p);
+    C = gretl_matrix_alloc(p, p);
+    X = gretl_matrix_copy_transpose(E);
+    R = gretl_matrix_alloc(p, n);
+    tmp = gretl_matrix_alloc(p, p);
+
+    if (S == NULL || V == NULL || C == NULL || X == NULL || 
+	R == NULL || tmp == NULL) {
+	err = 1;
+	goto bailout;
+    }
+
+    for (i=0; i<p; i++) {
+	x = gretl_matrix_get(S, i, i);
+	gretl_vector_set(V, i, 1.0 / sqrt(x));
+    }
+
+    err = gretl_matrix_diagonal_sandwich(V, S, C);
+    if (err) {
+	goto bailout;
+    }
+
+    gretl_matrix_print_to_prn(C, "\nResidual correlation matrix, C", prn);
+
+    evals = gretl_symmetric_matrix_eigenvals(C, 1);
+    if (evals == NULL) {
+	goto bailout;
+    }
+
+    pputs(prn, "Eigenvalues of the correlation matrix:\n\n");
+    for (i=0; i<p; i++) {
+	pprintf(prn, " %10g\n", evals[i]);
+    }
+    pputc(prn, '\n');
+
+    /* C should now contain eigenvectors of the original C:
+       relabel as 'H' for perspicuity */
+    H = C;
+#if 0
+    gretl_matrix_print_to_prn(H, "Eigenvectors, H", prn);
+#endif
+    gretl_matrix_copy_values(tmp, H);
+
+    /* make "tmp" into $H \Lambda^{-1/2}$ */
+    for (i=0; i<p; i++) {
+	for (j=0; j<p; j++) {
+	    x = gretl_matrix_get(tmp, i, j);
+	    x *= 1.0 / sqrt(evals[j]);
+	    gretl_matrix_set(tmp, i, j, x);
+	}
+    }
+
+    /* make S into $H \Lambda^{-1/2} H'$ */
+    gretl_matrix_multiply_mod(tmp, GRETL_MOD_NONE,
+			      H, GRETL_MOD_TRANSPOSE,
+			      S);
+
+#if 1
+    gretl_matrix_demean_by_row(X);
+#endif
+
+    /* compute VX', in X (don't need to subtract means, because these
+       are OLS residuals)
+    */
+    for (i=0; i<p; i++) {
+	for (j=0; j<n; j++) {
+	    x = gretl_matrix_get(X, i, j);
+	    x *= gretl_vector_get(V, i);
+	    gretl_matrix_set(X, i, j, x);
+	}
+    }
+
+    /* finally, compute $R' = H  \Lambda^{-1/2} H' V X'$ 
+       Doornik and Hansen, 1994, section 3 */
+    gretl_matrix_multiply(S, X, R);
+
+    /* Z_1 and Z_2 are p-vectors: use existing storage */
+    Z1 = V;
+    Z2 = gretl_matrix_reuse(tmp, p, 1);
+
+    for (i=0; i<p && !err; i++) {
+	get_moments(R, i, &skew, &kurt);
+	if (na(skew) || na(kurt)) {
+	    err = 1;
+	} else {
+	    double z1i = dh_root_b1_to_z1(skew, n);
+	    double z2i = dh_b2_to_z2(skew * skew, kurt, n);
+
+	    gretl_vector_set(Z1, i, z1i);
+	    gretl_vector_set(Z2, i, z2i);
+	}
+    }
+	
+    if (!err) {
+	X2 = gretl_vector_dot_product(Z1, Z1, &err);
+	X2 += gretl_vector_dot_product(Z2, Z2, &err);
+    }
+
+    if (na(X2)) {
+	pputs(prn, "Calculation of test statistic failed\n");
+    } else {
+	pputs(prn, "Test for multivariate normality of residuals\n");
+	pprintf(prn, "Doornik-Hansen Chi-square(%d) = %g, ", 2 * p, X2);
+	pprintf(prn, "with p-value = %g\n", chisq(X2, 2 * p));
+    }
+
+ bailout:
+
+    gretl_matrix_free(S);
+    gretl_matrix_free(V);
+    gretl_matrix_free(C);
+    gretl_matrix_free(X);
+    gretl_matrix_free(R);
+    gretl_matrix_free(tmp);
+
+    free(evals);
+
+    return err;
+}
+
 /**
  * get_freq:
  * @varno: ID number of variable to process.
@@ -755,6 +1153,7 @@ FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo,
     freq->endpt = malloc((nbins + 1) * sizeof *freq->endpt);
     freq->midpt = malloc(nbins * sizeof *freq->midpt);
     freq->f = malloc(nbins * sizeof *freq->f);
+
     if (freq->endpt == NULL || freq->midpt == NULL || freq->f == NULL) {
 	gretl_errno = E_ALLOC;
 	strcpy(gretl_errmsg, _("Out of memory for frequency distribution"));
@@ -781,7 +1180,7 @@ FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo,
 	freq->midpt[k] = freq->endpt[k] + .5 * binwidth;
     }
 
-    for (t=pdinfo->t1; t<pdinfo->t2; t++) {
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	xx = x[t];
 	if (na(xx)) {
 	    continue;
@@ -859,70 +1258,7 @@ int model_error_dist (const MODEL *pmod, double ***pZ,
 
 }
 
-#ifdef PACF_BY_OLS
-
-static int get_pacf (double *pacf, int m, int varnum, 
-		     double ***pZ, DATAINFO *pdinfo, PRN *prn)
-{
-    int i, j, err = 0;
-    int *laglist, *list;
-    int t1 = pdinfo->t1;
-    int v = pdinfo->v;
-    MODEL tmp;
-
-    pdinfo->t1 = 0; 
-
-    list = malloc((m + 3) * sizeof *list);
-    laglist = malloc(m * sizeof *laglist);
-    if (list == NULL || laglist == NULL) {
-	pdinfo->t1 = t1;
-	return 1;
-    }
-
-    /* add appropriate number of lags to data set */
-    for (i=1; i<=m; i++) {
-	int lnum = laggenr(varnum, i, pZ, pdinfo);
-
-	if (lnum < 0) {
-	    free(list);
-	    free(laglist);
-	    return 1;
-	}
-	laglist[i-1] = lnum; 
-    }
-
-    gretl_model_init(&tmp, pdinfo);
-
-    pdinfo->t1 = t1;
-
-    list[1] = varnum;
-    for (i=2; i<=m; i++) {
-	list[0] = i + 2;
-	list[2] = 0;
-	for (j=0; j<i; j++) {
-	    list[j+3] = laglist[j];
-	}
-	tmp = lsq(list, pZ, pdinfo, OLS, OPT_A, 0);
-	if ((err = tmp.errcode)) {
-	    fprintf(stderr, "error estimating model for pacf\n");
-	    break;
-	}
-	pacf[i-1] = tmp.coeff[i];
-	clear_model(&tmp, pdinfo);
-    }
-
-    dataset_drop_last_variables(pdinfo->v - v, pZ, pdinfo);
-    free(laglist);
-    free(list);
-
-    pdinfo->t1 = t1;
-
-    return err;
-}
-
-#else
-
-/* Durbin-Levinson algorithm */
+/* PACF via Durbin-Levinson algorithm */
 
 static int get_pacf (double *pacf, const double *acf, int m)
 {
@@ -959,8 +1295,6 @@ static int get_pacf (double *pacf, const double *acf, int m)
     return 0;
 }
 
-#endif
-
 int auto_acf_order (int pd, int nobs)
 {
     int m;
@@ -1146,11 +1480,7 @@ int corrgram (int varno, int order, double ***pZ,
     /* for confidence bands */
     pm = 1.96 / sqrt((double) nobs);
 
-#ifdef PACF_BY_OLS
-    err = pacf_err = get_pacf(pacf, pacf_m, varno, pZ, pdinfo, prn);
-#else
     err = pacf_err = get_pacf(pacf, acf, pacf_m);
-#endif
 
     if (!err) {
 	pacf[0] = acf[0];
@@ -1185,11 +1515,7 @@ int corrgram (int varno, int order, double ***pZ,
 	goto acf_getout;
     }
 
-    fprintf(fq, "# correlogram\n");
-
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
+    gretl_push_c_numeric_locale();
 
     /* create two separate plots, if both are OK */
     if (!pacf_err) {
@@ -1236,9 +1562,7 @@ int corrgram (int varno, int order, double ***pZ,
 	fputs("set nomultiplot\n", fq);
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
 
     fclose(fq);
 
@@ -1308,9 +1632,8 @@ static int fract_int_GPH (int n, double *hhat, double *omega, PRN *prn)
     }
 
     clear_model(&tmp);
-    free_Z(tmpZ, tmpdinfo);
-    clear_datainfo(tmpdinfo, CLEAR_FULL);
-    free(tmpdinfo);
+
+    destroy_dataset(tmpZ, tmpdinfo);
 
     return err;
 }
@@ -1639,7 +1962,7 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
     if (do_graph && gnuplot_init(PLOT_PERIODOGRAM, &fq) == 0) {
 	char titlestr[80];
 
-	fputs("# periodogram\n", fq);
+	fputs("# literal lines = 4\n", fq);
 	fputs("set xtics nomirror\n", fq); 
 
 	if (pdinfo->pd == 4) {
@@ -1727,15 +2050,14 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
     pputc(prn, '\n');
 
     if (do_graph) {
-#ifdef ENABLE_NLS
-	setlocale(LC_NUMERIC, "C");
-#endif
+	gretl_push_c_numeric_locale();
+
 	for (t=1; t<=nobs/2; t++) {
 	    fprintf(fq, "%d %f\n", t, savexx[t]);
 	}
-#ifdef ENABLE_NLS
-	setlocale(LC_NUMERIC, "");
-#endif
+
+	gretl_pop_c_numeric_locale();
+
 	fputs("e\n", fq);
 
 	fclose(fq);
@@ -1745,8 +2067,7 @@ int periodogram (int varno, double ***pZ, const DATAINFO *pdinfo,
 
     if (opt == 0) {
 	if (fract_int_GPH(nT, hhat, omega, prn)) {
-	    pprintf(prn, "\n%s\n",
-		    _("Fractional integration test failed"));
+	    pprintf(prn, "\n%s\n", _("Fractional integration test failed"));
 	}
 	fract_int_LWE((const double **) *pZ, varno, t1, t2, prn);
     }
@@ -2575,13 +2896,13 @@ real_mahalanobis_distance (const int *list, double ***pZ,
 
     if (!err) {
 	if (opt & OPT_V) {
-	    gretl_matrix_print(S, _("Covariance matrix"), prn);
+	    gretl_matrix_print_to_prn(S, _("Covariance matrix"), prn);
 	}
 	err = gretl_invert_symmetric_matrix(S);
 	if (err) {
 	    fprintf(stderr, "error inverting covariance matrix\n");
 	} else if (opt & OPT_V) {
-	    gretl_matrix_print(S, _("Inverse of covariance matrix"), prn);
+	    gretl_matrix_print_to_prn(S, _("Inverse of covariance matrix"), prn);
 	}
     }
 
@@ -2615,7 +2936,7 @@ real_mahalanobis_distance (const int *list, double ***pZ,
 		gretl_vector_set(xdiff, i, (*pZ)[vi][t] - xbar);
 	    }
 
-	    m = gretl_scalar_b_prime_X_b(xdiff, S, &err);
+	    m = gretl_scalar_b_X_b(xdiff, GRETL_MOD_TRANSPOSE, S, &err);
 
 	    get_obs_string(obs_string, t, pdinfo);
 	    pprintf(prn, "%8s ", obs_string);
@@ -2679,3 +3000,5 @@ MahalDist *get_mahal_distances (const int *list, double ***pZ,
 
     return md;
 }
+
+
