@@ -24,6 +24,7 @@
 #include "libset.h"
 #include "compat.h"
 #include "missing_private.h"
+#include "estim_private.h"
 
 /* There's a balancing act with 'TINY' here.  It's the minimum value
    for test that libgretl will accept before rejecting a
@@ -51,7 +52,7 @@
 
 /* private function prototypes */
 static int form_xpxxpy (const int *list, int t1, int t2, 
-			double **Z, int nwt, double rho, int pwe,
+			const double **Z, int nwt, double rho, int pwe,
 			double *xpx, double *xpy, const char *mask);
 static void regress (MODEL *pmod, double *xpy, double **Z, 
 		     int n, double rho);
@@ -59,11 +60,8 @@ static int cholbeta (double *xpx, double *xpy, double *coeff, double *rss,
 		     int nv);
 static void diaginv (double *xpx, double *xpy, double *diag, int nv);
 
-static double dwstat (int order, MODEL *pmod, double **Z);
-static double rhohat (int order, int t1, int t2, const double *uhat);
 static int hatvar (MODEL *pmod, int n, double **Z);
 static void omitzero (MODEL *pmod, const double **Z, const DATAINFO *pdinfo);
-static void tsls_omitzero (int *list, const double **Z, int t1, int t2);
 static int depvar_zero (int t1, int t2, int yno, int nwt, 
 			const double **Z);
 static int lagdepvar (const int *list, const double **Z, const DATAINFO *pdinfo); 
@@ -183,8 +181,8 @@ static int get_model_df (MODEL *pmod)
     int err = 0;
 
     pmod->ncoeff = pmod->list[0] - 1;
-
     pmod->dfd = pmod->nobs - pmod->ncoeff;
+
     if (pmod->dfd < 0) {
 	pmod->errcode = E_DF;
         sprintf(gretl_errmsg, _("No. of obs (%d) is less than no. "
@@ -544,7 +542,7 @@ lsq_check_for_missing_obs (MODEL *pmod, gretlopt opts,
 	if (pmod->ci == POOLED && pmod->missmask != NULL) {
 	    if (!model_mask_leaves_balanced_panel(pmod, pdinfo)) {
 		gretl_model_set_int(pmod, "unbalanced", 1);
-	    }
+	    } 
 	}
     }
 
@@ -668,17 +666,21 @@ int redundant_var (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, int trim)
 
     if (ret == 1) {
 	static char msg[ERRLEN];
-	int v = pmod->list[targ];
+	int rem, v = pmod->list[targ];
 
 	/* remove var from list and reduce number of coeffs */
 	gretl_list_delete_at_pos(pmod->list, targ);
-	pmod->ncoeff -= 1;
+	get_model_df(pmod);
 
 	/* compose a message */
 	if (trim == 0) {
 	    strcpy(msg, _("Omitted due to exact collinearity:"));
 	}
-	if (pdinfo->varname[v][0] != 0) {
+
+	rem = ERRLEN - 1 - strlen(msg);
+
+	if (pdinfo->varname[v][0] != 0 && 
+	    rem > strlen(pdinfo->varname[v]) + 1) {
 	    strcat(msg, " ");
 	    strcat(msg, pdinfo->varname[v]);
 	}
@@ -771,6 +773,7 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     mdl.t1 = pdinfo->t1;
     mdl.t2 = pdinfo->t2;
+    mdl.full_n = pdinfo->n;
     mdl.ci = ci;
 
     /* Doing weighted least squares? */
@@ -934,8 +937,8 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	/* calculate regression results, Cholesky style */
-	form_xpxxpy(mdl.list, mdl.t1, mdl.t2, *pZ, mdl.nwt, rho, 
-		    pwe, mdl.xpx, xpy, mdl.missmask);
+	form_xpxxpy(mdl.list, mdl.t1, mdl.t2, (const double **) *pZ, 
+		    mdl.nwt, rho, pwe, mdl.xpx, xpy, mdl.missmask);
 
 #ifdef XPX_DEBUG
 	for (i=0; i<=l0; i++) {
@@ -986,7 +989,7 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     if ((opt & OPT_T) && mdl.missmask == NULL) {
 	mdl.rho = rhohat(0, mdl.t1, mdl.t2, mdl.uhat);
-	mdl.dw = dwstat(0, &mdl, *pZ);
+	mdl.dw = dwstat(0, &mdl, (const double **) *pZ);
     } else {
 	mdl.rho = mdl.dw = NADBL;
     }
@@ -998,9 +1001,10 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* Generate model selection statistics */
-    gretl_calculate_criteria(mdl.criterion, 
-			     (mdl.ci == WLS)? mdl.ess_wt : mdl.ess, 
-			     mdl.nobs, mdl.ncoeff);
+    gretl_calculate_criteria((mdl.ci == WLS)? mdl.ess_wt : mdl.ess, 
+			     mdl.nobs, mdl.ncoeff, &mdl.lnL, 
+			     &mdl.criterion[C_AIC],
+			     &mdl.criterion[C_BIC]);
 
     /* hccm command or HC3a */
     if (jackknife) {
@@ -1046,7 +1050,7 @@ MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 */
 
 static int form_xpxxpy (const int *list, int t1, int t2, 
-			double **Z, int nwt, double rho, int pwe,
+			const double **Z, int nwt, double rho, int pwe,
 			double *xpx, double *xpy, const char *mask)
 {
     int i, j, t;
@@ -1177,8 +1181,6 @@ static int form_xpxxpy (const int *list, int t1, int t2,
     return 0; 
 }
 
-/* .......................................................... */
-
 static int make_ess (MODEL *pmod, double **Z)
 {
     int i, t, yno = pmod->list[1], l0 = pmod->list[0];
@@ -1220,7 +1222,7 @@ static void compute_r_squared (MODEL *pmod, double *y)
 	} else {
 	    pmod->rsq = gretl_corr_rsq(pmod->t1, pmod->t2, y, pmod->yhat);
 	    pmod->adjrsq = 
-		1 - ((1 - pmod->rsq) * (pmod->nobs - 1) / pmod->dfd);
+		1.0 - ((1.0 - pmod->rsq) * (pmod->nobs - 1.0) / pmod->dfd);
 	} 
     }
 
@@ -1258,6 +1260,7 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
 
     ysum = xpy[0];
     ypy = xpy[pmod->ncoeff + 1];
+
 #ifdef NO_LHS_ZERO
     if (floateq(ypy, 0.0)) { 
         pmod->errcode = E_YPY;
@@ -1303,7 +1306,7 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
     }
 
     if (floatlt(pmod->tss, 0.0) || floateq(pmod->tss, 0.0)) {
-       pmod->rsq = pmod->adjrsq = NADBL;
+	pmod->rsq = pmod->adjrsq = NADBL;
     } 
 
     hatvar(pmod, n, Z); 
@@ -1337,8 +1340,12 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
 
     diaginv(pmod->xpx, xpy, diag, pmod->ncoeff);
 
-    for (v=0; v<pmod->ncoeff; v++) { 
-       pmod->sderr[v] = pmod->sigma * sqrt(diag[v]); 
+    for (v=0; v<pmod->ncoeff; v++) {
+	if (diag[v] >= 0.0) {
+	    pmod->sderr[v] = pmod->sigma * sqrt(diag[v]);
+	} else {
+	    pmod->sderr[v] = 0.0;
+	}
     }
 
     free(diag); 
@@ -1347,14 +1354,15 @@ static void regress (MODEL *pmod, double *xpy, double **Z,
 }
 
 /*
-  cholbeta: does an in-place Choleski decomposition of xpx (lower
-  triangular matrix stacked in columns) and solves the normal
-  equations for coeff.  
+  cholbeta: does an in-place Cholesky decomposition of xpx (lower
+  triangular matrix stacked in columns) and solves for the
+  least-squares coefficient estimates.
 
-  xpx = X'X on input and Choleski decomposition on output
-  xpy = the X'y vector on input and Choleski-transformed t
+  xpx = X'X on input and Cholesky decomposition on output
+  xpy = the X'y vector on input and Cholesky-transformed t
         vector on output 
   coeff = array of estimated coefficients 
+  rss = pointer to receive regression sum of squares
   nv = number of regression coefficients including the constant
 
   The number of floating-point operations is basically 3.5 * nv^2
@@ -1367,6 +1375,10 @@ cholbeta (double *xpx, double *xpy, double *coeff, double *rss, int nv)
     int i, j, k, kk, l, jm1;
     double e, d, d1, d2, test, xx;
 
+    if (xpx[0] <= 0.0) {
+	return E_NAN;
+    }
+
     e = 1.0 / sqrt(xpx[0]);
     xpx[0] = e;
     xpy[1] *= e;
@@ -1392,7 +1404,7 @@ cholbeta (double *xpx, double *xpy, double *coeff, double *rss, int nv)
 #if 0
 	    fprintf(stderr, "cholbeta: test = %g\n", test);
 #endif
-	    if (rss != NULL) *rss = -1.0;
+	    *rss = -1.0;
 	    return E_SINGULAR;
         }
 	if (test < SMALL) {
@@ -1417,31 +1429,32 @@ cholbeta (double *xpx, double *xpy, double *coeff, double *rss, int nv)
 
     kk--;
 
-    /* find regression sum of squares */
-    if (rss != NULL) {
-	d = 0.0;
-	for (j=1; j<=nv; j++) {
-	    d += xpy[j] * xpy[j];
-	}
-	*rss = d;
+    /* regression sum of squares */
+    d = 0.0;
+    for (j=1; j<=nv; j++) {
+	d += xpy[j] * xpy[j];
     }
+    *rss = d;
 
-    /* solve for the regression coefficients */
-    if (coeff != NULL) {
-	for (j=0; j<nv-1; j++) {
-	    coeff[j] = 0.0;
-	}
-	coeff[nv-1] = xpy[nv] * xpx[kk];
-	for (j=nv-1; j>=1; j--) {
-	    d = xpy[j];
-	    for (i=nv-1; i>=j; i--) {
-		kk--;
-		d -= coeff[i] * xpx[kk];
-	    }
+    /* solve for the coefficients */
+    for (j=0; j<nv-1; j++) {
+	coeff[j] = 0.0;
+    }
+    coeff[nv-1] = xpy[nv] * xpx[kk];
+    for (j=nv-1; j>=1; j--) {
+	d = xpy[j];
+	for (i=nv-1; i>=j; i--) {
 	    kk--;
-	    coeff[j-1] = d * xpx[kk];
-	}  
-    } 
+	    d -= coeff[i] * xpx[kk];
+	}
+	kk--;
+	coeff[j-1] = d * xpx[kk];
+    }
+    for (j=0; j<nv; j++) {
+	if (isnan(coeff[j])) {
+	    return E_NAN;
+	}
+    }	
 
     return 0; 
 }
@@ -1469,7 +1482,9 @@ static void diaginv (double *xpx, double *xpy, double *diag, int nv)
         e = d * d;
         m = 0;
         if (l > 1) {
-	    for (j=1; j<=l-1; j++) m += nv - j;
+	    for (j=1; j<=l-1; j++) {
+		m += nv - j;
+	    }
 	}
         for (i=l+1; i<=nv; i++) {
             d = 0.0;
@@ -1493,7 +1508,7 @@ static void diaginv (double *xpx, double *xpy, double *diag, int nv)
  * makevcv:
  * @pmod: pointer to model.
  *
- * Inverts the Choleski-decomposed X'X and computes the 
+ * Inverts the Cholesky-decomposed X'X and computes the 
  * coefficient covariance matrix.
  * 
  * Returns: 0 on successful completion, error code on error.
@@ -1584,7 +1599,7 @@ int makevcv (MODEL *pmod)
     order is the order of autoregression, 0 for OLS.
 */
 
-static double dwstat (int order, MODEL *pmod, double **Z)
+double dwstat (int order, MODEL *pmod, const double **Z)
 {
     double diff, ut, ut1;
     double diffsq = 0.0;
@@ -1654,7 +1669,7 @@ static double altrho (int order, int t1, int t2, const double *uhat)
     order is the order of autoregression, 0 for OLS.
 */
 
-static double rhohat (int order, int t1, int t2, const double *uhat)
+double rhohat (int order, int t1, int t2, const double *uhat)
 {
     double ut, ut1, uu = 0.0, xx = 0.0;
     double rho;
@@ -1740,18 +1755,14 @@ static int hilu_plot (double *ssr, double *rho, int n)
     fputs("set xrange [-1.0:1.0]\n", fp);
     fputs("plot '-' using 1:2 w impulses\n", fp);
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
+    gretl_push_c_numeric_locale();
 
     for (i=0; i<n; i++) {
 	fprintf(fp, "%g %g\n", rho[i], ssr[i]);
     }
     fputs("e\n", fp);
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
 
     fclose(fp);
 
@@ -2026,473 +2037,6 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
     return rho;
 }
 
-/* .......................................................... */
-
-static int get_pos (const int *list)
-{
-    int i, ret = -1;
-
-    for (i=1; i<=list[0]; i++) {
-	if (list[i] == LISTSEP) {
-	    ret = i;
-	    break;
-	}
-    }
-
-    return ret;
-}
-
-void tsls_free_data (const MODEL *pmod)
-{
-    const char *endog = gretl_model_get_data(pmod, "endog");
-    double **X = gretl_model_get_data(pmod, "tslsX");
-    int i, m = 0;
-
-    if (endog != NULL && X != NULL) {
-	for (i=0; i<pmod->ncoeff; i++) {
-	    if (endog[i]) m++;
-	}
-	for (i=0; i<m; i++) {
-	    free(X[i]);
-	}
-    }
-}
-
-static int tsls_save_data (MODEL *pmod, const int *list, 
-			   double **Z, DATAINFO *pdinfo)
-{
-    double **X = NULL;
-    char *endog = NULL;
-    int addvars = list[0];
-    int i, j, k, pos, m, err = 0;
-    size_t esize, Xsize = list[0] * sizeof *X;
-    size_t es_old, xs_old;
-    int recycle_X = 0;
-    int recycle_e = 0;
-
-    pos = get_pos(pmod->list);
-    m = pos - 2;
-    esize = m * sizeof *endog;
-
-    /* re-use old pointers if applicable */
-
-    X = gretl_model_get_data_and_size(pmod, "tslsX", &xs_old);
-    if (X == NULL) {
-	X = malloc(Xsize);
-    } else if (Xsize != xs_old) {
-	tsls_free_data(pmod);
-	gretl_model_detach_data_item(pmod, "tslsX");
-	free(X);
-	X = malloc(Xsize);
-    } else {
-	recycle_X = 1;
-    }
-
-    endog = gretl_model_get_data_and_size(pmod, "endog", &es_old);
-    if (endog == NULL) {
-	endog = malloc(esize);
-    } else if (esize != es_old) {
-	gretl_model_detach_data_item(pmod, "endog");
-	free(endog);
-	endog = malloc(esize);
-    } else {
-	recycle_e = 1;
-    }
-
-    if (X == NULL || endog == NULL) {
-	free(X);
-	free(endog);
-	return E_ALLOC;
-    }
-
-    for (i=1; i<=list[0]; i++) {
-	k = pdinfo->v - 1 + i - addvars;
-	X[i-1] = Z[k];
-	Z[k] = NULL;
-    }
-
-    for (i=0; i<m; i++) {
-	k = pmod->list[i+2];
-	endog[i] = 0;
-	for (j=1; j<=list[0]; j++) {
-	    if (list[j] == k) {
-		endog[i] = 1;
-		break;
-	    }
-	}
-    }
-
-    /* now attach X and endog to the model */
-    if (!recycle_X) {
-	gretl_model_set_data(pmod, "tslsX", X, Xsize);
-    }
-    if (!recycle_e) {
-	gretl_model_set_data(pmod, "endog", endog, esize);
-    }
-
-    return err;
-}
-
-double *tsls_get_Xi (const MODEL *pmod, double **Z, int i)
-{
-    const char *endog;
-    double **X;
-    double *ret;
-
-    endog = gretl_model_get_data(pmod, "endog");
-    X = gretl_model_get_data(pmod, "tslsX");
-
-    if (endog == NULL || X == NULL) return NULL;
-
-    if (!endog[i]) {
-	ret = Z[pmod->list[i+2]];
-    } else {
-	int j, k = 0;
-
-	for (j=0; j<i; j++) {
-	    if (endog[j]) k++;
-	}
-	ret = X[k];
-    }
-
-    return ret;
-}
-
-/*
-  tsls_make_replist: determines which variables in reglist, when
-  checked against the predetermined and exogenous vars in instlist,
-  need to be instrumented, and populates replist accordingly
-*/
-
-static int 
-tsls_make_replist (const int *reglist, int *instlist, int *replist)
-{
-    int i, j, k = 0;
-    int endog, fixup = 0;
-
-    for (i=2; i<=reglist[0]; i++) {  
-	endog = 1;
-	for (j=1; j<=instlist[0]; j++) {
-	    if (instlist[j] == reglist[i]) {
-		endog = 0;
-		break;
-	    }
-	}
-	if (reglist[i] == 0 && endog) {
-	    /* const is in reglist but not instlist: needs fixing */
-	    fixup = 1;
-	} else if (endog) {
-	    replist[++k] = reglist[i];
-	} 
-    }
-
-    replist[0] = k;
-
-    if (fixup) {
-	/* add const to list of instruments */
-	instlist[0] += 1;
-	for (i=instlist[0]; i>=2; i--) {
-	    instlist[i] = instlist[i - 1];
-	}
-	instlist[1] = 0;
-    }
-
-    return 0;
-}
-
-/**
- * tsls_func:
- * @list: dependent variable plus list of regressors.
- * @pos_in: position in the list for the separator between list
- *   of variables and list of instruments (or 0 if unknown).
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the data set.
- * @opt: may contain %OPT_R for robust VCV, %OPT_S to save second-
- * stage regressors (%OPT_S is used in context of three-stage least 
- * squares), %OPT_N (no df correction).
- *
- * Estimate the model given in @list by means of Two-Stage Least
- * Squares.
- * 
- * Returns: a #MODEL struct, containing the estimates.
- */
-
-MODEL tsls_func (const int *list, int pos_in, double ***pZ, DATAINFO *pdinfo,
-		 gretlopt opt)
-{
-    int i, t;
-    int t1 = pdinfo->t1, t2 = pdinfo->t2;
-    int *reglist = NULL, *instlist = NULL, *replist = NULL;
-    int *s1list = NULL, *s2list = NULL;
-    int pos, nelem, orig_nvar = pdinfo->v;
-    MODEL tsls;
-
-    /* pos: position in list of the separator element between
-       dep. var. plus regressors and the list of instruments
-    */    
-    if (pos_in > 0) {
-	pos = pos_in;
-    } else {
-	pos = get_pos(list);
-    }
-
-    /* initialize model (in case we bail out early on error) */
-    gretl_model_init(&tsls);
-
-    *gretl_errmsg = '\0';
-
-    /* adjust sample range for missing observations */
-    varlist_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
-			  (const double **) *pZ); 
-
-    /* 
-       reglist: dependent var plus list of regressors
-       instlist: list of instruments
-       s1list: regression list for first-stage regressions
-       s2list: regression list for second-stage regression
-       replist: list of vars to be replaced by fitted values
-                for the second-stage regression
-    */
-    reglist = malloc(pos * sizeof *reglist);
-    instlist = malloc((list[0] - pos + 2) * sizeof *instlist);
-    s1list = malloc((list[0] - pos + 3) * sizeof *s1list);
-    s2list = malloc(pos * sizeof *s2list);
-    replist = malloc(pos * sizeof *replist);
-
-    if (reglist == NULL || instlist == NULL || s1list == NULL ||
-	s2list == NULL || replist == NULL) {
-	tsls.errcode = E_ALLOC;
-	goto tsls_bailout;
-    }
-
-    /* dep. var. plus regressors: first portion of input list */
-    reglist[0] = pos - 1;
-    for (i=1; i<pos; i++) {
-	reglist[i] = list[i];
-    }    
-
-    /* set up list of instruments: second portion of input list */
-    instlist[0] = list[0] - pos;
-    for (i=1; i<=instlist[0]; i++) {
-	instlist[i] = list[i + pos];
-    }	
-
-    /* drop any vars that are all zero, and reshuffle the constant
-       into first position among the independent vars 
-    */
-    tsls_omitzero(reglist, (const double **) *pZ, pdinfo->t1, pdinfo->t2);
-    rearrange_list(reglist);
-    tsls_omitzero(instlist, (const double **) *pZ, pdinfo->t1, pdinfo->t2);
-
-    /* initial composition of second-stage regression list (will be
-       modified below) 
-    */
-    for (i=0; i<pos; i++) {
-	s2list[i] = reglist[i];
-    }
-
-    /* determine the list of variables (replist) for which we need to
-       obtain fitted values in the first stage 
-    */
-    tsls_make_replist(reglist, instlist, replist);
-
-    /* check for order condition */
-    if (instlist[0] < reglist[0] - 1) {
-        sprintf(gretl_errmsg, 
-		_("Order condition for identification is not satisfied.\n"
-		"varlist 2 needs at least %d more variable(s) not in "
-		"varlist1."), reglist[0] - 1 - instlist[0]);
-	tsls.errcode = E_UNSPEC; 
-	goto tsls_bailout;
-    }
-
-    /* replist[0] holds the number of fitted vars to create */
-    if (dataset_add_series(replist[0], pZ, pdinfo)) {
-	tsls.errcode = E_ALLOC;
-	goto tsls_bailout;
-    } 
-
-    /* common setup for first-stage regressions: regressors are copied
-       from instlist */
-    s1list[0] = instlist[0] + 1;
-    for (i=2; i<=s1list[0]; i++) {
-	s1list[i] = instlist[i-1];
-    }    
-
-    /* 
-       deal with the variables for which instruments are needed: cycle
-       through the list of variables to be instrumented (replist), run
-       the first stage regression, and add the fitted values into the
-       data matrix Z
-    */
-    for (i=1; i<=replist[0]; i++) { 
-	int newv = orig_nvar + i - 1;
-	int j;
-
-	/* select the dependent variable */
-        s1list[1] = replist[i];
-
-	/* run the first-stage regression */
-	tsls = lsq(s1list, pZ, pdinfo, OLS, OPT_A, 0.0);
-	if (tsls.errcode) {
-	    goto tsls_bailout;
-	}
-
-	/* write the fitted values into data matrix, Z */
-	for (t=0; t<pdinfo->n; t++) {
-	    (*pZ)[newv][t] = tsls.yhat[t];
-	}
-
-	clear_model(&tsls);
-
-	/* give the fitted series the same name as the original */
-	strcpy(pdinfo->varname[newv], pdinfo->varname[replist[i]]);
-
-	/* substitute the newly created var into the right place in the
-	   second-stage regression list */
-	for (j=2; j<=s2list[0]; j++) {
-	    if (s2list[j] == replist[i]) {
-		s2list[j] = newv;
-		break;
-	    }
-	}
-    } 
-
-    /* second-stage regression */
-    tsls = lsq(s2list, pZ, pdinfo, OLS, OPT_NONE, 0.0);
-    if (tsls.errcode) {
-	goto tsls_bailout;
-    }
-
-    /* special: we need to use the original RHS vars to compute
-       residuals and associated statistics */
-
-    tsls.ess = 0.0;
-    for (t=tsls.t1; t<=tsls.t2; t++) {
-	double yh = 0.0;
-
-	if (model_missing(&tsls, t)) {
-	    continue;
-	}
-	for (i=0; i<tsls.ncoeff; i++) {
-	    yh += tsls.coeff[i] * (*pZ)[reglist[i+2]][t];
-	}
-	tsls.yhat[t] = yh; 
-	tsls.uhat[t] = (*pZ)[tsls.list[1]][t] - yh;
-	tsls.ess += tsls.uhat[t] * tsls.uhat[t];
-    }
-
-    if (tsls.ess <= 0.0) {
-	tsls.sigma = 0.0;
-    } else {
-	tsls.sigma = sqrt(tsls.ess / ((opt & OPT_N)? tsls.nobs : tsls.dfd));
-    }
-
-    /* computation of covariance matrix of parameter estimates */
-
-    if ((opt & OPT_R) || get_use_qr()) {
-	/* QR decomp in force, or robust standard errors called for */
-	qr_tsls_vcv(&tsls, (const double **) *pZ, opt);
-    } else {
-	double *xpx = NULL, *xpy = NULL, *diag = NULL;
-	int nxpx = tsls.ncoeff * (tsls.ncoeff + 1) / 2;
-
-	xpx = malloc(nxpx * sizeof *xpx);
-	xpy = malloc((s2list[0] + 1) * sizeof *xpy);
-	diag = malloc(tsls.ncoeff * sizeof *diag);
-
-	if (xpy == NULL || xpx == NULL || diag == NULL) {
-	    free(xpx);
-	    free(xpy);
-	    free(diag);
-	    tsls.errcode = E_ALLOC;
-	    goto tsls_bailout;
-	}
-
-	form_xpxxpy(s2list, tsls.t1, tsls.t2, *pZ, 0, 0.0, 0,
-		    xpx, xpy, tsls.missmask);
-
-	cholbeta(xpx, xpy, NULL, NULL, tsls.ncoeff);    
-	diaginv(xpx, xpy, diag, tsls.ncoeff);
-
-	for (i=0; i<tsls.ncoeff; i++) {
-	    tsls.sderr[i] = tsls.sigma * sqrt(diag[i]); 
-	}
-
-	free(diag); 
-	free(xpx);
-	free(xpy);
-    }
-
-    /* computation of additional statistics (R^2, F, etc.) */
-
-    tsls.rsq = gretl_corr_rsq(tsls.t1, tsls.t2, (*pZ)[tsls.list[1]],
-			      tsls.yhat);
-    tsls.adjrsq = 
-	1.0 - ((1.0 - tsls.rsq) * (tsls.nobs - 1.0) / tsls.dfd);
-    tsls.fstt = tsls.rsq * tsls.dfd / (tsls.dfn * (1.0 - tsls.rsq));
-    ls_aic_bic(&tsls);
-
-    if (tsls.missmask == NULL) {
-	/* no missing obs within sample range */
-	tsls.rho = rhohat(0, tsls.t1, tsls.t2, tsls.uhat);
-	tsls.dw = dwstat(0, &tsls, *pZ);
-    } else {
-	tsls.rho = tsls.dw = NADBL;
-    }
-
-    /* set command code on the model */
-    tsls.ci = TSLS;
-
-    /* write the full tsls list (dep. var. and regressors, followed by
-       instruments, possibly purged of zero elements) into the model
-       for future reference
-    */
-    nelem = reglist[0] + instlist[0];
-    tsls.list = realloc(tsls.list, (nelem + 2) * sizeof *tsls.list);
-    if (tsls.list == NULL) {
-	tsls.errcode = E_ALLOC;
-    } else {
-	int j = 1;
-
-	tsls.list[0] = nelem + 1;
-	for (i=1; i<=reglist[0]; i++) {
-	    tsls.list[j++] = reglist[i];
-	}
-	tsls.list[j++] = LISTSEP;
-	for (i=1; i<=instlist[0]; i++) {
-	    tsls.list[j++] = instlist[i];
-	}
-    }
-
- tsls_bailout:
-
-    free(reglist); 
-    free(instlist);
-    free(s1list); 
-    free(s2list);
-
-    /* save first-stage fitted values, if wanted */
-    if ((opt & OPT_S) && tsls.errcode == 0) {
-	tsls_save_data(&tsls, replist, *pZ, pdinfo);
-    }
-
-    free(replist);
-
-    /* delete first-stage fitted values from dataset */
-    dataset_drop_last_variables(pdinfo->v - orig_nvar, pZ, pdinfo);
-
-    if (tsls.errcode) {
-	model_count_minus();
-    }
-
-    /* restore original sample range */
-    pdinfo->t1 = t1;
-    pdinfo->t2 = t2;
-
-    return tsls;
-}
-
 /* Given an original regression list, augment it by adding the squares
    (and cross-products, if wanted) or logs of the original independent
    variables.  Generate these variables if need be.  Return the
@@ -2947,14 +2491,14 @@ int whites_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	pval = chisq(TR2, white.ncoeff - 1);
 
 	if (opt & OPT_S) {
-	    ModelTest *test;
+	    ModelTest *test = model_test_new(GRETL_TEST_WHITES);
 
-	    test = new_test_on_model(pmod, GRETL_TEST_WHITES);
 	    if (test != NULL) {
 		model_test_set_teststat(test, GRETL_STAT_TR2);
 		model_test_set_dfn(test, white.ncoeff - 1);
 		model_test_set_value(test, TR2);
 		model_test_set_pvalue(test, pval);
+		maybe_add_test_to_model(pmod, test);
 	    }	  
 	}
 
@@ -3194,7 +2738,7 @@ MODEL ar_func (const int *list, int pos, double ***pZ,
     }
     ar.fstt = ar.dfd * (tss - ar.ess) / (ar.dfn * ar.ess);
     ls_aic_bic(&ar);
-    ar.dw = dwstat(maxlag, &ar, *pZ);
+    ar.dw = dwstat(maxlag, &ar, (const double **) *pZ);
     ar.rho = rhohat(maxlag, ar.t1, ar.t2, ar.uhat);
 
     dataset_drop_last_variables(arlist[0] + 1 + reglist[0], pZ, pdinfo);
@@ -3246,6 +2790,7 @@ static void omitzero (MODEL *pmod, const double **Z, const DATAINFO *pdinfo)
 		sprintf(vnamebit, "%s ", pdinfo->varname[lv]);
 		strcat(gretl_msg, vnamebit);
 		dropmsg = 1;
+		v--;
 	    }
 	}
     }
@@ -3268,6 +2813,7 @@ static void omitzero (MODEL *pmod, const double **Z, const DATAINFO *pdinfo)
 		sprintf(vnamebit, "%s ", pdinfo->varname[lv]);
 		strcat(gretl_msg, vnamebit);
 		dropmsg = 1;
+		v--;
 	    }
 	}
     }
@@ -3277,21 +2823,6 @@ static void omitzero (MODEL *pmod, const double **Z, const DATAINFO *pdinfo)
     }
 }
 
-static void tsls_omitzero (int *list, const double **Z, int t1, int t2)
-{
-    int i, v;
-
-    for (i=2; i<=list[0]; i++) {
-        v = list[i];
-        if (gretl_iszero(t1, t2, Z[v])) {
-	    gretl_list_delete_at_pos(list, i);
-	    i--;
-	}
-    }
-}
-
-/* ...........................................................*/
-
 static int depvar_zero (int t1, int t2, int yno, int nwt,
 			const double **Z)
 {
@@ -3360,25 +2891,14 @@ lagdepvar (const int *list, const double **Z, const DATAINFO *pdinfo)
     return ret;
 }
 
-/**
- * arch_test:
- * @pmod: model to be tested.
- * @order: lag order for ARCH process.
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the data set.
- * @opt: mat contain %OPT_O to print covariance matrix, %OPT_S
- *       to save test results to model.
- * @prn: gretl printing struct.
- *
- * Test @pmod for Auto-Regressive Conditional Heteroskedasticity.  
- * If this effect is significant, re-restimate the model using 
- * weighted least squares.
- * 
- * Returns: a #MODEL struct, containing the estimates.
- */
+/* if 'full' is non-zero, do the whole thing (print the ARCH test
+   model, re-estimate if p-value is < .10); otherwise just do
+   the ARCH test itself and print the test result
+*/
 
-MODEL arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo, 
-		 gretlopt opt, PRN *prn)
+static MODEL 
+real_arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo, 
+		gretlopt opt, PRN *prn, int full)
 {
     MODEL archmod;
     int *wlist = NULL, *arlist = NULL;
@@ -3448,47 +2968,51 @@ MODEL arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	/* print results */
 	archmod.aux = AUX_ARCH;
 	archmod.order = order;
-
-	printmodel(&archmod, pdinfo, OPT_NONE, prn);
-
-	pprintf(prn, _("No of obs. = %d, unadjusted R^2 = %f\n"),
-		archmod.nobs, archmod.rsq);
 	LM = archmod.nobs * archmod.rsq;
 	xx = chisq(LM, order);
 
+	if (full) {
+	    printmodel(&archmod, pdinfo, OPT_NONE, prn);
+	    pprintf(prn, _("No of obs. = %d, unadjusted R^2 = %f\n"),
+		    archmod.nobs, archmod.rsq);
+	}
+
 	if (opt & OPT_S) {
-	    ModelTest *test;
+	    ModelTest *test = model_test_new(GRETL_TEST_ARCH);
 
-	    test = new_test_on_model(pmod, GRETL_TEST_ARCH);
 	    if (test != NULL) {
 		model_test_set_teststat(test, GRETL_STAT_TR2);
 		model_test_set_order(test, order);
 		model_test_set_dfn(test, order);
 		model_test_set_value(test, LM);
 		model_test_set_pvalue(test, xx);
+		maybe_add_test_to_model(pmod, test);
 	    }	    
-	}	
+	}
+
+	if (!full) {
+	    goto arch_test_exit;
+	}
 
 	record_test_result(LM, xx, "ARCH");
 
 	pprintf(prn, _("LM test statistic (%f) is distributed as Chi-square "
 		"(%d)\nArea to the right of LM = %f  "), LM, order, xx);
-	if (xx > 0.1) 
+
+	if (xx > 0.1) {
 	    pprintf(prn, "\n%s.\n%s.\n",
 		    _("ARCH effect is insignificant at the 10 percent level"),
 		    _("Weighted estimation not done"));
-	else {
+	} else {
 	    pprintf(prn, "\n%s.\n",
 		    _("ARCH effect is significant at the 10 percent level"));
-	    /* weighted estimation */
-	    wlist = malloc((pmod->list[0] + 2) * sizeof *wlist);
+	    /* do weighted estimation */
+	    wlist = gretl_list_new(pmod->list[0] + 1);
 	    if (wlist == NULL) {
 		archmod.errcode = E_ALLOC;
 	    } else {
-		wlist[0] = pmod->list[0] + 1;
 		nwt = wlist[1] = pdinfo->v - 1; /* weight var */
 		for (i=2; i<=wlist[0]; i++) {
 		    wlist[i] = pmod->list[i-1];
@@ -3514,6 +3038,8 @@ MODEL arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
+ arch_test_exit:
+
     if (arlist != NULL) free(arlist);
     if (wlist != NULL) free(wlist);
 
@@ -3522,6 +3048,55 @@ MODEL arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo,
     return archmod;
 }
 
+/**
+ * arch_test:
+ * @pmod: model to be tested.
+ * @order: lag order for ARCH process.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the data set.
+ * @opt: may contain %OPT_O to print covariance matrix, %OPT_S
+ *       to save test results to model.
+ * @prn: gretl printing struct.
+ *
+ * Tests @pmod for Auto-Regressive Conditional Heteroskedasticity.  
+ * If this effect is significant, re-restimates the model using 
+ * weighted least squares.
+ * 
+ * Returns: a #MODEL struct, containing the estimates.
+ */
+
+MODEL arch_test (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo, 
+		 gretlopt opt, PRN *prn)
+{
+    return real_arch_test(pmod, order, pZ, pdinfo, opt, prn, 1);
+}
+
+/**
+ * arch_test_simple:
+ * @pmod: model to be tested.
+ * @order: lag order for ARCH process.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the data set.
+ * @prn: gretl printing struct.
+ *
+ * Tests @pmod for Auto-Regressive Conditional Heteroskedasticity.  
+ * 
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int arch_test_simple (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo, 
+		      PRN *prn)
+{
+    MODEL amod;
+    int err;
+
+    amod = real_arch_test(pmod, order, pZ, pdinfo, OPT_S, prn, 0);
+    err = amod.errcode;
+    clear_model(&amod);
+
+    return err;
+}
+
 /**
  * arch_model:
  * @list: dependent variable plus list of regressors.
@@ -3552,7 +3127,7 @@ MODEL arch_model (const int *list, int order, double ***pZ, DATAINFO *pdinfo,
     } 
 
     /* FIXME: vcv option? */
-    amod = arch_test(&lmod, order, pZ, pdinfo, opt, prn);
+    amod = real_arch_test(&lmod, order, pZ, pdinfo, opt, prn, 1);
 
     free(lmod.list);
 
@@ -3575,7 +3150,7 @@ MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo)
 {
     MODEL lad_model;
     void *handle;
-    int (*lad_driver)(MODEL *, double **, DATAINFO *);
+    int (*lad_driver) (MODEL *, double **, DATAINFO *);
 
     /* run an initial OLS to "set the model up" and check for errors.
        the lad_driver function will overwrite the coefficients etc.
@@ -3588,6 +3163,7 @@ MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo)
     }
 
     lad_driver = get_plugin_function("lad_driver", &handle);
+
     if (lad_driver == NULL) {
 	fprintf(stderr, I_("Couldn't load plugin function\n"));
 	lad_model.errcode = E_FOPEN;
@@ -3604,128 +3180,51 @@ MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo)
 
 /**
  * arma:
- * @list: dependent variable plus AR and MA orders.
+ * @list: dependent variable, AR and MA orders, and any exogenous
+ * regressors.
  * @Z: data array.
  * @pdinfo: information on the data set.
- * @opt: options: may include %OPT_S to suppress intercept.
+ * @opt: options: may include %OPT_S to suppress intercept, %OPT_V
+ * for verbose results, %OPT_X to use X-12-ARIMA.
  * @PRN: for printing details of iterations (or %NULL). 
  *
- * Calculate ARMA estimates.
+ * Calculate ARMA estimates, using either native gretl code or
+ * by invoking X-12-ARIMA.
  * 
  * Returns: a #MODEL struct, containing the estimates.
  */
 
-MODEL arma (const int *list, const double **Z, DATAINFO *pdinfo, 
+MODEL arma (const int *list, const double **Z, const DATAINFO *pdinfo, 
 	    gretlopt opt, PRN *prn)
 {
     MODEL armod;
     void *handle;
-    MODEL (*arma_model) (const int *, const double **, DATAINFO *, gretlopt, PRN *);
+    MODEL (*arma_func) (const int *, const double **, const DATAINFO *, 
+			 gretlopt, PRN *);
 
     *gretl_errmsg = '\0';
 
-    arma_model = get_plugin_function("arma_model", &handle);
+    if (opt & OPT_X) {
+	arma_func = get_plugin_function("arma_x12_model", &handle);
+    } else {
+	arma_func = get_plugin_function("arma_model", &handle);
+    }
 
-    if (arma_model == NULL) {
+    if (arma_func == NULL) {
+	fprintf(stderr, I_("Couldn't load plugin function\n"));
 	gretl_model_init(&armod);
 	armod.errcode = E_FOPEN;
 	return armod;
     }
 
-    armod = (*arma_model) (list, Z, pdinfo, opt, prn);
+    armod = (*arma_func) (list, Z, pdinfo, opt, prn);
 
     close_plugin(handle);
-
     set_model_id(&armod);
 
     return armod;
 } 
 
-#ifdef HAVE_X12A
-
-/**
- * arma_x12a:
- * @list: dependent variable plus AR and MA orders
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the data set.
- * @ppaths: gretl path info struct (so we can find X-12-ARIMA).
- * @opt: may include OPT_V for verbose results.
- * @PRN: for printing details of iterations (or NULL).
- *
- * Calculate ARMA estimates, via a call to X-12-ARIMA.
- * 
- * Returns: a #MODEL struct, containing the estimates.
- */
-
-MODEL arma_x12 (const int *list, const double **Z, DATAINFO *pdinfo, 
-		const PATHS *ppaths, gretlopt opt, PRN *prn)
-{
-    MODEL armod;
-    void *handle;
-    MODEL (*arma_x12_model) (const int *, const double **, DATAINFO *, 
-			     const char *, const char *,
-			     gretlopt, int, PRN *);
-    int gui = gretl_in_gui_mode();
-
-    *gretl_errmsg = '\0';
-
-    arma_x12_model = get_plugin_function("arma_x12_model", &handle);
-    if (arma_x12_model == NULL) {
-	gretl_model_init(&armod);
-	armod.errcode = E_FOPEN;
-	return armod;
-    }
-
-    armod = (*arma_x12_model) (list, Z, pdinfo, ppaths->x12a, 
-			       ppaths->x12adir, opt, gui, prn);
-
-    close_plugin(handle);
-
-    set_model_id(&armod);
-
-    return armod;
-}  
-
-#endif /* HAVE_X12A */
-
-/**
- * logistic_model:
- * @list: dependent variable plus list of regressors.
- * @pZ: pointer to data matrix.
- * @pdinfo: information on the data set.
- * @param: 
- *
- * Estimate the model given in @list using the logistic transformation
- * of the dependent variable.
- * 
- * Returns: a #MODEL struct, containing the estimates.
- */
-
-MODEL logistic_model (const int *list, double ***pZ, DATAINFO *pdinfo,
-		      const char *param)
-{
-    MODEL lmod;
-    void *handle;
-    MODEL (*logistic_estimate) (const int *, double ***, DATAINFO *, const char *);
-
-    *gretl_errmsg = '\0';
-
-    logistic_estimate = get_plugin_function("logistic_estimate", &handle);
-    if (logistic_estimate == NULL) {
-	gretl_model_init(&lmod);
-	lmod.errcode = E_FOPEN;
-	return lmod;
-    }
-
-    lmod = (*logistic_estimate) (list, pZ, pdinfo, param);
-
-    close_plugin(handle);
-
-    set_model_id(&lmod);
-
-    return lmod;
-}
-
 /**
  * tobit_model:
  * @list: dependent variable plus list of regressors.
@@ -3884,6 +3383,22 @@ MODEL garch (const int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt,
     return gmod;
 } 
 
+/**
+ * pooled:
+ * @list: regression list (dependent variable plus independent 
+ * variables).
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the (panel) data set.
+ * @opt: can include %OPT_W to do weighted least squares
+ * using weights based on the error variance for the 
+ * respective cross-sectional units.
+ * @prn: for printing details of iterations (or %NULL).
+ *
+ * Calculate pooled OLS or WLS estimates for a panel dataset.
+ * 
+ * Returns: a #MODEL struct, containing the estimates.
+ */
+
 MODEL pooled (const int *list, double ***pZ, DATAINFO *pdinfo,
 	      gretlopt opt, PRN *prn)
 {
@@ -3914,6 +3429,21 @@ MODEL pooled (const int *list, double ***pZ, DATAINFO *pdinfo,
     return wmod;
 }
 
+/**
+ * groupwise_hetero_test:
+ * @pmod: pooled OLS model to be tested.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: information on the (panel) data set.
+ * @prn: for printing details of iterations (or %NULL).
+ *
+ * Calculates iterated WLS estimates using weights based on the error
+ * variance for the cross-sectional units and performs a Wald test
+ * for the null hypothesis that the error variance is uniform
+ * across the units.
+ * 
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
 int groupwise_hetero_test (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 			   PRN *prn)
 {
