@@ -19,9 +19,8 @@
 
 /* estimate.h for gretl:  set out structs and functions employed */
 
-#include <stdio.h>
-
-/* functions follow */
+#ifndef ESTIMATE_H
+#define ESTIMATE_H
 
 MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo, 
 	   GretlCmdIndex ci, gretlopt opt, double rho);
@@ -31,15 +30,9 @@ double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
 
 MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo); 
 
-MODEL arma (const int *list, const double **Z, DATAINFO *pdinfo, 
+MODEL arma (const int *list, const double **Z, const DATAINFO *pdinfo, 
 	    gretlopt opt, PRN *prn);
 
-MODEL arma_x12 (const int *list, const double **Z, DATAINFO *pdinfo, 
-		const PATHS *ppaths, gretlopt opt, PRN *prn);
-
-MODEL logistic_model (const int *list, double ***pZ, DATAINFO *pdinfo,
-		      const char *param);
-
 MODEL tobit_model (const int *list, double ***pZ, DATAINFO *pdinfo, 
 		   PRN *prn);
 
@@ -55,14 +48,6 @@ MODEL pooled (const int *list, double ***pZ, DATAINFO *pdinfo,
 int groupwise_hetero_test (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 			   PRN *prn);
 
-double *tsls_get_Xi (const MODEL *pmod, double **Z, int i);
-
-void tsls_free_data (const MODEL *pmod);
-
-MODEL tsls_func (const int *list, int pos_in, 
-		 double ***pZ, DATAINFO *pdinfo,
-		 gretlopt opt);
-
 MODEL hsk_func (const int *list, double ***pZ, DATAINFO *pdinfo);
 
 int whites_test (MODEL *pmod, 
@@ -77,6 +62,9 @@ MODEL arch_test (MODEL *pmod, int order,
 		 double ***pZ, DATAINFO *pdinfo, 
 		 gretlopt opt, PRN *prn);
 
+int arch_test_simple (MODEL *pmod, int order, double ***pZ, DATAINFO *pdinfo, 
+		      PRN *prn);
+
 MODEL arch_model (const int *list, int order, 
 		  double ***pZ, DATAINFO *pdinfo, 
 		  gretlopt opt, PRN *prn);
@@ -86,4 +74,6 @@ int makevcv (MODEL *pmod);
 int *augment_regression_list (const int *orig, int aux, 
 			      double ***pZ, DATAINFO *pdinfo);
 
+#endif /* ESTIMATE_H */
+
 
