@@ -360,7 +360,7 @@ has_real_exog_regressors (MODEL *pmod, const int *dvlags,
 	    if (xi != 0 && (dvlags == NULL || dvlags[i] == 0)) {
 		if (is_trend_variable(Z[xi], pdinfo->n)) {
 		    continue;
-		} else if (is_periodic_dummy(Z[xi], pdinfo->n)) {
+		} else if (is_periodic_dummy(Z[xi], pdinfo)) {
 		    continue;
 		} else {
 		    ret = 1;
@@ -381,7 +381,7 @@ static int
 fit_resid_init (int t1, int t2, int pre_n, const MODEL *pmod, 
 		const DATAINFO *pdinfo, FITRESID *fr)
 {
-    if (pre_n > t1) {
+    if (pre_n > t1) { /* is this right? */
 	pre_n = t1;
     }
 
@@ -530,7 +530,7 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 					       NULL);
 
 	/* forecast variance */
-	vyh = gretl_scalar_b_X_b_prime(Xs, V, NULL);
+	vyh = gretl_scalar_b_X_b(Xs, GRETL_MOD_NONE, V, NULL);
 	if (na(vyh)) {
 	    err = 1;
 	} else {
@@ -578,7 +578,7 @@ static int nls_fcast (Forecast *fc, const MODEL *pmod,
 	pdinfo->t1 = fc->t1;
 	pdinfo->t2 = fc->t2;
 	sprintf(formula, "$nl_y = %s", nlfunc);
-	err = generate(formula, pZ, pdinfo, NULL, OPT_P);
+	err = generate(formula, pZ, pdinfo, OPT_P);
     }
 
     if (!err) {
@@ -865,7 +865,8 @@ static int garch_fcast (Forecast *fc, MODEL *pmod,
     }
 
     if (fc->dvlags != NULL) {
-	int pmax, ns = fc->t2 - pmod->t2;
+	int ns = fc->t2 - pmod->t2;
+	int pmax = 0;
 	
 	if (ns > 0) {
 	    phi = garch_ldv_phi(pmod, xlist, fc->dvlags, &pmax);
@@ -996,8 +997,8 @@ arma_variance_machine (const double *phi, int p,
 static int arma_fcast (Forecast *fc, MODEL *pmod, 
 		       const double **Z, const DATAINFO *pdinfo)
 {
-    const double *phi;
-    const double *theta;
+    double *phi = NULL;
+    double *theta = NULL;
     const double *beta;
 
     double *psi = NULL;
@@ -1030,13 +1031,14 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 	tstart = pmod->t2 + 1;
     }
 
-    p = gretl_arma_model_get_AR_order(pmod);
-    q = gretl_arma_model_get_MA_order(pmod);
+    p = gretl_arma_model_get_max_AR_lag(pmod);
+    q = gretl_arma_model_get_max_MA_lag(pmod);
+
     xlist = model_xlist(pmod);
     yno = gretl_model_get_depvar(pmod);
 
-    DPRINTF(("forecasting variable %d (%s), obs %d to %d\n", yno, 
-	     pdinfo->varname[yno], fc->t1, fc->t2));
+    DPRINTF(("forecasting variable %d (%s), obs %d to %d, with p=%d, q=%d\n", yno, 
+	     pdinfo->varname[yno], fc->t1, fc->t2, p, q));
 
     if (xlist != NULL) {
 	xvars = xlist[0];
@@ -1044,9 +1046,12 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 	xvars = 0;
     }
 
-    phi = pmod->coeff + pmod->ifc; /* AR coeffs */
-    theta = phi + p;               /* MA coeffs */
-    beta = theta + q;              /* coeffs on indep vars */
+    err = gretl_arma_model_get_AR_MA_coeffs(pmod, &phi, &theta);
+    if (err) {
+	goto bailout;
+    }
+
+    beta = gretl_arma_model_get_x_coeffs(pmod);
 
     /* setup for forecast error variance */
     if (fc->sderr != NULL) {
@@ -1054,7 +1059,7 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
     }
 
     /* cut-off points for using actual rather than forecast
-       values of y in generating further forecasts */
+       values of y in generating further forecasts (FIXME??) */
     if (fc->method == FC_STATIC) {
 	ar_smax = fc->t2;
 	ma_smax = pmod->t2;
@@ -1072,6 +1077,7 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
     for (t=tstart; t<=fc->t2 && !err; t++) {
 	int miss = 0;
 	double yh = 0.0;
+	int lag;
 
 	tt = t - fc->offset;
 
@@ -1097,21 +1103,22 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 
 	/* AR contribution */
 	for (i=0; i<p && !miss; i++) {
-	    s = t - i - 1;
+	    lag = i + 1;
+	    s = t - lag;
 	    if (s < 0) {
 		yval = NADBL;
 	    } else if (s <= ar_smax) {
 		yval = Z[yno][s];
-		DPRINTF(("  AR: lag %d, y[%d] = %g\n", i+1, s, yval));
+		DPRINTF(("  AR: lag %d, y[%d] = %g\n", lag, s, yval));
 	    } else {
 		yval = fc->yhat[s - fc->offset];
-		DPRINTF(("  AR: lag %d, yhat[%d] = %g\n", i+1, s - fc->offset, yval));
+		DPRINTF(("  AR: lag %d, yhat[%d] = %g\n", lag, s - fc->offset, yval));
 	    }
 	    if (na(yval)) {
-		DPRINTF(("  AR: lag %d, s =%d, missing value\n", i+1, s));
+		DPRINTF(("  AR: lag %d, s =%d, missing value\n", lag, s));
 		miss = 1;
 	    } else {
-		DPRINTF(("  AR: lag %d, s=%d, using coeff %g\n", i+1, s, phi[i]));
+		DPRINTF(("  AR: lag %d, s=%d, using coeff %g\n", lag, s, phi[i]));
 		yh += phi[i] * yval;
 	    }
 	}
@@ -1120,13 +1127,14 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 
 	/* MA contribution */
 	for (i=0; i<q && !miss; i++) {
-	    s = t - i - 1;
+	    lag = i + 1;
+	    s = t - lag;
 	    if (s >= pmod->t1 && s <= ma_smax) {
-		DPRINTF(("  MA: lag %d, e[%d] = %g, coeff %g\n", i+1, s, 
+		DPRINTF(("  MA: lag %d, e[%d] = %g, coeff %g\n", lag, s, 
 			 pmod->uhat[s], theta[i]));
 		yh += theta[i] * pmod->uhat[s];
 	    } else if (fc->eps != NULL) {
-		DPRINTF(("  MA: lag %d, ehat[%d] = %g, coeff %g\n", i+1, s, 
+		DPRINTF(("  MA: lag %d, ehat[%d] = %g, coeff %g\n", lag, s, 
 			 fc->eps[s - fc->offset], theta[i]));
 		yh += theta[i] * fc->eps[s - fc->offset];
 	    }
@@ -1161,9 +1169,17 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 #endif
     }
 
+ bailout:
+
     if (psi != NULL) {
 	free(psi);
     }
+    if (phi != NULL) {
+	free(phi);
+    }
+    if (theta != NULL) {
+	free(theta);
+    }
 
     return err;
 }
@@ -1989,7 +2005,7 @@ fcast_get_t2max (const int *list, const int *dvlags, const MODEL *pmod,
 		continue;
 	    } else if (is_trend_variable(Z[vi], pdinfo->n)) {
 		continue;
-	    } else if (is_periodic_dummy(Z[vi], pdinfo->n)) {
+	    } else if (is_periodic_dummy(Z[vi], pdinfo)) {
 		continue;
 	    }
 	    if (na(Z[vi][t])) {
@@ -2015,18 +2031,20 @@ fcast_get_t2max (const int *list, const int *dvlags, const MODEL *pmod,
  * @i: 0-based index for the variable to forecast, within
  * the VAR system (the dependent variable in the ith equation,
  * counting from 0).
- * @t1: start of forecast range.
- * @t2: end of forecast range.
+ * @t1: start of range.
+ * @t2: end of range.
  * @pre_n: number of pre-forecast observations to display.
  * @Z: data array using which @var was estimated.
  * @pdinfo: dataset information.
  * @opt: if OPT_D, force a dynamic forecast; if OPT_S, force
  * a static forecast.  By default, the forecast is static within
  * the data range over which the model was estimated, and dynamic
- * out of sample (in cases where a dynamic forecast is meaningful).
+ * out of sample.
  *
  * Allocates a #FITRESID structure and fills it out with forecasts
  * based on @var, over the specified range of observations.  
+ * The first @pre_n observations, starting at @t1, will be not
+ * contain forecasts (FIXME).
  * 
  * Returns: pointer to allocated structure, or %NULL on failure.
  * The %err member of the returned object should be checked:
@@ -2039,27 +2057,28 @@ FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t1, int t2, int pre_n,
 {
     FITRESID *fr;
     const gretl_matrix *F;
-    const MODEL *pmod;
+    const MODEL *pmod = NULL;
     int nf = t2 - t1 + 1;
-    int yno, nobs = nf + pre_n;
-    int m, s, t;
+    int yno, m, s, t;
 
     if (nf <= 0) {
 	return NULL;
     }
 
-    pmod = gretl_var_get_model(var, i);
-    if (pmod == NULL) {
-	return NULL;
+    if (!var->ecm) {
+	pmod = gretl_VAR_get_model(var, i);
+	if (pmod == NULL) {
+	    return NULL;
+	}
     }
 
-    F = gretl_var_get_forecast_matrix(var, t1, t2, Z, pdinfo, opt);
+    F = gretl_VAR_get_forecast_matrix(var, t1, t2, pre_n, Z, pdinfo, opt);
     if (F == NULL) {
-	fprintf(stderr, "gretl_var_get_forecast_matrix() gave NULL\n");
+	fprintf(stderr, "gretl_VAR_get_forecast_matrix() gave NULL\n");
 	return NULL;
     }
 
-    fr = fit_resid_new(nobs);
+    fr = fit_resid_new(nf);
     if (fr == NULL) {
 	return NULL;
     }
@@ -2071,47 +2090,49 @@ FITRESID *get_VAR_forecast (GRETL_VAR *var, int i, int t1, int t2, int pre_n,
 	}
     }
 
-    fr->model_ci = VAR;
+    fr->model_ci = var->ci;
+    fr->pre_n = pre_n;
     fr->t1 = t1;
     fr->t2 = t2;
-    fr->pre_n = pre_n;  
 
-    yno = pmod->list[1];
+    if (var->ecm) {
+	yno = var->jinfo->list[i+1];
+    } else {
+	yno = pmod->list[1];
+    }
+
     strcpy(fr->depvar, pdinfo->varname[yno]);
 
-    m = gretl_var_get_n_equations(var);
+    m = var->neqns;
 
     nf = 0;
     for (s=0; s<fr->nobs; s++) {
-	t = s + fr->t1 - fr->pre_n;
+	t = s + fr->t1;
 	fr->actual[s] = Z[yno][t];
-	if (s < fr->pre_n) {
-	    fr->fitted[s] = pmod->yhat[t];
-	    if (fr->sderr != NULL) {
-		fr->sderr[s] = NADBL;
-	    }
-	} else {
-	    fr->fitted[s] = gretl_matrix_get(F, s - pre_n, i);
-	    if (!na(fr->fitted[s])) {
-		nf++;
-	    }
-	    if (fr->sderr != NULL) {
-		fr->sderr[s] = gretl_matrix_get(F, s - pre_n, i + m);
-	    }
+	fr->fitted[s] = gretl_matrix_get(F, s, i);
+	if (!na(fr->fitted[s])) {
+	    nf++;
+	}
+	if (fr->sderr != NULL) {
+	    fr->sderr[s] = gretl_matrix_get(F, s, i + m);
 	}
     }
 
     if (nf == 0) {
 	fr->err = E_MISSDATA;
     } else {
-	fr->tval = tcrit95(pmod->dfd);
+	if (var->ecm) {
+	    fr->df = var->T;
+	    /* asymptotic normal */
+	    fr->tval = 1.96;
+	} else {
+	    fr->df = pmod->dfd;
+	    fr->tval = tcrit95(fr->df);
+	}
 	fit_resid_set_dec_places(fr);
 	strcpy(fr->depvar, pdinfo->varname[yno]);
-	fr->df = pmod->dfd;
     }
 
-    fr->t1 -= pre_n;
-
     return fr;
 }
 
