@@ -56,7 +56,8 @@ struct atomset_ {
 #define must_know_children(f) (f == T_NOBS || f == T_MEAN || f == T_SUM || \
                                f == T_SD || f == T_VAR || f == T_SST || \
                                f == T_MEDIAN || f == T_MIN || f == T_MAX || \
-                               f == T_DIFF || f == T_LDIFF || f == T_T1 || f == T_T2 || \
+                               f == T_DIFF || f == T_LDIFF || f == T_SDIFF || \
+                               f == T_T1 || f == T_T2 || \
                                f == T_CUM || f == T_SORT || \
                                f == T_VARNUM || f == T_VECTOR || \
                                f == T_ISLIST || f == T_NELEM || \
@@ -177,10 +178,6 @@ static void real_stack_set_parentage (genatom **atoms, int n)
     }
 }
 
-/* below: ths static stuff is a big problem if genr is
-   called within aa genr. 
-*/
-
 static genatom *atom_stack (genatom *atom, atomset *aset, int op)
 {
     genatom *ret = NULL;
@@ -249,7 +246,7 @@ static genatom *atom_stack (genatom *atom, atomset *aset, int op)
     return ret;
 }
 
-int attach_atomset (GENERATE *genr)
+int attach_atomset (GENERATOR *genr)
 {
     atomset *aset = malloc(sizeof *aset);
 
@@ -270,7 +267,7 @@ int push_atom (genatom *atom)
     return (atom_stack(atom, atom->aset, STACK_PUSH) == NULL);
 }
 
-genatom *pop_atom (GENERATE *genr)
+genatom *pop_atom (GENERATOR *genr)
 {
     return atom_stack(NULL, genr->aset, STACK_POP);
 }
@@ -285,12 +282,12 @@ genatom *peek_child_atom (genatom *atom)
     return atom_stack(atom, atom->aset, STACK_PEEK_CHILDREN);
 }
 
-void reset_atom_stack (GENERATE *genr)
+void reset_atom_stack (GENERATOR *genr)
 {
     atom_stack(NULL, genr->aset, STACK_RESET);
 }
 
-void destroy_atom_stack (GENERATE *genr)
+void destroy_atom_stack (GENERATOR *genr)
 {
     if (genr->aset == NULL) return;
 
@@ -299,7 +296,7 @@ void destroy_atom_stack (GENERATE *genr)
     genr->aset = NULL;
 }
 
-void atom_stack_set_parentage (GENERATE *genr)
+void atom_stack_set_parentage (GENERATOR *genr)
 {
     atom_stack(NULL, genr->aset, STACK_DEPEND);
 }
@@ -309,22 +306,22 @@ void atom_eat_children (genatom *atom)
     atom_stack(atom, atom->aset, STACK_EAT_CHILDREN);
 }
 
-void atom_stack_bookmark (GENERATE *genr)
+void atom_stack_bookmark (GENERATOR *genr)
 {
     atom_stack(NULL, genr->aset, STACK_BOOKMARK);
 }
 
-void atom_stack_resume (GENERATE *genr)
+void atom_stack_resume (GENERATOR *genr)
 {
     atom_stack(NULL, genr->aset, STACK_RESUME);
 }
 
-int atom_stack_check_for_scalar (GENERATE *genr)
+int atom_stack_check_for_scalar (GENERATOR *genr)
 {
     return (atom_stack(NULL, genr->aset, STACK_SCALAR_CHECK) != NULL);
 }
 
-static double calc_stack (double val, int op, GENERATE *genr)
+static double calc_stack (double val, int op, GENERATOR *genr)
 {
     double *valstack = genr->valstack;
     int i;
@@ -342,15 +339,13 @@ static double calc_stack (double val, int op, GENERATE *genr)
 	    valstack[0] = val;
 	    genr->nvals += 1;
 	}
-    }
-    else if (op == STACK_POP && genr->nvals > 0) {
+    } else if (op == STACK_POP && genr->nvals > 0) {
 	x = valstack[0];
 	for (i=0; i<VALSTACK_SIZE-1; i++) {
 	    valstack[i] = valstack[i+1];
 	}
 	genr->nvals -= 1;
-    }
-    else if (op == STACK_RESET) {
+    } else if (op == STACK_RESET) {
 	for (i=0; i<VALSTACK_SIZE; i++) {
 	    valstack[i] = 0.0;
 	}
@@ -360,18 +355,18 @@ static double calc_stack (double val, int op, GENERATE *genr)
     return x;
 }
 
-int calc_push (double x, GENERATE *genr)
+int calc_push (double x, GENERATOR *genr)
 {
     calc_stack(x, STACK_PUSH, genr);
     return genr->err;
 }
 
-double calc_pop (GENERATE *genr)
+double calc_pop (GENERATOR *genr)
 {
     return calc_stack(0., STACK_POP, genr);
 }
 
-void reset_calc_stack (GENERATE *genr)
+void reset_calc_stack (GENERATOR *genr)
 {
     calc_stack(0., STACK_RESET, genr);
 }
