@@ -38,6 +38,7 @@ struct genatom_ {
     char level;
     char scalar;
     int varnum;
+    int varobs;
     int tmpvar;
     char lag;
     double val;
@@ -59,6 +60,7 @@ enum transformations {
     T_ATAN,
     T_DIFF,
     T_LDIFF, 
+    T_SDIFF,
     T_MEAN, 
     T_SD, 
     T_MIN,
@@ -94,6 +96,8 @@ enum transformations {
     T_MPOW,
     T_DNORM,
     T_CNORM,
+    T_GAMMA,
+    T_LNGAMMA,
     T_RESAMPLE,
     T_HPFILT,
     T_BKFILT,
@@ -120,10 +124,11 @@ enum genr_flags {
     GENR_PRIVATE      = 1 << 6
 };
 
-typedef struct _GENERATE GENERATE;
-
-struct _GENERATE {
+struct _GENERATOR {
     int err;
+    int done;
+    char orig_s[MAXLINE];
+    char lhs[USER_VLEN];
     unsigned char flags;
     double *xvec;
     int varnum;
@@ -134,28 +139,29 @@ struct _GENERATE {
     double **tmpZ;
     DATAINFO *pdinfo;
     double ***pZ;
-    MODEL *pmod;
     atomset *aset;
     double valstack[VALSTACK_SIZE];
     int nvals;
     char **S;
 };
 
-int attach_atomset (GENERATE *genr);
+int attach_atomset (GENERATOR *genr);
 int push_atom (genatom *atom);
-genatom *pop_atom (GENERATE *genr);
+genatom *pop_atom (GENERATOR *genr);
 genatom *pop_child_atom (genatom *atom);
 genatom *peek_child_atom (genatom *atom);
-void reset_atom_stack (GENERATE *genr);
-void destroy_atom_stack (GENERATE *genr);
-void atom_stack_set_parentage (GENERATE *genr);
+void reset_atom_stack (GENERATOR *genr);
+void destroy_atom_stack (GENERATOR *genr);
+void atom_stack_set_parentage (GENERATOR *genr);
 void atom_eat_children (genatom *atom);
-void atom_stack_bookmark (GENERATE *genr);
-void atom_stack_resume (GENERATE *genr);
-int atom_stack_check_for_scalar (GENERATE *genr);
+void atom_stack_bookmark (GENERATOR *genr);
+void atom_stack_resume (GENERATOR *genr);
+int atom_stack_check_for_scalar (GENERATOR *genr);
 
-int calc_push (double x, GENERATE *genr);
-double calc_pop (GENERATE *genr);
-void reset_calc_stack (GENERATE *genr);
+int calc_push (double x, GENERATOR *genr);
+double calc_pop (GENERATOR *genr);
+void reset_calc_stack (GENERATOR *genr);
+
+const char *get_genr_func_word (int fnum);
 
 #endif /* GENSTACK_H */
