@@ -21,6 +21,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <glib.h>
 
 #include "gretl_commands.h"
 
@@ -95,6 +96,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { LOOP,     "loop" },
     { MAHAL,    "mahal" },
     { MEANTEST, "meantest" },
+    { MLE,      "mle" },
     { MODELTAB, "modeltab" },
     { MPOLS,    "mpols" },
     { MULTIPLY, "multiply" },
@@ -126,6 +128,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { RUN,      "run" },
     { RUNS,     "runs" },
     { SCATTERS, "scatters" },
+    { SDIFF,    "sdiff" },
     { SET,      "set" },
     { SETOBS,   "setobs" },
     { SETMISS,  "setmiss" },
@@ -140,27 +143,17 @@ static struct gretl_cmd gretl_cmds[] = {
     { TABPRINT, "tabprint" },
     { TESTUHAT, "testuhat" },
     { TOBIT,    "tobit" },
+    { TRANSPOSE, "transpos" },
     { TSLS,     "tsls" },    
     { VAR,      "var" },
     { VARLIST,  "varlist" },
     { VARTEST,  "vartest" },
+    { VECM,     "vecm" },
     { VIF,      "vif" },
     { WLS,      "wls" },
     { NC,       NULL}
 }; 
 
-int gretl_command_number (const char *s)
-{    
-    int i;
-
-    for (i=0; i<NC; i++) {
-	if (!strcmp(s, gretl_cmds[i].cword)) {
-	    return gretl_cmds[i].cnum;
-	}
-    }
-    return 0;
-}
-
 const char *gretl_command_word (int i)
 {
     if (i >= 0 && i < NC) {
@@ -170,6 +163,44 @@ const char *gretl_command_word (int i)
     }
 }
 
+static GHashTable *ht;
+
+static void gretl_command_hash_init (void)
+{
+    int i;
+
+    ht = g_hash_table_new(g_str_hash, g_str_equal);
+
+    for (i=0; gretl_cmds[i].cword != NULL; i++) {
+	g_hash_table_insert(ht, (gpointer) gretl_cmds[i].cword, 
+			    GINT_TO_POINTER(gretl_cmds[i].cnum));
+    }
+}
+
+int gretl_command_number (const char *s)
+{    
+    gpointer p;
+    int ret = 0;
+
+    if (ht == NULL) {
+	gretl_command_hash_init();
+    }
+    
+    p = g_hash_table_lookup(ht, s);
+    if (p != NULL) {
+	ret = GPOINTER_TO_INT(p);
+    }
+
+    return ret;
+}
+
+void gretl_command_hash_cleanup (void)
+{
+    if (ht != NULL) {
+	g_hash_table_destroy(ht);
+    }
+}
+
 const char *gretl_command_complete_next (const char *s,
 					 int idx)
 {
