@@ -86,6 +86,7 @@ typedef enum {
     LOOP,
     MAHAL,
     MEANTEST,
+    MLE,
     MODELTAB,
     MPOLS,
     MULTIPLY,
@@ -117,6 +118,7 @@ typedef enum {
     RUN,
     RUNS,
     SCATTERS,
+    SDIFF,
     SET,
     SETOBS,
     SETMISS,
@@ -131,10 +133,12 @@ typedef enum {
     TABPRINT,
     TESTUHAT,
     TOBIT,
+    TRANSPOSE,
     TSLS,    
     VAR,
     VARLIST,
     VARTEST,
+    VECM,
     VIF,
     WLS,
     NC
@@ -195,5 +199,6 @@ const char *gretl_command_complete_next (const char *s, int idx);
 
 const char *gretl_command_complete (const char *s);
 
+void gretl_command_hash_cleanup (void);
 
 #endif /* COMMANDS_H */
