@@ -282,7 +282,7 @@ destroy_or_assign_local_vars (fncall *call, double ***pZ, DATAINFO *pdinfo,
 		copy_values_to_assignee(cv, i, *pZ, pdinfo);
 		saveit = 0;
 	    } else {
-		/* rename variable as caller desired: do not delete */
+		/* rename variable as caller desired: mark for preservation */
 		strcpy(pdinfo->varname[i], call->assv[v]);
 		STACK_LEVEL(pdinfo, i) -= 1; 
 	    }
@@ -852,6 +852,7 @@ static int parse_fn_element (char *s, char **parmv, char *ptype, int i,
 	sprintf(gretl_errmsg, "Unrecognized data type '%s'", tstr);
 	err = E_PARSE;
     } else if (ptype[i] == ARG_LIST && which == FN_RETURNS) {
+	/* FIXME? */
 	strcpy(gretl_errmsg, "A function cannot return a list");
 	err = 1;
     }
@@ -1149,11 +1150,131 @@ int gretl_function_append_line (const char *line)
     return err;
 }
 
+#define LOCAL_COPY_LISTS 1
+
+#ifdef LOCAL_COPY_LISTS
+
+/* expand by one element the mapping (in the form of two parallel
+   lists) between the ID numbers of "outer scope" variables and
+   the ID numbers of the corresponding local copies of those
+   variables */
+
+static int 
+grow_vars_mapping (int outv, int inv, int **p_outlist, int **p_inlist)
+{
+    int *outlist = NULL, *inlist = NULL;
+    int n;
+
+    if (*p_outlist != NULL) {
+	n = (*p_outlist)[0] + 2;
+    } else {
+	n = 2;
+    }
+
+    outlist = realloc(*p_outlist, n * sizeof *outlist);
+    if (outlist == NULL) {
+	return 1;
+    }
+
+    inlist = realloc(*p_inlist, n * sizeof *inlist);
+    if (inlist == NULL) {
+	*p_outlist = outlist;
+	return 1;
+    }
+
+    outlist[0] = inlist[0] = n - 1;
+    
+    outlist[n-1] = outv;
+    inlist[n-1] = inv;
+
+    *p_outlist = outlist;
+    *p_inlist = inlist;
+
+    return 0;
+}
+
+/* look up a variable by its "outer" ID number: if we've already made
+   a local copy of the variable, return the ID number of the copy
+   (otherwise return 0) */
+
+static int find_mapped_var (int v, const int *outlist, const int *inlist)
+{
+    int i, match = 0;
+
+    if (outlist != NULL && inlist != NULL) {
+	for (i=1; i<=outlist[0]; i++) {
+	    if (v == outlist[i]) {
+		match = inlist[i];
+		break;
+	    }
+	}
+    }
+
+    return match;
+}
+
+/* given a named list of variables supplied as an argument to a function,
+   make a revised version of the list in which the original variable
+   ID numbers are replaced by the ID numbers of local copies of those
+   variables */
+
+static int localize_list (const char *oldname, const char *newname,
+			  double ***pZ, DATAINFO *pdinfo,
+			  int **p_outlist, int **p_inlist)
+{
+    int *orig = NULL;
+    int *new = NULL;
+    int origv = pdinfo->v;
+    int i, v, match, err = 0;
+
+    orig = get_list_by_name(oldname);
+    if (orig == NULL) {
+	return 1;
+    }
+
+    new = gretl_list_copy(orig);
+    if (new == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=1; i<=orig[0] && !err; i++) {
+	v = orig[i];
+	if (v == pdinfo->v) {
+	    /* unknown variable */
+	    err = E_DATA;
+	} else if ((match = find_mapped_var(v, *p_outlist, *p_inlist))) {
+	    /* a local copy of this variable already exists */
+	    new[i] = match;
+	} else {
+	    /* add a local copy of the variable */
+	    err = dataset_copy_variable_as(v, pdinfo->varname[v], 
+					   pZ, pdinfo);
+	    if (!err) {
+		new[i] = pdinfo->v - 1;
+		err = grow_vars_mapping(v, pdinfo->v - 1, p_outlist, p_inlist);
+	    }
+	}
+    }
+	
+    if (!err) {
+	/* save the new list at appropriate stacking level */
+	err = stack_localized_list_as(new, newname);
+    } else {
+	dataset_drop_last_variables(pdinfo->v - origv, pZ, pdinfo);
+	free(new);
+    }
+
+    return err;
+}
+
+#endif
+
 static int check_and_allocate_function_args (ufunc *fun,
 					     int argc, char **argv, 
 					     double ***pZ,
 					     DATAINFO *pdinfo)
 {
+    int *outlist = NULL, *inlist = NULL;
     int i, v, err = 0;
 
     if (argc != fun->n_params) {
@@ -1176,6 +1297,9 @@ static int check_and_allocate_function_args (ufunc *fun,
 		v = varindex(pdinfo, argv[i]);
 		if (v < pdinfo->v && !pdinfo->vector[v]) {
 		    err = dataset_copy_variable_as(v, fun->params[i], pZ, pdinfo);
+		    if (!err) {
+			err = grow_vars_mapping(v, pdinfo->v - 1, &outlist, &inlist);
+		    }
 		} else {
 		    sprintf(gretl_errmsg, "argument %d (%s): not a scalar", i+1, argv[i]);
 		    err = 1;
@@ -1189,13 +1313,21 @@ static int check_and_allocate_function_args (ufunc *fun,
 	    }
 	    if (v < pdinfo->v && pdinfo->vector[v]) {
 		err = dataset_copy_variable_as(v, fun->params[i], pZ, pdinfo);
+		if (!err) {
+		    err = grow_vars_mapping(v, pdinfo->v - 1, &outlist, &inlist);
+		}
 	    } else {
 		sprintf(gretl_errmsg, "argument %d (%s): not a series", i+1, argv[i]);
 		err = 1;
 	    } 
 	} else if (fun->ptype[i] == ARG_LIST) {
 	    if (get_list_by_name(argv[i]) != NULL) {
+#ifdef LOCAL_COPY_LISTS
+		err = localize_list(argv[i], fun->params[i], pZ, pdinfo,
+				    &outlist, &inlist);
+#else
 		err = copy_named_list_as(argv[i], fun->params[i]);
+#endif
 	    } else {
 		sprintf(gretl_errmsg, "argument %d (%s): not a list", i+1, argv[i]);
 		err = 1;
@@ -1206,6 +1338,9 @@ static int check_and_allocate_function_args (ufunc *fun,
 	}
     }
 
+    free(outlist);
+    free(inlist);
+
     return err;
 }
 
@@ -1239,6 +1374,9 @@ static int check_function_assignments (ufunc *fun,
 	    } else {
 		asslist[i+1] = v;
 	    }
+	} else if (fun->rtype[i] == ARG_LIST) {
+	    /* FIXME? */
+	    fprintf(stderr, "requested return of list as '%s'\n", assv[i]);
 	} else {
 	    err = check_varname(assv[i]);
 	}
