@@ -23,6 +23,72 @@
 # include <glib.h>
 #endif
 
+#ifdef ENABLE_NLS
+
+static int numeric_c_locale_depth = 0;
+static char *numeric_locale = NULL;
+
+/**
+ * gretl_push_c_numeric_locale:
+ *
+ * Description:  Saves the current %LC_NUMERIC locale and sets it to "C"
+ * This way you can safely read write floating point numbers all in the
+ * same format.  You should make sure that code between
+ * gretl_push_c_numeric_locale() and gretl_pop_c_numeric_locale()
+ * doesn't do any setlocale calls or locale may end up in a strange setting.
+ * Also make sure to always pop the C numeric locale after you've pushed it.
+ * The calls can be nested.
+ **/
+
+void gretl_push_c_numeric_locale (void)
+{
+    if (numeric_c_locale_depth == 0) {
+	free(numeric_locale);
+	numeric_locale = gretl_strdup(setlocale(LC_NUMERIC, NULL));
+	setlocale(LC_NUMERIC, "C");
+    }
+    numeric_c_locale_depth++;
+}
+
+/**
+ * gretl_pop_c_numeric_locale:
+ *
+ * Description:  Restores the LC_NUMERIC locale to what it was 
+ * before the matching gretl_push_c_numeric_locale(). If these calls
+ * were nested, then this is a no-op until we get to the most outermost
+ * layer. Code in between these should not do any setlocale calls
+ * to change the %LC_NUMERIC locale or things may come out very strange.
+ **/
+
+void gretl_pop_c_numeric_locale (void)
+{
+    if (numeric_c_locale_depth == 0) {
+	return;
+    }
+
+    numeric_c_locale_depth--;
+
+    if (numeric_c_locale_depth == 0) {
+	setlocale(LC_NUMERIC, numeric_locale);
+	free(numeric_locale);
+	numeric_locale = NULL;
+    }
+}
+
+#else
+
+void gretl_push_c_numeric_locale (void)
+{
+    return;
+}
+
+void gretl_pop_c_numeric_locale (void)
+{
+    return;
+}
+
+#endif /* ENABLE_NLS */
+
 /**
  * doing_nls:
  *
@@ -518,7 +584,7 @@ struct l2sym {
     int ucs2val; /* corresponding UCS-2 code */
 };
 
-#ifndef WIN32
+# ifndef WIN32
 
 /* iso-8859-2 */
 
@@ -582,7 +648,7 @@ static struct l2sym l2table[] = {
     { 255, 729 }  /*  '. */
 };
 
-#else
+# else
 
 /* Windows codepage 1250 */
 
@@ -663,7 +729,7 @@ static struct l2sym l2table[] = {
     { 255,  729 }  /*  '. */
 };
 
-#endif
+# endif
 
 static int l2_lookup (int c)
 {
@@ -699,7 +765,7 @@ char *sprint_l2_to_html (char *targ, const char *s, size_t len)
     *p = '\0';
 
     while ((c = *s)) {
-#ifndef WIN32
+# ifndef WIN32
 	if (c > 160) {
 	    sprintf(p, "&#%d;", l2_lookup(c));
 	    p = strchr(p, ';') + 1;
@@ -709,14 +775,14 @@ char *sprint_l2_to_html (char *targ, const char *s, size_t len)
 	} else {
 	    *p++ = c;
 	}
-#else
+# else
 	if (c > 127) {
 	    sprintf(p, "&#%d;", l2_lookup(c));
 	    p = strchr(p, ';') + 1;
 	} else {
 	    *p++ = c;
 	}
-#endif
+# endif
 	s++;
 	if (p - targ > len - 8) {
 	    break;
@@ -807,22 +873,56 @@ char *get_month_name (char *mname, int m)
     strftime(mname, 7, "%b", &mt);
     *mname = tolower(*mname);
 
-#ifdef ENABLE_NLS
     real_iso_to_ascii(mname, gretl_cset_min);
-#endif
 
     return mname;
 }
 
 int get_utf_width (const char *str, int width)
 {
-#if defined(ENABLE_NLS) && defined(USE_GTK2)
+# ifdef USE_GTK2
     width += strlen(str) - g_utf8_strlen(str, -1);
-#endif
+# endif
 
     return width;
 }
 
+int get_translated_width (const char *str)
+{
+    int w = strlen(str);
+
+# ifdef USE_GTK2
+    w += w - g_utf8_strlen(str, -1);
+# endif
+
+    return w;
+}
+
+static int printing_to_console;
+
+char *maybe_iso_gettext (const char *msgid)
+{
+   if (printing_to_console) {
+       return iso_gettext(msgid);
+   } else {
+       return gettext(msgid);
+   }
+} 
+
+void check_for_console (PRN *prn)
+{
+    if (prn == NULL) return;
+
+    if (printing_to_standard_stream(prn)) {
+	printing_to_console = 1;
+    } else {
+	printing_to_console = 0;
+    }
+}
+
+void console_off (void)
+{
+    printing_to_console = 0;
+}
+
 #endif /* ENABLE_NLS */
-
-
