@@ -190,7 +190,34 @@ int remember_list (const int *list, const char *name, PRN *prn)
 }
 
 /**
- * copy_named_list:
+ * stack_localized_list_as:
+ * @list: the list to be saved.
+ * @name: the name to be given.
+ *
+ * For use in user-defined functions: take a list of
+ * variables that has been "localized" to function-scope
+ * and save it under the given @name.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int stack_localized_list_as (int *list, const char *name)
+{
+    saved_list *sl;
+    int err;
+
+    err = real_remember_list(list, name, 1, NULL);
+
+    if (!err) {
+	sl = list_stack[n_lists - 1];
+	sl->level += 1;
+    }
+
+    return err;
+}
+
+/**
+ * copy_named_list_as:
  * @orig: the name of the original list.
  * @new: the name to be given to the copy.
  *
@@ -345,7 +372,7 @@ int *gretl_null_list (void)
  * @src: an array of integers, the first element of which holds
  * a count of the number of elements following.
  *
- * Returns: an allocated copy @src.
+ * Returns: an allocated copy @src (or %NULL if @src is %NULL).
  */
 
 int *gretl_list_copy (const int *src)
@@ -552,6 +579,51 @@ int gretl_list_delete_at_pos (int *list, int pos)
     return 0;
 }
 
+/**
+ * gretl_list_purge_const:
+ * @list: an array of integers, the first element of which holds
+ * a count of the number of elements following.
+ *
+ * Checks @list from position 1 onward for the presence of a constant
+ * (the variable with ID number 0).  If this is found, it is deleted
+ * from list (that is, any following elements are moved forward by one
+ * and list[0] is decremented by 1).
+ *
+ * Returns: 1 if the constant was found and deleted, else 0.
+ */
+
+int gretl_list_purge_const (int *list)
+{
+    int i, gotc = 0;
+    int l0 = list[0];
+
+    /* handle the case where the constant comes last; if it's
+       the only element behind the list separator, remove both
+       the constant and the separator */
+    if (list[l0] == 0) {
+	gotc = 1;
+	list[0] -= 1;
+	if (list[l0 - 1] == LISTSEP) {
+	    list[l0 - 1] = 0;
+	    list[0] -= 1;
+	}
+    } else {
+	for (i=1; i<l0; i++) {
+	    if (list[i] == 0) {
+		for ( ; i<l0; i++) {
+		    list[i] = list[i+1];
+		}
+		list[l0] = 0;
+		list[0] -= 1;
+		gotc = 1;
+		break;
+	    }
+	}
+    }
+
+    return gotc;
+}
+
 /**
  * gretl_list_add:
  * @orig: an array of integers, the first element of which holds
@@ -620,7 +692,7 @@ int *gretl_list_add (const int *orig, const int *add, int *err)
  * a count of the number of elements following.
  * @err: location to receive error code.
  *
- * Creates a list containing all but the last elements of @orig.
+ * Creates a list containing all but the last element of @orig.
  *
  * Returns: new list on success, %NULL on error.
  */
@@ -973,6 +1045,99 @@ int gretl_list_has_const (const int *list)
     return 0;
 }
 
+/**
+ * gretl_list_separator_position:
+ * @list: an array of integer variable ID numbers, the first element
+ * of which holds a count of the number of elements following.
+ *
+ * Returns: if @list contains the separator for compound
+ * lists, #LISTSEP, the position in @list at which this is found,
+ * else 0.  The search begins at position 2.
+ */
+
+int gretl_list_separator_position (const int *list)
+{
+    int i;
+
+    for (i=2; i<=list[0]; i++) {
+        if (list[i] == LISTSEP) {
+	    return i;
+	}
+    }
+
+    return 0;
+}
+
+/**
+ * gretl_list_has_separator:
+ * @list: an array of integer variable ID numbers, the first element
+ * of which holds a count of the number of elements following.
+ *
+ * Returns: 1 if @list contains the separator for compound
+ * lists, #LISTSEP, else 0.  The search begins at position 2.
+ */
+
+int gretl_list_has_separator (const int *list)
+{
+    return gretl_list_separator_position(list) > 0;
+}
+
+/**
+ * gretl_list_split_on_separator:
+ * @list: source list.
+ * @plist1: pointer to accept first sub-list.
+ * @plist2: pointer to accept second sub-list.
+ *
+ * If @list contains the list separator, #LISTSEP, creates two
+ * sub-lists, one containing the elements of @list preceding
+ * the separator and one containing the elements following
+ * the separator.  The sub-lists are newly allocated, and assigned
+ * as the content of @plist1 and @plist2 respectively.
+ *
+ * Returns: 0 on success, %E_ALLOC is memory allocation fails,
+ * or %E_DATA if @list does not contain a separator.
+ */
+
+int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
+{
+    int *list1 = NULL, *list2 = NULL;
+    int i, n = -1;
+
+    for (i=1; i<list[0] && n<0; i++) {
+	if (list[i] == LISTSEP) {
+	    n = i;
+	}
+    }
+
+    if (n < 0) {
+	return 1;
+    }
+
+    list1 = gretl_list_new(n - 1);
+    if (list1 == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=1; i<n; i++) {
+	list1[i] = list[i];
+    }
+
+    list2 = gretl_list_new(list[0] - n);
+    if (list2 == NULL) {
+	free(list1);
+	return E_ALLOC;
+    }
+
+    for (i=1; i<=list2[0]; i++) {
+	list2[i] = list[i + n];
+    }
+
+    *plist1 = list1;
+    *plist2 = list2;
+    
+    return 0;
+}
+
 /**
  * gretl_list_duplicates:
  * @list: an array of integer variable ID numbers, the first element
@@ -994,10 +1159,17 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 
     if (ci == ARCH) {
 	start = 3;
+    } else if (ci == ARMA) {
+	for (i=list[0]-1; i>2; i--) {
+	    if (list[i] == LISTSEP) {
+		start = i+1;
+		break;
+	    }
+	}
     } else if (ci == LAGS && list[0] > 1 && list[2] == LISTSEP) {
 	start = 3;
-    } else if (ci == TSLS || ci == AR || ci == ARMA || 
-	       ci == SCATTERS || ci == MPOLS || ci == GARCH) {
+    } else if (ci == TSLS || ci == AR || ci == SCATTERS || 
+	       ci == MPOLS || ci == GARCH) {
 	for (i=2; i<list[0]; i++) {
 	    if (list[i] == LISTSEP) {
 		start = i+1;
