@@ -22,6 +22,11 @@
 
 /* #define LDEBUG 1 */
 
+/* minimum value of diagonal element of R (as in X = QR) that counts
+   as non-zero for the purpose of determining the rank of X */
+
+#define R_DIAG_MIN 1.0e-8
+
 typedef enum {
     GRETL_MATRIX_OK = 0,
     GRETL_MATRIX_NOMEM,
@@ -40,12 +45,53 @@ typedef enum {
 typedef struct _gretl_matrix gretl_matrix;
 typedef struct _gretl_matrix gretl_vector;
 
+struct _gretl_matrix {
+    int rows;
+    int cols;
+    int t;
+    double *val;
+};
+
+#define mdx(a,i,j)   ((j)*(a)->rows+(i))
+#define mdxtr(a,i,j) ((i)*(a)->rows+(j))
+
+/**
+ * gretl_matrix_cols:
+ * @m: matrix to query.
+ * 
+ * Gives the number of columns in @m. 
+ */
+
+#define gretl_matrix_cols(m) ((m == NULL)? 0 : m->cols)
+
+/**
+ * gretl_matrix_rows:
+ * @m: matrix to query.
+ * 
+ * Gives the number of rows in @m. 
+ */
+
+#define gretl_matrix_rows(m) ((m == NULL)? 0 : m->rows)
+
+/**
+ * gretl_vector_get_length:
+ * @v: vector to examine.
+ * 
+ * Gives the length of vector @v (without regard to whether
+ * it is a row or column vector).
+ */
+
+#define gretl_vector_get_length(v) ((v == NULL)? 0 : \
+                                    (v->cols > v->rows)? v->cols : \
+                                    v->rows)
+
 /**
  * gretl_vector_alloc:
  * @i: number of columns.
  *
- * Allocates a new #gretl_vector with %i columns.
+ * Allocates a new #gretl_vector with @i columns.
  */
+
 #define gretl_vector_alloc(i) gretl_matrix_alloc(1,(i))
 
 /**
@@ -54,6 +100,7 @@ typedef struct _gretl_matrix gretl_vector;
  *
  * Allocates a new column gretl_vector with @i rows.
  */
+
 #define gretl_column_vector_alloc(i) gretl_matrix_alloc((i),1)
 
 /**
@@ -62,12 +109,25 @@ typedef struct _gretl_matrix gretl_vector;
  *
  * Frees the vector @v and its associated storage.
  */
+
 #define gretl_vector_free(v) gretl_matrix_free(v)
 
+double gretl_matrix_get (const gretl_matrix *m, int i, int j);
+
+double gretl_vector_get (const gretl_vector *v, int i);
+
+int gretl_matrix_set (gretl_matrix *m, int i, int j, double x);
+
+int gretl_vector_set (gretl_vector *v, int i, double x);
+
 gretl_matrix *gretl_matrix_alloc (int rows, int cols);
 
+gretl_matrix *gretl_matrix_reuse (gretl_matrix *m, int rows, int cols);
+
 gretl_matrix *gretl_packed_matrix_alloc (int rows);
 
+gretl_matrix *gretl_identity_matrix_new (int n);
+
 gretl_matrix *gretl_matrix_copy (const gretl_matrix *m);
 
 gretl_matrix *gretl_matrix_copy_transpose (const gretl_matrix *m);
@@ -106,14 +166,6 @@ double *gretl_matrix_steal_data (gretl_matrix *m);
 int gretl_matrix_copy_values (gretl_matrix *targ, 
 			      const gretl_matrix *src);
 
-double gretl_matrix_get (const gretl_matrix *m, int i, int j);
-
-double gretl_vector_get (const gretl_vector *v, int i);
-
-int gretl_matrix_set (gretl_matrix *m, int i, int j, double x);
-
-int gretl_vector_set (gretl_vector *v, int i, double x);
-
 int gretl_matrix_add_to (gretl_matrix *targ, const gretl_matrix *src);
 
 int 
@@ -132,13 +184,27 @@ int gretl_matrix_multiply_mod (const gretl_matrix *a, GretlMatrixMod amod,
 int gretl_matrix_multiply (const gretl_matrix *a, const gretl_matrix *b,
 			   gretl_matrix *c);
 
+gretl_matrix *
+gretl_matrix_kronecker_product (const gretl_matrix *A, const gretl_matrix *B);
+
 double gretl_matrix_dot_product (const gretl_matrix *a, GretlMatrixMod amod,
 				 const gretl_matrix *b, GretlMatrixMod bmod,
 				 int *errp);
 
+double gretl_vector_dot_product (const gretl_vector *a, const gretl_vector *b,
+				 int *errp);
+
 gretl_matrix *gretl_matrix_dot_multiply (const gretl_matrix *a, 
 					 const gretl_matrix *b);
 
+double gretl_matrix_row_mean (const gretl_matrix *m, int row);
+
+double gretl_matrix_column_mean (const gretl_matrix *m, int col);
+
+void gretl_matrix_demean_by_row (gretl_matrix *m);
+
+void gretl_matrix_demean_by_column (gretl_matrix *m);
+
 gretl_matrix *gretl_matrix_vcv (gretl_matrix *m);
 
 double gretl_matrix_determinant (gretl_matrix *a);
@@ -147,7 +213,7 @@ double gretl_matrix_log_determinant (gretl_matrix *a);
 
 double gretl_matrix_log_abs_determinant (gretl_matrix *a);
 
-double gretl_vcv_log_determinant (gretl_matrix *a);
+double gretl_vcv_log_determinant (const gretl_matrix *m);
 
 int gretl_LU_solve (gretl_matrix *a, gretl_vector *b);
 
@@ -157,13 +223,25 @@ int gretl_invert_symmetric_matrix (gretl_matrix *a);
 
 int gretl_invert_diagonal_matrix (gretl_matrix *a);
 
+int gretl_SVD_invert_matrix (gretl_matrix *a);
+
+double gretl_symmetric_matrix_rcond (const gretl_matrix *m);
+
+int gretl_eigen_sort (double *evals, gretl_matrix *evecs, int rank);
+
 double *gretl_general_matrix_eigenvals (gretl_matrix *m, gretl_matrix *ev);
 
 double *gretl_symmetric_matrix_eigenvals (gretl_matrix *m,
 					  int eigenvecs);
 
+gretl_matrix *gretl_matrix_right_nullspace (const gretl_matrix *M);
+
 int gretl_matrix_cholesky_decomp (gretl_matrix *a);
 
+int gretl_matrix_QR_decomp (gretl_matrix *M, gretl_matrix *R);
+
+int gretl_matrix_QR_rank (gretl_matrix *R, char **pmask, int *errp);
+
 int gretl_matrix_ols (const gretl_vector *y, const gretl_matrix *X,
 		      gretl_vector *b, gretl_matrix *vcv,
 		      gretl_vector *uhat, double *s2);
@@ -178,15 +256,16 @@ int gretl_matrix_svd_ols (const gretl_vector *y, const gretl_matrix *X,
 			  gretl_vector *b, gretl_matrix *vcv,
 			  gretl_vector *uhat, double *s2);
 
-double gretl_scalar_b_prime_X_b (const gretl_vector *b, const gretl_matrix *X,
-				 int *errp);
-
-double gretl_scalar_b_X_b_prime (const gretl_vector *b, const gretl_matrix *X,
-				 int *errp);
+double gretl_scalar_b_X_b (const gretl_vector *b, GretlMatrixMod bmod,
+			   const gretl_matrix *X, int *errp);
 
 gretl_matrix *
-gretl_matrix_A_X_A_prime (const gretl_matrix *A, const gretl_matrix *X,
-			  int *errp);
+gretl_matrix_A_X_A (const gretl_matrix *A, GretlMatrixMod amod,
+		    const gretl_matrix *X, int *errp);
+
+int
+gretl_matrix_diagonal_sandwich (const gretl_vector *d, const gretl_matrix *X,
+				gretl_matrix *DXD);
 
 gretl_matrix *
 gretl_vcv_matrix_from_model (MODEL *pmod, const char *select);
@@ -194,18 +273,15 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select);
 gretl_vector *
 gretl_coeff_vector_from_model (const MODEL *pmod, const char *select);
 
-void gretl_matrix_print (const gretl_matrix *m, const char *msg, PRN *prn);
+void 
+gretl_matrix_print_to_prn (const gretl_matrix *m, const char *msg, PRN *prn);
+
+void gretl_matrix_print (const gretl_matrix *m, const char *msg);
 
 void gretl_matrix_set_int (gretl_matrix *m, int t);
 
 int gretl_matrix_get_int (const gretl_matrix *m);
 
-int gretl_vector_get_length (const gretl_vector *v);
-
-int gretl_matrix_cols (const gretl_matrix *m);
-
-int gretl_matrix_rows (const gretl_matrix *m);
-
 int gretl_is_identity_matrix (const gretl_matrix *m);
 
 int gretl_is_zero_vector (const gretl_vector *v);
@@ -215,4 +291,16 @@ gretl_covariance_matrix_from_varlist (const int *list, const double **Z,
 				      const DATAINFO *pdinfo, 
 				      gretl_matrix **means, int *errp);
 
+int gretl_matrix_row_to_array (const gretl_matrix *m, int i, double *x);
+
+gretl_matrix **gretl_matrix_array_alloc (int n);
+
+gretl_matrix **
+gretl_matrix_array_alloc_with_size (int n, int rows, int cols);
+
+void gretl_matrix_array_free (gretl_matrix **A, int n);
+
+gretl_matrix *gretl_matrix_data_subset (const int *list, const double **Z,
+					int t1, int t2, char **pmask);
+
 #endif /* GRETL_MATRIX_H */
