@@ -18,8 +18,11 @@
  */
 
 #include "libgretl.h"
+#include "objstack.h"
 
-#undef MODEL_DEBUG
+#include "glib.h"
+
+#define MDEBUG 0
 
 struct model_data_item_ {
     char *key;
@@ -146,6 +149,27 @@ int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, size_t size)
     return gretl_model_set_data_with_destructor(pmod, key, ptr, size, NULL);
 }
 
+/**
+ * gretl_model_set_list_as_data:
+ * @pmod: pointer to #MODEL.
+ * @key: key string, used in retrieval.
+ * @list: list to attach.
+ *
+ * Attaches @list to @pmod as data, recoverable via the key @key 
+ * using gretl_model_get_data().
+ *
+ * Returns: 0 on success, 1 on failure.
+ */
+
+int gretl_model_set_list_as_data (MODEL *pmod, const char *key, int *list)
+{
+    size_t size = (list[0] + 1) * sizeof *list;
+
+    return gretl_model_set_data_with_destructor(pmod, key, (void *) list, 
+						size, NULL);
+}
+
+
 /**
  * gretl_model_set_int:
  * @pmod: pointer to #MODEL.
@@ -303,6 +327,18 @@ double gretl_model_get_double (const MODEL *pmod, const char *key)
     return NADBL;
 }
 
+static void adjust_vecm_name (const char *orig, char *cname)
+{
+    int cnum;
+    char cc;
+
+    if (sscanf(orig, "EC%d%c", &cnum, &cc) == 2) {
+	sprintf(cname, "EC%d", cnum);
+    } else {
+	strcpy(cname, orig);
+    }
+}
+
 static void make_cname (const char *orig, char *cname)
 {
     char *p;
@@ -348,11 +384,14 @@ char *gretl_model_get_param_name (const MODEL *pmod, const DATAINFO *pdinfo,
     *targ = '\0';
 
     if (pmod != NULL) {
-	/* special treatment for ARCH, ARMA, GARCH, NLS */
+	/* special treatment for ARCH, ARMA, GARCH, NLS, MLE */
 	if (pmod->aux == AUX_ARCH) {
 	    make_cname(pdinfo->varname[pmod->list[i + 2]], targ);
-	} else if (pmod->ci == NLS || pmod->ci == ARMA || pmod->ci == GARCH) {
+	} else if (pmod->ci == NLS || pmod->ci == MLE ||
+		   pmod->ci == ARMA || pmod->ci == GARCH) {
 	    strcpy(targ, pmod->params[i + 1]);
+	} else if (pmod->aux == AUX_VECM) {
+	    adjust_vecm_name(pdinfo->varname[pmod->list[i + 2]], targ);
 	} else {
 	    strcpy(targ, pdinfo->varname[pmod->list[i + 2]]);
 	}
@@ -457,14 +496,14 @@ gretl_model_get_coeff_intervals (const MODEL *pmod,
 }
 
 /**
- * gretl_arma_model_get_AR_order:
+ * gretl_arma_model_get_nonseasonal_AR_order:
  * @pmod: pointer to gretl model.
  *
- * Returns: the autoregressive order of @pmod, or 0 if
+ * Returns: the non-seasonal autoregressive order of @pmod, or 0 if
  * @pmod is not an ARMA model.
  */
 
-int gretl_arma_model_get_AR_order (const MODEL *pmod)
+int gretl_arma_model_get_nonseasonal_AR_order (const MODEL *pmod)
 {
     int p = 0;
 
@@ -476,14 +515,14 @@ int gretl_arma_model_get_AR_order (const MODEL *pmod)
 }
 
 /**
- * gretl_arma_model_get_MA_order:
+ * gretl_arma_model_get_nonseasonal_MA_order:
  * @pmod: pointer to gretl model.
  *
- * Returns: the moving-average order of @pmod, or 0 if
+ * Returns: the non-seasonal moving-average order of @pmod, or 0 if
  * @pmod is not an ARMA model.
  */
 
-int gretl_arma_model_get_MA_order (const MODEL *pmod)
+int gretl_arma_model_get_nonseasonal_MA_order (const MODEL *pmod)
 {
     int q = 0;
 
@@ -494,6 +533,210 @@ int gretl_arma_model_get_MA_order (const MODEL *pmod)
     return q;
 }
 
+/**
+ * gretl_arma_model_get_max_AR_lag:
+ * @pmod: pointer to gretl model.
+ *
+ * Returns: the maximum autoregressive lag in @pmod, or 0 if
+ * @pmod is not an ARMA model.
+ */
+
+int gretl_arma_model_get_max_AR_lag (const MODEL *pmod)
+{
+    int pmax = 0;
+
+    if (pmod->ci == ARMA) {
+	int p, P, pd;
+
+	p = pmod->list[1];
+	P = gretl_model_get_int(pmod, "arma_P");
+
+	if (P == 0) {
+	    pmax = p;
+	} else {
+	    pd = gretl_model_get_int(pmod, "arma_pd");
+	    pmax = P * pd + p;
+	}
+    }
+
+    return pmax;
+}
+
+/**
+ * gretl_arma_model_get_max_MA_lag:
+ * @pmod: pointer to gretl model.
+ *
+ * Returns: the maximum moving-average lag in @pmod, or 0 if
+ * @pmod is not an ARMA model.
+ */
+
+int gretl_arma_model_get_max_MA_lag (const MODEL *pmod)
+{
+    int qmax = 0;
+
+    if (pmod->ci == ARMA) {
+	int q, Q, pd;
+
+	q = pmod->list[2];
+	Q = gretl_model_get_int(pmod, "arma_Q");
+	
+	if (Q == 0) {
+	    qmax = q;
+	} else {
+	    pd = gretl_model_get_int(pmod, "arma_pd");
+	    qmax = Q * pd + q;
+	}
+    }
+
+    return qmax;
+}
+
+/**
+ * gretl_arma_model_get_AR_MA_coeffs:
+ * @pmod: pointer to gretl model.
+ * @arvec: pointer to receive AR coeff vector.
+ * @mavec: pointer to receive MA coeff vector.
+ *
+ * Creates allocated copies of the AR and MA coefficient vectors from
+ * @pmod.  If @pmod includes seasonal ARMA terms, the coefficient vectors
+ * are suitably expanded, and include the interactions, if any, between 
+ * seasonal and non-seasonal terms.  The length of these vectors can
+ * be determined using gretl_arma_model_get_max_AR_lag() and
+ * gretl_arma_model_get_max_MA_lag() respectively.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_arma_model_get_AR_MA_coeffs (const MODEL *pmod,
+				       double **arvec,
+				       double **mavec)
+{
+    double *ac = NULL;
+    double *mc = NULL;
+    int err = 0;
+
+    if (pmod->ci != ARMA) {
+	err = 1;
+    } else {
+	const double *phi = NULL, *Phi = NULL;
+	const double *theta = NULL, *Theta = NULL;
+
+	int p = pmod->list[1];
+	int q = pmod->list[2];
+	int P = gretl_model_get_int(pmod, "arma_P");
+	int Q = gretl_model_get_int(pmod, "arma_Q");
+	int pd = gretl_model_get_int(pmod, "arma_pd");
+	int pmax, qmax;
+	int i, j, k;
+
+	pmax = (P > 0)? P * pd + p : p;
+	qmax = (Q > 0)? Q * pd + q : q;
+	
+	if (pmax > 0) {
+	    ac = malloc(pmax * sizeof *ac);
+	    if (ac == NULL) {
+		err = E_ALLOC;
+	    }
+	}
+
+	if (!err && qmax > 0) {
+	    mc = malloc(qmax * sizeof *ac);
+	    if (mc == NULL) {
+		free(ac);
+		ac = NULL;
+		err = E_ALLOC;
+	    }
+	}
+
+	if (!err) {
+	    phi = pmod->coeff + pmod->ifc; /* non-seasonal AR coeffs */
+	    Phi = phi + p;                 /* seasonal AR coeffs */
+	    theta = Phi + P;               /* non-seasonal MA coeffs */
+	    Theta = theta + q;             /* seasonal MA coeffs */
+	}
+
+	if (ac != NULL) {
+	    for (i=0; i<p; i++) {
+		ac[i] = phi[i];
+	    }	    
+	    if (P > 0) {
+		for (i=p; i<pmax; i++) {
+		    ac[i] = 0.0;
+		}
+		for (i=0; i<P; i++) {
+		    k = pd * (i+1) - 1;
+		    ac[k] += Phi[i];
+		    for (j=0; j<p; j++) {
+			ac[k+j+1] += Phi[i] * phi[j];
+		    }
+		}
+	    }		
+	}
+
+	if (mc != NULL) {
+	    for (i=0; i<q; i++) {
+		mc[i] = theta[i];
+	    }	    
+	    if (Q > 0) {
+		for (i=p; i<qmax; i++) {
+		    mc[i] = 0.0;
+		}
+		for (i=0; i<Q; i++) {
+		    k = pd * (i+1) - 1;
+		    mc[k] += Theta[i];
+		    for (j=0; j<q; j++) {
+			mc[k+j+1] += Theta[i] * theta[j];
+		    }
+		}
+	    }		
+	}
+    }
+
+    if (!err) {
+	*arvec = ac;
+	*mavec = mc;
+    }
+
+    return err;
+}
+
+/**
+ * gretl_arma_model_get_x_coeffs:
+ * @pmod: pointer to gretl model.
+ *
+ * Returns: pointer to the array of coefficients on the exogenous
+ * regressors in @pmod, or %NULL if the model is not ARMA or if there
+ * are no such regressors.
+ */
+
+const double *gretl_arma_model_get_x_coeffs (const MODEL *pmod)
+{
+    const double *xc = NULL;
+
+    if (pmod->ci == ARMA && gretl_model_get_int(pmod, "armax")) {
+	xc = pmod->coeff;
+	xc += pmod->ifc;
+	xc += pmod->list[1];
+	xc += pmod->list[2];
+	xc += gretl_model_get_int(pmod, "arma_P");
+	xc += gretl_model_get_int(pmod, "arma_Q");
+    }
+
+    return xc;
+}
+
+static int is_seasonal_arma (const MODEL *pmod)
+{
+    int ret = 0;
+
+    if (gretl_model_get_int(pmod, "arma_P") ||
+	gretl_model_get_int(pmod, "arma_Q")) {
+	ret = 1;
+    }
+
+    return ret;
+}
+
 /**
  * gretl_model_get_depvar:
  * @pmod: pointer to gretl model.
@@ -509,7 +752,7 @@ int gretl_model_get_depvar (const MODEL *pmod)
 	if (pmod->ci == GARCH) {
 	    dv = pmod->list[4];
 	} else if (pmod->ci == ARMA) {
-	    if (gretl_model_get_int(pmod, "seasonal")) {
+	    if (is_seasonal_arma(pmod)) {
 		dv = pmod->list[7];
 	    } else {
 		dv = pmod->list[4];
@@ -536,13 +779,8 @@ int *gretl_model_get_x_list (const MODEL *pmod)
     int i, nx;
 
     if (pmod->ci == ARMA) {
-	int start;
+	int start = (is_seasonal_arma(pmod))? 7 : 4;
 
-	if (gretl_model_get_int(pmod, "seasonal")) {
-	    start = 7;
-	} else {
-	    start = 4;
-	}
 	nx = pmod->list[0] - start + pmod->ifc;
 	if (nx > 0) {
 	    list = gretl_list_new(nx);
@@ -569,7 +807,7 @@ int *gretl_model_get_x_list (const MODEL *pmod)
 		}
 	    }
 	}
-    } else if (pmod->ci != NLS) {
+    } else if (pmod->ci != NLS && pmod->ci != MLE) {
 	nx = pmod->ncoeff;
 	list = gretl_list_new(nx);
 	if (list != NULL) {
@@ -874,7 +1112,13 @@ void gretl_model_init (MODEL *pmod)
 
     if (pmod == NULL) return;
 
+#if MDEBUG
+    fprintf(stderr, "gretl_model_init: pmod at %p\n", (void *) pmod);
+#endif
+
     pmod->ID = 0;
+    pmod->refcount = 1;
+    pmod->full_n = 0;
 
     pmod->ntests = 0;
     pmod->nparams = 0;
@@ -913,6 +1157,28 @@ void gretl_model_smpl_init (MODEL *pmod, const DATAINFO *pdinfo)
     pmod->smpl.t2 = pdinfo->t2;
 }
 
+static MODEL *real_gretl_model_new (int protected)
+{
+    MODEL *pmod = malloc(sizeof *pmod);
+
+#if MDEBUG
+    fprintf(stderr, "real_gretl_model_new: model at %p\n", (void *) pmod);
+#endif
+
+    if (pmod != NULL) {
+	gretl_model_init(pmod);
+    }
+
+    if (protected) {
+#if MDEBUG
+	fprintf(stderr, " protecting this model\n");
+#endif
+	gretl_model_protect(pmod);
+    }
+
+    return pmod;
+}
+
 /**
  * gretl_model_new:
  * 
@@ -924,13 +1190,79 @@ void gretl_model_smpl_init (MODEL *pmod, const DATAINFO *pdinfo)
 
 MODEL *gretl_model_new (void)
 {
-    MODEL *pmod = malloc(sizeof *pmod);
-
-    gretl_model_init(pmod);
-    return pmod;
+    return real_gretl_model_new(0);
 }
 
-/* .......................................................... */
+/**
+ * gretl_model_new_protected:
+ * 
+ * Allocates memory for a gretl #MODEL struct and initializes the struct,
+ * using gretl_model_init().  Unlike plain gretl_model_new(), this function
+ * creates a model which will not be freed by the mechanisms in objstack.c.
+ *
+ * Returns: pointer to model (or %NULL if allocation fails).
+ */
+
+MODEL *gretl_model_new_protected (void)
+{
+    return real_gretl_model_new(1);
+}
+
+/**
+ * gretl_model_array_new:
+ * @n: number of models in array.
+ * 
+ * Allocates memory for an array of @n gretl #MODEL structs and 
+ * initializes each model using gretl_model_init().
+ *
+ * Returns: pointer to models array (or %NULL if allocation fails).
+ */
+
+MODEL **gretl_model_array_new (int n)
+{
+    MODEL **models;
+    int i, j;
+
+    models = malloc(n * sizeof *models);
+
+    if (models != NULL) {
+	for (i=0; i<n; i++) {
+	    models[i] = gretl_model_new();
+	    if (models[i] == NULL) {
+		for (j=0; j<i; j++) {
+		    free(models[i]);
+		}
+		free(models);
+		models = NULL;
+		break;
+	    }
+	}
+    } 
+
+    return models;
+}
+
+/**
+ * gretl_model_array_destroy:
+ * @models: array of gretl models.
+ * @n: number of models in array.
+ * 
+ * Frees all resources associated with an array of models, which
+ * should have been obtained via gretl_model_array_new().
+ */
+
+void gretl_model_array_destroy (MODEL **models, int n)
+{
+    int i;
+
+    if (models != NULL) {
+	for (i=0; i<n; i++) {
+	    clear_model(models[i]);
+	    free(models[i]);
+	}
+	free(models);
+    }
+}
 
 static void clear_ar_info (MODEL *pmod)
 {
@@ -948,7 +1280,7 @@ static void clear_ar_info (MODEL *pmod)
     pmod->arinfo = NULL;
 }
 
-#ifdef MODEL_DEBUG
+#if MDEBUG > 1
 
 static void 
 debug_print_model_info (const MODEL *pmod, const char *msg)
@@ -979,12 +1311,29 @@ debug_print_model_info (const MODEL *pmod, const char *msg)
 	    (void *) pmod->data);
 }
 
-#endif /* MODEL_DEBUG */
+#endif /* MDEBUG */
 
-static void gretl_test_free (ModelTest *test)
+/**
+ * gretl_model_destroy_tests:
+ * @pmod: pointer to model.
+ *
+ * Clears any hypothesis test structs that have been attached
+ * to @pmod.
+ */
+
+void gretl_model_destroy_tests (MODEL *pmod)
 {
-    if (test->param != NULL) {
-	free(test->param);
+    if (pmod != NULL && pmod->ntests > 0) {
+	int i;
+
+	for (i=0; i<pmod->ntests; i++) {
+	    if (pmod->tests[i].param != NULL) {
+		free(pmod->tests[i].param);
+	    }
+	}
+	free(pmod->tests);
+	pmod->tests = NULL;
+	pmod->ntests = 0;
     }
 }
 
@@ -1005,7 +1354,11 @@ void clear_model (MODEL *pmod)
     if (pmod != NULL) {
 	int i;
 
-#ifdef MODEL_DEBUG
+#if MDEBUG
+	fprintf(stderr, "clear model: model at %p\n", (void *) pmod);
+#endif
+
+#if MDEBUG > 1
 	debug_print_model_info(pmod, "Doing clear_model");
 #endif
 	if (pmod->list) free(pmod->list);
@@ -1021,12 +1374,6 @@ void clear_model (MODEL *pmod)
 	if (pmod->arinfo) {
 	    clear_ar_info(pmod);
 	}
-	if (pmod->ntests) {
-	    for (i=0; i<pmod->ntests; i++) {
-		gretl_test_free(&pmod->tests[i]);
-	    }
-	    free(pmod->tests);
-	}
 	if (pmod->params) {
 	    for (i=0; i<pmod->nparams; i++) {
 		free(pmod->params[i]);
@@ -1036,14 +1383,58 @@ void clear_model (MODEL *pmod)
 	if (pmod->dataset) {
 	    free_model_dataset(pmod);
 	}
+	gretl_model_destroy_tests(pmod);
 	destroy_all_data_items(pmod);
     }
 
     /* this may be redundant */
+#if MDEBUG
+    fprintf(stderr, "clear_model, calling gretl_model_init\n");
+#endif
+#if 1
     gretl_model_init(pmod);
+#endif
 }
 
-/* ........................................................... */
+/**
+ * gretl_model_free:
+ * @pmod: pointer to #MODEL.
+ *
+ * Free allocated content of @pmod then the pointer itself,
+ * if the model's reference count has reached zero.
+ */
+
+void gretl_model_free (MODEL *pmod)
+{
+    if (pmod != NULL) {
+#if MDEBUG
+	fprintf(stderr, "gretl_model_free: pmod at %p, incoming refcount = %d\n",
+		(void *) pmod, pmod->refcount);
+#endif
+	pmod->refcount -= 1;
+	if (pmod->refcount <= 0) {
+	    clear_model(pmod);
+	    free(pmod);
+	}
+    }
+}
+
+/**
+ * gretl_model_free_on_exit:
+ * @pmod: pointer to #MODEL.
+ *
+ * Free allocated content of @pmod then the pointer itself,
+ * without regard to the model's reference count.
+ */
+
+void gretl_model_free_on_exit (MODEL *pmod)
+{
+    if (pmod != NULL) {
+	remove_model_from_stack(pmod);
+	clear_model(pmod);
+	free(pmod);
+    }
+}
 
 static void copy_test (ModelTest *targ, const ModelTest *src)
 {
@@ -1058,6 +1449,7 @@ static void copy_test (ModelTest *targ, const ModelTest *src)
     targ->teststat = src->teststat;
     targ->dfn = src->dfn;
     targ->dfd = src->dfd;
+    targ->order = src->order;
     targ->value = src->value;
     targ->pvalue = src->pvalue;
 }
@@ -1085,36 +1477,75 @@ static void gretl_test_init (ModelTest *test, ModelTestType ttype)
     test->type = ttype;
     test->order = 0;
     test->param = NULL;
-    test->teststat = 0;
+    test->teststat = GRETL_STAT_NONE;
     test->dfn = test->dfd = 0;
     test->value = test->pvalue = NADBL;
 }
 
+static int model_tests_differ (ModelTest *mt1, ModelTest *mt2)
+{
+    int ret = 0;
+
+    if (mt1->type != mt2->type) {
+	ret = 1;
+    } else if (mt1->order != mt2->order) {
+	ret = 1;
+    } else if (mt1->param != NULL && mt2->param != NULL &&
+	       strcmp(mt1->param, mt2->param)) {
+	ret = 1;
+    } else if (mt1->teststat != mt2->teststat) {
+	ret = 1;
+    } else if (mt1->value != mt2->value) {
+	ret = 1;
+    }
+
+    return ret;
+}
+
 /**
- * new_test_on_model:
- * @pmod: pointer to model.
+ * model_test_new:
  * @ttype: type of test to add.
  *
- * Adds a #ModelTest to @pmod, if a test of the given type has
- * not already been performed and recorded.
- *
- * Returns: model test pointer, or %NULL if the test is
- * already present or on failure.
+ * Returns: new #ModelTest pointer, or %NULL on failure.
  */
 
-ModelTest *new_test_on_model (MODEL *pmod, ModelTestType ttype)
+ModelTest *model_test_new (ModelTestType ttype)
+{
+    ModelTest *test = malloc(sizeof *test);
+
+    if (test != NULL) {
+	gretl_test_init(test, ttype);
+    }
+
+    return test;
+}
+
+/**
+ * maybe_add_test_to_model:
+ * @pmod: pointer to model.
+ * @test: model test to be added.
+ *
+ * Adds a #ModelTest to @pmod, if the test in question has
+ * not already been performed and recorded.
+ *
+ * Returns: 1 if the test was added, otherwise 0.
+ */
+
+int maybe_add_test_to_model (MODEL *pmod, ModelTest *test)
 {
     ModelTest *tests = NULL;
-    ModelTest *ret = NULL;
     int i, nt = pmod->ntests;
-    int done = 0;
+    int done = 0, add = 0;
+
+    if (test == NULL || test->teststat == GRETL_STAT_NONE) {
+	return 0;
+    }
 
     for (i=0; i<nt; i++) {
-	if (ttype == pmod->tests[i].type) {
-	    /* already done */
+	if (!model_tests_differ(test, &pmod->tests[i])) {
 	    done = 1;
 	}
-    }
+    } 
 
     if (!done) {
 	tests = realloc(pmod->tests, (nt + 1) * sizeof *tests);
@@ -1123,11 +1554,14 @@ ModelTest *new_test_on_model (MODEL *pmod, ModelTestType ttype)
     if (tests != NULL) {
 	pmod->tests = tests;
 	pmod->ntests += 1;
-	ret = &pmod->tests[nt];
-	gretl_test_init(ret, ttype);
+	copy_test(&pmod->tests[nt], test);
+	add = 1;
     }
 
-    return ret;
+    free(test->param);
+    free(test);
+
+    return add;
 }
 
 void model_test_set_teststat (ModelTest *test, unsigned char ts)
@@ -1184,7 +1618,9 @@ static int gretl_test_print_string (const ModelTest *test, PRN *prn)
 	N_("Test for omission of variables"),
 	N_("RESET test for specification"),
 	N_("Non-linearity test (squares)"),
-	N_("White's test for heteroskedasticity")
+	N_("White's test for heteroskedasticity"),
+	N_("Sargan over-identification test"),
+	N_("Hausman test")
     };
     char ordstr[16];
     char *param = NULL;
@@ -1268,7 +1704,9 @@ static int gretl_test_print_h_0 (const ModelTest *test, PRN *prn)
 	N_("parameters are zero for the variables"),
 	N_("specification is adequate"),
 	N_("relationship is linear"),
-	N_("heteroskedasticity not present")
+	N_("heteroskedasticity not present"),
+	N_("all instruments are valid"),
+	N_("OLS estimates are consistent")
     };
 
     if (test->type >= GRETL_TEST_MAX) {
@@ -1580,52 +2018,60 @@ static char *copy_missmask (const MODEL *pmod)
     return mask;
 }
 
-/**
- * copy_model:
- * @targ: pointer to #MODEL to copy to.
- * @src: pointer to #MODEL to copy from.
- * @pdinfo: pointer to dataset information.
- *
- * Does a deep copy of @src to @targ.  That is, @targ ends up with
- * its own allocated copies of all the pointer members of @src.
- *
- * Returns: 0 on success, 1 on failure.
- */
-
-int copy_model (MODEL *targ, const MODEL *src, const DATAINFO *pdinfo)
+static int copy_model (MODEL *targ, const MODEL *src)
 {
-    int i = src->list[0] - 1;
-    int m = i * (i + 1) / 2;
+    int k = src->ncoeff;
+    int m = k * (k + 1) / 2;
 
     /* monolithic copy of structure */
     *targ = *src;
 
     /* now work on pointer members */
     gretl_model_init_pointers(targ);
-    if ((targ->coeff = copyvec(src->coeff, src->ncoeff)) == NULL) 
+
+    if (src->coeff != NULL &&
+	(targ->coeff = copyvec(src->coeff, src->ncoeff)) == NULL) {
 	return 1;
-    if ((targ->sderr = copyvec(src->sderr, src->ncoeff))  == NULL)   
+    }
+
+    if (src->sderr != NULL &&
+	(targ->sderr = copyvec(src->sderr, src->ncoeff)) == NULL) {  
 	return 1;
-    if ((targ->uhat = copyvec(src->uhat, pdinfo->n)) == NULL) 
+    }
+
+    if (src->uhat != NULL && 
+	(targ->uhat = copyvec(src->uhat, src->full_n)) == NULL) {
 	return 1;
-    if ((targ->yhat = copyvec(src->yhat, pdinfo->n)) == NULL) 
+    }
+
+    if (src->yhat != NULL && 
+	(targ->yhat = copyvec(src->yhat, src->full_n)) == NULL) {
 	return 1;
+    }
+
     if (src->submask != NULL && 
-	(targ->submask = copy_subsample_mask(src->submask, pdinfo->n)) == NULL) 
+	(targ->submask = copy_subsample_mask(src->submask)) == NULL) { 
 	return 1;
+    }
+
     if (src->missmask != NULL && 
-	(targ->missmask = copy_missmask(src)) == NULL) 
+	(targ->missmask = copy_missmask(src)) == NULL) { 
 	return 1;
+    }
+
+    if (src->xpx != NULL && 
+	(targ->xpx = copyvec(src->xpx, m)) == NULL) {
+	return 1;
+    }
 
-    if (src->xpx != NULL &&
-	(targ->xpx = copyvec(src->xpx, m)) == NULL) return 1;
     if (src->vcv != NULL && 
-	(targ->vcv = copyvec(src->vcv, m)) == NULL) return 1;
+	(targ->vcv = copyvec(src->vcv, m)) == NULL) {
+	return 1;
+    }
 
-    if (src->arinfo != NULL) {
-	targ->arinfo = copy_ar_info(src->arinfo);
-	if (targ->arinfo == NULL) 
-	    return 1; 
+    if (src->arinfo != NULL && 
+	(targ->arinfo = copy_ar_info(src->arinfo)) == NULL) {
+	return 1; 
     }
 
     if (src->ntests > 0 && src->tests != NULL) {
@@ -1649,14 +2095,47 @@ int copy_model (MODEL *targ, const MODEL *src, const DATAINFO *pdinfo)
 	}
     }
 
-    m = src->list[0];
-    targ->list = malloc((m + 1) * sizeof *targ->list);
-    if (targ->list == NULL) return 1;
-    for (i=0; i<=m; i++) targ->list[i] = src->list[i];    
+    if (src->list != NULL && 
+	(targ->list = gretl_list_copy(src->list)) == NULL) {
+	return 1;
+    } 
 
     return 0;
 }
 
+/**
+ * gretl_model_copy:
+ * @pmod: pointer to #MODEL to copy.
+ *
+ * Does a deep copy of @pmod: allocates a new #MODEL pointer
+ * which has its own allocated copies of all the pointer
+ * members of @pmod.  The only feature of @pmod that is
+ * not duplicated is the reference count, which is set
+ * to zero in the copy. 
+ *
+ * Returns: the copied model, or %NULL on failure.
+ */
+
+MODEL *gretl_model_copy (const MODEL *pmod)
+{
+    MODEL *new = malloc(sizeof *new);
+
+    if (new != NULL) {
+	int err = copy_model(new, pmod);
+
+	if (err) {
+	    clear_model(new);
+	    free(new);
+	    new = NULL;
+	} else {
+	    /* nota bene */
+	    new->refcount = 0;
+	}
+    }
+
+    return new;
+}
+
 /**
  * swap_models:
  * @targ: pointer to pointer to #MODEL.
@@ -1671,8 +2150,17 @@ int swap_models (MODEL **targ, MODEL **src)
 {
     MODEL *tmp = *targ;
 
+#if MDEBUG
+    fprintf(stderr, "swap_models: %p <-> %p\n", *targ, *src);
+#endif
+
     *targ = *src;
     *src = tmp;
+
+    /* handle the case where we have a model that is currently
+       defined as the "last model" (objstack.c) */
+    maybe_swap_into_last_model(*targ, tmp);
+
     return 0;
 }
 
@@ -1680,6 +2168,8 @@ int is_model_cmd (const char *s)
 {
     int ret = 0;
 
+    /* FIXME mle? */
+
     if (s != NULL && *s != '\0') {
 	if (!strncmp(s, "ols", 3)  ||
 	    !strncmp(s, "corc", 4) ||
@@ -1737,21 +2227,25 @@ int command_ok_for_model (int test_ci, int model_ci)
 {
     int ok = 1;
 
+    if (model_ci == MLE) {
+	return 0;
+    }
+
     switch (test_ci) {
     case ADD:
     case ADDTO:
-    case COEFFSUM:
-    case VIF:
-	if (model_ci == NLS || model_ci == TSLS ||
-	    model_ci == ARMA || model_ci == GARCH) ok = 0;
-	break;
-
     case OMIT:
     case OMITFROM:
 	if (model_ci == NLS || model_ci == ARMA || 
 	    model_ci == GARCH) ok = 0;
 	break;
 
+    case COEFFSUM:
+    case VIF:
+	if (model_ci == NLS || model_ci == TSLS ||
+	    model_ci == ARMA || model_ci == GARCH) ok = 0;
+	break;
+
     case EQNPRINT:
 	if (model_ci != OLS) ok = 0; /* FIXME: unduly restrictive? */
 	break;
@@ -1771,9 +2265,11 @@ int command_ok_for_model (int test_ci, int model_ci)
     case HAUSMAN:
 	if (model_ci != POOLED) ok = 0;
 	break;
+
     case RESTRICT:
 	if (model_ci == LAD || model_ci == NLS) ok = 0;
 	break;
+
     case TESTUHAT:
 	/* do we really need to exclude garch? */
 	if (model_ci == TOBIT || model_ci == GARCH) ok = 0;
@@ -1836,6 +2332,10 @@ int highest_numbered_var_in_model (const MODEL *pmod,
     int i, v, vmax = 0;
     int gotsep = 0;
 
+    if (pmod->ci == MLE) {
+	return 0;
+    }
+
     for (i=1; i<=pmod->list[0]; i++) {
 	v = pmod->list[i];
 	if (v == LISTSEP) {
@@ -1889,12 +2389,301 @@ int mle_aic_bic (MODEL *pmod, int addk)
 
 double coeff_pval (const MODEL *pmod, double x, int df)
 {
-    if (0 && ML_ESTIMATOR(pmod->ci)) {
+    if (pmod->ci == MLE) {
+	return normal_pvalue_2(x);
+    } else if (0 && ML_ESTIMATOR(pmod->ci)) {
+	/* not ready */
 	return normal_pvalue_2(x);
     } else {
 	return t_pvalue_2(x, df);
     }
 }
 
+/**
+ * gretl_model_add_arma_varnames:
+ * @pmod: pointer to target model.
+ * @pdinfo: dataset information.
+ * @yno: ID number of dependent variable.
+ * @p: non-seasonal AR order.
+ * @q: non-seasonal MA order.
+ * @P: seasonal AR order.
+ * @Q: seasonal MA order.
+ * @r: number of exogenous regressors (excluding the constant).
+ * 
+ * Composes a set of names to be given to the regressors in an
+ * ARMA model.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
 
+int gretl_model_add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
+				   int yno, int p, int q, int P, int Q, 
+				   int r)
+{
+    int ptot = p + P;
+    int qtot = q + Q;
+    int np = ptot + qtot + r + pmod->ifc + 1;
+    int xstart;
+    int i, j, s;
 
+    const char *depvar;
+    size_t n;
+
+    pmod->params = malloc(np * sizeof pmod->params);
+    if (pmod->params == NULL) {
+	pmod->errcode = E_ALLOC;
+	return 1;
+    }
+
+    pmod->nparams = np;
+
+    for (i=0; i<np; i++) {
+	pmod->params[i] = malloc(VNAMELEN);
+	if (pmod->params[i] == NULL) {
+	    for (j=0; j<i; j++) {
+		free(pmod->params[j]);
+	    }
+	    free(pmod->params);
+	    pmod->params = NULL;
+	    pmod->nparams = 0;
+	    pmod->errcode = E_ALLOC;
+	    return 1;
+	}
+    }
+
+    strcpy(pmod->params[0], pdinfo->varname[yno]);
+
+    if (pmod->ifc) {
+	strcpy(pmod->params[1], pdinfo->varname[0]);
+	j = 2;
+    } else {
+	j = 1;
+    }
+
+    depvar = pmod->params[0];
+    n = strlen(depvar);
+
+    for (i=0; i<ptot; i++) {
+	if (i < p) {
+	    s = i + 1;
+	} else {
+	    s = (i - p + 1) * pdinfo->pd;
+	}
+	if (n < VNAMELEN - 4) {
+	    sprintf(pmod->params[j++], "%s(-%d)", depvar, s);
+	} else {
+	    sprintf(pmod->params[j++], "y(-%d)", s);
+	}
+    }
+
+    for (i=0; i<qtot; i++) {
+	if (i < q) {
+	    s = i + 1;
+	} else {
+	    s = (i - q + 1) * pdinfo->pd;
+	}    
+	sprintf(pmod->params[j++], "e(-%d)", s);
+    }
+
+    xstart = (P || Q)? 8 : 5;
+
+    for (i=0; i<r; i++) {
+	strcpy(pmod->params[j++], pdinfo->varname[pmod->list[xstart+i]]); 
+    }   
+
+    return 0;
+}
+
+/* try to tell if an OLS model with two independent variables is
+   actually a quadratic model (x_2 = x_1^2). */
+
+static int model_is_quadratic (const MODEL *pmod, const double **Z,
+			       const DATAINFO *pdinfo)
+{
+    const double *x1, *x2;
+    int t, ret = 1;
+
+    x1 = Z[pmod->list[3]];
+    x2 = Z[pmod->list[4]];
+
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(x1[t]) && x2[t] != x1[t] * x1[t]) {
+	    ret = 0;
+	    break;
+	}
+    }
+
+    return ret;
+}
+
+/**
+ * gretl_model_get_fitted_formula:
+ * @pmod: pointer to target model.
+ * @xvar: ID number of variable that _may_ be "x" in the model.
+ * @Z: data array.
+ * @pdinfo: dataset information.
+ * 
+ * If @pmod is a simple linear, quadratic or logistic model, 
+ * and if @xvar is in fact the "x" variable from the model, 
+ * returns a string representing the formula for generating the 
+ * fitted values as a function of x.  This formula may be used 
+ * in the context of a fitted versus actual plot.
+ *
+ * Returns: formula for fitted values, or %NULL if this is
+ * not available.
+ */
+
+char *gretl_model_get_fitted_formula (const MODEL *pmod, int xvar, 
+				      const double **Z,
+				      const DATAINFO *pdinfo)
+{
+    const double **mZ;
+    const DATAINFO *mdinfo;
+    char *ret = NULL;
+
+    /* only OLS and logistic are handled */
+    if (xvar == 0 || (pmod->ci != OLS && pmod->ci != LOGISTIC)) {
+	return NULL;
+    }
+
+    if (pmod->dataset != NULL) {
+	mZ = (const double **) pmod->dataset->Z;
+	mdinfo = pmod->dataset->dinfo;
+    } else {
+	mZ = Z;
+	mdinfo = pdinfo;
+    }
+
+    if (pmod->ci == LOGISTIC) {
+	if (pmod->ifc && pmod->ncoeff == 2 && xvar == pmod->list[3]) {
+	    double lmax = gretl_model_get_double(pmod, "lmax");
+
+	    if (!na(lmax)) {
+		ret = g_strdup_printf("yformula: %g/(1.0+exp(-(%g%s%g*x)))",
+				      lmax, pmod->coeff[0], 
+				      (pmod->coeff[1] >= 0)? "+" : "",
+				      pmod->coeff[1]);
+	    }
+	}
+    } else if (!pmod->ifc && pmod->ncoeff == 1 && xvar == pmod->list[2]) {
+	ret = g_strdup_printf("yformula: %g*x", pmod->coeff[0]);
+    } else if (pmod->ifc && pmod->ncoeff == 2 && xvar == pmod->list[3]) {
+	ret = g_strdup_printf("yformula: %g%s%g*x", pmod->coeff[0], 
+			      (pmod->coeff[1] >= 0)? "+" : "",
+			      pmod->coeff[1]);
+    } else if (pmod->ifc && pmod->ncoeff == 3 && xvar == pmod->list[3]) {
+	if (model_is_quadratic(pmod, mZ, mdinfo)) {
+	    ret = g_strdup_printf("yformula: %g%s%g*x%s%g*x**2", pmod->coeff[0], 
+				  (pmod->coeff[1] >= 0)? "+" : "",
+				  pmod->coeff[1], 
+				  (pmod->coeff[2] >= 0)? "+" : "",
+				  pmod->coeff[2]);
+	}
+    }
+	
+    return ret;
+}
+
+void gretl_model_set_name (MODEL *pmod, const char *name)
+{
+    if (pmod->name != NULL) {
+	free(pmod->name);
+    }
+
+    pmod->name = gretl_strdup(name);
+}
+
+const char *gretl_model_get_name (const MODEL *pmod)
+{
+    if (pmod != NULL) {
+	return pmod->name;
+    }
+
+    return NULL;
+}
+
+int gretl_model_stat_index (const char *s)
+{
+    char test[8] = {0};
+
+    strncat(test, s, 7);
+    lower(test);
+
+    if (!strcmp(test, "$ess"))  
+	return M_ESS;
+    if (!strcmp(test, "$t")) 
+	return M_T;
+    if (!strcmp(test, "$rsq"))  
+	return M_RSQ;
+    if (!strcmp(test, "$sigma"))  
+	return M_SIGMA;
+    if (!strcmp(test, "$df"))   
+	return M_DF;
+    if (!strcmp(test, "$ncoeff"))   
+	return M_NCOEFF;
+    if (!strcmp(test, "$lnl"))   
+	return M_LNL;
+    if (!strcmp(test, "$aic"))   
+	return M_AIC;
+    if (!strcmp(test, "$bic"))   
+	return M_BIC;
+    if (!strcmp(test, "$nrsq") || 
+	!strcmp(test, "$trsq")) 
+	return M_TRSQ;
+
+    return 0;
+}
+
+double gretl_model_get_scalar (const MODEL *pmod, int idx, int *err)
+{
+    double x = NADBL;
+
+    if (pmod == NULL) {
+	*err = E_BADSTAT;
+	return x;
+    }
+
+    if (*err) return x;
+
+    switch (idx) {  
+    case M_ESS:
+	x = pmod->ess;
+	break;
+    case M_RSQ:
+	x = pmod->rsq;
+	break;
+    case M_LNL:
+	x = pmod->lnL;
+	break;
+    case M_AIC:
+	x = pmod->criterion[C_AIC];
+	break;
+    case M_BIC:
+	x = pmod->criterion[C_BIC];
+	break;
+    case M_SIGMA:
+	if (pmod->nwt) x = pmod->sigma_wt;
+	else x = pmod->sigma;
+	break;
+    case M_TRSQ:
+	if (!na(pmod->rsq)) {
+	    x = pmod->nobs * pmod->rsq;
+	}
+	break;
+    case M_DF:
+	x = (double) pmod->dfd;
+	break;
+    case M_NCOEFF:
+	x = (double) pmod->ncoeff; /* is this always available? */
+	break;
+    case M_T:
+	x = (double) pmod->nobs;
+	break;	
+    }
+
+    if (na(x)) {
+	*err = E_BADSTAT;
+    }
+
+    return x;
+}
