@@ -20,6 +20,19 @@
 #ifndef GRETL_MODEL_H
 #define GRETL_MODEL_H
 
+enum model_stat_index {
+    M_ESS = 1,   /* error sum of squares */
+    M_T,         /* observations used */
+    M_RSQ,       /* R-squared */
+    M_SIGMA,     /* standard error of residuals */
+    M_DF,        /* degrees of freedom */
+    M_NCOEFF,    /* total number of estimated coefficients */
+    M_LNL,       /* log-likelihood */
+    M_AIC,       /* Akaike info criterion */
+    M_BIC,       /* Bayesian info criterion */
+    M_TRSQ,      /* T * R-squared, last model */
+};
+
 typedef struct CoeffIntervals_ CoeffIntervals;
 
 struct CoeffIntervals_ {
@@ -31,18 +44,6 @@ struct CoeffIntervals_ {
     int ifc;
 };
 
-/**
- * free_model:
- * @p: pointer to #MODEL.
- *
- * Free allocated content of MODEL then the pointer itself.
- */
-
-#define free_model(p) if (p != NULL) { \
-                             clear_model(p); \
-                             free(p); \
-                          }
-
 #define AR_MODEL(c) (c == AR || \
                      c == ARMA || \
                      c == CORC || \
@@ -58,6 +59,7 @@ struct CoeffIntervals_ {
 #define ML_ESTIMATOR(c) (c == ARMA || \
                          c == GARCH || \
                          c == LOGIT || \
+                         c == MLE || \
                          c == POISSON || \
                          c == PROBIT || \
                          c == TOBIT)
@@ -100,13 +102,21 @@ typedef enum {
     GRETL_TEST_RESET,
     GRETL_TEST_SQUARES,
     GRETL_TEST_WHITES,
+    GRETL_TEST_SARGAN,
+    GRETL_TEST_TSLS_HAUSMAN,
     GRETL_TEST_MAX
 } ModelTestType;
 
 MODEL *gretl_model_new (void);
 
+MODEL *gretl_model_new_protected (void);
+
 void gretl_model_init (MODEL *pmod);
 
+MODEL **gretl_model_array_new (int n);
+
+void gretl_model_array_destroy (MODEL **models, int n);
+
 void gretl_model_smpl_init (MODEL *pmod, const DATAINFO *pdinfo);
 
 void impose_model_smpl (const MODEL *pmod, DATAINFO *pdinfo);
@@ -115,12 +125,18 @@ void gretl_model_set_auxiliary (MODEL *pmod, ModelAuxCode aux);
 
 void clear_model (MODEL *pmod);
 
+void gretl_model_free (MODEL *pmod);
+
+void gretl_model_free_on_exit (MODEL *pmod);
+
 int gretl_model_set_data_with_destructor (MODEL *pmod, const char *key, 
 					  void *ptr, size_t size, 
 					  void (*destructor) (void *));
 
 int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, size_t size);
 
+int gretl_model_set_list_as_data (MODEL *pmod, const char *key, int *list);
+
 int gretl_model_destroy_data_item (MODEL *pmod, const char *key);
 
 int gretl_model_detach_data_item (MODEL *pmod, const char *key);
@@ -152,17 +168,25 @@ int gretl_model_get_depvar (const MODEL *pmod);
 
 int *gretl_model_get_x_list (const MODEL *pmod);
 
-int gretl_arma_model_get_AR_order (const MODEL *pmod);
+int gretl_arma_model_get_nonseasonal_AR_order (const MODEL *pmod);
 
-int gretl_arma_model_get_MA_order (const MODEL *pmod);
+int gretl_arma_model_get_nonseasonal_MA_order (const MODEL *pmod);
+
+int gretl_arma_model_get_max_AR_lag (const MODEL *pmod);
+
+int gretl_arma_model_get_max_MA_lag (const MODEL *pmod);
+
+int gretl_arma_model_get_AR_MA_coeffs (const MODEL *pmod,
+				       double **arvec,
+				       double **mavec);
+
+const double *gretl_arma_model_get_x_coeffs (const MODEL *pmod);
 
 int gretl_model_new_vcv (MODEL *pmod, int *nelem);
 
 VMatrix *gretl_model_get_vcv (MODEL *pmod, const DATAINFO *pdinfo);
 
-void debug_print_model_info (const MODEL *pmod, const char *msg);
-
-int copy_model (MODEL *targ, const MODEL *src, const DATAINFO *pdinfo);
+MODEL *gretl_model_copy (const MODEL *pmod);
 
 int swap_models (MODEL **targ, MODEL **src);
 
@@ -182,7 +206,9 @@ void model_count_minus (void);
 
 void set_model_id (MODEL *pmod);
 
-ModelTest *new_test_on_model (MODEL *pmod, ModelTestType ttype);
+ModelTest *model_test_new (ModelTestType ttype);
+
+int maybe_add_test_to_model (MODEL *pmod, ModelTest *test);
 
 void model_test_set_teststat (ModelTest *test, unsigned char ts);
 void model_test_set_order (ModelTest *test, int order);
@@ -196,6 +222,8 @@ void model_test_set_allocated_param (ModelTest *test, char *s);
 void gretl_model_test_print (const MODEL *pmod, int i, PRN *prn);
 void gretl_model_print_last_test (const MODEL *pmod, PRN *prn);
 
+void gretl_model_destroy_tests (MODEL *pmod);
+
 void model_list_to_string (int *list, char *buf);
 
 int highest_numbered_var_in_model (const MODEL *pmod, 
@@ -205,4 +233,20 @@ int mle_aic_bic (MODEL *pmod, int addk);
 
 double coeff_pval (const MODEL *pmod, double x, int df);
 
+int gretl_model_add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
+				   int yno, int p, int q, int P, int Q, 
+				   int r);
+
+char *gretl_model_get_fitted_formula (const MODEL *pmod, int xvar,
+				      const double **Z,
+				      const DATAINFO *pdinfo);
+
+void gretl_model_set_name (MODEL *pmod, const char *name);
+
+const char *gretl_model_get_name (const MODEL *pmod);
+
+int gretl_model_stat_index (const char *s);
+
+double gretl_model_get_scalar (const MODEL *pmod, int idx, int *err);
+
 #endif /* GRETL_MODEL_H */
