@@ -52,8 +52,6 @@ enum {
 
 static void ensure_slash (char *str);
 
-/* .......................................................... */
-
 static int add_gdt_suffix (char *fname)
 {
     int added = 0;
@@ -66,8 +64,6 @@ static int add_gdt_suffix (char *fname)
     return added;
 }
 
-/* .......................................................... */
-
 FILE *gretl_fopen (const char *filename, const char *mode)
 {
 #ifdef USE_G_FOPEN
@@ -93,8 +89,6 @@ gzFile gretl_gzopen (const char *filename, const char *mode)
 #endif
 }
 
-/* .......................................................... */
-
 int gretl_path_prepend (char *file, const char *path)
 {
     char temp[MAXLEN];
@@ -289,8 +283,6 @@ static int find_in_subdir (const char *topdir, char *fname, int code)
 
 #endif /* win32 vs posix */
 
-/* .......................................................... */
-
 static char *search_dir (char *fname, const char *topdir, int code)
 {
     FILE *test;
@@ -320,8 +312,6 @@ static char *search_dir (char *fname, const char *topdir, int code)
     return NULL;
 }
 
-/* .......................................................... */
-
 static int path_is_absolute (const char *fname)
 {
     int ret = 0;
@@ -338,8 +328,6 @@ static int path_is_absolute (const char *fname)
     return ret;
 }
 
-/* .......................................................... */
-
 static void make_path_absolute (char *fname, const char *orig)
 {
     char thisdir[MAXLEN];
@@ -423,6 +411,25 @@ char *addpath (char *fname, PATHS *ppaths, int script)
 	return fname;
     }
 
+#ifdef WIN32
+    /* try looking on the desktop? */
+    if (1) {
+	char *dtdir = desktop_path();
+	char *ret = NULL;
+
+	fname = tmp;
+	strcpy(fname, orig);
+
+	if (dtdir != NULL) {
+	    ret = search_dir(fname, dtdir, CURRENT_DIR);
+	    free(dtdir);
+	}
+	if (ret != NULL) {
+	    return ret;
+	}
+    }	    
+#endif
+
     fname = tmp;
     strcpy(fname, orig);
 
@@ -532,8 +539,6 @@ int getopenfile (const char *line, char *fname, PATHS *ppaths,
     return 0;
 }
 
-/* .......................................................... */
-
 enum paths_status_flags {
     STRING_TABLE_WRITTEN = 1 << 0
 };
@@ -543,6 +548,8 @@ struct INTERNAL_PATHS {
     char gnuplot[MAXLEN];
     char plotfile[MAXLEN];
     char libpath[MAXLEN];
+    char x12a[MAXLEN];
+    char x12adir[MAXLEN];
     char pngfont[32];
     unsigned char status;
 };
@@ -574,17 +581,15 @@ static void set_gretl_libpath (const char *path)
 #endif /* WIN32 */
 }
 
-/* .......................................................... */
-
 static void copy_paths_to_internal (const PATHS *paths)
 {
     strcpy(gretl_paths.userdir, paths->userdir);
     strcpy(gretl_paths.gnuplot, paths->gnuplot);
+    strcpy(gretl_paths.x12a, paths->x12a);
+    strcpy(gretl_paths.x12adir, paths->x12adir);
     strcpy(gretl_paths.pngfont, paths->pngfont);
 }
 
-/* .......................................................... */
-
 const char *gretl_lib_path (void)
 {
     return gretl_paths.libpath;
@@ -621,6 +626,16 @@ char *set_gretl_plotfile (const char *fname)
     return gretl_paths.plotfile;
 }
 
+const char *gretl_x12_arima (void)
+{
+    return gretl_paths.x12a;
+}
+
+const char *gretl_x12_arima_dir (void)
+{
+    return gretl_paths.x12adir;
+}
+
 const char *gretl_png_font (void)
 {
     return gretl_paths.pngfont;
@@ -655,8 +670,6 @@ static void ensure_slash (char *str)
     }
 }
 
-/* .......................................................... */
-
 void show_paths (const PATHS *ppaths)
 {
     printf(_("gretl: using these basic search paths:\n"));
@@ -667,8 +680,6 @@ void show_paths (const PATHS *ppaths)
     printf("gnuplot: %s\n", ppaths->gnuplot);
 }
 
-/* .......................................................... */
-
 #ifdef WIN32
 
 int set_paths (PATHS *ppaths, gretlopt opt)
@@ -716,11 +727,13 @@ int set_paths (PATHS *ppaths, gretlopt opt)
 	gretl_set_gui_mode(1);
 	if (opt & OPT_N) {
 	    /* force english */
-	    sprintf(ppaths->helpfile, "%sgretl_hlp.txt", ppaths->gretldir);
-	    sprintf(ppaths->cmd_helpfile, "%sgretlcli_hlp.txt", ppaths->gretldir);
+	    sprintf(ppaths->helpfile, "%sgretlgui_hlp.txt", ppaths->gretldir);
+	    sprintf(ppaths->cmd_helpfile, "%sgretlcmd_hlp.txt", ppaths->gretldir);
+	    sprintf(ppaths->cli_helpfile, "%sgretlcli_hlp.txt", ppaths->gretldir);
 	} else {
-	    sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretl_hlp.txt"));
-	    sprintf(ppaths->cmd_helpfile, "%s%s", ppaths->gretldir, _("gretlcli_hlp.txt"));
+	    sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlgui_hlp.txt"));
+	    sprintf(ppaths->cmd_helpfile, "%s%s", ppaths->gretldir, _("gretlcmd_hlp.txt"));
+	    sprintf(ppaths->cli_helpfile, "%s%s", ppaths->gretldir, _("gretlcli_hlp.txt"));
 	}
     } else { 
 	sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlcli_hlp.txt"));
@@ -795,11 +808,21 @@ int set_paths (PATHS *ppaths, gretlopt opt)
 	gretl_set_gui_mode(1);
 	if (opt & OPT_N) {
 	    /* force english */
-	    sprintf(ppaths->helpfile, "%sgretl.hlp", ppaths->gretldir);
+	    sprintf(ppaths->helpfile, "%sgretlgui.hlp", ppaths->gretldir);
+	    sprintf(ppaths->cli_helpfile, "%sgretlcli.hlp", ppaths->gretldir);
+#ifdef USE_GTK2
+	    sprintf(ppaths->cmd_helpfile, "%sgretlcmd.hlp", ppaths->gretldir);
+#else
 	    sprintf(ppaths->cmd_helpfile, "%sgretlcli.hlp", ppaths->gretldir);
+#endif
 	} else {
-	    sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretl.hlp"));
+	    sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlgui.hlp"));
+	    sprintf(ppaths->cli_helpfile, "%s%s", ppaths->gretldir, _("gretlcli.hlp"));
+#ifdef USE_GTK2
+	    sprintf(ppaths->cmd_helpfile, "%s%s", ppaths->gretldir, _("gretlcmd.hlp"));
+#else
 	    sprintf(ppaths->cmd_helpfile, "%s%s", ppaths->gretldir, _("gretlcli.hlp"));
+#endif
 	}
     } else {
 	sprintf(ppaths->helpfile, "%s%s", ppaths->gretldir, _("gretlcli.hlp"));
