@@ -36,7 +36,8 @@ typedef enum {
     GRETL_FORMAT_RTF_TXT   = 1 << 4,
     GRETL_FORMAT_EQN       = 1 << 5,
     GRETL_FORMAT_SELECTION = 1 << 6,
-    GRETL_FORMAT_CSV       = 1 << 7
+    GRETL_FORMAT_CSV       = 1 << 7,
+    GRETL_FORMAT_TABLE     = 1 << 8,
 } PrnFormat;
 
 /* functions follow */
@@ -55,12 +56,16 @@ const char *gretl_print_get_buffer (PRN *prn);
 
 void gretl_print_set_format (PRN *prn, PrnFormat format);
 
+void gretl_print_toggle_doc_flag (PRN *prn);
+
 int pprintf (PRN *prn, const char *template, ...);
 
 int pputs (PRN *prn, const char *s);
 
 int pputc (PRN *prn, int c);
 
+void gretl_prn_newline (PRN *prn);
+
 void gretl_print_flush_stream (PRN *prn);
 
 int printing_to_standard_stream (PRN *prn);
@@ -75,10 +80,14 @@ int plain_format (PRN *prn);
 
 int rtf_format (PRN *prn);
 
+int rtf_doc_format (PRN *prn);
+
 int tex_format (PRN *prn);
 
 int tex_doc_format (PRN *prn);
 
 int tex_eqn_format (PRN *prn);
 
+int csv_format (PRN *prn);
+
 #endif /* GRETL_PRN_H */
