@@ -19,6 +19,8 @@
 
 #include "libgretl.h"
 #include "system.h"
+#include "var.h"
+#include "objstack.h"
 #include "gretl_restrict.h"
 
 #define RDEBUG 0
@@ -34,16 +36,16 @@ struct restriction_ {
 };
 
 struct restriction_set_ {
-    int cross;                    /* 1 for cross-equation set, else 0 */
     int k;                        /* number of restrictions (rows) */
+    int cross;                    /* includes cross-equation restrictions? */
     char *mask;                   /* selection mask for coeffs */
     restriction **restrictions;
     MODEL *pmod;
     gretl_equation_system *sys;
+    GRETL_VAR *var;
     const DATAINFO *pdinfo;
 };
 
-
 static void destroy_restriction_set (gretl_restriction_set *rset);
 
 static int check_R_matrix (const gretl_matrix *R)
@@ -67,7 +69,24 @@ static int check_R_matrix (const gretl_matrix *R)
 }
 
 static int 
-get_R_column (const gretl_restriction_set *rset, int i, int j)
+get_R_vecm_column (const gretl_restriction_set *rset, int i, int j)
+{
+    const restriction *r = rset->restrictions[i];
+    int nb = gretl_VECM_n_beta(rset->var);
+    int col = r->coeff[j];
+    int k;
+
+    if (r->eq != NULL) {
+	for (k=0; k<r->eq[j]; k++) {
+	    col += nb;
+	}
+    }
+
+    return col;
+}
+
+static int 
+get_R_sys_column (const gretl_restriction_set *rset, int i, int j)
 {
     const restriction *r = rset->restrictions[i];
     const int *list;
@@ -82,6 +101,17 @@ get_R_column (const gretl_restriction_set *rset, int i, int j)
     return col;
 }
 
+static int get_R_column (const gretl_restriction_set *rset, int i, int j)
+{
+    if (rset->sys != NULL) {
+	return get_R_sys_column(rset, i, j);
+    } else if (rset->var != NULL) {
+	return get_R_vecm_column(rset, i, j);
+    } else {
+	return 0;
+    }
+}
+
 static double get_restriction_param (const restriction *r, int k)
 {
     double x = 0.0;
@@ -99,13 +129,13 @@ static double get_restriction_param (const restriction *r, int k)
 
 static int R_n_columns (gretl_restriction_set *rset)
 {
-    int cols = 0;
+    int i, cols = 0;
 
-    if (rset->cross) {
+    if (rset->var) {
+	cols = gretl_VECM_n_beta(rset->var);
+    } else if (rset->sys) {
 	cols = system_n_indep_vars(rset->sys);
     } else {
-	int i;
-
 	for (i=0; i<rset->pmod->ncoeff; i++) {
 	    if (rset->mask[i]) {
 		cols++;
@@ -143,13 +173,7 @@ restriction_set_form_matrices (gretl_restriction_set *rset,
 
     for (i=0; i<rset->k; i++) { 
 	r = rset->restrictions[i];
-	if (rset->cross) {
-	    for (j=0; j<r->nterms; j++) {
-		col = get_R_column(rset, i, j);
-		x = r->mult[j];
-		gretl_matrix_set(R, i, col, x);
-	    }
-	} else {
+	if (rset->pmod != NULL) {
 	    col = 0;
 	    for (j=0; j<rset->pmod->ncoeff; j++) {
 		if (rset->mask[j]) {
@@ -157,18 +181,34 @@ restriction_set_form_matrices (gretl_restriction_set *rset,
 		    gretl_matrix_set(R, i, col++, x);
 		}
 	    }
-	}
+	} else {	    
+	    for (j=0; j<r->nterms; j++) {
+		col = get_R_column(rset, i, j);
+		x = r->mult[j];
+		gretl_matrix_set(R, i, col, x);
+	    }
+	} 
 	gretl_vector_set(q, i, r->rhs);
     }
 
-
 #if RDEBUG
-    gretl_matrix_print(R, "R", NULL);
-    gretl_matrix_print(q, "q", NULL);
+    gretl_matrix_print(R, "R");
+    gretl_matrix_print(q, "q");
 #endif
 
-    *Rin = R;
-    *qin = q;
+    if (rset->var != NULL) {
+	gretl_matrix *D = gretl_matrix_right_nullspace(R);
+
+#if RDEBUG
+	gretl_matrix_print(D, "D");
+#endif
+	gretl_VAR_attach_restrictions(rset->var, D);
+	gretl_matrix_free(R);
+	gretl_matrix_free(q);
+    } else {
+	*Rin = R;
+	*qin = q;
+    }
 
     return 0;
 }
@@ -242,6 +282,7 @@ static int parse_b_bit (const char *s, int *eq, int *bnum)
 static int 
 parse_coeff_chunk (const char *s, double *x, int *eq, int *bnum)
 {
+    const char *s0 = s;
     int err = E_PARSE;
 
     *eq = 0;
@@ -263,7 +304,7 @@ parse_coeff_chunk (const char *s, double *x, int *eq, int *bnum)
     }
 
     if (err) {
-	sprintf(gretl_errmsg, _("parse error in '%s'\n"), s);
+	sprintf(gretl_errmsg, _("parse error in '%s'\n"), s0);
     } 
 
 #if RDEBUG
@@ -332,8 +373,8 @@ static restriction *restriction_new (int n, int cross)
 static restriction *
 augment_restriction_set (gretl_restriction_set *rset, int n_terms)
 {
-    int n = rset->k;
     restriction **rlist = NULL;
+    int n = rset->k;
 
     rlist = realloc(rset->restrictions, (n + 1) * sizeof *rlist);
     if (rlist == NULL) return NULL;
@@ -352,23 +393,21 @@ static void print_mult (double mult, int first, PRN *prn)
 {
     if (mult == 1.0) {
 	if (!first) pputs(prn, " + ");
-    }
-    else if (mult == -1.0) {
+    } else if (mult == -1.0) {
 	if (first) pputs(prn, "-");
 	else pputs(prn, " - ");
-    }
-    else if (mult > 0.0) {
+    } else if (mult > 0.0) {
 	if (first) pprintf(prn, "%g*", mult);
 	else pprintf(prn, " + %g*", mult);
-    }
-    else if (mult < 0.0) {
+    } else if (mult < 0.0) {
 	if (first) pprintf(prn, "%g*", mult);
 	else pprintf(prn, " - %g*", fabs(mult));
     }	
 }
 
 static void print_restriction (const gretl_restriction_set *rset,
-			       int j, PRN *prn)
+			       int j, const DATAINFO *pdinfo, 
+			       PRN *prn)
 {
     const restriction *r = rset->restrictions[j];
     char vname[24];
@@ -378,8 +417,17 @@ static void print_restriction (const gretl_restriction_set *rset,
 	print_mult(r->mult[i], i == 0, prn);
 	if (rset->cross) {
 	    pprintf(prn, "b[%d,%d]", r->eq[i] + 1, r->coeff[i]);
+	} else if (rset->var) {
+	    const int *list = gretl_VECM_list(rset->var);
+	    int li = (list == NULL)? 0 : r->coeff[i] + 1;
+
+	    if (li > 0 && li <= list[0]) {
+		pprintf(prn, "b[%s]", pdinfo->varname[list[li]]);
+	    } else {
+		pprintf(prn, "b[%d]", r->coeff[i]);
+	    }
 	} else {
-	    gretl_model_get_param_name(rset->pmod, rset->pdinfo, 
+	    gretl_model_get_param_name(rset->pmod, pdinfo, 
 				       r->coeff[i], vname);
 	    pprintf(prn, "b[%s]", vname);
 	}
@@ -388,7 +436,8 @@ static void print_restriction (const gretl_restriction_set *rset,
 }
 
 static void 
-print_restriction_set (const gretl_restriction_set *rset, PRN *prn)
+print_restriction_set (const gretl_restriction_set *rset, 
+		       const DATAINFO *pdinfo, PRN *prn)
 {
     int i;
 
@@ -405,7 +454,7 @@ print_restriction_set (const gretl_restriction_set *rset, PRN *prn)
 	} else {
 	    pputc(prn, ' ');
 	}
-	print_restriction(rset, i, prn);
+	print_restriction(rset, i, pdinfo, prn);
     }
 
 #if RDEBUG
@@ -434,7 +483,8 @@ add_term_to_restriction (restriction *r, double mult, int eq, int bnum, int i)
 
 static gretl_restriction_set *
 real_restriction_set_start (MODEL *pmod, const DATAINFO *pdinfo,
-			    gretl_equation_system *sys)
+			    gretl_equation_system *sys,
+			    GRETL_VAR *var)
 {
     gretl_restriction_set *rset;
 
@@ -444,17 +494,20 @@ real_restriction_set_start (MODEL *pmod, const DATAINFO *pdinfo,
     rset->pmod = pmod;
     rset->pdinfo = pdinfo;
     rset->sys = sys;
+    rset->var = var;
 
     rset->k = 0;
     rset->mask = NULL;
     rset->restrictions = NULL;
 
-    if (sys != NULL) {
+    if (rset->sys != NULL) {
+	rset->cross = 1;
+    } else if (rset->var != NULL && gretl_VECM_rank(rset->var) > 1) {
 	rset->cross = 1;
     } else {
 	rset->cross = 0;
     }
-    
+
     return rset;
 }
 
@@ -467,7 +520,17 @@ static int bnum_out_of_bounds (const gretl_restriction_set *rset,
 {
     int ret = 1;
 
-    if (rset->cross) {
+    if (rset->var) {
+	if (eq >= gretl_VECM_rank(rset->var)) {
+	    sprintf(gretl_errmsg, _("Equation number (%d) is out of range"), 
+		    eq + 1);
+	} else if (bnum >= gretl_VECM_n_beta(rset->var)) {
+	    sprintf(gretl_errmsg, _("Coefficient number (%d) is out of range"), 
+		    bnum);
+	} else {
+	    ret = 0;
+	}
+    } else if (rset->sys) {
 	const int *list = system_get_list(rset->sys, eq);
 
 	if (list == NULL) {
@@ -578,7 +641,13 @@ real_restriction_set_parse_line (gretl_restriction_set *rset,
     }
 
     if (!err) {
-	if (!sscanf(p, " = %lf", &r->rhs)) err = E_PARSE;
+	if (!sscanf(p, " = %lf", &r->rhs)) {
+	    err = E_PARSE;
+	} else if (rset->var != NULL && r->rhs != 0.0) {
+	    strcpy(gretl_errmsg, "VECM restrictions: the equations must "
+		   "be homogeneous");
+	    err = 1;
+	}
     }
 
     if (err) {
@@ -597,7 +666,9 @@ restriction_set_parse_line (gretl_restriction_set *rset, const char *line)
 	nx = rset->pmod->ncoeff;
     } else if (rset->sys != NULL) {
 	nx = system_n_indep_vars(rset->sys);
-    } 
+    } else if (rset->var != NULL) {
+	nx = gretl_VECM_n_beta(rset->var);
+    }
 
     if (rset->k >= nx) {
 	sprintf(gretl_errmsg, _("Too many restrictions (maximum is %d)"), 
@@ -609,15 +680,41 @@ restriction_set_parse_line (gretl_restriction_set *rset, const char *line)
     return real_restriction_set_parse_line(rset, line, 0);
 }
 
+/* special set-up for a set of restrictions for a VAR (vecm,
+   actually) */
+
+gretl_restriction_set *
+var_restriction_set_start (const char *line, GRETL_VAR *var)
+{
+    gretl_restriction_set *rset;
+
+    rset = real_restriction_set_start(NULL, NULL, NULL, var);
+    if (rset == NULL) {
+	strcpy(gretl_errmsg, _("Out of memory!"));
+	return NULL;
+    }
+
+    *gretl_errmsg = '\0';
+
+    if (real_restriction_set_parse_line(rset, line, 1)) {
+	if (*gretl_errmsg == '\0') {
+	    sprintf(gretl_errmsg, _("parse error in '%s'\n"), line);
+	}
+	return NULL;
+    }
+
+    return rset;
+}
+
 /* special set-up for a set of cross-equation restrictions, for
-   a system of equations */
+   a system of simultaneous equations */
 
 gretl_restriction_set *
 cross_restriction_set_start (const char *line, gretl_equation_system *sys)
 {
     gretl_restriction_set *rset;
 
-    rset = real_restriction_set_start(NULL, NULL, sys);
+    rset = real_restriction_set_start(NULL, NULL, sys, NULL);
     if (rset == NULL) {
 	strcpy(gretl_errmsg, _("Out of memory!"));
 	return NULL;
@@ -635,32 +732,44 @@ gretl_restriction_set *
 restriction_set_start (const char *line, MODEL *pmod, const DATAINFO *pdinfo)
 {
     gretl_restriction_set *rset = NULL;
-    char *sysname = NULL;
+    char *sname = NULL;
 
 #if RDEBUG
     fprintf(stderr, "restriction_set_start: line='%s'\n", line);
 #endif
 
     if (!strncmp(line, "restrict", 8)) {
-	sysname = get_system_name_from_line(line + 8);
+	sname = get_system_name_from_line(line);
     }
 
     /* are we applying a restriction to a named system of equations? */
-    if (sysname != NULL) {
-	gretl_equation_system *sys;
+    if (sname != NULL) {
+	gretl_equation_system *sys = NULL;
+	GRETL_VAR *var = NULL;
 
-	sys = get_equation_system_by_name(sysname, NULL);
-	if (sys == NULL) {
-	    sprintf(gretl_errmsg, "'%s': unrecognized name", sysname);
-	} else {
-	    rset = real_restriction_set_start(NULL, NULL, sys);
+	sys = get_equation_system_by_name(sname);
+	if (sys != NULL) {
+	    rset = real_restriction_set_start(NULL, NULL, sys, NULL);
 	    if (rset == NULL) {
 		strcpy(gretl_errmsg, _("Out of memory!"));
 	    }	    
+	} else {
+	    var = get_VECM_by_name(sname);
+	    if (var != NULL) {
+		rset = real_restriction_set_start(NULL, NULL, NULL, var);
+		if (rset == NULL) {
+		    strcpy(gretl_errmsg, _("Out of memory!"));
+		}
+	    }
 	}
-	free(sysname);
+
+	if (sys == NULL && var == NULL) {
+	    sprintf(gretl_errmsg, "'%s': unrecognized name", sname);
+	}
+
+	free(sname);
     } else {
-	rset = real_restriction_set_start(pmod, pdinfo, NULL);
+	rset = real_restriction_set_start(pmod, pdinfo, NULL, NULL);
 
 	if (rset == NULL) {
 	    strcpy(gretl_errmsg, _("Out of memory!"));
@@ -694,8 +803,8 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
     }
 
 #if RDEBUG
-    gretl_matrix_print(R, "R matrix", prn);
-    gretl_matrix_print(q, "q vector", prn);
+    gretl_matrix_print(R, "R matrix");
+    gretl_matrix_print(q, "q vector");
 #endif
 
     if ((err = check_R_matrix(R))) {
@@ -706,7 +815,7 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 	    err = E_ALLOC;
 	}
 	goto bailout;
-    }	
+    }
 
     b = gretl_coeff_vector_from_model(rset->pmod, rset->mask);
     vcv = gretl_vcv_matrix_from_model(rset->pmod, rset->mask);
@@ -722,8 +831,8 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
     }
 
 #if RDEBUG
-    gretl_matrix_print(vcv, "VCV matrix", prn);
-    gretl_matrix_print(b, "coeff vector", prn);
+    gretl_matrix_print(vcv, "VCV matrix");
+    gretl_matrix_print(b, "coeff vector");
 #endif  
 
     err = gretl_matrix_multiply(R, b, br);
@@ -733,7 +842,7 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
     }
 
 #if RDEBUG
-    gretl_matrix_print(br, "br", prn);
+    gretl_matrix_print(br, "br");
 #endif  
 
     if (!gretl_is_zero_vector(q)) {
@@ -751,10 +860,10 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 	Rv = vcv;
 	freeRv = 0;
     } else {
-	Rv = gretl_matrix_A_X_A_prime(R, vcv, &err);
+	Rv = gretl_matrix_A_X_A(R, GRETL_MOD_NONE, vcv, &err);
 	if (err) goto bailout;
 #if RDEBUG
-	gretl_matrix_print(Rv, "Rv", prn);
+	gretl_matrix_print(Rv, "Rv");
 #endif  
     }
 
@@ -765,7 +874,7 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 	goto bailout;
     }
     
-    F = gretl_scalar_b_prime_X_b(br, Rv, &err);
+    F = gretl_scalar_b_X_b(br, GRETL_MOD_TRANSPOSE, Rv, &err);
     if (err) {
 	pputs(prn, _("Failed to compute F statistic for test\n"));
 	goto bailout;
@@ -804,24 +913,30 @@ static int test_restriction_set (gretl_restriction_set *rset, PRN *prn)
 */
 
 int
-gretl_restriction_set_finalize (gretl_restriction_set *rset, PRN *prn)
+gretl_restriction_set_finalize (gretl_restriction_set *rset, 
+				const DATAINFO *pdinfo,
+				PRN *prn)
 {
     int err = 0;
 
-    if (rset->sys == NULL) {
-	/* single model */
-	err = restriction_set_make_mask(rset);
+    if (rset == NULL) {
+	return 1;
+    }
+
+    if (rset->var != NULL) {
+	/* vecm */
+	err = restriction_set_form_matrices(rset, NULL, NULL);
 	if (!err) {
-	    print_restriction_set(rset, prn);
-	    test_restriction_set(rset, prn);
-	    destroy_restriction_set(rset);
+	    print_restriction_set(rset, pdinfo, prn);
+	    gretl_VECM_test_beta(rset->var, prn);
 	}
-    } else {
+    } else if (rset->sys != NULL) {
+	/* simultaneous equations system */
 	gretl_matrix *R;
 	gretl_matrix *q;
 
 #if RDEBUG
-	print_restriction_set(rset, prn);
+	print_restriction_set(rset, pdinfo, prn);
 #endif
 	err = restriction_set_form_matrices(rset, &R, &q);
 	if (!err) {
@@ -837,7 +952,15 @@ gretl_restriction_set_finalize (gretl_restriction_set *rset, PRN *prn)
 	    system_set_restriction_matrices(rset->sys, R, q);
 	    destroy_restriction_set(rset);
 	}
-    }
+    } else {
+	/* single model */
+	err = restriction_set_make_mask(rset);
+	if (!err) {
+	    print_restriction_set(rset, pdinfo, prn);
+	    test_restriction_set(rset, prn);
+	    destroy_restriction_set(rset);
+	}
+    }	
 
     return err;
 }
