@@ -19,7 +19,7 @@
 
 #include "libgretl.h"
 #include "gretl_func.h"
-#include "system.h"
+#include "objstack.h"
 #include "cmd_private.h"
 #include "libset.h"
 
@@ -276,53 +276,65 @@ void printlist (const int *list, const char *msg)
 
 /* Compute model selection criteria */
 
-int gretl_calculate_criteria (double *x, double ess, int nobs, int ncoeff)
+int gretl_calculate_criteria (double ess, int nobs, int ncoeff,
+			      double *ll, double *aic, double *bic)
 {
-    if (na(ess) || ess <= 0.0 || ncoeff < 1 || nobs <= ncoeff) {
-	x[C_AIC] = NADBL;
-	x[C_BIC] = NADBL;
+    int err = 0;
 
-	return 1;
+    if (na(ess) || ess <= 0.0 || ncoeff < 1 || nobs <= ncoeff) {
+	*ll = NADBL;
+	*aic = NADBL;
+	*bic = NADBL;
+	err = 1;
     } else {
 	const double ln2pi1 = 2.837877066409345;
-	double ll;
 
 	errno = 0;
-	ll = -.5 * nobs * log(ess);
+
+	*ll = -.5 * nobs * log(ess);
 
 	if (errno == EDOM || errno == ERANGE) {
-	    x[C_AIC] = NADBL;
-	    x[C_BIC] = NADBL;
+	    *ll = NADBL;
+	    *aic = NADBL;
+	    *bic = NADBL;
 	} else {
-	    ll += -.5 * nobs * (ln2pi1 - log((double) nobs));
-	    x[C_AIC] = -2.0 * ll + 2 * ncoeff;
-	    x[C_BIC] = -2.0 * ll + ncoeff * log(nobs);
+	    *ll += -.5 * nobs * (ln2pi1 - log((double) nobs));
+	    *aic = -2.0 * *ll + 2 * ncoeff;
+	    *bic = -2.0 * *ll + ncoeff * log(nobs);
 	}
-
-	return 0;
     }
+
+    return err;
 }
 
 int ls_aic_bic (MODEL *pmod)
 {
-    return gretl_calculate_criteria(pmod->criterion, 
-				    pmod->ess, pmod->nobs,
-				    pmod->ncoeff);
+    double ll, aic, bic;
+    int err;
+
+    err = gretl_calculate_criteria(pmod->ess, pmod->nobs, pmod->ncoeff,
+				   &ll, &aic, &bic);
+
+    pmod->lnL = ll;
+    pmod->criterion[C_AIC] = aic;
+    pmod->criterion[C_BIC] = bic;
+
+    return err;
 }
 
 int gretl_print_criteria (double ess, int nobs, int ncoeff, PRN *prn)
 {
-    double x[2];
+    double ll, aic, bic;
     int err;
 
-    err = gretl_calculate_criteria(x, ess, nobs, ncoeff);
+    err = gretl_calculate_criteria(ess, nobs, ncoeff, &ll, &aic, &bic);
 
     if (err) {
 	pputs(prn, _("Error calculating model selection criteria\n"));
     } else {
 	pprintf(prn, _("Using ess = %g, %d observations, %d coefficients\n"), 
 		ess, nobs, ncoeff);
-	pprintf(prn, "\nAIC = %g\nBIC = %g\n\n", x[C_AIC], x[C_BIC]);
+	pprintf(prn, "\nAIC = %g\nBIC = %g\n\n", aic, bic);
     }
 
     return err;
@@ -679,7 +691,7 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
 	break;
     case LOGIT:
     case PROBIT:
-	*tmpmod = logit_probit(cmd.list, pZ, pdinfo, cmd.ci);
+	*tmpmod = logit_probit(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt);
 	break;
     case TOBIT:
 	*tmpmod = tobit_model(cmd.list, pZ, pdinfo, NULL);
@@ -808,8 +820,6 @@ int balanced_panel (const DATAINFO *pdinfo)
     return 1;
 }
 
-/* ........................................................... */
-
 double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo)
 {
     if (pdinfo->vector[i]) {
@@ -819,8 +829,6 @@ double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo)
     }	
 }
 
-/* ........................................................... */
-
 static void free_mp_varnames (mp_results *mpvals)
 {
     int i, n = mpvals->ncoeff + 1;
@@ -1042,10 +1050,11 @@ void libgretl_cleanup (void)
 
     gretl_rand_free();
     gretl_functions_cleanup();
-    gretl_equation_systems_cleanup();
+    gretl_saved_objects_cleanup();
     gretl_transforms_cleanup();
     libset_cleanup();
     gretl_lists_cleanup();
+    gretl_command_hash_cleanup();
 
     p = strstr(gretl_plotfile(), "gpttmp");
     if (p != NULL) {
@@ -1071,21 +1080,20 @@ record_or_get_test_result (double teststat, double pval, char *blurb,
 {
     static double val = NADBL;
     static double pv = NADBL;
-    static char info[128] = {0};
+    static char info[MAXLABEL] = {0};
 
     double ret = NADBL;
 
     if (code == SET_TEST_STAT) {
 	val = teststat;
 	pv = pval;
+	*info = '\0';
 	if (blurb != NULL) {
-	    strcpy(info, blurb);
-	} else {
-	    *info = '\0';
-	}
+	    strncat(info, blurb, MAXLABEL - 1);
+	} 
     } else if (code == GET_TEST_STAT || code == GET_TEST_PVAL) {
 	if (blurb != NULL) {
-	    strcpy(blurb, info);
+	    strncat(blurb, info, MAXLABEL - 1);
 	}
 	ret = (code == GET_TEST_STAT)? val : pv;
     } 
