@@ -51,8 +51,6 @@ typedef struct {
 
 static void get_optional_filename (const char *line, CMD *cmd);
 
-/* ........................................................... */
-
 static int trydatafile (char *line, CMD *cmd)
 {
     int i, m, n = strlen(line);
@@ -214,6 +212,7 @@ static int catch_command_alias (CMD *cmd)
                        c == LEVERAGE || \
                        c == LMTEST || \
                        c == LOOP || \
+                       c == MLE || \
                        c == MODELTAB || \
                        c == NLS || \
                        c == NULLDATA || \
@@ -234,6 +233,7 @@ static int catch_command_alias (CMD *cmd)
                        c == SYSTEM || \
                        c == TABPRINT || \
                        c == TESTUHAT || \
+                       c == TRANSPOSE || \
                        c == VARLIST || \
                        c == VIF)
 
@@ -251,7 +251,8 @@ static int catch_command_alias (CMD *cmd)
                             c == COINT || \
                             c == COINT2 || \
                             c == KPSS || \
-                            c == VAR)
+                            c == VAR || \
+                            c == VECM)
 
 #define DEFAULTS_TO_FULL_LIST(c) (c == CORR || \
                                   c == DIFF || \
@@ -260,6 +261,7 @@ static int catch_command_alias (CMD *cmd)
                                   c == LOGS || \
                                   c == PCA || \
                                   c == PRINT || \
+                                  c == SDIFF || \
                                   c == SMPL || \
                                   c == SQUARE || \
                                   c == STORE || \
@@ -269,11 +271,6 @@ static int catch_command_alias (CMD *cmd)
                                c == PRINT || \
                                c == STORE)
 
-#define HIDDEN_COMMAND(c) (c == FUNCERR || \
-                           c == REMEMBER)
-
-/* ........................................................... */
-
 static int flow_control (const char *line, double ***pZ, 
 			 DATAINFO *pdinfo, CMD *cmd)
 {
@@ -597,6 +594,7 @@ int auto_transform_ok (const char *s, int *lnum,
 	"logs",
 	"diff",
 	"ldiff",
+	"sdiff",
 	"square",
 	NULL
     };
@@ -643,10 +641,12 @@ int auto_transform_ok (const char *s, int *lnum,
     if (trans == 0) {
 	err = list_loggenr(genlist, pZ, pdinfo);
     } else if (trans == 1) {
-	err = list_diffgenr(genlist, pZ, pdinfo);
+	err = list_diffgenr(genlist, DIFF, pZ, pdinfo);
     } else if (trans == 2) {
-	err = list_ldiffgenr(genlist, pZ, pdinfo);
+	err = list_diffgenr(genlist, LDIFF, pZ, pdinfo);
     } else if (trans == 3) {
+	err = list_diffgenr(genlist, SDIFF, pZ, pdinfo);
+    } else if (trans == 4) {
 	err = list_xpxgenr(&genlist, pZ, pdinfo, opt);
     }
 
@@ -916,6 +916,7 @@ static int fix_semicolon_after_var (char *s)
 /* apparatus for checking that the "end" command is valid */
 
 #define COMMAND_CAN_END(c) (c == FUNC || \
+                            c == MLE || \
                             c == NLS || \
 			    c == RESTRICT || \
 			    c == SYSTEM)
@@ -1100,6 +1101,42 @@ static int maybe_print_object (const char *line, CMD *cmd)
     return ret;
 }
 
+static int trap_comments (char *s, CMD *cmd)
+{
+    int ret = 0;
+
+    if (*s == '#' || (*s == '/' && *(s+1) == '/')) {	
+	cmd->nolist = 1;
+	cmd->ci = CMD_COMMENT;
+	ret = 1;
+    } else if (strstr(s, " #") || strstr(s, "//")) {
+	int quoted = 0;
+	int braced = 0;
+
+	while (*s) {
+	    if (*s == '"') {
+		quoted = !quoted;
+	    } else if (!quoted) {
+		if (*s == '{') {
+		    braced++;
+		} else if (*s == '}') {
+		    braced--;
+		}
+	    }
+	    if (!quoted && !braced) {
+		if ((*s == ' ' && *(s+1) == '#') ||
+		    (*s == '/' && *(s+1) == '/')) {
+		    *s = '\0';
+		    break;
+		}
+	    }
+	    s++;
+	}
+    }
+
+    return ret;
+}
+
 /**
  * parse_command_line:
  * @line: the command line.
@@ -1125,11 +1162,10 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 
     *gretl_errmsg = '\0';
 
-    /* extract any options first */
-    cmd->opt = get_gretl_options(line, &cmd->errcode);
-    if (cmd->errcode) {
-	return cmd->errcode;
-    }
+    compress_spaces(line);
+#if 0
+    fprintf(stderr, "parsing '%s'\n", line);
+#endif
 
     /* look for ramu practice files */
     if (line[0] == '(' && line[1] == '*') {
@@ -1137,7 +1173,7 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 	gotdata = trydatafile(line, cmd);
     }
 
-    /* trap comments */
+    /* trap old-style comments */
     if (!gotdata) {
 	if (filter_comments(line, cmd)) {
 	    cmd->nolist = 1;
@@ -1147,9 +1183,13 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
     }
 
     /* also new-style comments */
-    if (*line == '#') {
-	cmd->nolist = 1;
-	cmd->ci = CMD_COMMENT;
+    if (trap_comments(line, cmd)) {
+	return cmd->errcode;
+    } 
+
+    /* extract any options */
+    cmd->opt = get_gretl_options(line, &cmd->errcode);
+    if (cmd->errcode) {
 	return cmd->errcode;
     }    
 
@@ -1372,17 +1412,17 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 	nf--;
 	pos = 0;
 	linelen = strlen(line);
-    } else if (cmd->ci == MULTIPLY) { 
-	char suffix[4];
+    } else if (cmd->ci == MULTIPLY || cmd->ci == VECM) { 
+	char extra[4];
 
-	sscanf(line, "%3s", suffix);
+	sscanf(line, "%3s", extra);
 	free(cmd->extra);
-	cmd->extra = gretl_strdup(suffix);
-	shift_string_left(line, strlen(suffix));
+	cmd->extra = gretl_strdup(extra);
+	shift_string_left(line, strlen(extra));
 	nf--;
 	pos = 0;
 	linelen = strlen(line);
-    }
+    } 
 
     if (cmd->ci == AR || cmd->ci == ARMA ||
 	cmd->ci == GARCH) {
@@ -1496,6 +1536,16 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 	    }
 	} /* end if isalpha(*field) */
 
+#if defined(USE_GTK2) || defined (HAVE_FNMATCH_H)
+	else if (*field == '*') {
+	    if (wildcard_expand_ok(field, &lnum, pdinfo, cmd)) {
+		/* handled, get on with it */
+		pos += strlen(field) + 1;
+		continue; 			
+	    }
+	}
+#endif
+
 	else if (isdigit(*field)) {
 	    /* could be the ID number of a variable */
 	    v = atoi(field);
@@ -1521,7 +1571,7 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 		    poly = 1;
 		}
 		continue;
-	    } else if (cmd->ci == VAR) {
+	    } else if (cmd->ci == VAR || cmd->ci == VECM) {
 		pos += strlen(field) + 1;
 		cmd->list[lnum++] = LISTSEP;
 		continue;
@@ -1600,6 +1650,10 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
 	cmd->errcode = E_ALLOC;
     }
 
+#if 0
+    printlist(cmd->list, "cmd->list");
+#endif
+
     if (cmd->errcode) {
 	cmd->context = 0;
     }
@@ -1609,15 +1663,6 @@ int parse_command_line (char *line, CMD *cmd, double ***pZ, DATAINFO *pdinfo)
     return cmd->errcode;
 }
 
-static void nl_strip (char *line)
-{
-    int n = strlen(line);
-
-    if (n > 0 && line[n-1] == '\n') {
-	line[n-1] = 0;
-    }
-}
-
 /**
  * help:
  * @cmdword: the command on which help is wanted.
@@ -1635,6 +1680,7 @@ static void nl_strip (char *line)
 int help (const char *cmdword, const char *helpfile, PRN *prn)
 {
     FILE *fp;
+    char word[9];
     char line[128];
     int i, j, ok;
 
@@ -1674,18 +1720,18 @@ int help (const char *cmdword, const char *helpfile, PRN *prn)
 
     ok = 0;
     while (fgets(line, sizeof line, fp) != NULL) {
-	nl_strip(line);
-	if (!strcmp(cmdword, line)) {
+	if (*line != '#') {
+	    continue;
+	}
+	sscanf(line + 2, "%8s", word);
+	if (!strcmp(cmdword, word)) {
 	    ok = 1;
-	    pputc(prn, '\n');
-	    while (fgets(line, MAXLEN, fp)) {
+	    pprintf(prn, "\n%s\n", word);
+	    while (fgets(line, sizeof line, fp)) {
 		if (*line == '#') {
 		    break;
 		}
-		nl_strip(line);
-		if (*line != '@') {
-		    pprintf(prn, "%s\n", line);
-		}		
+		pputs(prn, line);
 	    }
 	    break;
 	}
@@ -1769,31 +1815,27 @@ static int parse_criteria (const char *line, const double **Z,
  * option flag, or 0 if the option flag is not recognized.
  */
 
-int parseopt (const char **argv, int argc, char *fname,
-	      int *force_lang)
+int parseopt (const char **argv, int argc, char *fname, int *force_lang)
 {
     int opt = 0;
     const char *s = argv[1];
 
     *fname = '\0';
+    *force_lang = 0;
 
 #ifdef ENABLE_NLS
-    if (strcmp(s, "-e") == 0 || strncmp(s, "--english", 9) == 0) { 
+    if (!strcmp(s, "-e") || !strncmp(s, "--english", 9)) { 
 	*force_lang = ENGLISH;
-	if (--argc < 2) {
-	    return 0;
-	}
-	argv++;
-	s = argv[1];
-    }
-    if (strcmp(s, "-q") == 0 || strncmp(s, "--basque", 8) == 0) { 
+    } else if (!strcmp(s, "-q") || !strncmp(s, "--basque", 8)) { 
 	*force_lang = BASQUE;
+    }
+    if (*force_lang) {
 	if (--argc < 2) {
 	    return 0;
 	}
 	argv++;
 	s = argv[1];
-    }
+    }	
 #endif
 
     if (strcmp(s, "-b") == 0 || strncmp(s, "--batch", 7) == 0) 
@@ -2026,6 +2068,22 @@ static int more_coming (const CMD *cmd, int i)
     return 0;
 }
 
+static int n_separators (const int *list)
+{
+    int i, nsep = 0;
+
+    for (i=2; i<list[0]; i++) {
+	if (list[i] == LISTSEP) {
+	    nsep++;
+	}
+    }
+
+    return nsep;
+}
+
+#define listsep_switch(c) (c == AR || c == GARCH || c == ARMA || \
+                           c == MPOLS)
+
 #define hold_param(c) (c == TSLS || c == AR || c == ARMA || c == CORRGM || \
                        c == MPOLS || c == SCATTERS || c == GNUPLOT || \
                        c == LOGISTIC || c == GARCH || c == EQUATION || \
@@ -2039,11 +2097,14 @@ print_cmd_list (const CMD *cmd, const DATAINFO *pdinfo,
 		int batch, int echo_stdout, char leadchar,
 		int *stdlen, int *prnlen, PRN *prn)
 {
+    int use_varnames = 1;
     char first[16];
-    int i, gotsep = 1;
+    int nsep, gotsep, i;
 
-    if (cmd->ci == AR || cmd->ci == ARMA || cmd->ci == GARCH) {
-	gotsep = 0;
+    nsep = n_separators(cmd->list);
+
+    if (cmd->ci == AR || cmd->ci == GARCH || cmd->ci == ARMA) {
+	use_varnames = 0;	
     }
 
     if (cmd->ci == REMEMBER && cmd->extra != NULL) {
@@ -2073,6 +2134,8 @@ print_cmd_list (const CMD *cmd, const DATAINFO *pdinfo,
 	if (cmd->ci == REMEMBER) {
 	    fputs(" =", stdout);
 	    *stdlen += 2;
+	} else if (cmd->ci == VECM) {
+	    *stdlen += printf(" %s", cmd->extra);
 	}
     }
 
@@ -2092,7 +2155,10 @@ print_cmd_list (const CMD *cmd, const DATAINFO *pdinfo,
 	}
 	if (cmd->ci == REMEMBER) {
 	    *prnlen += pputs(prn, " =");
-	}	
+	} else if (cmd->ci == VECM) {
+	    pprintf(prn, " %s", cmd->extra);
+	    *prnlen += 1 + strlen(cmd->extra);
+	}
     }
 
 #if 0
@@ -2106,7 +2172,10 @@ print_cmd_list (const CMD *cmd, const DATAINFO *pdinfo,
     }
 #endif
 
+    gotsep = 0;
+
     for (i=1; i<=cmd->list[0]; i++) {
+
 	if (cmd->list[i] == LISTSEP) {
 	    if (echo_stdout) {
 		*stdlen += printf(" ;");
@@ -2114,12 +2183,15 @@ print_cmd_list (const CMD *cmd, const DATAINFO *pdinfo,
 	    if (!batch) {
 		*prnlen += pputs(prn, " ;");
 	    }
-	    gotsep = (cmd->ci != MPOLS)? 1 : 0;
+	    gotsep++;
+	    if (listsep_switch(cmd->ci) && gotsep == nsep) {
+		use_varnames = !use_varnames;
+	    }
 	    continue;
 	}
 
 	if (echo_stdout) {
-	    if (gotsep) {
+	    if (use_varnames) {
 		*stdlen += cmd_list_print_var(cmd, i, pdinfo, 1, prn);
 	    } else {
 		*stdlen += printf(" %d", cmd->list[i]);
@@ -2131,7 +2203,7 @@ print_cmd_list (const CMD *cmd, const DATAINFO *pdinfo,
 	}
 
 	if (!batch) {
-	    if (gotsep) {
+	    if (use_varnames) {
 		*prnlen += cmd_list_print_var(cmd, i, pdinfo, 0, prn);
 	    } else {
 		char numstr[12];
@@ -2279,8 +2351,12 @@ void echo_cmd (const CMD *cmd, const DATAINFO *pdinfo, const char *line,
 	const char *flagstr;
 	int ci = cmd->ci;
 
-	if (cmd->ci == END && !strcmp(cmd->param, "nls")) {
-	    ci = NLS;
+	if (cmd->ci == END) {
+	    if (!strcmp(cmd->param, "nls")) {
+		ci = NLS;
+	    } else if (!strcmp(cmd->param, "mle")) {
+		ci = MLE;
+	    }
 	}
 	flagstr = print_flags(cmd->opt, ci);
 	len = strlen(flagstr);
@@ -2578,6 +2654,7 @@ static int add_obs (int n, double ***pZ, DATAINFO *pdinfo, PRN *prn)
 #define list_genr_command(c) (c == DIFF || \
                               c == LDIFF || \
                               c == LOGS || \
+                              c == SDIFF || \
                               c == SQUARE)
 
 /* common code for command-line and GUI client programs, where the
@@ -2623,7 +2700,8 @@ int simple_commands (CMD *cmd, const char *line,
 
     case COINT2:
 	order = atoi(cmd->param);
-	err = johansen_test(order, cmd->list, pZ, datainfo, cmd->opt, prn);
+	err = johansen_test_simple(order, cmd->list, pZ, datainfo, 
+				   cmd->opt, prn);
 	break;
 
     case CORR:
@@ -2687,9 +2765,15 @@ int simple_commands (CMD *cmd, const char *line,
 	break;
 
     case DIFF:
-	err = list_diffgenr(genlist, pZ, datainfo);
+    case LDIFF:
+    case SDIFF:
+	err = list_diffgenr(genlist, cmd->ci, pZ, datainfo);
 	if (err) {
-	    pputs(prn, _("Error adding first differences of variables.\n"));
+	    if (cmd->ci == LDIFF) {
+		pputs(prn, _("Error adding log differences of variables.\n"));
+	    } else if (cmd->ci == DIFF) {
+		pputs(prn, _("Error adding first differences of variables.\n"));
+	    }
 	} else {
 	    maybe_list_vars(datainfo, prn);
 	}
@@ -2704,15 +2788,6 @@ int simple_commands (CMD *cmd, const char *line,
 	err = kpss_test(order, cmd->list[1], pZ, datainfo, cmd->opt, prn);
 	break;
 
-    case LDIFF:
-	err = list_ldiffgenr(genlist, pZ, datainfo);
-	if (err) {
-	    pputs(prn, _("Error adding log differences of variables.\n"));
-	} else {
-	    maybe_list_vars(datainfo, prn);
-	}
-	break;
-
     case LAGS:
 	order = atoi(cmd->param);
 	err = list_laggenr(order, cmd->list, pZ, datainfo); 
@@ -2901,6 +2976,10 @@ int simple_commands (CMD *cmd, const char *line,
 	} 
 	break;
 
+    case TRANSPOSE:
+	err = transpose_data(pZ, datainfo);
+	break;
+
     default:
 	break;
     }
@@ -2916,13 +2995,17 @@ int simple_commands (CMD *cmd, const char *line,
     return err;
 }
 
+#define CDEBUG 0
+
 /**
  * get_command_index:
  * @line: command line.
  * @cmd: pointer to gretl command struct.
  *
  * Parse @line and assign to the %ci field of @cmd the index number of
- * the command embedded in @line.
+ * the command embedded in @line.  Note: this is a "lite" version of
+ * parse_command_line().  It is used when commands are being stacked
+ * for execution within a loop.
  *
  * Returns: 1 on error, otherwise 0.
  */
@@ -2930,11 +3013,18 @@ int simple_commands (CMD *cmd, const char *line,
 int get_command_index (const char *line, CMD *cmd)
 {
     static int context;
+    int done = 0;
 
     while (isspace(*line)) {
 	line++;
     }
 
+    cmd->ci = 0;
+
+#if CDEBUG
+    fprintf(stderr, "get_command_index: line='%s'\n", line);
+#endif
+
     if (*line == '#' || *line == '(') {
 	cmd->nolist = 1;
 	cmd->ci = CMD_COMMENT;
@@ -2947,27 +3037,48 @@ int get_command_index (const char *line, CMD *cmd)
 	return 0;
     }
 
+#if CDEBUG
+    fprintf(stderr, " got command word = '%s'\n", cmd->word);
+#endif
+
     /* subsetted commands (e.g. "deriv" in relation to "nls") */
     if (!strcmp(cmd->word, "end")) {
 	context = 0;
 	cmd->ci = END;
+	done = 1;
     } else if (context && strcmp(cmd->word, "equation")) {
 	/* "equation" occurs in the SYSTEM context, but it is
-	   a command in its own right */
+	   a command in its own right, so we don't set cmd->ci
+	   to the context value */
 	cmd->ci = context;
+#if CDEBUG
+	fprintf(stderr, " context (static) = %d, ci = %d\n", context, cmd->ci);
+#endif
+	done = 1;
     } else if (catch_command_alias(cmd)) {
-	; /* cmd->ci is set OK */
-    } else if ((cmd->ci = gretl_command_number(cmd->word)) == 0) {
-	if (!plausible_genr_start(line, cmd)) {
+#if CDEBUG
+	fprintf(stderr, " caught command alias, ci = %d\n", cmd->ci);
+#endif
+	done = 1;
+    } 
+
+    if (!done) {
+	cmd->ci = gretl_command_number(cmd->word);
+#if CDEBUG
+	fprintf(stderr, " gretl_command_number(%s) gave %d\n", cmd->word, cmd->ci);
+#endif
+	if (cmd->ci == 0 && !plausible_genr_start(line, cmd)) {
 	    cmd->errcode = 1;
 	    sprintf(gretl_errmsg, _("command '%s' not recognized"), 
 		    cmd->word);
 	    return 1;
 	}
-    }
+    }	
 
     if (cmd->ci == NLS) {
 	context = NLS;
+    } else if (cmd->ci == MLE) {
+	context = MLE;
     }
 
     if (!strcmp(line, "end loop")) {
@@ -3113,8 +3224,4 @@ const char *gretl_cmd_get_savename (const CMD *cmd)
     return cmd->savename;
 }
 
-/* keep track of input streams */
-
-
-
 
