@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <math.h>
+#include <limits.h>
 
 #include <zlib.h>
 
@@ -71,7 +72,7 @@
 #define MAXLABEL 128  /* maximum length of descriptive labels for variables */
 #define MAXLEN   512  /* max length of regular "long" strings */
 #define ERRLEN   256  /* max length of libgretl error messages */
-#define MAXDISP   20  /* max length of "display names" for variables */
+#define MAXDISP   32  /* max length of "display names" for variables */
 #define VNAMELEN  14  /* space allocated for var names (including termination) */
 #define USER_VLEN  9  /* max allocation for varname constructed by user */
 #define OBSLEN    11  /* space allocated for obs strings (including termination) */
@@ -85,7 +86,6 @@
 
 #define LISTSEP            999
 #define PMAX_NOT_AVAILABLE 666
-#define OPT_NONE           0L
 
 /* numbers smaller than the given limit will print as zero */
 #define screen_zero(x)  ((fabs(x) > 1.0e-13)? x : 0.0)
@@ -122,6 +122,7 @@ enum progress_return_flags {
 };
 
 enum test_stats {
+    GRETL_STAT_NONE,
     GRETL_STAT_NORMAL_CHISQ,
     GRETL_STAT_TR2,
     GRETL_STAT_F,
@@ -133,6 +134,7 @@ enum test_stats {
 };
 
 enum gretl_opt_flags {
+    OPT_NONE = 0,
     OPT_A = 1 <<  0,
     OPT_B = 1 <<  1,
     OPT_C = 1 <<  2,
@@ -153,9 +155,19 @@ enum gretl_opt_flags {
     OPT_V = 1 << 17,
     OPT_W = 1 << 18,
     OPT_X = 1 << 19,
-    OPT_Z = 1 << 20
+    OPT_Z = 1 << 20,
+    OPT_UNSET = ULONG_MAX
 };
 
+typedef enum {
+    OP_EQ  = '=',
+    OP_GT  = '>',
+    OP_LT  = '<',
+    OP_NEQ = 21,
+    OP_GTE = 22,
+    OP_LTE = 23
+} GretlOp;
+
 typedef unsigned long gretlopt;
 
 typedef struct VARINFO_ VARINFO;
@@ -202,7 +214,7 @@ struct DATAINFO_ {
     char decpoint;      /* character used to represent decimal point */
     char **S;           /* to hold observation markers */
     char *descrip;      /* to hold info on data sources etc. */
-    unsigned char *vector; /* hold info on vars: vector versus scalar */
+    char *vector;       /* hold info on vars: vector versus scalar */
     char *submask;      /* subsampling mask */
     void *data;         /* all-purpose pointer */
 };
@@ -221,6 +233,7 @@ struct PATHS_ {
     char scriptdir[MAXLEN];
     char helpfile[MAXLEN];
     char cmd_helpfile[MAXLEN];
+    char cli_helpfile[MAXLEN];
     char datfile[MAXLEN];
     char binbase[MAXLEN];
     char ratsbase[MAXLEN];
@@ -247,14 +260,15 @@ struct SAMPLE_ {
 };
 
 struct ARINFO_ {
-    int *arlist;                /* list of autoreg lags */
-    double *rho;                /* array of autoreg. coeffs. */
-    double *sderr;              /* and their standard errors */
+    int *arlist;          /* list of autoreg lags */
+    double *rho;          /* array of autoreg. coeffs. */
+    double *sderr;        /* and their standard errors */
 };
 
 /* struct to hold model results */
 struct MODEL_ {
     int ID;                      /* ID number for model */
+    int refcount;                /* for saving/deleting */
     int t1, t2, nobs;            /* starting observation, ending
                                     observation, and number of obs */
     char *submask;               /* keep track of sub-sample in force
@@ -262,6 +276,7 @@ struct MODEL_ {
     char *missmask;              /* missing observations mask */
     SAMPLE smpl;                 /* numeric start and end of current sample
                                     when model was estimated */
+    int full_n;                  /* full length of dataset on estimation */
     int ncoeff, dfn, dfd;        /* number of coefficents; degrees of
                                     freedom in numerator and denominator */
     int *list;                   /* list of variables by ID number */
@@ -338,6 +353,7 @@ struct mp_results_ {
 #include "dataset.h"
 #include "estimate.h"
 #include "generate.h"
+#include "genrfuncs.h"
 #include "compare.h"
 #include "gretl_intl.h"
 #include "gretl_list.h"
@@ -359,12 +375,14 @@ struct mp_results_ {
 #include "nonparam.h"
 #include "options.h"
 #include "discrete.h"
+#include "adf_kpss.h"
 #include "subsample.h"
 #include "calendar.h"
 #include "plugins.h"
 #include "nls.h"
 #include "missing.h"
 #include "transforms.h"
+#include "tsls.h"
 #ifdef WIN32
 # include "gretl_win32.h"
 #endif
