@@ -34,7 +34,7 @@ struct _cmplx {
 #define NO_RBAR_SQ(a) (a == AUX_SQ || a == AUX_LOG || a == AUX_WHITE || a == AUX_AR)
 
 static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-			int i, int longnames, PRN *prn);
+			int i, PRN *prn);
 static int rtf_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
 			    int i, PRN *prn);
 static void depvarstats (const MODEL *pmod, PRN *prn);
@@ -54,8 +54,6 @@ static void print_poisson_offset (const MODEL *pmod, const DATAINFO *pdinfo,
 				  PRN *prn);
 static void print_ll (const MODEL *pmod, PRN *prn);
 
-/* ......................................................... */ 
-
 static void noconst (const MODEL *pmod, PRN *prn)
 {
     if (na(pmod->rsq)) return;
@@ -521,6 +519,7 @@ const char *estimator_string (int ci, PRN *prn)
     else if (ci == POISSON) return N_("Poisson");
     else if (ci == POOLED) return N_("Pooled OLS");
     else if (ci == NLS) return N_("NLS");
+    else if (ci == MLE) return N_("ML");
     else if (ci == LOGISTIC) return N_("Logistic");
     else if (ci == GARCH) return N_("GARCH");
     else if (ci == CORC) {
@@ -541,7 +540,11 @@ static const char *
 my_estimator_string (const MODEL *pmod, PRN *prn)
 {
     if (pmod->ci == ARMA) {
-	return (pmod->list[0] > 4)? N_("ARMAX") : N_("ARMA");
+	if (gretl_model_get_int(pmod, "armax")) {
+	    return N_("ARMAX");
+	} else {
+	    return N_("ARMA");
+	}
     } else if (pmod->ci == WLS) {
 	if (gretl_model_get_int(pmod, "iters")) {
 	    return N_("Maximum Likelihood");
@@ -723,15 +726,62 @@ static void garch_vcv_line (const MODEL *pmod, PRN *prn)
     }
 }
 
-static void model_print_newline (PRN *prn)
+static void qml_vcv_line (PRN *prn)
 {
-    if (tex_format(prn)) {
-	pputs(prn, "\\\\\n");
-    } else if (rtf_format(prn)) {
-	pputs(prn, "\\par\n");
-    } else {
-	pputc(prn, '\n');
+    const char *vcvstr = N_("QML standard errors");
+
+    pprintf(prn, "%s\n", (plain_format(prn))? _(vcvstr) : I_(vcvstr));
+}
+
+static void tex_vecm_depvar_name (char *s, const char *vname)
+{
+    char tmp[9];
+    int gotit = 0;
+
+    if (sscanf(vname, "d_%8s", tmp)) {
+	char myvar[16];
+
+	tex_escape(myvar, tmp);
+	sprintf(s, "$\\Delta$%s", myvar);
+	gotit = 1;
     }
+
+    if (!gotit) {
+	tex_escape(s, vname); 
+    }    
+}
+
+void print_model_vcv_info (const MODEL *pmod, PRN *prn)
+{
+    if (pmod->aux == AUX_SCR || gretl_model_get_int(pmod, "hac_lag")) {
+	hac_vcv_line(pmod, prn);
+    } else if (gretl_model_get_int(pmod, "hc")) {
+	hc_vcv_line(pmod, prn);
+    } else if (gretl_model_get_int(pmod, "garch_vcv")) {
+	garch_vcv_line(pmod, prn);
+    } else if ((pmod->ci == LOGIT || pmod->ci == PROBIT) && 
+	       gretl_model_get_int(pmod, "robust")) {
+	qml_vcv_line(prn);
+    }
+}
+
+static void print_tsls_droplist (const MODEL *pmod, 
+				 const DATAINFO *pdinfo,
+				 PRN *prn)
+{
+    const int *dlist = gretl_model_get_data(pmod, "tsls_droplist");
+    int i, v;
+
+    pputs(prn, _("Omitted due to exact collinearity:"));
+    for (i=1; i<=dlist[0]; i++) {
+	v = dlist[i];
+	if (v < pdinfo->v) {
+	    pprintf(prn, " %s", pdinfo->varname[v]);
+	} else {
+	    pprintf(prn, " %d", v);
+	}
+    }
+    pputc(prn, '\n');
 }
 
 static void print_model_heading (const MODEL *pmod, 
@@ -739,13 +789,15 @@ static void print_model_heading (const MODEL *pmod,
 				 gretlopt opt, 
 				 PRN *prn)
 {
-    char startdate[OBSLEN], enddate[OBSLEN], vname[16];
+    char startdate[OBSLEN], enddate[OBSLEN], vname[32];
     int t1 = pmod->t1, t2 = pmod->t2;
     int tex = tex_format(prn);
     int utf = plain_format(prn);
 
-    ntodate(startdate, t1, pdinfo);
-    ntodate(enddate, t2, pdinfo);
+    if (pmod->aux != AUX_VAR && pmod->aux != AUX_VECM) {
+	ntodate(startdate, t1, pdinfo);
+	ntodate(enddate, t2, pdinfo);
+    }
 
     switch (pmod->aux) {
     case AUX_SQ:
@@ -795,6 +847,10 @@ static void print_model_heading (const MODEL *pmod,
 	pprintf(prn, "\n%s %d: ", 
 		(utf)? _("Equation") : I_("Equation"), pmod->ID);
 	break;
+    case AUX_VECM:
+	pprintf(prn, "%s %d: ", 
+		(utf)? _("Equation") : I_("Equation"), pmod->ID);
+	break;
     case AUX_ADD:
     default:
 	if (pmod->ID < 0 || (opt & OPT_S)) {
@@ -807,7 +863,9 @@ static void print_model_heading (const MODEL *pmod,
 	break;
     }
 
-    if (pmod->aux == AUX_SYS) {
+    if (pmod->aux == AUX_VAR || pmod->aux == AUX_VECM) {
+	;
+    } else if (pmod->aux == AUX_SYS) {
 	pprintf(prn, (utf)?
 		_("%s estimates using the %d observations %s%s%s") :
 		I_("%s estimates using the %d observations %s%s%s"),
@@ -822,7 +880,7 @@ static void print_model_heading (const MODEL *pmod,
 		    I_("%s estimates using %d observations from %s%s%s"),
 		    _(my_estimator_string(pmod, prn)), 
 		    pmod->nobs, startdate, (tex)? "--" : "-", enddate);
-	    model_print_newline(prn);
+	    gretl_prn_newline(prn);
 	    pprintf(prn, "%s: %d",
 		    (utf)? _("Missing or incomplete observations dropped") :
 		    I_("Missing or incomplete observations dropped"), mc);
@@ -842,16 +900,17 @@ static void print_model_heading (const MODEL *pmod,
 		_(my_estimator_string(pmod, prn)), 
 		pmod->nobs);
 	if (effn > 0) {
-	    model_print_newline(prn);
+	    gretl_prn_newline(prn);
 	    pprintf(prn, (utf)? _("Included %d cross-sectional units") :
 		    I_("Included %d cross-sectional units"), effn);
 	}
     }
 
-    model_print_newline(prn);
+    if (pmod->aux != AUX_VAR && pmod->aux != AUX_VECM) {
+	gretl_prn_newline(prn);
+    }
 
-    /* special names for dependent variable in cases of certain sorts
-       of auxiliary regressions */
+    /* special formulations for dependent variable in various cases */
     if (pmod->aux == AUX_SQ || pmod->aux == AUX_LOG) {
 	pprintf(prn, "%s: %s", 
 		(utf)? _("Dependent variable") : I_("Dependent variable"),
@@ -869,17 +928,32 @@ static void print_model_heading (const MODEL *pmod,
 	pprintf(prn, "%s: %s", 
 		(utf)? _("Dependent variable") : I_("Dependent variable"),
 		(tex)? vname : pmod->params[0]);
+    } else if (pmod->ci == MLE) {
+	if (tex) {
+	    pprintf(prn, "\\verb!%s!", pmod->params[0]);
+	} else {
+	    pputs(prn, pmod->params[0]);
+	}
     } else { 
-	int v = (pmod->ci == ARMA || pmod->ci == GARCH)? 
-	    pmod->list[4] : pmod->list[1];
+	int v = gretl_model_get_depvar(pmod);
 
-	if (tex) tex_escape(vname, pdinfo->varname[v]);
-	pprintf(prn, "%s: %s", 
-		(utf)? _("Dependent variable") : I_("Dependent variable"),
-		(tex)? vname : pdinfo->varname[v]);
+	if (tex) {
+	    if (pmod->aux == AUX_VECM) {
+		tex_vecm_depvar_name(vname, pdinfo->varname[v]);
+	    } else {
+		tex_escape(vname, pdinfo->varname[v]);
+	    }
+	}
+	if (pmod->aux == AUX_VAR || pmod->aux == AUX_VECM) {
+	    pputs(prn, (tex)? vname : pdinfo->varname[v]);
+	} else {
+	    pprintf(prn, "%s: %s", 
+		    (utf)? _("Dependent variable") : I_("Dependent variable"),
+		    (tex)? vname : pdinfo->varname[v]);
+	}
     }
 
-    model_print_newline(prn);
+    gretl_prn_newline(prn);
 
     /* supplementary strings below the estimator and sample info */
 
@@ -888,21 +962,15 @@ static void print_model_heading (const MODEL *pmod,
 	int method = gretl_model_get_int(pmod, "method");
 
 	if (method != SYS_FIML && method != SYS_LIML) {
-	    print_tsls_instruments (pmod->list, pdinfo, prn);
+	    print_tsls_instruments(pmod->list, pdinfo, prn);
 	}
     }
 
     /* VCV variants */
-    if (pmod->aux == AUX_SCR || gretl_model_get_int(pmod, "hac_lag")) {
-	hac_vcv_line(pmod, prn);
-    } else if (gretl_model_get_int(pmod, "hc")) {
-	hc_vcv_line(pmod, prn);
-    } else if (gretl_model_get_int(pmod, "garch_vcv")) {
-	garch_vcv_line(pmod, prn);
-    }
+    print_model_vcv_info(pmod, prn);
 
     /* WLS on panel data */
-    else if (gretl_model_get_int(pmod, "unit_weights") && !pmod->aux) {
+    if (gretl_model_get_int(pmod, "unit_weights") && !pmod->aux) {
 	if (tex) {
 	    pputs(prn, "\\\\\n");
 	}
@@ -935,13 +1003,30 @@ static void print_model_heading (const MODEL *pmod,
 	}
     } 
 
-    /* message about new variable created */
+    /* y-hat formula for logistic regression */
+    else if (pmod->ci == LOGISTIC) {
+	if (tex) {
+	    pprintf(prn, "$\\hat{y} = %g / (1 + e^{-X\\hat{\\beta}})$\n", 
+		    gretl_model_get_double(pmod, "lmax"));  
+	} else {
+	    pprintf(prn, "yhat = %g / (1 + exp(-X*b))\n",  
+		    gretl_model_get_double(pmod, "lmax"));
+	}
+    }
+
+    /* message about collinear variables dropped */
     if (plain_format(prn) && gretl_msg[0] != '\0' &&
 	strstr(gretl_msg, _("Replaced")) == NULL &&
 	strstr(gretl_msg, _("Generated")) == NULL) {
 	pprintf(prn, "%s\n", gretl_msg);
     }
 
+    /* ditto, but special for TSLS */
+    if (plain_format(prn) && pmod->ci == TSLS &&
+	gretl_model_get_data(pmod, "tsls_droplist") != NULL) {
+	print_tsls_droplist(pmod, pdinfo, prn);
+    }    
+
     if (pmod->missmask == NULL && gretl_model_get_int(pmod, "wt_dummy")) { 
 	/* FIXME alt formats */
 	pprintf(prn, "%s %d\n", 
@@ -967,16 +1052,20 @@ static void model_format_start (PRN *prn)
 	}
 	pputs(prn, "\\begin{center}\n");
     } else if (rtf_format(prn)) {
-	pputs(prn, "{\\rtf1\\par\n\\qc ");
+	if (rtf_doc_format(prn)) {
+	    pputs(prn, "{\\rtf1\\par\n\\qc ");
+	} else {
+	    pputs(prn, "\\par\n\\qc ");
+	}
     }
 }
 
 #define RTF_COEFF_ROW  "\\trowd \\trqc \\trgaph30\\trleft-30\\trrh262" \
-                       "\\cellx500\\cellx1900\\cellx3300\\cellx4700\\cellx6100" \
+                       "\\cellx1900\\cellx3300\\cellx4700\\cellx6100" \
                        "\\cellx7500\\cellx8000\n\\intbl"
 
 #define RTF_DISCRETE_ROW  "\\trowd \\trqc \\trgaph30\\trleft-30\\trrh262" \
-                       "\\cellx500\\cellx1900\\cellx3300\\cellx4700\\cellx6100" \
+                       "\\cellx1900\\cellx3300\\cellx4700\\cellx6100" \
                        "\\cellx8000\n\\intbl"
 
 #define RTF_ROOT_ROW   "\\trowd \\trqc \\trgaph30\\trleft-30\\trrh262" \
@@ -986,13 +1075,15 @@ static void model_format_start (PRN *prn)
 
 static void print_coeff_table_start (const MODEL *pmod, PRN *prn, int discrete)
 {
+    int use_param = pmod->ci == NLS || pmod->ci == MLE;
+
     if (plain_format(prn)) {
 	if (discrete) {
 	    pputs(prn, _("      VARIABLE      COEFFICIENT        STDERROR"
 			   "       T STAT       SLOPE\n"));
 	    pprintf(prn, "                                                 "
 		    "                 %s\n", _("(at mean)"));
-	} else if (pmod->ci == NLS) {
+	} else if (use_param) {
 	    pputs(prn, _("      PARAMETER      ESTIMATE          STDERROR"
 			   "       T STAT   2Prob(t > |T|)\n\n"));
 	} else {
@@ -1003,7 +1094,7 @@ static void print_coeff_table_start (const MODEL *pmod, PRN *prn, int discrete)
     } else {
 	char col1[16], col2[16];
 
-	if (pmod->ci == NLS) {
+	if (use_param) {
 	    strcpy(col1, N_("Parameter"));
 	    strcpy(col2, N_("Estimate"));
 	} else {
@@ -1038,7 +1129,6 @@ static void print_coeff_table_start (const MODEL *pmod, PRN *prn, int discrete)
 	if (rtf_format(prn)) {
 	    if (discrete) {
 		pprintf(prn, "{" RTF_DISCRETE_ROW
-			" \\qr \\cell"
 			" \\qc {\\i %s}\\cell"
 			" \\qc {\\i %s}\\cell"
 			" \\qc {\\i %s}\\cell"
@@ -1049,7 +1139,6 @@ static void print_coeff_table_start (const MODEL *pmod, PRN *prn, int discrete)
 			I_("t-statistic"), I_("Slope"));
 	    } else {
 		pprintf(prn, "{" RTF_COEFF_ROW
-			" \\qr \\cell"
 			" \\qc {\\i %s}\\cell"
 			" \\qc {\\i %s}\\cell"
 			" \\qc {\\i %s}\\cell"
@@ -1083,54 +1172,27 @@ static void model_format_end (PRN *prn)
 	if (tex_doc_format(prn)) {
 	    pputs(prn, "\n\\end{document}\n"); 
 	}
-    } else if (rtf_format(prn)) {
+    } else if (rtf_doc_format(prn)) {
 	pputs(prn, "\n}\n");
     }
 } 
 
-static int long_coeff_names (const MODEL *pmod, const DATAINFO *pdinfo)
-{
-    int ret = 0;
-
-    if (pmod->aux != AUX_ARCH &&
-	pmod->ci != NLS && 
-	pmod->ci != ARMA && 
-	pmod->ci != GARCH) {
-	int i, len;
-
-	for (i=0; i<pmod->ncoeff; i++) {
-	    len = strlen(pdinfo->varname[pmod->list[i+2]]);
-	    if (len > 8) {
-		ret = 1;
-		break;
-	    }
-	}
-    }
-
-    return ret;
-}
-
 static int 
 print_coefficients (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 {
     int i, err = 0, gotnan = 0;
-    int longnames = 0;
     int gn = -1;
 
     if (pmod->ci == GARCH) {
 	gn = pmod->list[0] - 4;
     }
 
-    if (plain_format(prn)) {
-	longnames = long_coeff_names(pmod, pdinfo);
-    }
-
     for (i=0; i<pmod->ncoeff; i++) {
 	if (plain_format(prn)) {
 	    if (i == gn) {
 		pputc(prn, '\n');
 	    }
-	    err = print_coeff(pdinfo, pmod, i, longnames, prn);
+	    err = print_coeff(pdinfo, pmod, i, prn);
 	} else if (tex_format(prn)) {
 	    if (i == gn) {
 		pputs(prn, "\\\\ \n");
@@ -1247,9 +1309,11 @@ static void print_whites_results (const MODEL *pmod, PRN *prn)
 static void print_ll (const MODEL *pmod, PRN *prn)
 {
     if (plain_format(prn)) {
-	pprintf(prn, "  %s = %.3f\n", _("Log-likelihood"), pmod->lnL);
+	pprintf(prn, "  %s = %.*g\n", _("Log-likelihood"), GRETL_DIGITS,
+		pmod->lnL);
     } else if (rtf_format(prn)) {
-	pprintf(prn, RTFTAB "%s = %.3f\n", I_("Log-likelihood"), pmod->lnL);
+	pprintf(prn, RTFTAB "%s = %.*g\n", I_("Log-likelihood"), GRETL_DIGITS,
+		pmod->lnL);
     } else if (tex_format(prn)) {
 	char xstr[32];
 
@@ -1280,6 +1344,8 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	return 0;
     }
 
+    /* FIXME utf vs iso-88* */
+
     if (!plain_format(prn)) {
 	model_format_start(prn);
     } else if (pmod->ci == ARMA || pmod->ci == GARCH || 
@@ -1291,6 +1357,14 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	if (iters > 0) {
 	    pprintf(prn, _("Convergence achieved after %d iterations\n"), iters);
 	}
+    } else if (pmod->ci == MLE) {
+	int fncount = gretl_model_get_int(pmod, "fncount");
+	int grcount = gretl_model_get_int(pmod, "grcount");
+
+	if (fncount > 0) {
+	    pprintf(prn, _("Function evaluations: %d\n"), fncount);
+	    pprintf(prn, _("Evaluations of gradient: %d\n"), grcount);
+	}
     }
 
     print_model_heading(pmod, pdinfo, opt, prn);
@@ -1386,7 +1460,9 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	goto close_format;
     }
 
-    if (!pmod->ifc && pmod->ci != NLS && plain_format(prn)) {
+    if (!pmod->ifc && pmod->ci != NLS && pmod->aux != AUX_VAR
+	&& pmod->aux != AUX_JOHANSEN && pmod->aux != AUX_VECM
+	&& pmod->ci != MLE && plain_format(prn)) {
 	noconst(pmod, prn);
     }
     
@@ -1416,7 +1492,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 
 	rsqline(pmod, prn);
 
-	if (pmod->ci != NLS) {
+	if (pmod->ci != NLS && pmod->aux != AUX_VECM) {
 	    Fline(pmod, prn);
 	}
 
@@ -1424,7 +1500,8 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	    if (pmod->ci == OLS || pmod->ci == VAR ||
 		(pmod->ci == WLS && gretl_model_get_int(pmod, "wt_dummy"))) {
 		dwline(pmod, prn);
-		if (pmod->ci != VAR && gretl_model_get_int(pmod, "ldepvar")) {
+		if (pmod->ci != VAR && pmod->aux != AUX_VECM &&
+		    gretl_model_get_int(pmod, "ldepvar")) {
 		    dhline(pmod, prn);
 		} 
 	    }
@@ -1436,7 +1513,10 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 
 	if (pmod->ci == ARMA) {
 	    print_arma_stats(pmod, prn);
-	} else {
+	} else if (pmod->aux != AUX_VECM) {
+	    if (pmod->ci == OLS) {
+		print_ll(pmod, prn);
+	    }
 	    info_stats_lines(pmod, prn);
 	}
 
@@ -1460,6 +1540,13 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	print_middle_table_end(prn);
     }
 
+    else if (pmod->ci == MLE) {
+	print_middle_table_start(prn);
+	print_ll(pmod, prn);
+	info_stats_lines(pmod, prn);
+	print_middle_table_end(prn);
+    }	
+
     else if (pmod->ci == HSK || pmod->ci == ARCH ||
 	     (pmod->ci == WLS && !gretl_model_get_int(pmod, "wt_dummy"))) {
 
@@ -1511,7 +1598,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	print_middle_table_end(prn);
     }
 
-    if (plain_format(prn) && 
+    if (plain_format(prn) && pmod->ci != MLE &&
 	pmod->ci != ARMA && pmod->ci != NLS && !pmod->aux) {
 	pval_max_line(pmod, pdinfo, prn);
     }
@@ -1537,8 +1624,6 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
     return gotnan;
 }
 
-/* ......................................................... */ 
-
 static void print_pval_str (double pval, char *str)
 {
     if (pval < .00001) {
@@ -1549,7 +1634,7 @@ static void print_pval_str (double pval, char *str)
 }
 
 static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-			int i, int longnames, PRN *prn)
+			int i, PRN *prn)
 {
     double t, pvalue = 999.0;
     int gotnan = 0;
@@ -1557,16 +1642,7 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
     char varname[24];
 
     gretl_model_get_param_name(pmod, pdinfo, i, varname);
-
-    if (longnames) {
-	pprintf(prn, " %13s ", varname);
-    } else if (pmod->ci == GARCH) {
-	pprintf(prn, "      %8s ", varname);
-    } else if (pmod->ci == ARMA) {
-	pprintf(prn, " %3d) %8s ", i + 1, varname);
-    } else {
-	pprintf(prn, " %3d) %8s ", pmod->list[i + 2], varname);
-    }
+    pprintf(prn, " %13s ", varname);
 
     bufspace((strlen(varname) > 12)? 1 : 2, prn);
 
@@ -1630,8 +1706,8 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	} else if (pvalue < 0.10) {
 	    pputs(prn, " *");
 	}
-    } else if (pmod->list[i+2] != 0 && 
-	     (pmod->ci == LOGIT || pmod->ci == PROBIT)) { 
+    } else if ((pmod->ci == LOGIT || pmod->ci == PROBIT) &&
+	       pmod->list[i+2] != 0) { 
 	double *slopes = gretl_model_get_data(pmod, "slopes");
 
 	if (slopes != NULL) {
@@ -1666,9 +1742,7 @@ static int rtf_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 
     pputs(prn, RTF_COEFF_ROW);
 
-    pprintf(prn, " \\qr %d\\cell \\ql %s\\cell", 
-	    ((pmod->ci == ARMA || pmod->ci == GARCH)? i+1 : pmod->list[i+2]), 
-	    varname);
+    pprintf(prn, "\\ql %s\\cell", varname);
 
     if (isnan(pmod->coeff[i]) || na(pmod->coeff[i])) {
 	pprintf(prn, " \\qc %s\\cell", I_("undefined"));
@@ -1819,6 +1893,43 @@ const char *root_fmt = "%8s%3d%17.4f%11.4f%11.4f%11.4f\n";
 const char *roots_sep = "  -----------------------------------------"
                         "------------------";
 
+static void print_root (double rx, double ix, double mod, double fr,
+			int i, int hline, PRN *prn)
+{
+     if (plain_format(prn)) {
+	 pprintf(prn, root_fmt, _("Root"), i, rx, ix, mod, fr);
+     } else if (tex_format(prn)) {
+	 pprintf(prn, "& %s & %d & $%.4f$ & $%.4f$ & $%.4f$ & $%.4f$ \\\\ ",
+		 I_("Root"), i, rx, ix, mod, fr);
+	 if (hline) {
+	     pputs(prn, "\\hline\n");
+	 } else {
+	     pputc(prn, '\n');
+	 }
+     } else if (rtf_format(prn)) {
+	 pputs(prn, RTF_ROOT_ROW);
+	 pprintf(prn, "\\ql \\cell \\ql %s %d \\cell"
+		 " \\qr %.4f\\cell"
+		 " \\qr %.4f\\cell"
+		 " \\qr %.4f\\cell"
+		 " \\qr %.4f\\cell \\intbl \\row\n",
+		 I_("Root"), i, rx, ix, mod, fr);
+     }
+}
+
+static void root_start (const char *tag, PRN *prn)
+{
+    if (plain_format(prn)) {
+	pprintf(prn, "  %s\n", _(tag));
+    } else if (tex_format(prn)) {
+	pprintf(prn, "%s \\\\ \n", I_(tag));
+    } else if (rtf_format(prn)) {
+	pputs(prn, RTF_ROOT_ROW);
+	pprintf(prn, "\\ql %s\\cell\\ql \\cell\\ql \\cell\\ql \\cell\\ql \\cell"
+		"\\ql\\cell \\intbl \\row\n", I_(tag));
+    }
+}
+
 static void print_arma_roots (const MODEL *pmod, PRN *prn)
 {
     cmplx *roots = gretl_model_get_data(pmod, "roots");
@@ -1826,7 +1937,9 @@ static void print_arma_roots (const MODEL *pmod, PRN *prn)
     if (roots != NULL) {
 	int p = pmod->list[1];
 	int q = pmod->list[2];
-	int i;
+	int P = gretl_model_get_int(pmod, "arma_P");
+	int Q = gretl_model_get_int(pmod, "arma_Q");
+	int i, k, hline;
 	double mod, fr;
 
 	if (plain_format(prn)) {
@@ -1847,15 +1960,8 @@ static void print_arma_roots (const MODEL *pmod, PRN *prn)
 	}
 
 	if (p > 0) {
-	    if (plain_format(prn)) {
-		pprintf(prn, "  %s\n", _("AR"));
-	    } else if (tex_format(prn)) {
-		pprintf(prn, "%s \\\\ \n", I_("AR"));
-	    } else if (rtf_format(prn)) {
-		pputs(prn, RTF_ROOT_ROW);
-		pprintf(prn, "\\ql %s\\cell\\ql \\cell\\ql \\cell\\ql \\cell\\ql \\cell"
-			"\\ql\\cell \\intbl \\row\n", I_("AR"));
-	    }
+	    k = 1;
+	    root_start(N_("AR"), prn);
 	    for (i=0; i<p; i++) {
 		if (roots[i].i != 0) {
 		    mod = roots[i].r * roots[i].r + roots[i].i * roots[i].i;
@@ -1864,37 +1970,19 @@ static void print_arma_roots (const MODEL *pmod, PRN *prn)
 		    mod = fabs(roots[i].r);
 		}
 		fr = atan2(roots[i].i, roots[i].r) / (2.0 * M_PI);
-		if (plain_format(prn)) {
-		    pprintf(prn, root_fmt, _("Root"), i + 1, 
-			    roots[i].r, roots[i].i, mod, fr);
-		} else if (tex_format(prn)) {
-		    pprintf(prn, "& %s & %d & $%.4f$ & $%.4f$ & $%.4f$ & $%.4f$ \\\\ ",
-			    I_("Root"), i + 1, roots[i].r, roots[i].i, mod, fr);
-		    if (i == p - 1 && q == 0) pputs(prn, "\\hline\n");
-		    else pputc(prn, '\n');
-		} else if (rtf_format(prn)) {
-		    pputs(prn, RTF_ROOT_ROW);
-		    pprintf(prn, "\\ql \\cell \\ql %s %d \\cell"
-			    " \\qr %.4f\\cell"
-			    " \\qr %.4f\\cell"
-			    " \\qr %.4f\\cell"
-			    " \\qr %.4f\\cell \\intbl \\row\n",
-			    I_("Root"), i + 1, roots[i].r, roots[i].i, mod, fr);
+		if (i == p - 1 && q == 0 && P == 0 && Q == 0) {
+		    hline = 1;
+		} else {
+		    hline = 0;
 		}
+		print_root(roots[i].r, roots[i].i, mod, fr, k++, hline, prn);
 	    }
-	} 
+	}
 
-	if (q > 0) {
-	    if (plain_format(prn)) {
-		pprintf(prn, "  %s\n", _("MA"));
-	    } else if (tex_format(prn)) {
-		pprintf(prn, "%s \\\\ \n", I_("MA"));
-	    } else if (rtf_format(prn)) {
-		pputs(prn, RTF_ROOT_ROW);
-		pprintf(prn, "\\ql %s\\cell\\ql \\cell\\ql \\cell\\ql \\cell\\ql \\cell"
-			"\\ql\\cell \\intbl \\row\n", I_("MA"));
-	    }
-	    for (i=p; i<p+q; i++) {
+	if (P > 0) {
+	    k = 1;
+	    root_start(N_("AR (seasonal)"), prn);
+	    for (i=p; i<p+P; i++) {
 		if (roots[i].i != 0) {
 		    mod = roots[i].r * roots[i].r + roots[i].i * roots[i].i;
 		    mod = sqrt(mod);
@@ -1902,23 +1990,52 @@ static void print_arma_roots (const MODEL *pmod, PRN *prn)
 		    mod = fabs(roots[i].r);
 		}
 		fr = atan2(roots[i].i, roots[i].r) / (2.0 * M_PI);
-		if (plain_format(prn)) {
-		    pprintf(prn, root_fmt, _("Root"), i - p + 1, 
-			    roots[i].r, roots[i].i, mod, fr);
-		} else if (tex_format(prn)) {
-		    pprintf(prn, "& %s & %d & $%.4f$ & $%.4f$ & $%.4f$ & $%.4f$ \\\\ ",
-			    I_("Root"), i - p + 1, roots[i].r, roots[i].i, mod, fr);
-		    if (i == p + q - 1) pputs(prn, "\\hline\n");
-		    else pputc(prn, '\n');
-		} else if (rtf_format(prn)) {
-		    pputs(prn, RTF_ROOT_ROW);
-		    pprintf(prn, "\\ql \\cell \\ql %s %d \\cell"
-			    " \\qr %.4f\\cell"
-			    " \\qr %.4f\\cell"
-			    " \\qr %.4f\\cell"
-			    " \\qr %.4f\\cell \\intbl \\row\n",
-			    I_("Root"), i - p + 1, roots[i].r, roots[i].i, mod, fr);
+		if (i == p + P - 1 && q == 0 && Q == 0) {
+		    hline = 1;
+		} else {
+		    hline = 0;
 		}
+		print_root(roots[i].r, roots[i].i, mod, fr, k++, hline, prn);
+	    }
+	}
+
+	if (q > 0) {
+	    k = 1;
+	    root_start(N_("MA"), prn);
+	    for (i=p+P; i<p+P+q; i++) {
+		if (roots[i].i != 0) {
+		    mod = roots[i].r * roots[i].r + roots[i].i * roots[i].i;
+		    mod = sqrt(mod);
+		} else {
+		    mod = fabs(roots[i].r);
+		}
+		fr = atan2(roots[i].i, roots[i].r) / (2.0 * M_PI);
+		if (i == p + P + q - 1 && Q == 0) {
+		    hline = 1;
+		} else {
+		    hline = 0;
+		}
+		print_root(roots[i].r, roots[i].i, mod, fr, k++, hline, prn);
+	    }
+	}
+
+	if (Q > 0) {
+	    k = 1;
+	    root_start(N_("MA (seasonal)"), prn);
+	    for (i=p+P+q; i<p+P+q+Q; i++) {
+		if (roots[i].i != 0) {
+		    mod = roots[i].r * roots[i].r + roots[i].i * roots[i].i;
+		    mod = sqrt(mod);
+		} else {
+		    mod = fabs(roots[i].r);
+		}
+		fr = atan2(roots[i].i, roots[i].r) / (2.0 * M_PI);
+		if (i == p + P + q + Q - 1) {
+		    hline = 1;
+		} else {
+		    hline = 0;
+		}
+		print_root(roots[i].r, roots[i].i, mod, fr, k++, hline, prn);
 	    }
 	}
 
@@ -1970,7 +2087,7 @@ print_poisson_offset (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 	    pprintf(prn, "\n %13s         1.0\n", namestr);
 	} else if (rtf_format(prn)) {
 	    pputs(prn, RTF_COEFF_ROW);
-	    pprintf(prn, " \\qr \\cell\\ql %s\\cell\\qc 1.0\\cell", namestr);
+	    pprintf(prn, "\\ql %s\\cell\\qc 1.0\\cell", namestr);
 	    pputs(prn, "\\qc \\cell\\qc \\cell \\qc \\cell \\intbl \\row\n");
 	} else if (tex_format(prn)) {
 	    char tmp[32];
@@ -2040,14 +2157,13 @@ static void plain_print_act_pred (const int *ap, PRN *prn)
     pputs(prn, _("Predicted"));
     pputc(prn, '\n');
     bufspace(leftlen + 3, prn);
-    pprintf(prn, "%*d   %*d\n\n", numwidth, 0, numwidth, 1);
-    bufspace(leftlen, prn);
-    pprintf(prn, "0  %*d   %*d\n", numwidth, ap[0], numwidth, ap[1]);
+    pprintf(prn, "%*d   %*d\n", numwidth, 0, numwidth, 1);
     bufspace(2, prn);
-    pputs(prn, _("Actual"));  
-    pputc(prn, '\n');
+    pputs(prn, _("Actual"));
+    pprintf(prn, " 0  %*d   %*d\n", numwidth, ap[0], numwidth, ap[1]);
     bufspace(leftlen, prn);
     pprintf(prn, "1  %*d   %*d\n", numwidth, ap[2], numwidth, ap[3]);
+    pputc(prn, '\n');
 }
 
 static void print_discrete_statistics (const MODEL *pmod, 
