@@ -25,6 +25,8 @@
 #include "libset.h"
 #include "compat.h"
 #include "cmd_private.h"
+#include "var.h"
+#include "objstack.h"
 
 #include <time.h>
 #include <unistd.h>
@@ -40,13 +42,6 @@ typedef long double bigval;
 typedef double bigval;
 #endif
 
-enum inequalities {
-    GT = 1,
-    LT,
-    GTE,
-    LTE
-};
-
 enum loop_types {
     COUNT_LOOP,
     WHILE_LOOP,
@@ -81,15 +76,8 @@ typedef struct {
 /* below: used for special "progressive" loop */ 
 
 typedef struct {
-    int ID;                      /* ID number for model */
-    int ci;                      /* command index for model */
-    int t1, t2, nobs;            /* starting observation, ending
-                                    observation, and number of obs */
-    int ncoeff, dfn, dfd;        /* number of coefficents; degrees of
-                                    freedom in numerator and denominator */
-    int *list;                   /* list of variables by ID number */
-    int ifc;                     /* = 1 if the equation includes a constant,
-                                    else = 0 */
+    int ID;                 /* ID number for model */
+    MODEL *model0;          /* copy of initial model */
     bigval *sum_coeff;      /* sums of coefficient estimates */
     bigval *ssq_coeff;      /* sums of squares of coeff estimates */
     bigval *sum_sderr;      /* sums of estimated std. errors */
@@ -304,7 +292,6 @@ int ok_in_loop (int ci, const LOOPSET *loop)
     }
 
     /* modeling commands */
-
     else if (ci == ARMA ||
 	     ci == CORC ||
 	     ci == GARCH ||
@@ -318,10 +305,17 @@ int ok_in_loop (int ci, const LOOPSET *loop)
 	return 1;
     }
 
-    else if (ci == NLS || ci == END) {
+    /* vector models */
+    else if (ci == VAR || ci == VECM) {
+	ok = 1;
+    }
+    
+    /* nonlinear models */
+    else if (ci == NLS || ci == MLE || ci == END) {
 	ok = 1;
     }
 
+    /* basic model tests */
     else if (ci == ADD || ci == OMIT) {
 	ok = 1;
     }
@@ -329,8 +323,6 @@ int ok_in_loop (int ci, const LOOPSET *loop)
     return ok;
 }
 
-/* ......................................................  */
-
 static int loop_attach_child (LOOPSET *loop, LOOPSET *child)
 {
     LOOPSET **children;
@@ -384,13 +376,13 @@ int opstr_to_op (const char *s)
     int op = LOOP_VAL_BAD;
 
     if (strstr(s, ">=")) {
-	op = GTE;
+	op = OP_GTE;
     } else if (strstr(s, ">")) {
-	op = GT;
+	op = OP_GT;
     } else if (strstr(s, "<=")) {
-	op = LTE;
+	op = OP_LTE;
     } else if (strstr(s, "<")) {
-	op = LT;
+	op = OP_LT;
     } 
     
     return op;
@@ -778,8 +770,8 @@ static int allocate_each_strings (LOOPSET *loop, int n)
     return err;
 }
 
-static int each_strings_from_named_list (LOOPSET *loop, char *s,
-					 int *nf)
+static int each_strings_from_named_list (LOOPSET *loop, const DATAINFO *pdinfo,
+					 char *s, int *nf)
 {
     int *list;
     int err = 0;
@@ -793,6 +785,24 @@ static int each_strings_from_named_list (LOOPSET *loop, char *s,
 	err = allocate_each_strings(loop, list[0]);
     }
 
+#if 1 
+    /* when cashing out list-members, use varnames rather than numbers */
+    if (!err) {
+	int i, li;
+
+	for (i=1; i<=list[0] && !err; i++) {
+	    li = list[i];
+	    if (li < 0 || li >= pdinfo->v) {
+		err = 1;
+	    } else {
+		loop->eachstrs[i-1] = gretl_strdup(pdinfo->varname[li]);
+		if (loop->eachstrs[i-1] == NULL) {
+		    err = 1;
+		}
+	    }
+	}
+    }
+#else
     if (!err) {
 	char numstr[16];
 	int i, li;
@@ -810,6 +820,7 @@ static int each_strings_from_named_list (LOOPSET *loop, char *s,
 	    }
 	}
     }
+#endif
 
     if (err && loop->eachstrs != NULL) {
 	destroy_each_strings(loop, list[0]);
@@ -912,7 +923,7 @@ parse_as_each_loop (LOOPSET *loop, const DATAINFO *pdinfo, char *s)
 	err = each_strings_from_list_of_vars(loop, pdinfo, s, &nf);
     } else if (nf == 1) {
 	/* named list? */
-	err = each_strings_from_named_list(loop, s, &nf);
+	err = each_strings_from_named_list(loop, pdinfo, s, &nf);
     } else {
 	/* simple list of values */
 	err = allocate_each_strings(loop, nf);
@@ -1114,7 +1125,7 @@ parse_loopline (char *line, LOOPSET *ploop, int loopstack,
     if (!err && !na(loop->init.val) && !na(loop->right.val)) {
 	int nt = loop->right.val - loop->init.val;
 
-	if (nt <= 0) {
+	if (loop->type != FOR_LOOP && nt <= 0) {
 	    strcpy(gretl_errmsg, _("Loop count missing or invalid\n"));
 	    err = 1;
 	}
@@ -1193,22 +1204,22 @@ eval_numeric_condition (int op, double xl, double xr)
 #if LOOP_DEBUG
     char opstr[4];
 
-    if (op == GT) strcpy(opstr, "GT");
-    else if (op == GTE) strcpy(opstr, "GTE");
-    else if (op == LTE) strcpy(opstr, "LTE");
-    else if (op == LT) strcpy(opstr, "LT");
+    if (op == OP_GT) strcpy(opstr, "GT");
+    else if (op == OP_GTE) strcpy(opstr, "GTE");
+    else if (op == OP_LTE) strcpy(opstr, "LTE");
+    else if (op == OP_LT) strcpy(opstr, "LT");
     else strcpy(opstr, "??");
     fprintf(stderr, "** eval_numeric_condition: xl=%g, xr=%g, "
 	    "op=%s\n", xl, xr, opstr);
 #endif
 
-    if (op == GT) {
+    if (op == OP_GT) {
 	cont = (xl > xr);
-    } else if (op == GTE) {
+    } else if (op == OP_GTE) {
 	cont = (xl >= xr);
-    } else if (op == LTE) {
+    } else if (op == OP_LTE) {
 	cont = (xl <= xr);
-    } else if (op == LT) {
+    } else if (op == OP_LT) {
 	cont = (xl < xr);
     }
 
@@ -1274,8 +1285,6 @@ loop_condition (LOOPSET *loop, double **Z, DATAINFO *pdinfo)
     return ok;
 }
 
-/* ......................................................  */
-
 static void controller_init (controller *clr)
 {
     clr->val = NADBL;
@@ -1379,7 +1388,7 @@ void gretl_loop_destroy (LOOPSET *loop)
 
     if (loop->models != NULL) {
 	for (i=0; i<loop->nmod; i++) {
-	    free_model(loop->models[i]);
+	    gretl_model_free(loop->models[i]);
 	}
 	free(loop->models);
     } 
@@ -1437,24 +1446,26 @@ void gretl_loop_destroy (LOOPSET *loop)
 static int loop_model_init (LOOP_MODEL *lmod, const MODEL *pmod,
 			    int id)
 {
-    int i, ncoeff = pmod->ncoeff;
+    int i, nc = pmod->ncoeff;
 
-    lmod->sum_coeff = malloc(ncoeff * sizeof *lmod->sum_coeff);
-    if (lmod->sum_coeff == NULL) return 1;
+    lmod->model0 = gretl_model_copy(pmod);
+    if (lmod->model0 == NULL) {
+	return E_ALLOC;
+    }
 
-    lmod->ssq_coeff = malloc(ncoeff * sizeof *lmod->ssq_coeff);
+    lmod->sum_coeff = malloc(nc * sizeof *lmod->sum_coeff);
+    if (lmod->sum_coeff == NULL) return E_ALLOC;
+
+    lmod->ssq_coeff = malloc(nc * sizeof *lmod->ssq_coeff);
     if (lmod->ssq_coeff == NULL) goto cleanup;
 
-    lmod->sum_sderr = malloc(ncoeff * sizeof *lmod->sum_sderr);
+    lmod->sum_sderr = malloc(nc * sizeof *lmod->sum_sderr);
     if (lmod->sum_sderr == NULL) goto cleanup;
 
-    lmod->ssq_sderr = malloc(ncoeff * sizeof *lmod->ssq_sderr);
+    lmod->ssq_sderr = malloc(nc * sizeof *lmod->ssq_sderr);
     if (lmod->ssq_sderr == NULL) goto cleanup;
 
-    lmod->list = gretl_list_copy(pmod->list);
-    if (lmod->list == NULL) goto cleanup;
-
-    for (i=0; i<ncoeff; i++) {
+    for (i=0; i<nc; i++) {
 #ifdef ENABLE_GMP
 	mpf_init(lmod->sum_coeff[i]);
 	mpf_init(lmod->ssq_coeff[i]);
@@ -1466,12 +1477,7 @@ static int loop_model_init (LOOP_MODEL *lmod, const MODEL *pmod,
 #endif
     }
 
-    lmod->ncoeff = ncoeff;
-    lmod->t1 = pmod->t1;
-    lmod->t2 = pmod->t2;
-    lmod->nobs = pmod->nobs;
     lmod->ID = id;
-    lmod->ci = pmod->ci;
 
     return 0;
 
@@ -1480,7 +1486,7 @@ static int loop_model_init (LOOP_MODEL *lmod, const MODEL *pmod,
     free(lmod->sum_sderr);
     free(lmod->ssq_sderr);
 
-    return 1;
+    return E_ALLOC;
 }
 
 /**
@@ -1612,8 +1618,8 @@ static int add_loop_model_record (LOOPSET *loop, int cmdnum)
 
 static int add_loop_model (LOOPSET *loop)
 {
-    int err = 0;
     int nm = loop->nmod + 1;
+    int err = 0;
 
     loop->lmodels = realloc(loop->lmodels, nm * sizeof *loop->lmodels);
     if (loop->lmodels == NULL) {
@@ -1714,7 +1720,6 @@ static int add_loop_print (LOOPSET *loop, const int *list, int cmdnum)
  * Returns: 0 on successful completion.
  */
 
-
 static int update_loop_print (LOOPSET *loop, int cmdnum, 
 			      const int *list, double ***pZ, 
 			      const DATAINFO *pdinfo)
@@ -1755,7 +1760,6 @@ static int update_loop_print (LOOPSET *loop, int cmdnum,
  * @prn: gretl printing struct.
  *
  * Print out the results after completion of the loop @loop.
- *
  */
 
 static void print_loop_results (LOOPSET *loop, const DATAINFO *pdinfo, 
@@ -1807,14 +1811,12 @@ static void print_loop_results (LOOPSET *loop, const DATAINFO *pdinfo,
     }
 }
 
-/* ......................................................  */
-
 static void free_loop_model (LOOP_MODEL *lmod)
 {
 #ifdef ENABLE_GMP
     int i;
 
-    for (i=0; i<lmod->ncoeff; i++) {
+    for (i=0; i<lmod->model0->ncoeff; i++) {
 	mpf_clear(lmod->sum_coeff[i]);
 	mpf_clear(lmod->sum_sderr[i]);
 	mpf_clear(lmod->ssq_coeff[i]);
@@ -1826,10 +1828,9 @@ static void free_loop_model (LOOP_MODEL *lmod)
     free(lmod->sum_sderr);
     free(lmod->ssq_coeff);
     free(lmod->ssq_sderr);
-    free(lmod->list);
-}
 
-/* ......................................................  */
+    gretl_model_free(lmod->model0);
+}
 
 static void free_loop_print (LOOP_PRINT *lprn)
 {
@@ -1975,24 +1976,25 @@ LOOPSET *add_to_loop (char *line, int ci, gretlopt opt,
     return NULL;
 }
 
-/* ......................................................... */ 
+/* FIXME: additional model info such as robust std errs method */
 
 static void print_loop_model (LOOP_MODEL *lmod, int loopnum,
 			      const DATAINFO *pdinfo, PRN *prn)
 {
-    int i;
     char startdate[OBSLEN], enddate[OBSLEN];
+    int i;
 
-    ntodate(startdate, lmod->t1, pdinfo);
-    ntodate(enddate, lmod->t2, pdinfo);
+    ntodate(startdate, lmod->model0->t1, pdinfo);
+    ntodate(enddate, lmod->model0->t2, pdinfo);
 
     pputc(prn, '\n');
     pprintf(prn, _("%s estimates using the %d observations %s-%s\n"),
-	    _(estimator_string(lmod->ci, prn)), lmod->t2 - lmod->t1 + 1, 
+	    _(estimator_string(lmod->model0->ci, prn)), lmod->model0->nobs, 
 	    startdate, enddate);
+    print_model_vcv_info(lmod->model0, prn);
     pprintf(prn, _("Statistics for %d repetitions\n"), loopnum); 
     pprintf(prn, _("Dependent variable: %s\n\n"), 
-	    pdinfo->varname[lmod->list[1]]);
+	    pdinfo->varname[lmod->model0->list[1]]);
 
     pputs(prn, _("                     mean of      std. dev. of     mean of"
 		 "     std. dev. of\n"
@@ -2001,14 +2003,13 @@ static void print_loop_model (LOOP_MODEL *lmod, int loopnum,
 		 "      Variable     coefficients   coefficients   std. errors"
 		 "    std. errors\n\n"));
 
-    for (i=0; i<lmod->ncoeff; i++) {
+    for (i=0; i<lmod->model0->ncoeff; i++) {
 	print_loop_coeff(pdinfo, lmod, i, loopnum, prn);
     }
+
     pputc(prn, '\n');
 }
 
-/* ......................................................... */ 
-
 static void print_loop_coeff (const DATAINFO *pdinfo, 
 			      const LOOP_MODEL *lmod, 
 			      int c, int n, PRN *prn)
@@ -2045,8 +2046,8 @@ static void print_loop_coeff (const DATAINFO *pdinfo,
 	mpf_set_d(sd2, 0.0);
     }
 
-    pprintf(prn, " %3d) %8s ", lmod->list[c+2], 
-	   pdinfo->varname[lmod->list[c+2]]);
+    pprintf(prn, " %3d) %8s ", lmod->model0->list[c+2], 
+	   pdinfo->varname[lmod->model0->list[c+2]]);
 
     pprintf(prn, "%#14g %#14g %#14g %#14g\n", mpf_get_d(c1), mpf_get_d(sd1), 
 	    mpf_get_d(c2), mpf_get_d(sd2));
@@ -2067,16 +2068,14 @@ static void print_loop_coeff (const DATAINFO *pdinfo,
     var2 = (lmod->ssq_sderr[c] - n * m2 * m2) / n;
     sd2 = (var2 <= 0.0)? 0 : sqrt((double) var2);
 
-    pprintf(prn, " %3d) %8s ", lmod->list[c+2], 
-	   pdinfo->varname[lmod->list[c+2]]);
+    pprintf(prn, " %3d) %8s ", lmod->model0->list[c+2], 
+	   pdinfo->varname[lmod->model0->list[c+2]]);
 
     pprintf(prn, "%#14g %#14g %#14g %#14g\n", (double) m1, (double) sd1, 
 	    (double) m2, (double) sd2);
 #endif
 }
 
-/* ......................................................... */ 
-
 static void print_loop_prn (LOOP_PRINT *lprn, int n,
 			    const DATAINFO *pdinfo, PRN *prn)
 {
@@ -2122,8 +2121,6 @@ static void print_loop_prn (LOOP_PRINT *lprn, int n,
     pputc(prn, '\n');
 }
 
-/* ......................................................... */ 
-
 static int print_loop_store (LOOPSET *loop, PRN *prn)
 {
     int i, t;
@@ -2165,9 +2162,7 @@ static int print_loop_store (LOOPSET *loop, PRN *prn)
     fprintf(fp, "<description>\n%s\n</description>\n", xmlbuf);
     free(xmlbuf);
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
+    gretl_push_c_numeric_locale();
 
     /* print info on variables */
     fprintf(fp, "<variables count=\"%d\">\n", loop->nstore);
@@ -2204,17 +2199,13 @@ static int print_loop_store (LOOPSET *loop, PRN *prn)
 
     fprintf(fp, "</observations>\n</gretldata>\n");
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
 
     fclose(fp);
 
     return 0;
 }
 
-/* ......................................................... */ 
-
 static int get_prnnum_by_id (LOOPSET *loop, int id)
 {
     int i;
@@ -2266,6 +2257,10 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
     int idx = 0;
     int err = 0;
 
+#if LOOP_DEBUG
+    fprintf(stderr, "subst_dollar_targ:\n original: '%s'\n", str);
+#endif
+
     if (loop->type == FOR_LOOP) {
 	sprintf(targ, "$%s", pdinfo->varname[loop->left.vnum]);
 	targlen = strlen(targ);
@@ -2316,6 +2311,10 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 	free(q);	
     }
 
+#if LOOP_DEBUG
+    fprintf(stderr, " after: '%s'\n", str);
+#endif
+
     return err;
 }
 
@@ -2469,12 +2468,18 @@ make_dollar_substitutions (char *str, const LOOPSET *loop,
     return err;
 }
 
+/* Below, in loop_exec, as of October 29, 2005, the handling of the
+   multiple models is probably not right yet -- needs more work in
+   light of the new object naming/saving mechanism in gretl.
+*/
+
 int loop_exec (LOOPSET *loop, char *line,
 	       double ***pZ, DATAINFO **ppdinfo, 
 	       MODEL **models, PRN *prn)
 {
     CMD cmd;
     MODEL *lastmod = models[0];
+    GRETL_VAR *var;
     char errline[MAXLINE];
     char linecpy[MAXLINE];
     int m = 0;
@@ -2591,7 +2596,7 @@ int loop_exec (LOOPSET *loop, char *line,
 		break;
 
 	    case GENR:
-		err = generate(linecpy, pZ, *ppdinfo, lastmod, cmd.opt);
+		err = generate(linecpy, pZ, *ppdinfo, cmd.opt);
 		if (loop_is_verbose(loop) && !err) { 
 		    print_gretl_msg(prn);
 		}
@@ -2620,7 +2625,7 @@ int loop_exec (LOOPSET *loop, char *line,
 		    }
 		} 
 
-		/* estimate the model called for */
+		/* estimate the model called for, using models[0] */
 		clear_model(models[0]);
 
 		if (cmd.ci == OLS || cmd.ci == WLS || cmd.ci == HCCM) {
@@ -2643,13 +2648,13 @@ int loop_exec (LOOPSET *loop, char *line,
 		    *models[0] = lsq(cmd.list, pZ, *ppdinfo, cmd.ci, cmd.opt, rho);
 		}
 
-		if ((err = (models[0])->errcode)) {
+		if ((err = models[0]->errcode)) {
 		    break;
-		}
+		} 
 
 		if (loop_is_progressive(loop)) {
 		    if (loop->iter == 0 && loop_model_init(&loop->lmodels[loop->nmod - 1], 
-						       models[0], j)) { 
+							   models[0], j)) { 
 			gretl_errmsg_set(_("Failed to initialize model for loop\n"));
 			err = 1;
 			break;
@@ -2658,18 +2663,20 @@ int loop_exec (LOOPSET *loop, char *line,
 			err = 1;
 			break;
 		    }
-		    lastmod = models[0];
+		    set_as_last_model(models[0], EQUATION);
 		} else if (cmd.opt & OPT_P) {
 		    /* deferred printing of model results */
 		    m = get_modnum_by_cmdnum(loop, j);
-		    swap_models(&models[0], &loop->models[m]);
-		    (loop->models[m])->ID = j;
+		    swap_models(&models[0], &loop->models[m]); /* direction? */
+		    loop->models[m]->ID = j;
 		    lastmod = loop->models[m];
+		    set_as_last_model(loop->models[m], EQUATION); /* FIXME? */
 		    model_count_minus();
 		} else {
-		    (models[0])->ID = ++modnum; /* FIXME? */
+		    models[0]->ID = ++modnum; /* FIXME? */
 		    printmodel(models[0], *ppdinfo, cmd.opt, prn);
 		    lastmod = models[0];
+		    set_as_last_model(models[0], EQUATION);
 		}
 		break;
 
@@ -2695,32 +2702,36 @@ int loop_exec (LOOPSET *loop, char *line,
 		} else {
 		    swap_models(&models[0], &models[1]);
 		    lastmod = models[0];
+		    set_as_last_model(models[0], EQUATION);
 		    clear_model(models[1]);
 		}
 		break;	
 
+	    case MLE:
 	    case NLS:
 		if (loop_is_progressive(loop) || (cmd.opt & OPT_P)) {
 		    err = 1;
 		} else {
-		    err = nls_parse_line(linecpy, (const double **) *pZ, *ppdinfo);
+		    err = nls_parse_line(cmd.ci, linecpy, (const double **) *pZ, 
+					 *ppdinfo, prn);
 		    if (err) {
 			errmsg(err, prn);
 		    } else {
-			gretl_cmd_set_context(&cmd, NLS);
+			gretl_cmd_set_context(&cmd, cmd.ci);
 		    }
 		}
 		break;
 
 	    case END:
-		if (!strcmp(cmd.param, "nls")) {
+		if (!strcmp(cmd.param, "nls") || !strcmp(cmd.param, "mle")) {
 		    clear_model(models[0]);
-		    *models[0] = nls(pZ, *ppdinfo, prn);
+		    *models[0] = nls(pZ, *ppdinfo, cmd.opt, prn);
 		    if ((err = (models[0])->errcode)) {
 			errmsg(err, prn);
 		    } else {
 			printmodel(models[0], *ppdinfo, cmd.opt, prn);
 			lastmod = models[0];
+			set_as_last_model(models[0], EQUATION);
 		    }
 		} else {
 		    err = 1;
@@ -2747,7 +2758,7 @@ int loop_exec (LOOPSET *loop, char *line,
 		break;
 
 	    case PRINTF:
-		err = do_printf(linecpy, pZ, *ppdinfo, models[0], prn);
+		err = do_printf(linecpy, pZ, *ppdinfo, prn);
 		break;
 
 	    case SMPL:
@@ -2797,6 +2808,22 @@ int loop_exec (LOOPSET *loop, char *line,
 		batch_pvalue(linecpy, (const double **) *pZ, *ppdinfo, prn);
 		break;
 
+	    case VAR:
+	    case VECM:
+		if (cmd.ci == VAR) {
+		    var = full_VAR(atoi(cmd.param), cmd.list, pZ, *ppdinfo, 
+				   cmd.opt, prn);
+		} else {
+		    var = vecm(atoi(cmd.param), atoi(cmd.extra), cmd.list, 
+			   pZ, *ppdinfo, cmd.opt, prn);
+		}
+		if (var == NULL) {
+		    err = 1;
+		} else {
+		    set_as_last_model(var, VAR);
+		}
+		break;
+
 	    default: 
 		/* not reachable (since commands were screened in advance) */
 		pprintf(prn, _("command: '%s'\nThis is not available in a loop.\n"),
@@ -2834,8 +2861,8 @@ int loop_exec (LOOPSET *loop, char *line,
     }
 
     if (lastmod != models[0]) {
-	/* to get genr commands that reference model statistics --
-	   after the loop has finished -- to come out right
+	/* to get commands that reference models[0], after the loop
+	   has finished, to come out right
 	*/
 	swap_models(&models[0], &loop->models[m]);
     }
@@ -2961,7 +2988,7 @@ int if_eval (const char *line, double ***pZ, DATAINFO *pdinfo)
 
     sprintf(formula, "__iftest=%s", line);
 
-    err = generate(formula, pZ, pdinfo, NULL, OPT_P);
+    err = generate(formula, pZ, pdinfo, OPT_P);
 #if IFDEBUG
     fprintf(stderr, "if_eval: generate returned %d\n", err);
 #endif
