@@ -34,10 +34,8 @@ LOOPSET *add_to_loop (char *line, int ci, gretlopt opt,
 
 void gretl_loop_destroy (LOOPSET *loop);
 
-void get_cmd_ci (const char *line, CMD *cmd);
-
 int loop_exec (LOOPSET *loop, char *line,
 	       double ***pZ, DATAINFO **ppdinfo, 
-	       MODEL **models, int *echo_off, PRN *prn);
+	       MODEL **models, PRN *prn);
 
 #endif /* MONTE_CARLO_H */
