@@ -22,7 +22,7 @@
 
 typedef struct _nls_spec nls_spec;
 
-nls_spec *nls_spec_new (const DATAINFO *pdinfo);
+nls_spec *nls_spec_new (int ci, const DATAINFO *pdinfo);
 
 void nls_spec_destroy (nls_spec *spec);
 
@@ -39,13 +39,14 @@ nls_spec_set_regression_function (nls_spec *spec,
 
 void nls_spec_set_t1_t2 (nls_spec *spec, int t1, int t2);
 
-int nls_parse_line (const char *line, const double **Z,
-		    const DATAINFO *pdinfo);
+int nls_parse_line (int ci, const char *line, const double **Z,
+		    const DATAINFO *pdinfo, PRN *prn);
 
-MODEL nls (double ***pZ, DATAINFO *pdinfo, PRN *prn);
+MODEL nls (double ***pZ, DATAINFO *pdinfo, gretlopt opt, PRN *prn);
 
 MODEL model_from_nls_spec (nls_spec *spec, double ***pZ, 
-			   DATAINFO *pdinfo, PRN *prn);
+			   DATAINFO *pdinfo, gretlopt opt, 
+			   PRN *prn);
 
 double get_default_nls_toler (void);
 
