@@ -33,6 +33,7 @@
                        c == LOGIT || \
                        c == OLS || \
                        c == OMIT || \
+                       c == MLE || \
                        c == NLS || \
                        c == POISSON || \
                        c == POOLED || \
@@ -59,9 +60,13 @@ struct flag_match {
 */
 
 struct gretl_option gretl_opts[] = {
+    { ADD,      OPT_B, "both" },
     { ADD,      OPT_I, "silent" },
     { ADD,      OPT_Q, "quiet" },
+    { ADD,      OPT_T, "inst" },
+    { ADDTO,    OPT_B, "both" },
     { ADDTO,    OPT_Q, "quiet" },
+    { ADDTO,    OPT_T, "inst" },
     { ADF,      OPT_N, "nc" }, 
     { ADF,      OPT_C, "c" }, 
     { ADF,      OPT_R, "ctt" },     
@@ -69,11 +74,16 @@ struct gretl_option gretl_opts[] = {
     { ADF,      OPT_V, "verbose" },
     { ADF,      OPT_Q, "quiet" },
     { ARMA,     OPT_N, "native" },
-    { ARMA,     OPT_S, "suppress-const" },    
+    { ARMA,     OPT_S, "nc" },    
     { ARMA,     OPT_V, "verbose" },
     { ARMA,     OPT_X, "x-12-arima" },
     { BXPLOT,   OPT_O, "notches" },
     { COINT,    OPT_N, "nc" },
+    { COINT2,   OPT_A, "crt" },
+    { COINT2,   OPT_D, "seasonals" },
+    { COINT2,   OPT_N, "nc" },
+    { COINT2,   OPT_R, "rc" },
+    { COINT2,   OPT_T, "ct" },
     { COINT2,   OPT_V, "verbose" },
     { EQNPRINT, OPT_O, "complete" },
     { TABPRINT, OPT_O, "complete" },
@@ -111,21 +121,28 @@ struct gretl_option gretl_opts[] = {
     { LMTEST,   OPT_S, "squares" }, 
     { LMTEST,   OPT_P, "panel" },
     { LMTEST,   OPT_W, "white" },
+    { LOGIT,    OPT_R, "robust" },
     { LOOP,     OPT_P, "progressive" },
     { LOOP,     OPT_Q, "quiet" },
     { LOOP,     OPT_V, "verbose" },
     { MAHAL,    OPT_S, "save" },
     { MAHAL,    OPT_V, "vcv" },
     { MEANTEST, OPT_O, "unequal-vars" },
+    { MLE,      OPT_V, "verbose" },
+    { NLS,      OPT_V, "verbose" },
     { OLS,      OPT_N, "no-df-corr" },
     { OLS,      OPT_O, "vcv" }, 
     { OLS,      OPT_P, "print-final" },
     { OLS,      OPT_R, "robust" },
     { OLS,      OPT_Q, "quiet" },
     { OLS,      OPT_S, "simple-print" },
+    { OMIT,     OPT_B, "both" },
     { OMIT,     OPT_I, "silent" },
     { OMIT,     OPT_Q, "quiet" },
+    { OMIT,     OPT_T, "inst" },
+    { OMITFROM, OPT_B, "both" },
     { OMITFROM, OPT_Q, "quiet" },
+    { OMITFROM, OPT_T, "inst" },
     { OUTFILE,  OPT_A, "append" },
     { OUTFILE,  OPT_C, "close" },
     { OUTFILE,  OPT_W, "write" },
@@ -142,6 +159,7 @@ struct gretl_option gretl_opts[] = {
     { PRINT,    OPT_O, "byobs" },
     { PRINT,    OPT_N, "no-dates" },
     { PRINT,    OPT_T, "ten" },
+    { PROBIT,   OPT_R, "robust" },
     { QUIT,     OPT_X, "exit" },
     { SETOBS,   OPT_C, "stacked-cross-section" },
     { SETOBS,   OPT_S, "stacked-time-series" },
@@ -163,12 +181,26 @@ struct gretl_option gretl_opts[] = {
     { STORE,    OPT_R, "gnu-R" },
     { STORE,    OPT_T, "traditional" },
     { STORE,    OPT_Z, "gzipped" },
+    { STORE,    OPT_X, "omit-obs" },
     { TOBIT,    OPT_V, "verbose" },
     { TSLS,     OPT_R, "robust" },  
     { TSLS,     OPT_S, "save" },
-    { VAR,      OPT_V, "impulse-responses" },
-    { VAR,      OPT_Q, "quiet" },  
+    { VAR,      OPT_D, "seasonals" },
+    { VAR,      OPT_F, "variance-decomp" },
+    { VAR,      OPT_I, "impulse-responses" },
+    { VAR,      OPT_N, "nc" },
+    { VAR,      OPT_Q, "quiet" }, 
     { VAR,      OPT_R, "robust" }, 
+    { VECM,     OPT_A, "crt" },
+    { VECM,     OPT_D, "seasonals" },
+    { VECM,     OPT_F, "variance-decomp" },
+    { VECM,     OPT_I, "impulse-responses" },
+    { VECM,     OPT_N, "nc" },
+    { VECM,     OPT_Q, "quiet" },
+    { VECM,     OPT_R, "rc" },
+    { VECM,     OPT_T, "ct" },
+    { VECM,     OPT_V, "verbose" },
+    { WLS,      OPT_R, "robust" },    
     { WLS,      OPT_Q, "quiet" },
     { 0,        0L,    NULL }
 };
@@ -371,6 +403,7 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 	} else if (is_long_opt(longopt)) {
 	    /* recognized option, but not valid for the command */
 	    sprintf(gretl_errmsg, "Invalid option '--%s'", longopt);
+	    fprintf(stderr, " line='%s', ci = %d\n", line, ci);
 	    *err = 1;
 	    return 0L;
 	} else {
@@ -448,6 +481,8 @@ gretlopt get_gretl_options (char *line, int *err)
 
     if (strstr(line, "end nls")) {
 	ci = NLS;
+    } else if (strstr(line, "end mle")) {
+	ci = MLE;
     } else {
 	ci = gretl_command_number(cmdword);
     }
@@ -492,8 +527,8 @@ gretlopt get_gretl_options (char *line, int *err)
 
 const char *print_flags (gretlopt oflags, int ci)
 {
-    static char flagstr[64];
-    char fbit[20];
+    static char flagstr[256];
+    char fbit[32];
     int i;
 
     flagstr[0] = '\0';
