@@ -32,9 +32,6 @@ enum {
     P_LAD,
     P_VIF,
     P_LEVERAGE,
-#ifdef WIN32
-    P_LONGNAME,
-#endif
     P_MP_OLS,
     P_PANEL_DATA,
     P_PCA,
@@ -46,14 +43,16 @@ enum {
     P_NISTCHECK,
     P_ARMA,
     P_ARMA_X12,
-    P_LOGISTIC,
     P_TOBIT,
     P_GARCH,
     P_AUDIO,
     P_URCDIST,
     P_KERNEL,
     P_FRACTAL,
-    P_POISSON
+    P_POISSON,
+    P_MAILER,
+    P_EVIEWS_IMPORT,
+    P_STATA_IMPORT
 } plugin_codes;
 
 struct plugin_info {
@@ -74,9 +73,6 @@ struct plugin_info plugins[] = {
     { P_LAD,             "lad" },
     { P_VIF,             "vif" },
     { P_LEVERAGE,        "leverage" },
-#ifdef WIN32
-    { P_LONGNAME,        "longname" },
-#endif
     { P_MP_OLS,          "mp_ols" },
     { P_PANEL_DATA,      "panel_data" },
     { P_PCA,             "pca" },
@@ -88,23 +84,29 @@ struct plugin_info plugins[] = {
     { P_NISTCHECK,       "nistcheck" },
     { P_ARMA,            "arma" },
     { P_ARMA_X12,        "arma_x12" },
-    { P_LOGISTIC,        "logistic" },
     { P_TOBIT,           "tobit" },
     { P_GARCH,           "garch" },
     { P_AUDIO,           "audio" },
     { P_URCDIST,         "urcdist" },
     { P_KERNEL,          "kernel" },
     { P_FRACTAL,         "fractals" },
-    { P_POISSON,         "poisson" }
+    { P_POISSON,         "poisson" },
+    { P_MAILER,          "mailer" },
+    { P_EVIEWS_IMPORT,   "eviews_import" },
+    { P_STATA_IMPORT,    "stata_import" },
 };  
 
 struct plugin_function plugin_functions[] = { 
     /* data importers */
     { "excel_get_data", P_EXCEL_IMPORT },
     { "wbook_get_data", P_GNUMERIC_IMPORT },
+    { "wf1_get_data",   P_EVIEWS_IMPORT },
+    { "dta_get_data",   P_STATA_IMPORT },
 
-    /* Johansen cointegration test */
-    { "johansen_eigenvals", P_JOHANSEN },
+    /* Johansen cointegration test and VECM */
+    { "johansen_analysis",        P_JOHANSEN },
+    { "johansen_bootstrap_round", P_JOHANSEN },
+    { "vecm_beta_test",           P_JOHANSEN },
 
     /* least absolute deviations */
     { "lad_driver", P_LAD },
@@ -153,14 +155,9 @@ struct plugin_function plugin_functions[] = {
     /* NIST test suite */
     { "run_nist_tests", P_NISTCHECK },    
 
-#ifdef WIN32
-    { "real_unmangle", P_LONGNAME },
-#endif
-    
     /* modeling */
     { "arma_model",        P_ARMA },
     { "arma_x12_model",    P_ARMA_X12 },
-    { "logistic_estimate", P_LOGISTIC },
     { "tobit_estimate",    P_TOBIT },
     { "garch_model",       P_GARCH },
     { "poisson_estimate",  P_POISSON },
@@ -178,6 +175,9 @@ struct plugin_function plugin_functions[] = {
     /* Hurst exponent estimation */
     { "hurst_exponent",    P_FRACTAL },
 
+    /* Send email */
+    { "email_file",    P_MAILER },
+
     /* sentinel */
     { NULL, 0 }
 };
