@@ -23,6 +23,7 @@
 #include "version.h"
 #include "libset.h"
 #include "forecast.h"
+#include "gretl_func.h"
 
 #include <time.h>
 
@@ -91,7 +92,7 @@ void session_time (PRN *prn)
 
 void logo (void)
 {
-    printf(_("gretl version %s\n"), version_string);
+    printf(_("gretl version %s\n"), GRETL_VERSION);
     puts(_("Copyright Ramu Ramanathan and Allin Cottrell"));
     puts(_("This is free software with ABSOLUTELY NO WARRANTY"));
 }
@@ -113,7 +114,7 @@ void gui_logo (PRN *prn)
 	prn = myprn;
     }
 	
-    pprintf(prn, _("gretl: gui client for gretl version %s,\n"), version_string);
+    pprintf(prn, _("gretl: gui client for gretl version %s,\n"), GRETL_VERSION);
     pputs(prn, _("copyright Allin Cottrell.\n"));
     pputs(prn, _("This is free software with ABSOLUTELY NO WARRANTY.\n"));
 
@@ -144,7 +145,7 @@ void gui_script_logo (PRN *prn)
 {
     time_t runtime = time(NULL);
 
-    pprintf(prn, _("gretl version %s\n"), version_string);
+    pprintf(prn, _("gretl version %s\n"), GRETL_VERSION);
     pprintf(prn, "%s: %s\n", _("Current session"), print_time(&runtime));
 }
 
@@ -430,7 +431,7 @@ static void cut_extra_zero (char *numstr, int digits)
    decimal point will be printed in the same position for all
    numbers printed this way.  The total width of the number
    string (including possible padding on left or right) is 
-   2*P + 5 characters, where P denotes the precision ("digits"). 
+   2 * P + 5 characters, where P denotes the precision ("digits"). 
 */
 
 void gretl_print_fullwidth_double (double x, int digits, PRN *prn)
@@ -455,7 +456,11 @@ void gretl_print_fullwidth_double (double x, int digits, PRN *prn)
     p = strchr(numstr, decpoint);
     if (p != NULL) {
 	forept = p - numstr;
+    } else {
+	/* handle case of no decimal point, added Sept 2, 2005 */
+	forept = strlen(numstr);
     }
+
     tmp = digits + 1 - forept;
     *final = 0;
     for (i=0; i<tmp; i++) {
@@ -484,6 +489,53 @@ void gretl_print_value (double x, PRN *prn)
     gretl_print_fullwidth_double(x, GRETL_DIGITS, prn);  
 }
 
+/**
+ * print_contemporaneous_covariance_matrix:
+ * @m: covariance matrix.
+ * @ldet: log-determinant of @m.
+ * @prn: gretl printing struct.
+ * 
+ * Print to @prn the covariance matrix @m, with correlations
+ * above the diagonal, and followed by the log determinant.
+ */
+
+void
+print_contemp_covariance_matrix (const gretl_matrix *m, 
+				 double ldet, PRN *prn)
+{
+    int rows = gretl_matrix_rows(m);
+    int cols = gretl_matrix_cols(m);
+    int jmax = 1;
+    char numstr[16];
+    double x;
+    int i, j;
+
+    pprintf(prn, "%s\n(%s)\n\n",
+	    _("Cross-equation VCV for residuals"),
+	    _("correlations above the diagonal"));
+
+    for (i=0; i<rows; i++) {
+	for (j=0; j<jmax; j++) {
+	    pprintf(prn, "%#13.5g", gretl_matrix_get(m, i, j));
+	}
+	for (j=jmax; j<cols; j++) {
+	    x = gretl_matrix_get(m, i, i) * gretl_matrix_get(m, j, j);
+	    x = sqrt(x);
+	    x = gretl_matrix_get(m, i, j) / x;
+	    sprintf(numstr,"(%.3f)", x); 
+	    pprintf(prn, "%13s", numstr);
+	}
+	pputc(prn, '\n');
+	if (jmax < cols) {
+	    jmax++;
+	}
+    }
+
+    if (!na(ldet)) {
+	pprintf(prn, "\n%s = %g\n", _("log determinant"), ldet);
+    }
+}
+
 /**
  * outcovmx:
  * @pmod: pointer to model.
@@ -696,22 +748,30 @@ static void fit_resid_head (const FITRESID *fr,
     pputs(prn, "\n\n");
 }
 
-/*  skips to new page and prints names of variables
-    from v1 to v2 */
+/* prints names of variables in @list, positions v1 to v2 */
 
-static void varheading (int v1, int v2, 
-			const DATAINFO *pdinfo, const int *list,
-			PRN *prn)
+static void varheading (const int *list, int v1, int v2, 
+			const DATAINFO *pdinfo, PRN *prn)
 {
     int i;
-        
-    pputs(prn, "\n     Obs ");
 
-    for (i=v1; i<=v2; i++) { 
-	pprintf(prn, "%13s", pdinfo->varname[list[i]]);
+    if (csv_format(prn)) {
+	pputs(prn, "Obs");
+	pputc(prn, pdinfo->delim);
+	for (i=v1; i<=v2; i++) { 
+	    pprintf(prn, "%s", pdinfo->varname[list[i]]);
+	    if (i < v2) {
+		pputc(prn, pdinfo->delim);
+	    } 
+	}
+	pputc(prn, '\n');
+    } else {
+	pputs(prn, "\n     Obs ");
+	for (i=v1; i<=v2; i++) { 
+	    pprintf(prn, "%13s", pdinfo->varname[list[i]]);
+	}
+	pputs(prn, "\n\n");
     }
-
-    pputs(prn, "\n\n");
 }
 
 /**
@@ -1031,6 +1091,39 @@ static int bufprintnum (char *buf, double x, int signif, int width)
     return 0;
 }
 
+/**
+ * obs_marker_init:
+ * @pdinfo: data information struct.
+ *
+ * Check the length to which observation markers should
+ * be printed, in a tabular context.  (We don't want to
+ * truncate 10-character date strings by chopping off the day.)
+ */
+
+static int oprintlen = 8;
+
+void obs_marker_init (const DATAINFO *pdinfo)
+{
+    int t, datestrs = 0;
+
+    if (pdinfo->markers) {
+	for (t=0; t<pdinfo->n; t++) {
+	    if (strlen(pdinfo->S[t]) == 10 && 
+		isdigit(pdinfo->S[t][0]) &&
+		strchr(pdinfo->S[t], '/')) {
+		datestrs = 1;
+		break;
+	    }
+	}
+    } 
+
+    if (datestrs) {
+	oprintlen = 10;
+    } else {
+	oprintlen = 8;
+    }
+}
+
 /**
  * print_obs_marker:
  * @t: observation number.
@@ -1038,21 +1131,82 @@ static int bufprintnum (char *buf, double x, int signif, int width)
  * @prn: gretl printing struct.
  *
  * Print a string (label, date or obs number) representing the given @t.
- *
  */
 
 void print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn)
 {
-    if (pdinfo->markers) { 
-	pprintf(prn, "%8s ", pdinfo->S[t]); 
-    } else {
-	char tmp[OBSLEN]; 
+    char tmp[OBSLEN] = {0};
 
+    if (pdinfo->markers) { 
+	strncat(tmp, pdinfo->S[t], oprintlen);
+	pprintf(prn, "%*s ", oprintlen, tmp); 
+    } else {
 	ntodate(tmp, t, pdinfo);
 	pprintf(prn, "%8s ", tmp);
     }
 }
 
+/**
+ * varlist:
+ * @pdinfo: data information struct.
+ * @prn: gretl printing struct
+ *
+ * Prints a list of the names of the variables currently defined.
+ */
+
+void varlist (const DATAINFO *pdinfo, PRN *prn)
+{
+    int level = 0;
+    int i, j, n;
+
+    if (gretl_executing_function()) {
+	level = gretl_function_stack_depth();
+
+	n = 0;
+	for (i=0; i<pdinfo->v; i++) {
+	    if (STACK_LEVEL(pdinfo, i) == level) {
+		n++;
+	    }
+	}
+    } else {
+	n = pdinfo->v;
+    }
+
+    pprintf(prn, _("Listing %d variables:\n"), n);
+
+    j = 1;
+    for (i=0; i<pdinfo->v; i++) {
+	if (level > 0 && STACK_LEVEL(pdinfo, i) != level) {
+	    continue;
+	}
+	pprintf(prn, "%3d) %-10s", i, pdinfo->varname[i]);
+	if (j % 5 == 0) {
+	    pputc(prn, '\n');
+	}
+	j++;
+    }
+
+    if (n % 5) pputc(prn, '\n');
+
+    pputc(prn, '\n');
+}
+
+/**
+ * maybe_list_vars:
+ * @pdinfo: data information struct.
+ * @prn: gretl printing struct
+ *
+ * Prints a list of the names of the variables currently defined,
+ * unless gretl messaging is turned off.
+ */
+
+void maybe_list_vars (const DATAINFO *pdinfo, PRN *prn)
+{
+    if (gretl_messages_on()) {
+	varlist(pdinfo, prn);
+    }
+}
+
 /* See if there is a variable that is an outcome of sorting,
    and has sorted case-markers attached to it.  If so,
    we'll arrange to print the case-markers in the correct
@@ -1243,7 +1397,7 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
 	    /* starting a new block of variables */
 	    v2 = (ncol > nvj5)? nvj5 : ncol;
 	    v2 += j5;
-	    varheading(v1, v2, pdinfo, plist, prn);
+	    varheading(plist, v1, v2, pdinfo, prn);
 
 	    if (pause && j > 0 && takenotes(1)) {
 		goto endprint;
@@ -1307,6 +1461,111 @@ int printdata (const int *list, const double **Z, const DATAINFO *pdinfo,
     return err;
 }
 
+/**
+ * print_data_sorted:
+ * @list: list of variables to print.
+ * @obsvec: list of observation numbers.
+ * @Z: data matrix.
+ * @pdinfo: data information struct.
+ * @prn: gretl printing struct.
+ *
+ * Print the data for the variables in @list, using the sort order 
+ * given in @obsvec.  The first element of @obsvec must contain the
+ * number of observations that follow.  By default, printing is plain 
+ * text, formatted in columns using space characters, but if the @prn 
+ * format is set to %GRETL_FORMAT_CSV then printing respects the user's 
+ * choice of column delimiter.
+ *
+ * Returns: 0 on successful completion, non-zero code on error.
+ */
+
+int print_data_sorted (const int *list, const int *obsvec, 
+		       const double **Z, const DATAINFO *pdinfo, 
+		       PRN *prn)
+{
+    char sdelim[2] = {0};
+    int csv = csv_format(prn);
+    int *pmax = NULL; 
+    double xx;
+    char obs_string[OBSLEN];
+    char line[128];
+    int T = obsvec[0];
+    int i, s, t;
+
+    /* must have a list of up to 5 variables... */
+    if (list == NULL || list[0] > 5) {
+	return E_DATA;
+    }
+
+    /* ...with no scalars or bad variable numbers */
+    for (i=1; i<=list[0]; i++) {
+	if (list[i] >= pdinfo->v || !pdinfo->vector[list[i]]) {
+	    return E_DATA;
+	}
+    }
+
+    /* and T must be in bounds */
+    if (T > pdinfo->n - pdinfo->t1) {
+	return E_DATA;
+    }
+
+    pmax = malloc(list[0] * sizeof *pmax);
+    if (pmax == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=1; i<=list[0]; i++) {
+	pmax[i-1] = get_signif(Z[list[i]] + pdinfo->t1, T);
+    }
+
+    varheading(list, 1, list[0], pdinfo, prn);
+
+    if (csv) {
+	sdelim[0] = pdinfo->delim;
+    }
+
+    /* print data by observations */
+    for (s=0; s<T; s++) {
+	t = obsvec[s+1];
+	if (t >= pdinfo->n) {
+	    continue;
+	}
+	get_obs_string(obs_string, t, pdinfo);
+	if (csv) {
+	    sprintf(line, "%s", obs_string);
+	    strcat(line, sdelim);
+	} else {
+	    sprintf(line, "%8s ", obs_string);
+	}
+	for (i=1; i<=list[0]; i++) {
+	    xx = Z[list[i]][t];
+	    if (na(xx)) {
+		if (csv) {
+		    strcat(line, "NA");
+		} else {
+		    strcat(line, "             ");
+		}
+	    } else { 
+		if (csv) {
+		    bufprintnum(line, xx, pmax[i-1], 0);
+		} else {
+		    bufprintnum(line, xx, pmax[i-1], 13);
+		}
+	    }
+	    if (csv && i < list[0]) {
+		strcat(line, sdelim);
+	    }
+	}
+	pputs(prn, line);
+	pputc(prn, '\n');
+    } 
+
+    pputc(prn, '\n');
+    free(pmax);
+
+    return 0;
+}
+
 int
 text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 {
@@ -1315,6 +1574,8 @@ text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
 
     fit_resid_head(fr, pdinfo, prn); 
 
+    obs_marker_init(pdinfo);
+
     for (t=0; t<fr->nobs; t++) {
 	print_obs_marker(t + fr->t1, pdinfo, prn);
 
@@ -1390,7 +1651,7 @@ int text_print_forecast (const FITRESID *fr,
     pputc(prn, '\n');
 
     if (do_errs) {
-	if (fr->model_ci == ARMA) {
+	if (fr->model_ci == ARMA || fr->model_ci == VECM) {
 	    pprintf(prn, _(" For 95%% confidence intervals, z(.025) = %.2f\n"), 
 		    1.96);
 	} else {
@@ -1421,6 +1682,8 @@ int text_print_forecast (const FITRESID *fr,
 	}
     }
 
+    obs_marker_init(pdinfo);
+
     for (t=fr->pre_n; t<fr->nobs; t++) {
 	print_obs_marker(t + fr->t1, pdinfo, prn);
 	fcast_print_x(fr->actual[t], 15, pmax, prn);
@@ -1429,6 +1692,7 @@ int text_print_forecast (const FITRESID *fr,
 	    pputc(prn, '\n');
 	    continue;
 	}
+
 	fcast_print_x(fr->fitted[t], 15, pmax, prn);
 
 	if (do_errs) {
@@ -1659,7 +1923,7 @@ static char *date_string (const char *s, double **Z, const DATAINFO *pdinfo)
     char *ret = NULL;
     int v, t = -1;
 
-    if (sscanf(s, "date(%15[^)])", darg)) {
+    if (!strncmp(s, "date(", 5) && sscanf(s + 5, "%15[^)])", darg)) {
 	if (isdigit(*darg)) {
 	    t = atoi(darg);
 	} else {
@@ -1700,8 +1964,7 @@ static int get_conversion (const char *s, int *skip)
 }
 
 static int real_do_printf (const char *line, double ***pZ, 
-			   DATAINFO *pdinfo, MODEL *pmod,
-			   PRN *prn, int t)
+			   DATAINFO *pdinfo, PRN *prn, int t)
 {
     const char *p;
     char format[128];
@@ -1841,8 +2104,7 @@ static int real_do_printf (const char *line, double ***pZ,
 		    xvals[i] = (*pZ)[v][0];
 		}
 	    } else {
-		err = get_generated_value(argv, &xvals[i], pZ, pdinfo, 
-					  pmod, t);
+		err = get_generated_value(argv, &xvals[i], pZ, pdinfo, t);
 	    }
 	}
 
@@ -1900,10 +2162,9 @@ static int real_do_printf (const char *line, double ***pZ,
 }
 
 int do_printf (const char *line, double ***pZ, 
-	       DATAINFO *pdinfo, MODEL *pmod,
-	       PRN *prn)
+	       DATAINFO *pdinfo, PRN *prn)
 {
-    return real_do_printf(line, pZ, pdinfo, pmod, prn, -1);
+    return real_do_printf(line, pZ, pdinfo, prn, -1);
 }
 
 /* originating command is of form:
@@ -1932,7 +2193,7 @@ int generate_obs_markers (double ***pZ, DATAINFO *pdinfo, char *s)
 
 	for (t=0; t<pdinfo->n && !err; t++) {
 	    gretl_print_reset_buffer(prn);
-	    err = real_do_printf(s, pZ, pdinfo, NULL, prn, t);
+	    err = real_do_printf(s, pZ, pdinfo, prn, t);
 	    if (!err) {
 		buf = gretl_print_get_buffer(prn);
 		pdinfo->S[t][0] = '\0';
@@ -1977,9 +2238,9 @@ int in_usa (void)
  * @buf: source buffer
  *
  * This function (which works rather like fgets) must be initialized 
- * via the call: bufgets(NULL, 0, buf);
+ * via the call: bufgets_init(buf);
  * 
- * Returns: @s (%NULL if nothing more can be read from @buf).
+ * Returns: @s (or %NULL if nothing more can be read from @buf).
  */
 
 char *bufgets (char *s, size_t size, const char *buf)
@@ -1996,7 +2257,7 @@ char *bufgets (char *s, size_t size, const char *buf)
     /* mechanism for resetting p */
     if (s == NULL || size == 0) {
 	p = NULL;
-	return 0;
+	return NULL;
     }
 
     /* start at beginning of buffer */
@@ -2042,3 +2303,15 @@ char *bufgets (char *s, size_t size, const char *buf)
 
     return s;
 }
+
+/**
+ * bufgets_init:
+ * @buf: source buffer.
+ *
+ * Initializes a text buffer for use with bufgets(). 
+ */
+
+void bufgets_init (const char *buf)
+{
+    bufgets(NULL, 0, buf);
+}
