@@ -45,13 +45,27 @@ void printcorr (const VMatrix *corrmat, PRN *prn);
 void print_smpl (const DATAINFO *pdinfo, 
 		 int fulln, PRN *prn); 
 
+void
+print_contemp_covariance_matrix (const gretl_matrix *m, 
+				 double ldet, PRN *prn);
+
 int outcovmx (MODEL *pmod, const DATAINFO *pdinfo, PRN *prn);
 
+void obs_marker_init (const DATAINFO *pdinfo);
+
 void print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn);
 
+void varlist (const DATAINFO *pdinfo, PRN *prn);
+
+void maybe_list_vars (const DATAINFO *pdinfo, PRN *prn);
+
 int printdata (const int *list, const double **Z, const DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn);
 
+int print_data_sorted (const int *list, const int *obsvec, 
+		       const double **Z, const DATAINFO *pdinfo, 
+		       PRN *prn);
+
 int text_print_fit_resid (const FITRESID *fr, 
 			  const DATAINFO *pdinfo, 
 			  PRN *prn);
@@ -77,8 +91,7 @@ void bufspace (int n, PRN *prn);
 void gretl_printxn (double x, int n, PRN *prn);
 
 int do_printf (const char *line, double ***pZ, 
-	       DATAINFO *pdinfo, MODEL *pmod,
-	       PRN *prn);
+	       DATAINFO *pdinfo, PRN *prn);
 
 int generate_obs_markers (double ***pZ, DATAINFO *pdinfo, char *s);
 
@@ -86,6 +99,8 @@ int in_usa (void);
 
 char *bufgets (char *s, size_t size, const char *buf);
 
+void bufgets_init (const char *buf);
+
 void scroll_pause (void);
 
 int scroll_pause_or_quit (void);
