@@ -35,6 +35,40 @@ static void getdf (const char *str);
 
 const char negval[] = N_("\nEnter x value (value < 0 will exit menu): "); 
 
+/**
+ * binomial_cdf:
+ * @k: maximum number of successes.
+ * @n: number of trials.
+ * @p: probability of success on each trial.
+ *
+ * Returns: the probability of @k or less successes on
+ * @n trials given binomial probability @p.
+ */
+
+double binomial_cdf (int k, int n, double p)
+{
+    double x = bdtr(k, n, p);
+
+    return (isnan(x))? NADBL : x;
+}
+
+/**
+ * binomial_pvalue:
+ * @k: number of successes.
+ * @n: number of trials.
+ * @p: probability of success on each trial.
+ *
+ * Returns: the probability of @k + 1 or more successes on
+ * @n trials given binomial probability @p.
+ */
+
+double binomial_pvalue (int k, int n, double p)
+{
+    double x = bdtrc(k, n, p);
+
+    return (isnan(x))? NADBL : x;
+}
+
 /**
  * x_factorial:
  * @x: input value.
@@ -257,6 +291,82 @@ static double get_number_or_val (const char *s,
     return NADBL;
 }
 
+static int val_from_string (const char *s, const double **Z,
+			    const DATAINFO *pdinfo, 
+			    double *px, int *pn)
+{
+    double x = NADBL;
+    int n = 0, err = 0;
+
+    if (isalpha((unsigned char) *s)) {
+	int v = varindex(pdinfo, s);
+
+	if (v < pdinfo->v) {
+	    if (pdinfo->vector[v]) {
+		x = Z[v][pdinfo->t1];
+	    } else {
+		x = Z[v][0];
+	    }
+	    if (na(x)) {
+		strcpy(gretl_errmsg, _("Missing values encountered"));
+		err = 1;
+	    } else {
+		n = (int) x;
+	    }
+	} else {
+	    sprintf(gretl_errmsg, _("Unknown variable '%s'"), s);
+	    err = 1;
+	}
+    } else if (*s != '\0') {
+	if (check_atof(s)) {
+	    err = 1;
+	} else {
+	    n = atoi(s);
+	    x = atof(s);
+	}
+    }	
+
+    *px = x;
+    *pn = n;
+
+    return err;
+}
+
+static char normalize_stat (char c)
+{
+    switch (c) {
+    case '1':
+    case 'z':
+    case 'n':
+    case 'N':
+	return 'z'; /* Normal */
+    case '2':
+    case 't':
+	return 't'; /* Student's t */
+    case '3':
+    case 'c':
+    case 'x':
+    case 'X':
+	return 'X'; /* Chi-square */
+    case '4':
+    case 'f':
+    case 'F':
+	return 'F'; /* F */
+    case '5':
+    case 'g':
+    case 'G':
+	return 'G'; /* Gamma */
+    case '6':
+    case 'b':
+    case 'B':
+	return 'B'; /* Binomial */
+    default:
+	break;
+    }
+
+    return 0;
+}
+
 /**
  * batch_pvalue:
  * @str: the command line, which should be of one of the following forms:
@@ -265,117 +375,73 @@ static double get_number_or_val (const char *s,
  * pvalue 3 df x (Chi-square);
  * pvalue 4 dfn dfd x (F-distribution); or
  * pvalue 5 mean variance x (Gamma distribution).
+ * pvalue 6 prob n x (Binomial distribution).
  * @Z: the data matrix.
  * @pdinfo: data information struct.
  * @prn: gretl printing struct.
  * 
  * Returns: the probability that a random variable distributed as
  * specified in the command line @str exceeds the value indicated
- * in @str, or a negative number in case of failure.
- *
+ * in @str, or #NADBL in case of failure.
  */
 
 double batch_pvalue (const char *str, 
 		     const double **Z, const DATAINFO *pdinfo, 
                      PRN *prn)
 {
-    int i, df1 = 0, df2 = 0;
-    char stat = 0;
-    double xx = NADBL, mean = 0, variance = 0, xval = 0, tmp;
-    char cmd[7], df1str[9], df2str[9], fstr[9]; 
-    int gotvar, err = 0;
+    int n1 = 0, n2 = 0, n3 = 0;
+    double x1 = 0, x2 = 0, x3 = 0;
+    char st = 0;
+    double tmp, pv = NADBL;
+    char s1[9] = {0};
+    char s2[9] = {0};
+    char s3[9] = {0};
+    char cmd[7];
+    int err = 0;
 
     for (;;) {
-	if (sscanf(str, "%c,%[^,],%[^,],%s", &stat, df1str, df2str, fstr) == 4)
+	if (sscanf(str, "%c,%[^,],%[^,],%s", &st, s1, s2, s3) == 4) {
 	    break;
-	else *df1str = *df2str = *fstr = '\0';
-	if (sscanf(str, "%c,%[^,],%s", &stat, df1str, fstr) == 3)
+	}
+	*s1 = *s2 = *s3 = '\0';
+	if (sscanf(str, "%c,%[^,],%s", &st, s1, s3) == 3) {
 	    break;
-	else *df1str = *df2str = *fstr = '\0';
-	if (sscanf(str, "%c,%s", &stat, fstr) == 2)
+	} 
+	*s1 = *s2 = *s3 = '\0';
+	if (sscanf(str, "%c,%s", &st, s3) == 2) {
 	    break;
-	else *df1str = *df2str = *fstr = '\0';
-	if (sscanf(str, "%s %c %s %s %s", cmd, &stat, df1str, df2str, fstr) == 5)
+	} 
+	*s1 = *s2 = *s3 = '\0';
+	if (sscanf(str, "%s %c %s %s %s", cmd, &st, s1, s2, s3) == 5) {
 	    break;
-	else *df1str = *df2str = *fstr = '\0';
-	if (sscanf(str, "%s %c %s %s", cmd, &stat, df1str, fstr) == 4)
+	} 
+	*s1 = *s2 = *s3 = '\0';
+	if (sscanf(str, "%s %c %s %s", cmd, &st, s1, s3) == 4) {
 	    break;
-	else *df1str = *df2str = *fstr = '\0';
-	if (sscanf(str, "%s %c %s", cmd, &stat, fstr) == 3)
+	} 
+	*s1 = *s2 = *s3 = '\0';
+	if (sscanf(str, "%s %c %s", cmd, &st, s3) == 3) {
 	    break;
-	else *df1str = *df2str = *fstr = '\0';
+	} 
+	*s1 = *s2 = *s3 = '\0';
 	break;
     }
 
-    if (isalpha((unsigned char) *df1str)) {
-	gotvar = 0;
-	for (i=0; i<pdinfo->v; i++) {
-	    if (strcmp(df1str, pdinfo->varname[i]) == 0) {
-		gotvar = 1;
-		df1 = (int) Z[i][0];
-		mean = Z[i][0];
-		break;
-	    }
-	}
-	if (!gotvar) {
-	    sprintf(gretl_errmsg, _("Unknown variable '%s'"), df1str);
-	    err = 1;
-	}
-    } else {
-	if (*df1str && check_atof(df1str)) {
-	    err = 1;
-	} else {
-	    df1 = atoi(df1str);
-	    mean = atof(df1str);
-	}
+    st = normalize_stat(st);
+
+    if (st == 0) {
+	pputs(prn, _("\nunrecognized pvalue code\n"));
+	return NADBL;
     }
 
-    if (isalpha((unsigned char) *df2str)) {
-	gotvar = 0;
-	for (i=0; i<pdinfo->v; i++) {
-	    if (strcmp(df2str, pdinfo->varname[i]) == 0) {
-		gotvar = 1;
-		df2 = (int) Z[i][0];
-		variance = Z[i][0];
-		break;
-	    }
-	}
-	if (!gotvar) {
-	    sprintf(gretl_errmsg, _("Unknown variable '%s'"), df2str);
-	    err = 1;
-	}
-    } else {
-	if (*df2str && check_atof(df2str)) {
-	    err = 1;
-	} else {
-	    df2 = atoi(df2str);
-	    variance = atof(df2str);
-	}
+    err = val_from_string(s1, Z, pdinfo, &x1, &n1);
+
+    if (!err) {
+	err = val_from_string(s2, Z, pdinfo, &x2, &n2);
     }
 
-    if (isalpha((unsigned char) *fstr)) {
-	gotvar = 0;
-	for (i=0; i<pdinfo->v; i++) {
-	    if (strcmp(fstr, pdinfo->varname[i]) == 0) {
-		gotvar = 1;
-		xval = get_xvalue(i, Z, pdinfo);
-		if (na(xval)) {
-		    pputs(prn, _("\nstatistic has missing value code\n"));
-		    return NADBL;
-		}		
-		break;
-	    }
-	}
-	if (!gotvar) {
-	    sprintf(gretl_errmsg, _("Unknown variable '%s'"), fstr);
-	    err = 1;
-	}
-    } else {
-	if (*fstr && check_atof(fstr)) {
-	    err = 1;
-	} else {
-	    xval = atof(fstr);
-	}
+    if (!err) {
+	err = val_from_string(s3, Z, pdinfo, &x3, &n3);
     }
 
     if (err) {
@@ -383,96 +449,106 @@ double batch_pvalue (const char *str,
 	return NADBL;
     }
 
-    switch (stat) {
+    /* check for missing params */
+    if (st == 'z' && !*s3) {
+	err = 1;
+    } else if ((st == 't' || st == 'X') && (!*s1 || !*s3)) {
+	err = 1;
+    } else if ((st == 'F' || st == 'G' || st == 'B') &&
+	(!*s1 || !*s2 || !*s3)) {
+	err = 1;
+    }
+
+    if (err) {
+	pputs(prn, _("\npvalue: missing parameter\n"));
+	return NADBL;
+    }	
+
+    switch (st) {
 
-    case '1':
     case 'z':
-    case 'n':
-	tmp = xval;
-	if (xval > 0.0) tmp = -tmp;
-	xx = normal_cdf(tmp);
-	if (xx < 0) {
-	    pputs(prn, _("\np-value calculation failed\n"));
-	    return -1;
-	}	
-	pprintf(prn, _("\nStandard normal: area to the %s "
-		"of %g = %g\n"), (xval > 0)? _("right"): _("left"), 
-		xval, xx);
-	pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
-		2.0 * xx, 1.0 - 2.0 * xx);
-	return xx;
+	tmp = x3;
+	if (x3 > 0.0) tmp = -tmp;
+	pv = normal_cdf(tmp);
+	if (pv < 0) {
+	    pv = NADBL;
+	} else if (!na(pv)) {	
+	    pprintf(prn, _("\nStandard normal: area to the %s "
+			   "of %g = %g\n"), (x3 > 0)? _("right"): _("left"), 
+		    x3, pv);
+	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
+		    2.0 * pv, 1.0 - 2.0 * pv);
+	}
+	break;
 
-    case '2':
     case 't':
-	if (!*fstr || !*df1str) {
-	    pputs(prn, _("\npvalue for t: missing parameter\n"));
-	    return -1;
+	pv = t_pvalue_2(x3, n1);
+	if (pv < 0) {
+	    pv = NADBL;
+	} else if (!na(pv)) {
+	    pv *= 0.5;
+	    pprintf(prn, _("\nt(%d): area to the %s of %g = %g\n"), 
+		    n1, (x3 > 0)? _("right"): _("left"),
+		    x3, pv);
+	    pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
+		    2.0 * pv, 1.0 - 2.0 * pv);
 	}
-	xx = t_pvalue_2(xval, df1);
-	if (xx < 0) {
-	    pputs(prn, _("\np-value calculation failed\n"));
-	    return -1;
-	}
-	xx *= 0.5;
-	pprintf(prn, _("\nt(%d): area to the %s of %g = %g\n"), 
-		df1, (xval > 0)? _("right"): _("left"),
-		xval, xx);
-	pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
-		2.0 * xx, 1.0 - 2.0 * xx);
-	return xx;
+	break;
 
-    case '3':
-    case 'c':
-    case 'x':
     case 'X':
-	if (!*fstr || !*df1str) {
-	    pputs(prn, _("\npvalue for chi-square: missing parameter\n"));
-	    return -1;
+	pv = chisq(x3, n1);
+	if (pv < 0) {
+	    pv = NADBL;
+	} else if (!na(pv)) {
+	    pprintf(prn, _("\nChi-square(%d): area to the right of %g = %g\n"), 
+		    n1, x3, pv);
+	    pprintf(prn, _("(to the left: %g)\n"), 1.0 - pv);
 	}
-	xx = chisq(xval, df1);
-	if (xx < 0) {
-	    pputs(prn, _("\np-value calculation failed\n"));
-	    return -1;
-	}
-	pprintf(prn, _("\nChi-square(%d): area to the right of %g = %g\n"), 
-		df1, xval, xx);
-	pprintf(prn, _("(to the left: %g)\n"), 1.0 - xx);
-	return xx;
+	break;
 
-    case '4':
-    case 'f':
     case 'F':
-	if (!*fstr || !*df1str || !*df2str) {
-	    pputs(prn, _("\npvalue for F: missing parameter\n"));
-	    return -1;
+	pv = fdist(x3, n1, n2);
+	if (pv < 0) {
+	    pv = NADBL;
+	} else if (!na(pv)) {
+	    pprintf(prn, _("\nF(%d, %d): area to the right of %g = %g\n"), 
+		    n1, n2, x3, pv);
+	    pprintf(prn, _("(to the left: %g)\n"), 1.0 - pv);
 	}
-	xx = fdist(xval, df1, df2);
-	if (xx < 0) {
-	    pputs(prn, _("\np-value calculation failed\n"));
-	    return -1;
-	}
-	pprintf(prn, _("\nF(%d, %d): area to the right of %g = %g\n"), 
-		df1, df2, xval, xx);
-	pprintf(prn, _("(to the left: %g)\n"), 1.0 - xx);
-	return xx;
+	break;
 
-    case '5':
-    case 'g':
     case 'G':
-	xx = gamma_dist(mean, variance, xval, 2);
-	if (na(xx))
-	    pputs(prn, _("\nError computing gamma distribution\n"));
-	else
+	pv = gamma_dist(x1, x2, x3, 2);
+	if (pv < 0) {
+	    pv = NADBL;
+	} else if (!na(pv)) {
 	    pprintf(prn, _("\nGamma (mean %g, variance %g, shape %g, scale %g):"
-		    "\n area to the right of %g = %g\n"), 
-		    mean, variance, mean*mean/variance, variance/mean,
-		    xval, 1.0 - xx);
-	return xx;
+			   "\n area to the right of %g = %g\n"), 
+		    x1, x2, x1*x1/x2, x2/x1,
+		    x3, 1.0 - pv);
+	}
+	break;
+
+    case 'B':
+	pv = binomial_pvalue(n3, n2, x1);
+	if (pv < 0) {
+	    pv = NADBL;
+	} else if (!na(pv)) {
+	    pprintf(prn, _("\nBinomial (p = %g, n = %d):"
+			   "\n Prob(x > %d) = %g\n"), 
+		    x1, n2, n3, pv);
+	}
+	break;
 
     default:
-	pputs(prn, _("\nunrecognized pvalue code\n"));
-	return NADBL;
+	break;
     }
+
+    if (na(pv)) {
+	pputs(prn, _("\nError computing pvalue\n"));
+    }
+
+    return pv;
 }
 
 static void putxx (double xx)
@@ -541,8 +617,6 @@ void interact_pvalue (void)
     } while (ans[0] == 'Y' || ans[0] == 'y');
 }
 
-/* ........................................................ */
-
 static void pnormal (void)
 {
     double xx, zx; 
@@ -673,26 +747,35 @@ static double chi_crit_a (double a, int df)
     }
 }
 
-static int parse_critical_input (const char *str, int *i, 
-				 int *df, int *n)
+static int 
+parse_critical_input (const char *s, int *df, int *n)
 {
-    *i = -1;
+    int ret = -1;
 
-    if (sscanf(str, "critical F %d %d", df, n) == 2) *i = 3;
-    else if (sscanf(str, "critical X %d", df) == 1) *i = 2;
-    else if (sscanf(str, "critical t %d", df) == 1) *i = 1;
-    else if (sscanf(str, "critical d %d", n) == 1) *i = 4;
-    else if (!strncmp(str, "critical N", 10)) *i = 0;
+    if (sscanf(s, "F %d %d", df, n) == 2) {
+	ret ='F';
+    } else if (sscanf(s, "X %d", df)) {
+	ret = 'X';
+    } else if (sscanf(s, "t %d", df)) {
+	ret = 't';
+    } else if (sscanf(s, "d %d", n)) {
+	ret = 'd';
+    } else if (*s == 'N' || *s == 'z') {
+	ret = 'z';
+    } 
 
-    return (*i == -1);
+    return ret;
 }
 
 /**
  * print_critical:
  * @line: the command line, which should be of one of the following forms:
+ * critical z (Normal); or
+ * critical N (Normal); or
  * critical t df (student's t); or
  * critical X df (chi-square); or
- * critical F dfn dfd (F distribution).
+ * critical F dfn dfd (F distribution); or
+ * critical d n (Durbin-Watson, sample size n).
  * @prn: gretl printing struct.
  *
  * Prints critical values for the specified distribution at the
@@ -709,66 +792,68 @@ int print_critical (const char *line, PRN *prn)
     void (*dw)(int, PRN *) = NULL;
     void (*tcrit)(int, PRN *, int) = NULL;
     void (*chicrit)(int, PRN *, int) = NULL;
-    int i, n = -1, df = -1, err = 0;
+    int st, n = -1, df = -1;
+    int err = 0;
 
-    if (parse_critical_input(line, &i, &df, &n)) {
+    st = parse_critical_input(line + 9, &df, &n);
+
+    if (st < 0) {
 	pputs(prn, _("Invalid input\n"));
 	err = 1;
-    }
-
-    if ((0 < i && i < 4 && df <= 0) || (i == 3 && n <= 0)) {
+    } else if ((st == 't' || st == 'X' || st == 'F') && df <= 0) {
+	pputs(prn, _("Invalid degrees of freedom\n"));
+    } else if (st == 'F' && n <= 0) {
 	pputs(prn, _("Invalid degrees of freedom\n"));
 	err = 1;
-    }
-    else if (i == 4 && n <= 0) {
+    } else if (st == 'd' && n <= 0) {
 	pputs(prn, _("Invalid sample size\n"));
 	err = 1;
     }    
 
     if (err) return 1;
 
-    switch (i) {
-    case 0: /* normal */
+    switch (st) {
+    case 'z': /* normal */
 	funp = norm_table = get_plugin_function("norm_lookup", &handle);
 	break;
-    case 1: /* t */
+    case 't': /* t */
 	funp = tcrit = get_plugin_function("t_lookup", &handle);
 	break;
-    case 2: /* chi-square */
+    case 'X': /* chi-square */
 	funp = chicrit = get_plugin_function("chisq_lookup", &handle);
 	break;
-    case 3: /* F */
+    case 'F': /* F */
 	break;
-    case 4: /* DW */
+    case 'd': /* DW */
 	funp = dw = get_plugin_function("dw_lookup", &handle);
 	break;
     default:
 	break;
     }
 
-    if (i != 3 && funp == NULL)  {
+    if (st != 'F' && funp == NULL)  {
 	pputs(prn, _("Couldn't load plugin function\n"));
 	return 1;
     }
     
-    switch (i) {
-    case 0:
+    switch (st) {
+    case 'z':
 	(*norm_table)(prn, 0);
 	break;
-    case 1:
+    case 't':
 	(*tcrit)(df, prn, 0);
 	break;
-    case 2:
+    case 'X':
 	(*chicrit)(df, prn, 0);
 	break;	
-    case 3:
+    case 'F':
 	pprintf(prn, _("Approximate critical values of F(%d, %d)\n\n"),
 		df, n);
 	pprintf(prn, _(" 10%% in right tail %.2f\n"), f_crit_a(.10, df, n));
 	pprintf(prn, "  5%%               %.2f\n", f_crit_a(.05, df, n));	
 	pprintf(prn, "  1%%               %.2f\n", f_crit_a(.01, df, n));
 	break;
-    case 4:
+    case 'd':
 	(*dw)(n, prn);
 	break;
     default:
@@ -785,59 +870,63 @@ int print_critical (const char *line, PRN *prn)
 static int parse_genr_critical_input (const char *str, 
 				      const double **Z, 
 				      const DATAINFO *pdinfo,
-				      int *i, int *dfn, int *dfd, 
+				      int *st, int *dfn, int *dfd, 
 				      double *a)
 {
     char dfnstr[VNAMELEN], dfdstr[VNAMELEN];
-    char astr[VNAMELEN];
+    char astr[32];
     double val;
-    *i = -1;
+    int err = 0;
 
     dfnstr[0] = dfdstr[0] = astr[0] = '\0';
 
-    if (sscanf(str, "F,%8[^,],%8[^,],%8s", dfnstr, dfdstr, astr) == 3) {
-	*i = 3;
-    } else if (sscanf(str, "X,%8[^,],%8s", dfnstr, astr) == 2) {
-	*i = 2;
-    } else if (sscanf(str, "t,%8[^,],%8s", dfnstr, astr) == 2) {
-	*i = 1;
-    } else if (sscanf(str, "N,%8s", astr) == 1) {
-	*i = 0;
+    if (sscanf(str, "F,%8[^,],%8[^,],%24s", dfnstr, dfdstr, astr) == 3) {
+	*st = 'F';
+    } else if (sscanf(str, "X,%8[^,],%24s", dfnstr, astr) == 2) {
+	*st = 'X';
+    } else if (sscanf(str, "t,%8[^,],%24s", dfnstr, astr) == 2) {
+	*st = 't';
+    } else if (sscanf(str, "N,%24s", astr) || sscanf(str, "z,%24s", astr)) {
+	*st = 'z';
 	*dfn = 500;
+    } else {
+	err = 1;
     }
 
-    if (*i == -1) return 1;
+    if (err) return err;
 
-    if (dfnstr[0] != '\0') {
+    if (*dfnstr != '\0') {
 	val = get_number_or_val(dfnstr, Z, pdinfo);
 	if (na(val)) {
-	    *i = -1;
+	    err = 1;
 	} else {
 	    *dfn = val;
 	}
     }
-    if (dfdstr[0] != '\0') {
+
+    if (*dfdstr != '\0') {
 	val = get_number_or_val(dfdstr, Z, pdinfo);
 	if (na(val)) {
-	    *i = -1;
+	    err = 1;
 	} else {
 	    *dfd = val;
 	}
     }
-    if (astr[0] != '\0') {
+
+    if (*astr != '\0') {
 	*a = get_number_or_val(astr, Z, pdinfo);
 	if (na(*a) || *a < 0.0) {
-	    *i = -1;
+	    err = 1;
 	}
     }
 
-    return (*i == -1);
+    return err;
 }
 
 double genr_get_critical (const char *line, const double **Z, 
 			  const DATAINFO *pdinfo)
 {
-    double alpha, ret = NADBL;
+    double alpha = 0.0, ret = NADBL;
     int st, dfn = -1, dfd = -1;
 
     if (parse_genr_critical_input(line, Z, pdinfo,
@@ -845,23 +934,39 @@ double genr_get_critical (const char *line, const double **Z,
 	return NADBL;
     }
 
-    if ((0 < st && st < 4 && dfn <= 0) || (st == 3 && dfd <= 0)) {
+    if ((st == 't' || st == 'X' || st == 'F') && dfn <= 0) {
 	strcpy(gretl_errmsg, _("Invalid degrees of freedom\n"));
 	return NADBL;
-    }
+    } else if (st == 'F' && dfd <= 0) {
+	strcpy(gretl_errmsg, _("Invalid degrees of freedom\n"));
+	return NADBL;
+    }	
 
-    if (st == 3) {
+    if (st == 'F') {
 	ret = f_crit_a(alpha, dfn, dfd);
-    } else if (st == 2) {
+	if (ret < 0) {
+	    ret = NADBL;
+	}
+    } else if (st == 'X') {
 	ret = chi_crit_a(alpha, dfn);
+	if (ret < 0) {
+	    ret = NADBL;
+	}
+    } else if (st == 't') {
+	if (alpha > 0.5) {
+	    ret = stdtri(dfn, 1.0 - alpha);
+	} else {
+	    ret = -stdtri(dfn, alpha);
+	}
     } else {
-	ret = sqrt(f_crit_a(2.0 * alpha, 1, dfn));
+	/* normal */
+	if (alpha > 0.5) {
+	    ret = ndtri(1.0 - alpha);
+	} else {
+	    ret = -ndtri(alpha);
+	}
     } 
 
-    if (ret < 0) {
-	ret = NADBL;
-    }
-
     return ret;
 }
 
@@ -870,7 +975,7 @@ double genr_get_critical (const char *line, const double **Z,
    Draws upon the pascal code specialf.pas by Bent Nielsen.
 */
 
-static const double tolerance = 1e-7;
+static const double gamma_tol = 1e-7;
 
 /* internal functions */
 
@@ -879,33 +984,32 @@ static double gamma_integral_expansion (double lambda, double x);
 static double gamma_integral_fraction (double lambda, double x);
 static double gammadist_wilson_hilferty (double shape, double scale, double x);
 
-/* exported functions */
+/* Control 1 : s1, s2 = shape, scale
+           2 : s1, s2 = expectation, variance
+   Returns NADBL on error 
+*/
 
 double gamma_dist (double s1, double s2, double x, int control)
-     /* Control 1 : s1, s2 = shape, scale
-                2 : s1, s2 = expectation, variance
-        Returns NADBL (-999.0) on error 
-     */
 {
-    double shape = 0, scale = 0, xx;
+    double shape, scale, xx;
 
-    switch (control) {
-    case 1: 
+    if (control == 1) {
 	shape = s1; 
 	scale = s2; 
-	break;
-    case 2: 
-	scale = s2/s1; 
-	shape = s1/scale; 
-	break;
-    }
-    if ((shape > 20) && (x/scale < 0.9*shape) && (x > 1))
+    } else {
+	scale = s2 / s1; 
+	shape = s1 / scale; 
+    }	
+
+    if ((shape > 20) && (x / scale < 0.9 * shape) && (x > 1)) {
 	xx = gammadist_wilson_hilferty(shape, scale, x);
-    else {
-	xx = gamma_integral(shape, x/scale);
-	if (na(xx)) return xx;
-	xx /= cephes_gamma(shape);
+    } else {
+	xx = gamma_integral(shape, x / scale);
+	if (!na(xx)) {
+	    xx /= cephes_gamma(shape);
+	}
     }
+
     return xx;
 }
 
@@ -917,19 +1021,23 @@ static double gamma_integral_expansion (double lambda, double x)
 	Note that the series is alternating.
      */
 {
-    double x1, x2, x3, xx;
+    double xx, x1 = 1.0;
+    double x2, x3 = 1.0 / lambda;
     int i = 0;
 
-    x1 = 1;
-    x3 = 1/lambda;
     do {
 	i++;
 	x1 *= (-x)/i;
-	x2 = x1/(lambda + i);
+	x2 = x1 / (lambda + i);
 	x3 += x2;
-    } while (fabs(x2) >= tolerance && i <= 100);
-    xx = x3 * exp(lambda*log(x));
-    if (i == 100) return NADBL;
+    } while (fabs(x2) >= gamma_tol && i <= 100);
+
+    if (i == 100) {
+	xx = NADBL;
+    } else {
+	xx = x3 * exp(lambda * log(x));
+    }
+
     return xx;
 }
 
@@ -943,25 +1051,32 @@ static double gamma_integral_fraction (double lambda, double x)
 	See also Schwartz p. 120.      
      */
 {
-    double a = 1-lambda;
-    double b = a+x+1;
-    double p1 = 1, p2 = 1+x;
-    double q1 = x, q2 = b*x;
-    double d, p0, q0, r1, r2, xx;
+    double a = 1 - lambda;
+    double b = a + x + 1;
+    double p1 = 1, p2 = 1 + x;
+    double q1 = x, q2 = b * x;
+    double r2 = p2 / q2;
+    double d, p0, q0, r1, xx;
     int c = 0;
 
-    r2 = p2/q2;
     do {
 	p0 = p1; p1 = p2; q0 = q1; q1 = q2; r1 = r2;
-	a = a+1; b = b+2; c = c+1; d = a*c;
-	p2 = b*p1 - d*p0;
-	q2 = b*q1 - d*q0;
-	if (fabs(q2) > 0)  r2 = p2/q2;
-    } while (!((fabs(r2-r1) < tolerance ) || (fabs(r2-r1) < tolerance * r2)
-	       || (c == 100)));
-    xx = cephes_gamma(lambda);
-    xx -= exp(-x + lambda * log(x)) * r2;
-    if (c == 100) return NADBL;
+	a = a + 1; b = b + 2; c = c + 1; d = a * c;
+	p2 = b * p1 - d * p0;
+	q2 = b * q1 - d * q0;
+	if (fabs(q2) > 0) {
+	    r2 = p2 / q2;
+	}
+	xx = fabs(r2 - r1);
+    } while (!((xx < gamma_tol ) || (xx < gamma_tol * r2) || (c == 100)));
+
+    if (c == 100) {
+	xx = NADBL;
+    } else {
+	xx = cephes_gamma(lambda);
+	xx -= exp(-x + lambda * log(x)) * r2;
+    }
+
     return xx;
 }
 
@@ -969,16 +1084,16 @@ static double gamma_integral (double lambda, double x)
 {
     double xx;
 
-    if (x < 0)  { 
-	return NADBL;
-    }
-    if (x < tolerance) {
-	xx = 0;
-    } else if ((x <= 1) || (x < 0.9*lambda)) {
+    if (x < 0.0)  { 
+	xx = NADBL;
+    } else if (x < gamma_tol) {
+	xx = 0.0;
+    } else if (x <= 1.0 || x < 0.9 * lambda) {
 	xx = gamma_integral_expansion(lambda, x);
     } else {
 	xx = gamma_integral_fraction(lambda, x);
     }
+
     return xx;
 }
 
@@ -989,12 +1104,13 @@ static double gammadist_wilson_hilferty (double shape, double scale, double x)
         vol 1, 2nd ed, Wiley 1994
      */
 {
-    double xx, df, xscaled;
+    double df = 2.0 * shape;
+    double xscaled = x * 2.0 / scale;
+    double xx;
+
+    xx = exp(log(xscaled/df)/3) - 1 + (double)(2) / 9 / df;
+    xx *= sqrt(9 * df / 2);
 
-    df = 2 * shape;
-    xscaled = x * 2/scale;
-    xx = exp(log(xscaled/df)/3) - 1 + (double)(2)/9/df;
-    xx *= sqrt(9*df/2);
     return normal_cdf(xx);
 } 
 
