@@ -20,7 +20,6 @@
 /* random.c for gretl */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 
 #include <time.h>
 
@@ -38,37 +37,51 @@
 static GRand *gretl_rand;
 #endif
 
+static unsigned int useed;
+
+unsigned int get_gretl_random_seed (void)
+{
+    return useed;
+}
+
 /**
  * gretl_rand_init:
  *
  * Initialize gretl's PRNG, using the system time as seed.
- *
  */
 
 void gretl_rand_init (void)
 {
+    useed = time(NULL);
 #ifdef HAVE_G_RAND
     gretl_rand = g_rand_new();
-    gretl_rand_set_seed((guint32) time(NULL));
+    gretl_rand_set_seed((guint32) useed);
 #else
-    init_genrand(time(NULL));
+    init_genrand(useed);
 #endif
 }
 
 /**
  * gretl_rand_set_seed:
- * @seed: the chosen seed value
+ * @seed: the chosen seed value.
  *
  * Set a specific (and hence reproducible) seed for gretl's PRNG.
- *
+ * But if the value 0 is given for @seed, set the seed using
+ * the system time (which is the default when libgretl is
+ * initialized).
  */
 
 void gretl_rand_set_seed (unsigned int seed)
 {
+    if (seed == 0) {
+	useed = time(NULL);
+    } else {
+	useed = seed;
+    }
 #ifdef HAVE_G_RAND
-    g_rand_set_seed(gretl_rand, seed);
+    g_rand_set_seed(gretl_rand, useed);
 #else
-    init_genrand(seed);
+    init_genrand(useed);
 #endif
 }
 
@@ -81,7 +94,6 @@ void gretl_rand_set_seed (unsigned int seed)
  * Fill the selected range of array @a with pseudo-random drawings
  * from the uniform distribution on 0-1, using the Mersenne
  * Twister.
- *
  */
 
 void gretl_uniform_dist (double *a, int t1, int t2) 
@@ -107,7 +119,6 @@ void gretl_uniform_dist (double *a, int t1, int t2)
  * from the standard distribution, using the Mersenne Twister for
  * uniform input and the Box-Muller method for converting to the
  * normal distribution.
- *
  */
 
 void gretl_normal_dist (double *a, int t1, int t2) 
@@ -130,22 +141,12 @@ void gretl_normal_dist (double *a, int t1, int t2)
     }
 }
 
-unsigned int gretl_rand_int (void)
-{
-#ifdef HAVE_G_RAND
-    return g_rand_int(gretl_rand);
-#else
-    return genrand_int32();
-#endif
-}
-
 /**
  * gretl_rand_int_max:
  * @max: the maximum value (open)
  *
  * Returns: a pseudo-random unsigned int in the interval [0, @max)
  * using the Mersenne Twister.
- *
  */
 
 unsigned int gretl_rand_int_max (unsigned int max)
@@ -160,8 +161,7 @@ unsigned int gretl_rand_int_max (unsigned int max)
 /**
  * gretl_rand_free:
  *
- * Free the gretl_rand structure.
- *
+ * Free the gretl_rand structure (may be called at program exit).
  */
 
 void gretl_rand_free (void)
