@@ -68,13 +68,10 @@ double dot_atof (const char *s)
 {
     double x;
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
+    gretl_push_c_numeric_locale();
     x = atof(s);
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif    
+    gretl_pop_c_numeric_locale();
+
     return x;
 }
 
@@ -293,14 +290,16 @@ int numeric_string (const char *str)
     char *test;
     int ret = 1;
 
+    if (str == NULL || *str == '\0') {
+	return 0;
+    }
+
     if (!strcmp(str, "inf") || !strcmp(str, "nan")) {
 	/* could be variable names: they are not valid numbers */
 	return 0;
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
+    gretl_push_c_numeric_locale();
 
     errno = 0;
 
@@ -309,9 +308,7 @@ int numeric_string (const char *str)
 	ret = 0;
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
 
     return ret;
 }
@@ -374,10 +371,13 @@ char *lower (char *str)
 
 char *gretl_strdup (const char *src)
 {
-    char *targ = malloc(strlen(src) + 1);
+    char *targ = NULL;
 
-    if (targ != NULL) {
-        strcpy(targ, src);
+    if (src != NULL) {
+	targ = malloc(strlen(src) + 1);
+	if (targ != NULL) {
+	    strcpy(targ, src);
+	}
     }
 
     return targ;
@@ -396,7 +396,7 @@ char *gretl_strndup (const char *src, size_t n)
 {
     char *targ = NULL;
 
-    if (n > 0) {
+    if (src != NULL && n > 0) {
 	size_t len = strlen(src);
 
 	if (len > n) {
@@ -577,12 +577,16 @@ char *shift_string_left (char *str, size_t move)
 
 char *chopstr (char *str)
 {
-    int i = strspn(str, " \t");
+    int i, n = strspn(str, " \t");
 
-    shift_string_left(str, i);
+    if (n > 0) {
+	shift_string_left(str, n);
+    }
 
-    for (i = strlen(str) - 1; i >= 0; i--) {
-	if (isspace((unsigned char) str[i]) || str[i] == '\r') {
+    n = strlen(str);
+
+    for (i = n - 1; i >= 0; i--) {
+	if (isspace(str[i]) || str[i] == '\r') {
 	    str[i] = '\0';
 	} else {
 	    break;
@@ -974,7 +978,7 @@ real_get_obs_string (char *obs, int t, const DATAINFO *pdinfo, int full)
 
 /**
  * get_obs_string:
- * @obs: char array big enough to hold the observation (OBSLEN).
+ * @obs: char array big enough to hold the observation (#OBSLEN).
  * @t: zero-based observation number.
  * @pdinfo: pointer to dataset information.
  *
@@ -1049,6 +1053,53 @@ char *colonize_obs (char *obs)
     return obs;
 }
 
+/**
+ * modify_obs_for_csv:
+ * @s: observation string (date).
+ * @pd: data frequency.
+ *
+ * Modifies the observation string corresponding to obervation @t to
+ * producing a form suitable for a CSV file.  This applies only to
+ * time series data. The string @s should be obtained by calling
+ * ntodate();
+ */
+
+void modify_date_for_csv (char *s, int pd)
+{
+    if (pd == 4) {
+	charsub(s, ':', 'Q');
+    } else {
+	charsub(s, ':', '-');
+    }
+}
+
+/**
+ * csv_obs_string_to_prn:
+ * @t: 0-based observation number.
+ * @pdinfo: data set information struct.
+ * @prn: printing struct.
+ *
+ * Prints the observation string corresponding to obervation @t to
+ * @prn, in a format suitable for a CSV file.
+ */
+
+void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn)
+{
+    if (pdinfo->S != NULL) {
+	pprintf(prn, "%s%c", pdinfo->S[t], pdinfo->delim);
+    } else if (pdinfo->structure != CROSS_SECTION) {
+	char tmp[OBSLEN];
+
+	ntodate_full(tmp, t, pdinfo);
+	if (quarterly_or_monthly(pdinfo)) {
+	    modify_date_for_csv(tmp, pdinfo->pd);
+	    pprintf(prn, "\"%s\"%c", tmp, pdinfo->delim);
+	} else {
+	    pprintf(prn, "\"'%s\"%c", tmp, pdinfo->delim);
+	}
+    }
+}	
+
 const char *print_time (const time_t *timep)
 {
     static char timestr[48];
@@ -1228,7 +1279,9 @@ int build_path (const char *dir, const char *fname, char *path,
 {
     size_t len;
 
-    if (dir == NULL || fname == NULL || path == NULL) return 1;
+    if (dir == NULL || fname == NULL || path == NULL) {
+	return 1;
+    }
 
     *path = '\0';
     strcat(path, dir);
@@ -1250,7 +1303,9 @@ int build_path (const char *dir, const char *fname, char *path,
         strcat(path, fname);
     }
 
-    if (ext != NULL) strcat(path, ext);
+    if (ext != NULL) {
+	strcat(path, ext);
+    }
 
     return 0;
 }
