@@ -46,13 +46,26 @@ static double **fullZ;
 static DATAINFO *fullinfo;
 static const DATAINFO *peerinfo;
 
-/* .......................................................... */
+#define SUBMASK_SENTINEL 127
 
-char *copy_subsample_mask (const char *src, int n)
+static int get_submask_length (const char *s)
 {
+    int n = 1;
+
+    while (*s != SUBMASK_SENTINEL) {
+	n++;
+	s++;
+    }
+
+    return n;
+}
+
+char *copy_subsample_mask (const char *src)
+{
+    int n = get_submask_length(src);
     char *ret = NULL;
 
-    if (n != 0 && src != NULL) {
+    if (src != NULL) {
 	ret = malloc(n * sizeof *ret);
 	if (ret != NULL) {
 	    memcpy(ret, src, n);
@@ -62,8 +75,6 @@ char *copy_subsample_mask (const char *src, int n)
     return ret;
 }
 
-/* .......................................................... */
-
 void maybe_free_full_dataset (const DATAINFO *pdinfo)
 {
     if (pdinfo == peerinfo) {
@@ -92,15 +103,13 @@ static void sync_dataset_elements (const DATAINFO *pdinfo)
     fullinfo->descrip = pdinfo->descrip;      
 }
 
-/* .......................................................... */
-
 static int 
 attach_subsample_to_datainfo (DATAINFO *subinfo, const double **Z, 
 			      const DATAINFO *pdinfo)
 {
     int s, t;
 
-    subinfo->submask = malloc(pdinfo->n * sizeof *subinfo->submask);
+    subinfo->submask = malloc((pdinfo->n + 1) * sizeof *subinfo->submask);
     if (subinfo->submask == NULL) {
 	return E_ALLOC;
     }
@@ -117,6 +126,8 @@ attach_subsample_to_datainfo (DATAINFO *subinfo, const double **Z,
 	subinfo->submask[t] = Z[s][t];
     }
 
+    subinfo->submask[pdinfo->n] = SUBMASK_SENTINEL;
+
     return 0;
 }
 
@@ -139,7 +150,7 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	/* sync, in case anything has moved */
 	sync_dataset_elements(pdinfo);
 
-	pmod->submask = copy_subsample_mask(pdinfo->submask, fullinfo->n);
+	pmod->submask = copy_subsample_mask(pdinfo->submask);
 	if (pmod->submask == NULL) {
 	    err = E_ALLOC;
 	}
@@ -148,8 +159,6 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
     return err;
 }
 
-/* .......................................................... */
-
 static double *get_old_mask (double **Z, const DATAINFO *pdinfo)
 {
     double *mask = NULL;
@@ -231,7 +240,7 @@ static int sn_from_tmp_dummy (double ***pZ, DATAINFO *pdinfo,
     /* + 4 to skip the command word "smpl" */
     sprintf(formula, "__tmpmsk=%s", line + 4);
 
-    err = generate(formula, pZ, pdinfo, NULL, OPT_P);
+    err = generate(formula, pZ, pdinfo, OPT_P);
     *gretl_msg = '\0';
 
     if (err) {
@@ -387,7 +396,7 @@ static int block_retained (const char *s)
     int nrows = 0;
     int ret = 0;
 
-#if 0
+#if SUBDEBUG
     fprintf(stderr, "block_retained: s = '%s'\n", s);
 #endif
 
@@ -412,11 +421,20 @@ static char *make_panel_signature (const DATAINFO *pdinfo,
 {
     char *sig;
     size_t sz = pdinfo->n + pdinfo->n / pdinfo->pd;
+    int t;
 
     sig = calloc(sz, 1);
 
+#if SUBDEBUG
+    for (t=0; t<pdinfo->n; t++) {
+	if (t >= mt1 && t <= mt2) {
+	    fprintf(stderr, "mask[%d]=%c\n", t-mt1, mask[t-mt1]);
+	}
+    }
+#endif
+
     if (sig != NULL) {
-	int t, i = 0;
+	int i = 0;
 
 	for (t=0; t<pdinfo->n; t++) {
 	    if (t > 0 && t % pdinfo->pd == 0) {
@@ -426,8 +444,10 @@ static char *make_panel_signature (const DATAINFO *pdinfo,
 		sig[i++] = '0';
 	    } else {
 		if (neg) {
-		    sig[i++] = (mask[t - mt1] != 1)? '1' : '0';
+		    /* using model missmask: '1' for missing vs '0' */
+		    sig[i++] = (mask[t - mt1] != '1')? '1' : '0';
 		} else {
+		    /* using sample missmask: 0 for missing vs 1 */
 		    sig[i++] = (mask[t - mt1] != 0)? '1' : '0';
 		}
 	    }
@@ -458,6 +478,10 @@ real_mask_leaves_balanced_panel (const char *mask,
     for (i=0; i<nblocks; i++) {
 	char *sig_n = sig + i * (pdinfo->pd + 1);
 
+#if SUBDEBUG
+	fprintf(stderr, "block %d: sig_n = '%s'\n", i, sig_n);
+#endif
+
 	if (block_retained(sig_n)) {
 	    if (ok_blocks == 0) {
 		sig_0 = sig_n;
@@ -801,8 +825,6 @@ int restrict_sample (const char *line,
     return 0;
 }
 
-/* .......................................................... */
-
 enum {
     SMPL_T1,
     SMPL_T2
@@ -852,7 +874,7 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
 
     nf = count_fields(line);
 
-#ifdef SUBDEBUG
+#if SUBDEBUG
     fprintf(stderr, "set_sample: line='%s', nf=%d, pdinfo=%p\n", 
 	    line, nf, (void *) pdinfo);
     if (pdinfo != NULL) {
@@ -1078,7 +1100,7 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo, gretlopt opt)
     *gretl_errmsg = '\0';
     *gretl_msg = '\0';
 
-#ifdef SUBDEBUG
+#if SUBDEBUG
     fprintf(stderr, "restore_full_sample: pZ=%p, ppdinfo=%p opt=%lu\n",
 	    (void *) pZ, (void *) ppdinfo, opt);
     fprintf(stderr, "*pZ=%p, *ppdinfo=%p\n",
@@ -1148,8 +1170,6 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo, gretlopt opt)
     return 0;
 }
 
-/* ........................................................... */
-
 int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn)
 {
     int i, t, tmiss;
@@ -1164,7 +1184,9 @@ int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn)
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	tmiss = 0;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (is_hidden_variable(i, pdinfo) || !pdinfo->vector[i]) continue;
+	    if (is_hidden_variable(i, pdinfo) || !pdinfo->vector[i]) {
+		continue;
+	    }
 	    if (na((*pZ)[i][t])) {
 		if (missvec[i] == 0) {
 		    missvec[0] += 1;
@@ -1255,7 +1277,7 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
     char *mask = NULL;
 
     if (fullZ == NULL || fullinfo == NULL) {
-#ifdef SUBDEBUG
+#if SUBDEBUG
 	fputs("add_subsampled_dataset_to_model: got NULL full dataset\n",
 	      stderr);
 #endif
@@ -1263,7 +1285,7 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
     }
 
     if (pmod->dataset != NULL) {
-#ifdef SUBDEBUG
+#if SUBDEBUG
 	fputs("add_subsampled_dataset_to_model: job already done\n",
 	      stderr);
 #endif	
@@ -1277,7 +1299,7 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	/* no subsample info: model was estimated on full dataset,
 	   so we reconstruct the full dataset */
 	sn = fullinfo->n;
-#ifdef SUBDEBUG
+#if SUBDEBUG
 	fprintf(stderr, "pmod->submask = NULL, set sn = %d\n", sn);
 #endif
     } else {
@@ -1288,7 +1310,7 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	if (mask == NULL) {
 	    return E_ALLOC;
 	}
-#ifdef SUBDEBUG
+#if SUBDEBUG
 	fprintf(stderr, "setting mask from pmod->submask: fullinfo->n = %d\n", 
 		fullinfo->n);
 #endif
@@ -1310,7 +1332,7 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	return E_ALLOC;
     }
 
-#ifdef SUBDEBUG
+#if SUBDEBUG
     fprintf(stderr, "pmod->dataset allocated at %p\n", 
 	    (void *) pmod->dataset);
 #endif
@@ -1325,7 +1347,7 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	return E_ALLOC;
     }
 
-#ifdef SUBDEBUG
+#if SUBDEBUG
     fprintf(stderr, "dataset created, copying series info\n");
 #endif
 
@@ -1337,7 +1359,7 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 			   (const double **) fullZ, fullinfo,
 			   mask);
 
-#ifdef SUBDEBUG
+#if SUBDEBUG
     fputs("data copied to subsampled dataset\n", stderr);
 #endif
 
@@ -1359,7 +1381,7 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 
     free(mask);
 
-#ifdef SUBDEBUG
+#if SUBDEBUG
     fputs("add_subsampled_dataset_to_model: success\n", stderr);
 #endif
 
@@ -1369,12 +1391,10 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 void free_model_dataset (MODEL *pmod)
 {
     if (pmod->dataset != NULL) {
-#ifdef SUBDEBUG
+#if SUBDEBUG
 	fprintf(stderr, "Deep freeing model->dataset\n");
 #endif
-	free_Z(pmod->dataset->Z, pmod->dataset->dinfo);
-	clear_datainfo(pmod->dataset->dinfo, CLEAR_FULL);
-	free(pmod->dataset->dinfo);
+	destroy_dataset(pmod->dataset->Z, pmod->dataset->dinfo);
 	free(pmod->dataset);
 	pmod->dataset = NULL;
     }
