@@ -22,6 +22,7 @@
 #include "libgretl.h"
 #include "system.h"
 #include "system_private.h"
+#include "objstack.h"
 
 enum {
     OP_PLUS,
@@ -91,76 +92,6 @@ const char *toofew = N_("An equation system must have at least two equations");
 static void destroy_ident (identity *pident);
 static int make_instrument_list (gretl_equation_system *sys);
 
-
-/* ------------------------------------------------------------ */
-
-static gretl_equation_system **system_stack;
-static int n_systems;
-
-gretl_equation_system *
-get_equation_system_by_name (const char *sysname, int *snum)
-{
-    int i;
-
-    for (i=0; i<n_systems; i++) {
-	if (!strcmp(sysname, system_stack[i]->name)) {
-	    if (snum != NULL) {
-		*snum = i;
-	    }
-	    return system_stack[i];
-	}
-    }
-
-    return NULL;
-}
-
-static int stack_system (gretl_equation_system *sys, PRN *prn)
-{
-    gretl_equation_system *orig;
-    int snum;
-
-    /* only feasible for named systems */
-    if (sys == NULL || sys->name == NULL) {
-	return 1;
-    }
-
-    orig = get_equation_system_by_name(sys->name, &snum);
-
-    if (orig != NULL) {
-	/* replace existing system of same name */
-	gretl_equation_system_destroy(orig);
-	system_stack[snum] = sys;
-	pprintf(prn, "Replaced equation system '%s'\n", sys->name);
-    } else {
-	gretl_equation_system **sstack;
-
-	sstack = realloc(system_stack, (n_systems + 1) * sizeof *sstack);
-	if (sstack == NULL) {
-	    return E_ALLOC;
-	}
-	system_stack = sstack;
-	system_stack[n_systems++] = sys;
-	pprintf(prn, "Added equation system '%s'\n", sys->name);
-    }
-
-    return 0;
-}
-
-void gretl_equation_systems_cleanup (void)
-{
-    int i;
-
-    for (i=0; i<n_systems; i++) {
-	gretl_equation_system_destroy(system_stack[i]);
-    }
-
-    free(system_stack);
-    system_stack = NULL;
-    n_systems = 0;
-}
-
-/* ------------------------------------------------------------ */
-
 static void 
 print_system_identity (const identity *pident, const DATAINFO *pdinfo, 
 		       PRN *prn)
@@ -211,12 +142,12 @@ print_equation_system_info (const gretl_equation_system *sys,
 
 }
 
-static int gretl_system_method_from_string (const char *str)
+int gretl_system_method_from_string (const char *s)
 {
     int i = 0;
 
     while (gretl_system_method_strings[i] != NULL) {
-	if (!strcmp(str, gretl_system_method_strings[i])) {
+	if (!strcmp(s, gretl_system_method_strings[i])) {
 	    return i;
 	}
 	i++;
@@ -225,6 +156,24 @@ static int gretl_system_method_from_string (const char *str)
     return i;
 }
 
+const char *system_method_full_string (int method)
+{
+    if (method >= SYS_SUR && method < SYS_MAX) {
+	return gretl_system_long_strings[method];
+    } else {
+	return NULL;
+    }
+}
+
+const char *system_method_short_string (int method)
+{
+    if (method >= SYS_SUR && method < SYS_MAX) {
+	return gretl_system_method_strings[method];
+    } else {
+	return NULL;
+    }
+}
+
 static gretl_equation_system *
 gretl_equation_system_new (int method, const char *name)
 {
@@ -247,6 +196,7 @@ gretl_equation_system_new (int method, const char *name)
 	sys->name = NULL;
     }
 
+    sys->refcount = 0;
     sys->method = method;
 
     sys->t1 = sys->t2 = 0;
@@ -261,9 +211,9 @@ gretl_equation_system_new (int method, const char *name)
     sys->iters = 0;
     sys->flags = 0;
 
-    sys->ll = sys->llu = 0.0;
-    sys->X2 = 0.0;
-    sys->ess = 0.0;
+    sys->ll = sys->llu = NADBL;
+    sys->X2 = NADBL;
+    sys->ess = NADBL;
     sys->diag = 0.0;
     sys->bdiff = 0.0;
 
@@ -303,10 +253,11 @@ static void system_clear_results (gretl_equation_system *sys)
 
     sys->t1 = sys->t2 = 0;
 
-    sys->ll = 0.0;
-    sys->llu = 0.0;
-    sys->X2 = 0.0;
-    sys->ess = 0.0;
+    sys->ll = NADBL;
+    sys->llu = NADBL;
+    sys->X2 = NADBL;
+    sys->ess = NADBL;
+
     sys->diag = 0.0;
     sys->bdiff = 0.0;
 
@@ -331,27 +282,17 @@ static void system_clear_results (gretl_equation_system *sys)
     }
 }
 
-static void gretl_equation_system_clear (gretl_equation_system *sys)
-{
-    if (sys == NULL || sys->lists == NULL) return;
-
-    sys->flags = 0;
-    sys->method = -1;
-
-    /* if restrictions in place, reset the restricted flag */
-    if (sys->R != NULL && sys->q != NULL) {
-	sys->flags |= GRETL_SYS_RESTRICT;
-    }
-
-    system_clear_results(sys);
-}
-
 void gretl_equation_system_destroy (gretl_equation_system *sys)
 {
     int i;
 
     if (sys == NULL || sys->lists == NULL) return;
 
+    sys->refcount -= 1;
+    if (sys->refcount > 0) {
+	return;
+    }
+
     for (i=0; i<sys->n_equations; i++) {
 	free(sys->lists[i]);
     }
@@ -432,10 +373,29 @@ int gretl_equation_system_append (gretl_equation_system *sys,
 char *get_system_name_from_line (const char *s)
 {
     char *name = NULL;
-    const char *p;
+    const char *p = strstr(s, " name");
     int pchars = 0;
 
-    while (isspace((unsigned char) *s)) s++;
+    if (p != NULL) {
+	p += 5;
+    } else {
+	p = strstr(s, "estimate ");
+	if (p == NULL) {
+	    p = strstr(s, "restrict ");
+	}
+	if (p != NULL) {
+	    p += 9;
+	}	
+    }
+
+    if (p == NULL) {
+	return NULL;
+    }
+
+    s = p;
+    while (isspace((unsigned char) *s) || *s == '=') {
+	s++;
+    }
 
     if (*s == '"') {
 	if (*(s + 1) != '\0') s++;
@@ -465,14 +425,30 @@ char *get_system_name_from_line (const char *s)
 
 /* parse a system estimation method out of a command line */
 
-static int get_estimation_method (const char *s)
+static int get_estimation_method_from_line (const char *s)
 {
+    const char *p = strstr(s, " method");
     char mstr[9];
     int method = -1;
 
-    while (isspace((unsigned char) *s)) s++;
+    if (p != NULL) {
+	p += 7;
+    } else {
+	p = strstr(s, " type");
+	if (p != NULL) {
+	    p += 5;
+	}
+    }
 
-    if (sscanf(s, "%8s", mstr) == 1) {
+    if (p == NULL) {
+	return -1;
+    }
+
+    while (isspace((unsigned char) *p) || *p == '=') {
+	p++;
+    }
+
+    if (sscanf(p, "%8s", mstr) == 1) {
 	lower(mstr);
 	method = gretl_system_method_from_string(mstr);
     }
@@ -480,58 +456,33 @@ static int get_estimation_method (const char *s)
     return method;
 }
 
-static char *system_start_get_name (const char *s)
+static void 
+system_set_save_flags (gretl_equation_system *sys, const char *s)
 {
-    char *sysname = NULL;
-    const char *p = strstr(s, "system name=");
+    s = strstr(s, " save");
 
-    if (p != NULL) {
-	sysname = get_system_name_from_line(p + 12);
+    if (s != NULL) {
+	s += 5;
+	if (*s != ' ' && *s != '=') {
+	    return;
+	}
+	if (strstr(s, "resids") || strstr(s, "uhat")) {
+	    sys->flags |= GRETL_SYSTEM_SAVE_UHAT;
+	}
+	if (strstr(s, "fitted") || strstr(s, "yhat")) {
+	    sys->flags |= GRETL_SYSTEM_SAVE_YHAT;
+	}
     }
-
-    return sysname;
-}
-
-static int system_start_get_method (const char *s)
-{
-    int method = -1;
-    int offset = 14;
-    const char *p;
-
-    p = strstr(s, "system method=");
-
-    if (p == NULL) {
-	/* backward compatibility */
-	p = strstr(s, "system type=");
-	offset = 12;
-    }
-
-    if (p != NULL) {
-	method = get_estimation_method(p + offset);
-    }
-
-    return method;
-}
-
-static int named_system_get_method (const char *s)
-{
-    int method = -1;
-    const char *p = strstr(s, "method=");
-
-    if (p != NULL) {
-	method = get_estimation_method(p + 7);
-    }
-
-    return method;
 }
 
 /* Start compiling an equation system in response to gretl's "system"
-   command: the command must specify either a "method" (estimation
-   method) or a name for the system.  In the former case (method
-   given, but no name), the system will be estimated as soon as its
-   definition is complete, then it will be destroyed.  In the latter
-   case the system definition is saved on a stack, and it can be
-   estimated via various methods (the "estimate" command).
+   command: the command must specify a "method" (estimation method)
+   and/or a name for the system.  If a method is given, the system
+   will be estimated as soon as its definition is complete.  If a name
+   is given, the system definition is saved on a stack, and it can
+   subsequently be estimated via various methods (the "estimate"
+   command).  If both a name and an estimation method are given, the
+   system is both estimated and saved.
 */
 
 gretl_equation_system *system_start (const char *line)
@@ -540,7 +491,7 @@ gretl_equation_system *system_start (const char *line)
     char *sysname = NULL;
     int method;
 
-    method = system_start_get_method(line);
+    method = get_estimation_method_from_line(line);
 
     if (method == SYS_MAX) {
 	/* invalid method was given */
@@ -548,13 +499,12 @@ gretl_equation_system *system_start (const char *line)
 	return NULL;
     }
 
-    if (method < 0) {
-	/* no method was specified: look for a name */
-	sysname = system_start_get_name(line);
-	if (sysname == NULL) {
-	    strcpy(gretl_errmsg, _(badsystem));
-	    return NULL;
-	}
+    sysname = get_system_name_from_line(line);
+
+    if (method < 0 && sysname == NULL) {
+	/* neither a method nor a name was specified */
+	strcpy(gretl_errmsg, _(badsystem));
+	return NULL;
     }
 
     sys = gretl_equation_system_new(method, sysname);
@@ -562,14 +512,7 @@ gretl_equation_system *system_start (const char *line)
 	return NULL;
     }
 
-    if (strstr(line, "save=")) {
-	if (strstr(line, "resids") || strstr(line, "uhat")) {
-	    sys->flags |= GRETL_SYSTEM_SAVE_UHAT;
-	}
-	if (strstr(line, "fitted") || strstr(line, "yhat")) {
-	    sys->flags |= GRETL_SYSTEM_SAVE_YHAT;
-	}
-    }
+    system_set_save_flags(sys, line);
 
     if (sysname != NULL) {
 	free(sysname);
@@ -642,7 +585,7 @@ system_print_F_test (const gretl_equation_system *sys,
 	goto bailout;
     }
 
-    F = gretl_scalar_b_prime_X_b(Rbq, RvR, &err);
+    F = gretl_scalar_b_X_b(Rbq, GRETL_MOD_TRANSPOSE, RvR, &err);
     if (err) {
 	pputs(prn, "Matrix multiplication failed in F test\n");
 	goto bailout;
@@ -685,13 +628,13 @@ system_print_LR_test (const gretl_equation_system *sys,
     pputc(prn, '\n');    
 }
 
-static int sys_test_type (gretl_equation_system *sys, gretlopt opt)
+static int sys_test_type (gretl_equation_system *sys)
 {
     int ret = SYS_NO_TEST;
 
     if (system_n_restrictions(sys) > 0) {
 	if (sys->method == SYS_SUR || sys->method == SYS_WLS) {
-	    if (opt & OPT_T) {
+	    if (sys->flags & GRETL_SYS_ITERATE) {
 		ret = SYS_LR_TEST;
 	    } else {
 		ret = SYS_F_TEST;
@@ -776,7 +719,7 @@ static int estimate_with_test (gretl_equation_system *sys,
 
 /* driver function for the routines in the "sysest" plugin */
 
-static int 
+int 
 gretl_equation_system_estimate (gretl_equation_system *sys, 
 				double ***pZ, DATAINFO *pdinfo, 
 				gretlopt opt, PRN *prn)
@@ -789,6 +732,9 @@ gretl_equation_system_estimate (gretl_equation_system *sys,
 
     *gretl_errmsg = 0;
 
+    /* in case we're re-estimating */
+    system_clear_results(sys);
+
     err = make_instrument_list(sys);
     if (err) goto system_bailout;
     
@@ -803,7 +749,7 @@ gretl_equation_system_estimate (gretl_equation_system *sys,
 	goto system_bailout;
     }
 
-    stest = sys_test_type(sys, opt);
+    stest = sys_test_type(sys);
 
     if (stest == SYS_TEST_NOTIMP) {
 	pprintf(prn, _("Sorry, command not available for this estimator"));
@@ -822,20 +768,17 @@ gretl_equation_system_estimate (gretl_equation_system *sys,
 	close_plugin(handle);
     }
 
-    if (sys->name == NULL) {
-	/* discard the system */
-	gretl_equation_system_destroy(sys);
-    } else {
-	/* retain the system for possible re-estimation */
-	gretl_equation_system_clear(sys);
-    }
+    if (!err) {
+	set_as_last_model(sys, SYSTEM);
+    } 
 
     return err;
 }
 
 /* Finalize an equation system in response to "end system".  If the
-   system has no name but has a method specified, we go ahead and
-   estimate it; otherwise we save it on a stack of defined systems.
+   system has a specified name, we save it on a stack of defined
+   systems.  If it has a specified estimation method, we go ahead
+   and estimate it.  If it has both, we do both.
 */
 
 int gretl_equation_system_finalize (gretl_equation_system *sys, 
@@ -843,6 +786,8 @@ int gretl_equation_system_finalize (gretl_equation_system *sys,
 				    PRN *prn)
 {
     gretlopt opt = OPT_NONE;
+    int err = 0;
+
     *gretl_errmsg = 0;
 
     if (sys == NULL) {
@@ -856,18 +801,78 @@ int gretl_equation_system_finalize (gretl_equation_system *sys,
 	return 1;
     }
 
-    if (sys->name != NULL) {
-	/* save the system for subsequent estimation */
-	return stack_system(sys, prn);
-    }
-
-    if (sys->method < 0 || sys->method >= SYS_MAX) {
+    if (sys->method >= SYS_MAX) {
 	strcpy(gretl_errmsg, _(badsystem));
 	gretl_equation_system_destroy(sys);
 	return 1;
+    }    
+
+#if 1 /* for cli program */
+    if (sys->name != NULL) {
+	/* save the system for subsequent estimation */
+	err = stack_system_as(sys, sys->name);
+    }
+#endif
+
+    if (!err && sys->method >= 0) {
+	err = gretl_equation_system_estimate(sys, pZ, pdinfo, opt, prn);
     }
 
-    return gretl_equation_system_estimate(sys, pZ, pdinfo, opt, prn);
+    return err;
+}
+
+static void 
+adjust_sys_flags_for_method (gretl_equation_system *sys, int method)
+{
+    char old_flags = sys->flags;
+
+    sys->flags = 0;
+
+    if (old_flags & GRETL_SYS_ITERATE) {
+	/* the iterate option is only available for WLS, SUR or 3SLS */
+	if (sys->method == SYS_WLS || sys->method == SYS_SUR ||
+	    sys->method == SYS_3SLS) {
+	    sys->flags |= GRETL_SYS_ITERATE;
+	}
+    }
+
+    /* by default, apply a df correction for single-equation methods */
+    if (sys->method == SYS_OLS || sys->method == SYS_WLS ||
+	sys->method == SYS_TSLS || sys->method == SYS_LIML) {
+	if (old_flags & GRETL_SYSTEM_DFCORR) {
+	    sys->flags |= GRETL_SYSTEM_DFCORR;
+	}
+    } 
+
+    if (old_flags & GRETL_SYS_VCV_GEOMEAN) {
+	sys->flags |= GRETL_SYS_VCV_GEOMEAN;
+    }    
+}
+
+static void 
+set_sys_flags_from_opt (gretl_equation_system *sys, gretlopt opt)
+{
+    sys->flags = 0;
+
+    if (opt & OPT_T) {
+	/* the iterate option is only available for WLS, SUR or 3SLS */
+	if (sys->method == SYS_WLS || sys->method == SYS_SUR ||
+	    sys->method == SYS_3SLS) {
+	    sys->flags |= GRETL_SYS_ITERATE;
+	}
+    }
+
+    /* by default, apply a df correction for single-equation methods */
+    if (sys->method == SYS_OLS || sys->method == SYS_WLS ||
+	sys->method == SYS_TSLS || sys->method == SYS_LIML) {
+	if (!(opt & OPT_N)) {
+	    sys->flags |= GRETL_SYSTEM_DFCORR;
+	}
+    } 
+
+    if (opt & OPT_M) {
+	sys->flags |= GRETL_SYS_VCV_GEOMEAN;
+    } 
 }
 
 /* Implement the "estimate" command, which must give the name of a pre-defined
@@ -888,13 +893,13 @@ int estimate_named_system (const char *line, double ***pZ, DATAINFO *pdinfo,
 	return 1;
     }
 
-    sysname = get_system_name_from_line(line + 9);
+    sysname = get_system_name_from_line(line);
     if (sysname == NULL) {
 	strcpy(gretl_errmsg, "estimate: no system name was provided");
 	return 1;
     }
 
-    sys = get_equation_system_by_name(sysname, NULL);
+    sys = get_equation_system_by_name(sysname);
     if (sys == NULL) {
 	sprintf(gretl_errmsg, "'%s': unrecognized name", sysname);
 	free(sysname);
@@ -903,7 +908,11 @@ int estimate_named_system (const char *line, double ***pZ, DATAINFO *pdinfo,
 
     free(sysname);
 
-    method = named_system_get_method(line);
+    method = get_estimation_method_from_line(line);
+    if (method < 0 || method >= SYS_MAX) {
+	method = sys->method;
+    }
+
     if (method < 0 || method >= SYS_MAX) {
 	strcpy(gretl_errmsg, "estimate: no valid method was specified");
 	return 1;
@@ -911,28 +920,33 @@ int estimate_named_system (const char *line, double ***pZ, DATAINFO *pdinfo,
 
     sys->method = method;
 
-    if (opt & OPT_T) {
-	/* the iterate option is available for WLS, SUR or 3SLS */
-	if (method != SYS_WLS && method != SYS_SUR && method != SYS_3SLS) {
-	    opt ^= OPT_T;
-	}
+    if (opt == OPT_UNSET) {
+	opt = OPT_NONE;
+	adjust_sys_flags_for_method(sys, method);
+    } else {
+	set_sys_flags_from_opt(sys, opt);
     }
 
-    /* by default, apply a df correction for single-equation methods */
-    if (method == SYS_OLS || method == SYS_WLS ||
-	method == SYS_TSLS || method == SYS_LIML) {
-	if (!(opt & OPT_N)) {
-	    sys->flags |= GRETL_SYSTEM_DFCORR;
-	}
+    /* if restrictions are in place, reset the restricted flag */
+    if (sys->R != NULL && sys->q != NULL) {
+	sys->flags |= GRETL_SYS_RESTRICT;
     } 
 
-    if (opt & OPT_M) {
-	sys->flags |= GRETL_SYS_VCV_GEOMEAN;
-    }    
-
     return gretl_equation_system_estimate(sys, pZ, pdinfo, opt, prn);
 }
 
+int estimate_saved_equation_system (gretl_equation_system *sys, 
+				    double ***pZ, DATAINFO *pdinfo,
+				    PRN *prn)
+{
+    char line[128];
+    int err;
+
+    sprintf(line, "estimate \"%s\"", sys->name);
+    err = estimate_named_system(line, pZ, pdinfo, OPT_UNSET, prn);
+    return err;
+}
+
 static int get_real_list_length (const int *list)
 {
     int i, len = list[0];
@@ -978,6 +992,19 @@ const char *gretl_system_short_string (const MODEL *pmod)
     return gretl_system_short_strings[i];
 }
 
+void gretl_system_set_name (gretl_equation_system *sys, const char *name)
+{
+    if (sys->name != NULL && !strcmp(sys->name, name)) {
+	return;
+    }
+
+    if (sys->name != NULL) {
+	free(sys->name);
+    }
+
+    sys->name = gretl_strdup(name);
+}
+
 const char *gretl_system_get_name (const gretl_equation_system *sys)
 {
     return sys->name;
@@ -1047,12 +1074,76 @@ void system_set_iters (gretl_equation_system *sys, int n)
     sys->iters = n;
 }
 
+int system_normality_test (const gretl_equation_system *sys, PRN *prn)
+{
+    int err = 0;
+
+    if (sys->uhat == NULL || sys->sigma == NULL) {
+	err = 1;
+    } else {
+	err = gretl_system_normality_test(sys->uhat, sys->sigma, prn);
+    }
+
+    return err;
+}
+
+void make_system_data_info (gretl_equation_system *sys, int eqn, 
+			    DATAINFO *pdinfo, int v, int code)
+{
+    if (code == GRETL_SYSTEM_SAVE_UHAT) {
+	sprintf(pdinfo->varname[v], "uhat_s%02d", eqn);
+	if (sys->method == SYS_SUR) {
+	    sprintf(VARLABEL(pdinfo, v), _("SUR residual, equation %d"), eqn);
+	} else if (sys->method == SYS_3SLS) {
+	    sprintf(VARLABEL(pdinfo, v), _("3SLS residual, equation %d"), eqn);
+	} else {
+	    sprintf(VARLABEL(pdinfo, v), "system residual, equation %d", eqn);
+	}
+    } else if (code == GRETL_SYSTEM_SAVE_YHAT) {
+	sprintf(pdinfo->varname[v], "yhat_s%02d", eqn);
+	if (sys->method == SYS_SUR) {
+	    sprintf(VARLABEL(pdinfo, v), _("SUR fitted value, equation %d"), eqn);
+	} else if (sys->method == SYS_3SLS) {
+	    sprintf(VARLABEL(pdinfo, v), _("3SLS fitted value, equation %d"), eqn);
+	} else {
+	    sprintf(VARLABEL(pdinfo, v), "system fitted value, equation %d", eqn);
+	}
+    }
+}
+
+int gretl_system_add_resids_to_dataset (const char *sysname, int eqnum,
+					double ***pZ, DATAINFO *pdinfo)
+{
+    gretl_equation_system *sys;
+    int v, t;
+
+    sys = get_equation_system_by_name(sysname);
+    if (sys == NULL || sys->uhat == NULL) {
+	return 1;
+    }
+
+    if (dataset_add_series(1, pZ, pdinfo)) return E_ALLOC;
+
+    v = pdinfo->v - 1;
+
+    for (t=0; t<pdinfo->n; t++) {
+	if (t < sys->t1 || t > sys->t2) {
+	    (*pZ)[v][t] = NADBL;
+	} else {
+	    (*pZ)[v][t] = gretl_matrix_get(sys->uhat, t - sys->t1, eqnum);
+	}
+    }
+
+    make_system_data_info(sys, eqnum + 1, pdinfo, v, GRETL_SYSTEM_SAVE_UHAT);
+
+    return 0;
+}
+
 /* simple accessor functions */
 
-const char *system_get_full_string (const gretl_equation_system *sys,
-				    gretlopt opt)
+const char *system_get_full_string (const gretl_equation_system *sys)
 {
-    if (opt & OPT_T) {
+    if (sys->flags & GRETL_SYS_ITERATE) {
 	static char sysstr[64];
 
 	sprintf(sysstr, _("iterated %s"), gretl_system_long_strings[sys->method]);
@@ -1202,8 +1293,6 @@ gretl_matrix *system_get_sigma (const gretl_equation_system *sys)
     return sys->sigma;
 }
 
-
-
 MODEL *system_get_model (const gretl_equation_system *sys, int i)
 {
     if (sys->models == NULL || i >= sys->n_equations) {
@@ -1223,6 +1312,11 @@ double system_get_llu (const gretl_equation_system *sys)
     return sys->llu;
 }
 
+double system_get_ess (const gretl_equation_system *sys)
+{
+    return sys->ess;
+}
+
 double system_get_X2 (const gretl_equation_system *sys)
 {
     return sys->X2;
@@ -1562,7 +1656,7 @@ int
 system_parse_line (gretl_equation_system *sys, const char *line,
 		   const DATAINFO *pdinfo)
 {
-    int err = 1;
+    int err = 0;
 
     *gretl_errmsg = '\0';
 
@@ -1572,6 +1666,8 @@ system_parse_line (gretl_equation_system *sys, const char *line,
 	err = add_aux_list_to_sys(sys, line + 5, pdinfo, ENDOG_LIST);
     } else if (strncmp(line, "instr", 5) == 0) {
 	err = add_aux_list_to_sys(sys, line + 5, pdinfo, INSTR_LIST);
+    } else {
+	err = E_PARSE;
     }
 
     if (err) {
@@ -1707,6 +1803,3 @@ system_set_restriction_matrices (gretl_equation_system *sys,
 
     sys->flags |= GRETL_SYS_RESTRICT;
 }
-
-
-
