@@ -35,13 +35,8 @@ typedef enum {
 
 gretl_equation_system *system_start (const char *line);
 
-gretl_equation_system *
-get_equation_system_by_name (const char *sysname, int *snum);
-
 char *get_system_name_from_line (const char *s);
 
-void gretl_equation_systems_cleanup (void);
-
 int gretl_equation_system_append (gretl_equation_system *sys, 
 				  const int *list);
 
@@ -53,13 +48,21 @@ int gretl_equation_system_finalize (gretl_equation_system *sys,
 				    double ***pZ, DATAINFO *pdinfo,
 				    PRN *prn);
 
+int 
+gretl_equation_system_estimate (gretl_equation_system *sys, 
+				double ***pZ, DATAINFO *pdinfo, 
+				gretlopt opt, PRN *prn);
+
 int estimate_named_system (const char *line, double ***pZ, DATAINFO *pdinfo, 
 			   gretlopt opt, PRN *prn);
 
+int estimate_saved_equation_system (gretl_equation_system *sys, 
+				    double ***pZ, DATAINFO *pdinfo,
+				    PRN *prn);
+
 void gretl_equation_system_destroy (gretl_equation_system *sys);
 
-const char *system_get_full_string (const gretl_equation_system *sys,
-				    gretlopt opt);
+const char *system_get_full_string (const gretl_equation_system *sys);
 
 int system_save_uhat (const gretl_equation_system *sys);
 int system_save_yhat (const gretl_equation_system *sys);
@@ -91,6 +94,8 @@ int *compose_tsls_list (gretl_equation_system *sys, int i);
 int system_get_depvar (const gretl_equation_system *sys, int i);
 
 const char *gretl_system_short_string (const MODEL *pmod);
+
+void gretl_system_set_name (gretl_equation_system *sys, const char *name);
 const char *gretl_system_get_name (const gretl_equation_system *sys);
 
 const gretl_matrix *system_get_R_matrix (const gretl_equation_system *sys);
@@ -98,6 +103,10 @@ const gretl_matrix *system_get_q_matrix (const gretl_equation_system *sys);
 
 int system_get_method (const gretl_equation_system *sys);
 
+int gretl_system_method_from_string (const char *s);
+const char *system_method_full_string (int method);
+const char *system_method_short_string (int method);
+
 int *system_get_endog_vars (const gretl_equation_system *sys);
 int *system_get_instr_vars (const gretl_equation_system *sys);
 
@@ -115,6 +124,7 @@ MODEL *system_get_model (const gretl_equation_system *sys, int i);
 double system_get_ll (const gretl_equation_system *sys);
 double system_get_llu (const gretl_equation_system *sys);
 double system_get_X2 (const gretl_equation_system *sys);
+double system_get_ess (const gretl_equation_system *sys);
 double system_get_diag_stat (const gretl_equation_system *sys);
 
 void system_set_ll (gretl_equation_system *sys, double ll);
@@ -140,4 +150,10 @@ void
 system_set_restriction_matrices (gretl_equation_system *sys,
 				 gretl_matrix *R, gretl_matrix *q);
 
+int 
+system_normality_test (const gretl_equation_system *sys, PRN *prn);
+
+int gretl_system_add_resids_to_dataset (const char *sysname, int eqnum,
+					double ***pZ, DATAINFO *pdinfo);
+
 #endif /* GRETL_EQUATION_SYSTEM_H */
