@@ -20,6 +20,9 @@
 /* texprint.c for gretl - LaTeX output from modeling */
 
 #include "libgretl.h"
+#include "johansen.h"
+
+#define tex_screen_zero(x)  ((fabs(x) > 1.0e-17)? x : 0.0)
 
 static void tex_modify_exponent (char *numstr)
 {
@@ -27,8 +30,11 @@ static void tex_modify_exponent (char *numstr)
 
     if (p != NULL) {
 	int expon = atoi(p + 1);
-
+#if 0
 	sprintf(p, "\\times 10^{%d}", expon);
+#else
+	sprintf(p, "\\mbox{e%s%d}", (expon > 0)? "+" : "", expon);
+#endif
     }
 }
 
@@ -41,13 +47,41 @@ static void tex_print_float (double x, PRN *prn)
 {
     char number[48];
 
-    x = screen_zero(x);
+    x = tex_screen_zero(x);
 
     sprintf(number, "%#.*g", GRETL_DIGITS, x);
     tex_modify_exponent(number);
     pprintf(prn, "%s", (x < 0.0)? (number + 1) : number);
 }
 
+static void tex_print_signed_float (double x, PRN *prn)
+{
+    char number[48];
+
+    x = tex_screen_zero(x);
+
+    sprintf(number, "%#.*g", GRETL_DIGITS, x);
+    tex_modify_exponent(number);
+
+    if (x < 0.0) {
+	pprintf(prn, "$-$%s", number + 1);
+    } else {
+	pputs(prn, number);
+    }
+}
+
+static void tex_print_math_float (double x, PRN *prn)
+{
+    char number[48];
+
+    x = tex_screen_zero(x);
+
+    sprintf(number, "%#.*g", GRETL_DIGITS, x);
+    tex_modify_exponent(number);
+
+    pprintf(prn, "$%s$", number);
+}
+
 /**
  * tex_escape:
  * @targ: target string (must be pre-allocated)
@@ -238,6 +272,32 @@ static void tex_lagname (char *s, const DATAINFO *pdinfo, int v)
     }
 }
 
+static void tex_vecm_varname (char *s, const DATAINFO *pdinfo, int v)
+{
+    const char *lbl = VARLABEL(pdinfo, v);
+    int cvnum;
+    int gotit = 0;
+
+    if (sscanf(pdinfo->varname[v], "EC%d", &cvnum)) {
+	sprintf(s, "EC%d$_{t-1}$", cvnum);
+	gotit = 1;
+    } else if (strlen(lbl) > 2) {
+	char myvar[24], tmp[9];
+	int lag;
+
+	lbl += 2;
+	if (sscanf(lbl, "d_%8[^(](t - %d)", tmp, &lag) == 2) {
+	    tex_escape(myvar, tmp);
+	    sprintf(s, "$\\Delta$%s$_{t-%d}$", myvar, lag);
+	    gotit = 1;
+	}
+    }
+
+    if (!gotit) {
+	tex_escape(s, pdinfo->varname[v]); 
+    }
+}
+
 int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
 		     int i, PRN *prn)
 {
@@ -273,6 +333,8 @@ int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	tex_garch_coeff_name(tmp, pmod->params[i+1], 0);
     } else if (pmod->ci == VAR) {
 	tex_lagname(tmp, pdinfo, pmod->list[i+2]);
+    } else if (pmod->aux == AUX_VECM) {
+	tex_vecm_varname(tmp, pdinfo, pmod->list[i+2]);
     } else {
 	tex_escape(tmp, pdinfo->varname[pmod->list[i+2]]);
     }
@@ -311,6 +373,142 @@ int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
     return 0;
 }
 
+void tex_print_VECM_omega (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *prn)
+{
+    char vname[32];
+    const int *list = vecm->jinfo->list;
+    double x;
+    int i, j;
+
+    pprintf(prn, "%s\n\n", I_("Cross-equation covariance matrix"));
+    pputs(prn, "\\vspace{1em}\n");
+
+    pputs(prn, "\\begin{tabular}{");
+    pputs(prn, "l");
+    for (i=0; i<vecm->neqns; i++) {
+	pputs(prn, "r");
+    }
+    pputs(prn, "}\n & ");
+
+    for (i=0; i<vecm->neqns; i++) {
+	tex_escape(vname, pdinfo->varname[list[i+1]]);
+	pprintf(prn, "$\\Delta$%s ", vname);
+	if (i == vecm->neqns - 1) {
+	    pputs(prn, "\\\\\n");
+	} else {
+	    pputs(prn, "& ");
+	}
+    }
+    pputc(prn, '\n');
+
+    for (i=0; i<vecm->neqns; i++) {
+	tex_escape(vname, pdinfo->varname[list[i+1]]);
+	pprintf(prn, "$\\Delta$%s & ", vname);
+	for (j=0; j<vecm->neqns; j++) {
+	    x = gretl_matrix_get(vecm->S, i, j);
+	    tex_print_math_float(x, prn);
+	    if (j == vecm->neqns - 1) {
+		pputs(prn, "\\\\\n");
+	    } else {
+		pputs(prn, " & ");
+	    }
+	}
+    }
+
+    pputs(prn, "\\end{tabular}\n\n");
+    pputs(prn, "\\vspace{1em}\n");
+
+    pputs(prn, "\\noindent\n");
+    pprintf(prn, "%s = ", I_("determinant"));
+    tex_print_math_float(exp(vecm->ldet), prn);
+    pputs(prn, "\\\\\n");
+}
+
+void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATAINFO *pdinfo, PRN *prn)
+{
+    char s[16];
+    JohansenInfo *jv = vecm->jinfo;
+    int rows = gretl_matrix_rows(jv->Beta);
+    int i, j;
+    double x;
+
+    pputs(prn, "\\noindent\n");
+    pputs(prn, _("Cointegrating vectors"));
+    if (jv->Bse != NULL) {
+	pprintf(prn, " (%s)\n", _("standard errors in parentheses"));
+    } 
+
+    pputs(prn, "\n\\vspace{1em}\n");
+
+    pputs(prn, "\\begin{tabular}{");
+    pputs(prn, "l");
+    for (i=0; i<jv->rank; i++) {
+	pputs(prn, "r");
+    }
+    pputs(prn, "}\n");
+
+    for (i=0; i<rows; i++) {
+	if (i < jv->list[0]) {
+	    tex_escape(s, pdinfo->varname[jv->list[i+1]]);
+	    pprintf(prn, "%s$_{t-1}$ & ", s);
+	} else if (jv->code == J_REST_CONST) {
+	    pputs(prn, "const & ");
+	} else if (jv->code == J_REST_TREND) {
+	    pputs(prn, "trend & ");
+	}
+
+	/* coefficients */
+	for (j=0; j<jv->rank; j++) {
+	    x = gretl_matrix_get(jv->Beta, i, j);
+	    if (jv->Bse == NULL) {
+		x /= gretl_matrix_get(jv->Beta, j, j);
+	    }
+	    tex_print_signed_float(x, prn);
+	    if (j == jv->rank - 1) {
+		pputs(prn, "\\\\\n");
+	    } else {
+		pputs(prn, "& ");
+	    }	    
+	}
+
+	if (jv->Bse != NULL) {
+	    /* standard errors */
+	    pputs(prn, " & ");
+	    for (j=0; j<jv->rank; j++) {
+		if (i < jv->rank) {
+		    x = 0.0;
+		} else {
+		    x = gretl_matrix_get(jv->Bse, i - jv->rank, j);
+		}
+		pputc(prn, '(');
+		tex_print_float(x, prn);
+		pputc(prn, ')');
+		if (j == jv->rank - 1) {
+		    pputs(prn, "\\\\\n");
+		} else {
+		    pputs(prn, "& ");
+		}
+	    }
+	}
+    }
+
+    pputs(prn, "\\end{tabular}\n\n\\vspace{1em}\n");
+}
+
+void tex_print_VAR_ll_stats (GRETL_VAR *var, PRN *prn)
+{
+    pprintf(prn, "\\noindent\n%s = ", I_("Log-likelihood"));
+    tex_print_math_float(var->ll, prn);
+    pputs(prn, "\\par\n");
+
+    pprintf(prn, "\\noindent\n%s = ", I_("Determinant of covariance matrix"));
+    tex_print_math_float(exp(var->ldet), prn);
+    pputs(prn, "\\par\n");
+
+    pprintf(prn, "\\noindent\n%s $= %.4f$ \\par\n", I_("AIC"), var->AIC);
+    pprintf(prn, "\\noindent\n%s $= %.4f$ \\par\n", I_("BIC"), var->BIC);
+}
+
 static PRN *make_tex_prn (int ID, char *texfile,
 			  int eqn, int doc)
 {
