@@ -29,6 +29,7 @@
 #include <time.h>
 
 #include "libgretl.h"
+#include "version.h"
 
 #include "importer.h"
 #include "biff.h"
@@ -37,15 +38,18 @@ static void free_sheet (void);
 static int allocate_row_col (int row, int col, wbook *book);
 static char *copy_unicode_string (unsigned char *src, int remlen, 
 				  int *skip, int *slop);
-static char *convert8to7 (const unsigned char *s, int count);
+static char *convert8to7 (const char *s, int count);
 static char *convert16to7 (const unsigned char *s, int count);
 static char *make_string (char *str);
 
-#undef EDEBUG
-#undef MEMDEBUG
+int debug_print;
 
-#define EXCEL_IMPORTER
-#include "import_common.c"
+#ifdef WIN32
+# include "build.h"
+char debug_fname[FILENAME_MAX];
+FILE *fdb;
+static void make_debug_fname (void);
+#endif
 
 #define cell_record(r) (r == BIFF_LABEL || \
                         r == BIFF_STRING || \
@@ -72,9 +76,64 @@ int sstsize = 0, sstnext = 0;
 struct sheetrow *rows = NULL;
 int nrows = 0;
 
+#define EXCEL_IMPORTER
+#include "import_common.c"
+
 const char *adjust_rc = N_("Perhaps you need to adjust the "
 			   "starting column or row?");
 
+#ifdef WIN32
+static void make_debug_fname (void)
+{
+    read_reg_val(HKEY_CURRENT_USER, "gretl", "userdir", debug_fname);
+    strcat(debug_fname, "xls.log");
+}
+
+static void open_debug_stream (void)
+{
+    if (debug_fname[0] != '\0') {
+	fdb = fopen(debug_fname, "w");
+    }
+}
+#endif
+
+static int dprintf (const char *format, ...)
+{
+    va_list args;
+    int len = 0;
+
+#ifdef WIN32
+    if (debug_print) {
+	if (fdb == NULL) {
+	    open_debug_stream();
+	}
+	if (fdb != NULL) {
+	    va_start(args, format);
+	    len = vfprintf(fdb, format, args);
+	    va_end(args);
+	    fflush(fdb);
+	}
+    }
+#else
+    if (debug_print) {
+	va_start(args, format);
+	len = vfprintf(stderr, format, args);
+	va_end(args);
+	fflush(stderr);
+    }
+#endif
+
+    return len;
+}
+
+static void print_version (void)
+{
+#ifdef WIN32
+    dprintf("gretl, version %s, %s\n", GRETL_VERSION, BUILD_DATE);
+#else
+    dprintf("gretl, version %s\n", GRETL_VERSION); 
+#endif
+}
 
 static double get_le_double (const unsigned char *rec) 
 {
@@ -203,30 +262,65 @@ static int check_copy_string (struct sheetrow *prow, int row, int col,
 		}
 	    }
 	    *p = '\0';
-#ifdef EDEBUG
-	    fprintf(stderr, "converting sst[%d] '%s' to numeric as %s\n", idx, s, q);
-#endif
+	    dprintf("converting sst[%d] '%s' to numeric as %s\n", idx, s, q);
 	    prow->cells[col] = g_strdup_printf("%s", q);
 	    free(q);
 	    return 0;
 	}
     }
 
-#ifdef EDEBUG
-    fprintf(stderr, "copying sst[%d] '%s' into place\n", idx, s);
-#endif
+    dprintf("copying sst[%d] '%s' into place\n", idx, s);
     prow->cells[col] = g_strdup_printf("\"%s", s);
 
     return 0;
 }
 
+static int is_date_format (int fmt)
+{
+    int ret = 0;
+
+    switch (fmt) {
+    case 14:
+    case 15:
+    case 16:
+    case 17:
+    case 18:
+    case 19:
+    case 20:
+    case 21:
+    case 22:
+    case 45:
+    case 46:
+    case 47:
+    case 50:
+	ret = 1;
+	break;
+    default:
+	break;
+    }
+
+    return ret;
+}
+
+static int wbook_find_format (wbook *book, int xfref)
+{
+    int fmt = -1;
+
+    if (book->xf_list != NULL && xfref < book->xf_list[0]) {
+	fmt = book->xf_list[xfref + 1];
+    }
+
+    return fmt;
+}
+
+#undef FORMAT_INFO
+
 static int process_item (BiffQuery *q, wbook *book, PRN *prn) 
 {
     struct sheetrow *prow = NULL;
     static char **string_targ;
     static int slop; /* SST overslop */
     unsigned char *ptr = NULL;
-    int xfref = 0;
     int i = 0, j = 0;
     double val;
 
@@ -236,15 +330,23 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	if (row_col_err(i, j, prn)) {
 	    return 1;
 	}
-	xfref = EX_GETXF(q);
-    }
+	if (q->ls_op == BIFF_NUMBER || q->ls_op == BIFF_RK) {
+	    guint16 xfref = EX_GETXF(q);
+	    int fmt = wbook_find_format(book, xfref);
 
-#ifdef FORMAT_INFO
-    if (q->ls_op == BIFF_NUMBER || q->ls_op == BIFF_RK) {
-	fprintf(stderr, "Numeric cell (%d, %d), format ref = %d\n", 
-		i, j, xfref);
-    }
+#if 0
+	    fprintf(stderr, "Numeric cell (%d, %d), XF index = %d, fmt = %d\n", 
+		    i, j, (int) xfref, fmt);
 #endif
+	    if (i == book->row_offset + 1 && 
+		j == book->col_offset && 
+		is_date_format(fmt)) {
+		fprintf(stderr, "Testing first obs cell (%d, %d): date format %d\n", 
+			i, j, fmt);
+		book->flags |= FIRST_COL_DATE_FORMAT;
+	    }
+	}
+    }
 
     switch (q->ls_op) {
 
@@ -264,10 +366,9 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    return 1;
 	}
 
-#ifdef EDEBUG
-	fprintf(stderr, "Got SST: allocated for %d strings (%d bytes), %p\n", 
+	dprintf("Got SST: allocated for %d strings (%d bytes), %p\n", 
 		sstsize, sstsize * sizeof *sst, (void *) sst);
-#endif
+
 	for (k=oldsz; k<sstsize; k++) {
 	    /* careful: initialize all pointers to NULL */
 	    sst[k] = NULL;
@@ -277,10 +378,8 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 
 	for (k=oldsz; k<sstsize; k++) {
 	    remlen = q->length - (ptr - q->data);
-#ifdef EDEBUG
-	    fprintf(stderr, "Working on sst[%d], data offset=%d, remlen=%d\n", 
+	    dprintf("Working on sst[%d], data offset=%d, remlen=%d\n", 
 		    k, (int) (ptr - q->data), remlen);
-#endif
 	    if (remlen <= 0) {
 		break;
 	    }
@@ -296,10 +395,8 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
     }	
 
     case BIFF_CONTINUE: 
-#ifdef EDEBUG
-	fprintf(stderr, "Got CONTINUE, sstnext = %d, len = %d\n", 
+	dprintf("Got CONTINUE, sstnext = %d, len = %d\n", 
 		sstnext, (int) q->length);
-#endif
 	if (sstnext > 0) {
 	    int k, skip, remlen;
 
@@ -315,9 +412,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 		if (remlen <= 0) {
 		    break;
 		}
-#ifdef EDEBUG
-		fprintf(stderr, "Working on sst[%d]\n", k);
-#endif
+		dprintf("Working on sst[%d]\n", k);
 		sst[k] = copy_unicode_string(ptr, remlen, &skip, &slop);
 		ptr += skip;
 	    }
@@ -328,9 +423,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	break;
 			   
     case BIFF_LABEL: 
-#ifdef EDEBUG
-	fprintf(stderr, "Got LABEL, row=%d, col=%d\n", i, j);
-#endif
+	dprintf("Got LABEL, row=%d, col=%d\n", i, j);
 	if (allocate_row_col(i, j, book)) {
 	    return 1;
 	} else {
@@ -338,14 +431,12 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	
 	    prow = rows + i;
 	    ptr = q->data + 8;
-	    prow->cells[j] = make_string(convert8to7(ptr, len));
+	    prow->cells[j] = make_string(convert8to7((char *) ptr, len));
 	}
 	break;
   
     case BIFF_LABELSST:
-#ifdef EDEBUG
-	fprintf(stderr, "Got LABELSST, row=%d, col=%d\n", i, j);
-#endif
+	dprintf("Got LABELSST, row=%d, col=%d\n", i, j);
 	if (allocate_row_col(i, j, book)) {
 	    return 1;
 	} else {
@@ -373,9 +464,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    val = get_le_double(q->data + 6);
 	    prow = rows + i;
 	    prow->cells[j] = g_strdup_printf("%.10g", val);
-#ifdef EDEBUG
-	    fprintf(stderr, "Got NUMBER (%g), row=%d, col=%d\n", val, i, j);
-#endif
+	    dprintf("Got NUMBER (%g), row=%d, col=%d\n", val, i, j);
 	}
 	break;
 
@@ -386,20 +475,14 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    val = biff_get_rk(q->data + 6);
 	    prow = rows + i;
 	    prow->cells[j] = g_strdup_printf("%.10g", val);
-#ifdef EDEBUG
-	    fprintf(stderr, "Got RK (%g), row=%d, col=%d\n", val, 
-		    i, j);
-#endif
+	    dprintf("Got RK (%g), row=%d, col=%d\n", val, i, j);
 	}
 	break;
 
     case BIFF_MULRK: {
 	int k, ncols = (q->length - 6) / 6;
 
-#ifdef EDEBUG
-	fprintf(stderr, "Got MULRK, row=%d, first_col=%d, ncols=%d\n", 
-		i, j, ncols);
-#endif
+	dprintf("Got MULRK, row=%d, first_col=%d, ncols=%d\n", i, j, ncols);
 	for (k=0; k<ncols; k++) {
 	    if (allocate_row_col(i, j, book)) {
 		return 1;
@@ -407,18 +490,14 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    val = biff_get_rk(q->data + 6 + 6 * k);
 	    prow = rows + i; /* might have moved */
 	    prow->cells[j] = g_strdup_printf("%.10g", val);
-#ifdef EDEBUG
-	    fprintf(stderr, " MULRK[col=%d] = %g\n", j, val);
-#endif
+	    dprintf(" MULRK[col=%d] = %g\n", j, val);
 	    j++;
 	}
 	break;
     }
 
     case BIFF_FORMULA:  
-#ifdef EDEBUG
-	fprintf(stderr, "Got FORMULA, row=%d, col=%d\n", i, j);
-#endif
+	dprintf("Got FORMULA, row=%d, col=%d\n", i, j);
 	if (allocate_row_col(i, j, book)) {
 	    return 1;
 	} else {
@@ -445,9 +524,7 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    } else {
 		/* floating-point */
 		val = get_le_double(ptr);
-#ifdef EDEBUG
-		fprintf(stderr, " floating-point value = %g\n", val);
-#endif
+		dprintf(" floating-point value = %g\n", val);
 		if (isnan(val)) {
 		    fprintf(stderr, "Got a NaN\n");
 		    prow->cells[j] = g_strdup("-999.0");
@@ -460,16 +537,12 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 
     case BIFF_STRING: 
 	if (string_targ == NULL) {
-#ifdef EDEBUG
-	    fprintf(stderr, "String record without preceding string formula\n");
-#endif
+	    dprintf("String record without preceding string formula\n");
 	} else {
 	    char *tmp = copy_unicode_string(q->data, 0, NULL, NULL);
 
 	    *string_targ = make_string(tmp);
-#ifdef EDEBUG
-	    fprintf(stderr, "Filled out string formula with '%s'\n", *string_targ);	
-#endif
+	    dprintf("Filled out string formula with '%s'\n", *string_targ);	
 	    string_targ = NULL;
 	}
 	break;
@@ -479,18 +552,24 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	    fprintf(stderr, "BOF when current sheet is not flushed\n");
 	    return 1;
 	}
-#ifdef EDEBUG
 	if (1) {
 	    unsigned version, boftype;
 
 	    version = MS_OLE_GET_GUINT16(q->data + 0);
 	    boftype = MS_OLE_GET_GUINT16(q->data + 2);
-	    fprintf(stderr, "Got BOF: version=%x, type=%x\n", 
-		    version, boftype);
+	    dprintf("Got BOF: version=%x, type=%x\n", version, boftype);
 	}
-#endif
 	break;
 
+    case BIFF_FORMAT: {
+	int idx = MS_OLE_GET_GUINT16(q->data + 0);
+
+	if (idx >= 14 && idx <= 17) {
+	    fprintf(stderr, "Got date format: index %d\n", idx);
+	}
+	break;
+    }
+
 #ifdef FORMAT_INFO
     case BIFF_COLINFO:
 	fprintf(stderr, "Got BIFF_COLINFO: col range (%d, %d), XF index %d\n",
@@ -511,15 +590,6 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	}
 	break;
     }
-
-    case BIFF_FORMAT: {
-	int idx = MS_OLE_GET_GUINT16(q->data + 0);
-
-	if ((idx >= 14 && idx <= 17) || idx >= 164) {
-	    fprintf(stderr, "Got BIFF_FORMAT: index %d\n", idx);
-	}
-	break;
-    }
 #endif
 
     default: 
@@ -610,14 +680,9 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
     } 
 
     while (!err && ms_biff_query_next(q)) {
-#ifdef EDEBUG
-	fprintf(stderr, "At %lu: q->opcode=0x%02x\n", 
-		(unsigned long) q->streamPos, q->opcode);
-#endif
+	dprintf("At %lu: q->opcode=0x%02x\n", (unsigned long) q->streamPos, q->opcode);
 	if (q->opcode == BIFF_EOF) {
-#ifdef EDEBUG
-	    fprintf(stderr, "got MSEOF at %lu\n", (unsigned long) stream->position);
-#endif
+	    dprintf("got MSEOF at %lu\n", (unsigned long) stream->position);
 	    eofcount++;
 
 	    if (eofcount == 1) {
@@ -641,12 +706,13 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
 
 	if (handled_record(q)) {
 	    err = process_item(q, book, prn);
-	} 
-#ifdef EDEBUG
-	else {
-	    fprintf(stderr, "skipping unhandled opcode 0x%02x\n", q->opcode);
+	} else if (q->ms_op == 0x02 && q->ls_op == BIFF_ROW) {
+	    dprintf("Got BIFF_ROW\n");
+	} else if (q->opcode == BIFF_DBCELL) {
+	    dprintf("Got BIFF_DBCELL\n");
+	} else {
+	    dprintf("skipping unhandled opcode 0x%02x\n", q->opcode);
 	}
-#endif
     }
 
     ms_biff_query_destroy(q);
@@ -679,16 +745,15 @@ copy_unicode_string (unsigned char *src, int remlen,
     int this_skip = 3, skip_to_next = 3;
     int csize = (flags & 0x01)? 2 : 1;
 
-#ifdef EDEBUG
-    fprintf(stderr, "copy_unicode_string: count = %d, csize = %d\n",
+    dprintf("copy_unicode_string: count = %d, csize = %d\n",
 	    count, csize);
+
     if (flags & 0x08) {
-	fprintf(stderr, " contains Rich-Text info\n");
+	dprintf(" contains Rich-Text info\n");
     }
     if (flags & 0x04) {
-	fprintf(stderr, " contains Far-East info\n");
+	dprintf(" contains Far-East info\n");
     }    
-#endif
 
     skip_to_next += count * csize;
 
@@ -726,14 +791,17 @@ copy_unicode_string (unsigned char *src, int remlen,
 	}
     }
 
-    if (csize == 1) {
-	return convert8to7(src + this_skip, count);
+    if (count > 64) {
+	/* let's not mess with excessive strings */
+	return g_strdup("bigstr");
+    } else if (csize == 1) {
+	return convert8to7((char *) src + this_skip, count);
     } else { 
 	return convert16to7(src + this_skip, count);
     }
 }
 
-static char *convert8to7 (const unsigned char *s, int count) 
+static char *convert8to7 (const char *s, int count) 
 {
     char *dest;
 
@@ -750,18 +818,15 @@ static char *convert8to7 (const unsigned char *s, int count)
 	strcpy(dest, "varname");
     }
 
-#ifdef EDEBUG
-    fprintf(stderr, "convert8to7: returning '%s'\n", dest);
-#endif
+    dprintf("convert8to7: returning '%s'\n", dest);
 
     return dest;
 }
 
 static char *convert16to7 (const unsigned char *s, int count) 
 {
-    char *p, *dest;
-    int i, j;
-    guint16 u;
+    char *dest;
+    int i, u, j = 0;
 
     dest = malloc(USER_VLEN);
     if (dest == NULL) {
@@ -770,24 +835,37 @@ static char *convert16to7 (const unsigned char *s, int count)
 
     memset(dest, 0, USER_VLEN);
 
-    p = dest;
-    j = 0;
+#if 1
     for (i=0; i<count && j<USER_VLEN-1; i++) {
 	u = MS_OLE_GET_GUINT16(s);
 	s += 2;
 	if ((isalnum(u) || ispunct(u)) && u < 128) {
-	    *p++ = u;
-	    j++;
+	    dest[j++] = u;
 	}
     }
+#else
+    printf("convert16to7: count = %d\n", count);
+    for (i=0; i<count; i++) {
+	u = MS_OLE_GET_GUINT16(s);
+	s += 2;
+	if ((isalnum(u) || ispunct(u) || u == ' ') && u < 128) {
+	    putchar(u);
+	    if (j<USER_VLEN-1) {
+		dest[j++] = u;
+	    }
+	    if (i % 56 == 0) {
+		putchar('\n');
+	    }	
+	}
+    }
+    printf("\nconvert16to7: returning '%s'\n", dest);
+#endif
 
     if (*dest == '\0') {
 	strcpy(dest, "varname");
     }
 
-#ifdef EDEBUG
-    fprintf(stderr, "convert16to7: returning '%s'\n", dest);
-#endif
+    dprintf("convert16to7: returning '%s'\n", dest);
 
     return dest;    
 }
@@ -813,10 +891,8 @@ static int allocate_row_col (int i, int j, wbook *book)
 
     started = 1;
 
-#ifdef MEMDEBUG
-    fprintf(stderr, "allocate: row=%d, col=%d, nrows=%d\n",
-	    i, j, nrows);
-#endif
+    dprintf("allocate: row=%d, col=%d, nrows=%d\n", i, j, nrows);
+
     if (i >= nrows) {
 	new_nrows = (i / 16 + 1) * 16;
 
@@ -829,28 +905,18 @@ static int allocate_row_col (int i, int j, wbook *book)
 	rows = myrows;
 
 	for (k=nrows; k<new_nrows; k++) {
-#ifdef MEMDEBUG
-	    fprintf(stderr, "allocate: initing rows[%d]\n", k);
-#endif
+	    dprintf("allocate: initing rows[%d]\n", k);
 	    row_init(&rows[k]);
-#ifdef MEMDEBUG
-	    fprintf(stderr, "rows[%d].end=%d\n", i, rows[k].end);
-#endif
+	    dprintf("rows[%d].end=%d\n", i, rows[k].end);
 	}
 	nrows = new_nrows;
     }
 
-#ifdef MEMDEBUG
-    fprintf(stderr, "allocate: col=%d and rows[%d].end = %d\n",
-	    j, i, rows[i].end);
-#endif
+    dprintf("allocate: col=%d and rows[%d].end = %d\n", j, i, rows[i].end);
 
     if (j >= rows[i].end) {
 	newcol = (j / 16 + 1) * 16;
-#ifdef MEMDEBUG
-	fprintf(stderr, "allocate: reallocing rows[%d].cells to size %d\n", 
-		i, newcol);
-#endif
+	dprintf("allocate: reallocing rows[%d].cells to size %d\n", i, newcol);
 	cells = realloc(rows[i].cells, newcol * sizeof *cells);
 
 	if (cells == NULL) {
@@ -876,9 +942,7 @@ static void free_sheet (void)
 {
     int i, j;
 
-#ifdef MEMDEBUG
-    printf("free_sheet(), nrows=%d\n", nrows);
-#endif
+    dprintf("free_sheet(), nrows=%d\n", nrows);
 
     /* free shared string table */
     if (sst != NULL) {
@@ -892,24 +956,17 @@ static void free_sheet (void)
     if (rows != NULL) {
 	for (i=0; i<nrows; i++) {
 	    if (rows[i].cells == NULL) {
-#ifdef MEMDEBUG
-		fprintf(stderr, "rows[%d].cells = NULL, skipping free\n", i);
-#endif
+		dprintf("rows[%d].cells = NULL, skipping free\n", i);
 		continue;
 	    }
 	    for (j=0; j<rows[i].end; j++) {
 		if (rows[i].cells[j] != NULL) {
-#ifdef MEMDEBUG
-		    fprintf(stderr, "Freeing rows[%d].cells[%d] at %p\n",
+		    dprintf("Freeing rows[%d].cells[%d] at %p\n",
 			    i, j, (void *) rows[i].cells[j]);
-#endif
 		    free(rows[i].cells[j]); 
 		}
 	    }
-#ifdef MEMDEBUG
-	    fprintf(stderr, "Freeing rows[%d].cells at %p\n",
-		    i, (void *) rows[i].cells);
-#endif
+	    dprintf("Freeing rows[%d].cells at %p\n", i, (void *) rows[i].cells);
 	    free(rows[i].cells);
 	}
 	free(rows);
@@ -921,59 +978,6 @@ static void free_sheet (void)
 
 #define IS_STRING(v) ((v[0] == '"'))
 
-static int consistent_date_labels (int row_offset, int col_offset)
-{
-    int i, startrow = 1 + row_offset;
-    int pd = 0, pdbak = 0;
-    double x, xbak = 0.0;
-
-    fprintf(stderr, "testing for consistent date labels in col %d\n", 
-	    col_offset);
-
-    for (i=startrow; i<nrows; i++) {
-	char *test = rows[i].cells[col_offset];
-
-	if (*test == '\0') {
-	    fprintf(stderr, " no: blank cell at row %d\n", i + 1);
-	    return 0;
-	}
-
-	/* skip quote */
-	if (*test == '"' || *test == '\'') {
-	    test++;
-	}
-
-	pd = label_is_date(test);
-
-	if (pd == 0) {
-	    fprintf(stderr, " no: label '%s' on row %d is not a date\n", 
-		    test, i + 1);
-	    return 0;
-	}
-
-	x = atof(test);
-
-	if (i == startrow) {
-	    pdbak = pd;
-	} else { 
-	    if (pd != pdbak) {
-		fprintf(stderr, " no: got inconsistent data frequencies %d and %d\n",
-			pdbak, pd);
-		return 0;
-	    }
-	    if (x <= xbak) {
-		fprintf(stderr, " no: got %g <= %g\n", x, xbak);
-		return 0;
-	    }
-	}
-	xbak = x;
-    }
-
-    fprintf(stderr, " yes: data frequency = %d\n", pd);
-
-    return pd;
-}
-
 /* check for full set of strings in first column to be read (which may
    be at an offset into the worksheet)
  */
@@ -985,20 +989,16 @@ static int first_col_strings (wbook *book)
     int ret = 1;
     
     for (i=startrow; i<nrows; i++) {
-#ifdef EDEBUG
-	fprintf(stderr, "book->row_offset=%d, i=%d\n", book->row_offset, i);
-	fprintf(stderr, "rows = %p\n", (void *) rows);
-#endif
+	dprintf("book->row_offset=%d, i=%d\n", book->row_offset, i);
+	dprintf("rows = %p\n", (void *) rows);
 	if (rows == NULL || rows[i].cells == NULL || 
 	    rows[i].cells[j] == NULL ||
 	    !IS_STRING(rows[i].cells[j])) {
 	    ret = 0;
 	    break;
 	}
-#ifdef EDEBUG
-	fprintf(stderr, "first_col_strings: rows[%d].cells[%d]: '%s'\n", i, j,
+	dprintf("first_col_strings: rows[%d].cells[%d]: '%s'\n", i, j,
 		rows[i].cells[j]);
-#endif
     }
 
     return ret;
@@ -1035,17 +1035,12 @@ check_all_varnames (wbook *book, int ncols, const char *blank_col, int skip)
 	    continue;
 	}
 	if (rows[i].cells[j] == NULL) {
-#ifdef EDEBUG
-	    fprintf(stderr, "got_varnames: rows[%d].cells[%d] is NULL\n",
-		    i, j);
-#endif
+	    dprintf("got_varnames: rows[%d].cells[%d] is NULL\n", i, j);
 	    return VARNAMES_NULL;
 	}
 
-#ifdef EDEBUG
-	fprintf(stderr, "got_varnames: rows[%d].cells[%d] is '%s'\n",
-		i, j, rows[i].cells[j]);
-#endif
+	dprintf("got_varnames: rows[%d].cells[%d] is '%s'\n", i, j, 
+		rows[i].cells[j]);
 
 	if (!IS_STRING(rows[i].cells[j])) {
 	    return VARNAMES_NOTSTR;
@@ -1121,22 +1116,15 @@ check_data_block (wbook *book, int ncols, const char *blank_col,
 	    continue;
 	}
 	for (i=startrow; i<nrows; i++) {
-#ifdef EDEBUG
-	    fprintf(stderr, "data_block: looking at rows[%d], end = %d\n", 
-		    i, rows[i].end);
-#endif
+	    dprintf("data_block: looking at rows[%d], end = %d\n", i, rows[i].end);
 	    if (rows[i].cells  == NULL) {
-#ifdef EDEBUG
-		fprintf(stderr, "data_block: rows[%d].cells = NULL\n", i);
-#endif
+		dprintf("data_block: rows[%d].cells = NULL\n", i);
 		ret = -1;
 	    } else if (j >= rows[i].end) {
-		fprintf(stderr, "data_block: short row, fell off the end\n");
+		dprintf("data_block: short row, fell off the end\n");
 		ret = -1;
 	    } else if (rows[i].cells[j] == NULL) {
-#ifdef EDEBUG
-		fprintf(stderr, "data_block: rows[%d].cells[%d] = NULL\n", i, j);
-#endif
+		dprintf("data_block: rows[%d].cells[%d] = NULL\n", i, j);
 		rows[i].cells[j] = g_strdup("-999.0");
 		ret = -1;
 	    } else if (IS_STRING(rows[i].cells[j])) {
@@ -1176,20 +1164,20 @@ n_vars_from_col (int totcols, char *blank_col, int offset, int first_special)
 	if (!blank_col[i]) nv++;
     }
 
-#ifdef EDEBUG
-    fprintf(stderr, "n_vars_from_col: totcols=%d, nv=%d\n", totcols, nv);
-#endif
+    dprintf("n_vars_from_col: totcols=%d, nv=%d\n", totcols, nv);
 
     return nv;
 }
 
-static int transcribe_data (double **Z, DATAINFO *pdinfo, 
-			    int row_offset, int totcols, int startcol, 
+static int transcribe_data (double **Z, DATAINFO *pdinfo, wbook *book,
+			    int totcols, int startcol, 
 			    char *blank_col)
 {
+    int roff = book->row_offset;
     int i, t, j = 1;
 
     for (i=startcol; i<totcols; i++) {
+	int ts, missing = 0;
 
 	if (blank_col[i]) {
 	    continue;
@@ -1200,32 +1188,38 @@ static int transcribe_data (double **Z, DATAINFO *pdinfo,
 	}
 
 	pdinfo->varname[j][0] = 0;
-	strncat(pdinfo->varname[j], rows[row_offset].cells[i] + 1, 
+	strncat(pdinfo->varname[j], rows[roff].cells[i] + 1, 
 		USER_VLEN - 1);
-#ifdef EDEBUG
-	fprintf(stderr, "accessing rows[%d].cells[%d] at %p\n",
-		row_offset, i, (void *) rows[row_offset].cells[i]);
-	fprintf(stderr, "set varname[%d] = '%s'\n", j, pdinfo->varname[j]);
-#endif
+	dprintf("accessing rows[%d].cells[%d] at %p\n",
+		roff, i, (void *) rows[roff].cells[i]);
+	dprintf("set varname[%d] = '%s'\n", j, pdinfo->varname[j]);
 
 	for (t=0; t<pdinfo->n; t++) {
-	    int ts = t + 1 + row_offset;
+	    if (book->missmask != NULL) {
+		while (book->missmask[t]) {
+		    Z[j][t++] = NADBL;
+		    missing++;
+		}
+	    }
+
+	    ts = t + 1 + roff - missing;
 
 	    if (rows[ts].cells == NULL || i >= rows[ts].end ||
 		rows[ts].cells[i] == NULL) {
 		continue;
 	    }
-#ifdef EDEBUG
-	    fprintf(stderr, "accessing rows[%d].cells[%d] at %p\n", ts, i,
+
+	    dprintf("accessing rows[%d].cells[%d] at %p\n", ts, i,
 		    (void *) rows[ts].cells[i]);
-	    fprintf(stderr, "setting Z[%d][%d] = rows[%d].cells[%d] "
+	    dprintf("setting Z[%d][%d] = rows[%d].cells[%d] "
 		    "= '%s'\n", j, t, i, ts, rows[ts].cells[i]);
-#endif
+
 	    Z[j][t] = atof(rows[ts].cells[i]);
 	    if (Z[j][t] == -999.0) {
 		Z[j][t] = NADBL;
 	    }
 	}
+
 	j++;
     }
 
@@ -1316,7 +1310,7 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     int datacols, totcols;
     struct string_err strerr;
     char *blank_col = NULL;
-    int i, t;
+    int i, t, pd = 0;
     int err = 0;
 
     newinfo = datainfo_new();
@@ -1325,9 +1319,7 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	return 1;
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
+    gretl_push_c_numeric_locale();
 
     wbook_init(&book);
 
@@ -1347,12 +1339,14 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	} else {
 	    wsheet_menu(&book, 0);
 	}
+	debug_print = book.debug;
+	if (debug_print) {
+	    print_version();
+	}
     }
 
-#ifdef EDEBUG
-    fprintf(stderr, "sheet selected=%d; import offsets: col=%d, row=%d\n",
+    dprintf("sheet selected=%d; import offsets: col=%d, row=%d\n",
 	    book.selected, book.col_offset, book.row_offset);
-#endif
 
     if (book.selected == -1) {
 	/* canceled */
@@ -1426,19 +1420,24 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     }	    
 
     /* do we have a first column containing dates? */
-    if (obs_column_heading(rows[book.row_offset].cells[book.col_offset])) {
-	int pd = consistent_date_labels(book.row_offset, book.col_offset);
 
-	if (pd) {
-	    time_series_setup(rows[1 + book.row_offset].cells[book.col_offset],
-			      newinfo, pd, NULL, &time_series, &label_strings);
-	}
+    if (book_numeric_dates(book)) {
+	pd = pd_from_numeric_dates(nrows, book.row_offset, book.col_offset, NULL,
+				   &book);
+    } else if (obs_column_heading(rows[book.row_offset].cells[book.col_offset])) {
+	pd = consistent_date_labels(nrows, book.row_offset, book.col_offset, NULL); 
     }
 
+    if (pd) {
+	time_series_setup(rows[1 + book.row_offset].cells[book.col_offset],
+			  newinfo, pd, NULL, &time_series, &label_strings,
+			  book.flags);
+    }    
+
     /* number of variables and observations for import dataset */
     newinfo->v = n_vars_from_col(totcols, blank_col, book.col_offset,
 				 time_series || label_strings);
-    newinfo->n = nrows - 1 - book.row_offset;
+    newinfo->n = nrows - 1 - book.row_offset + book.totmiss;
     fprintf(stderr, "newinfo->v = %d, newinfo->n = %d\n",
 	    newinfo->v, newinfo->n);
 
@@ -1461,7 +1460,7 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     } 
 
     /* OK: actually populate the dataset */
-    transcribe_data(newZ, newinfo, book.row_offset, totcols,
+    transcribe_data(newZ, newinfo, &book, totcols,
 		    book.col_offset + (time_series || label_strings),
 		    blank_col);
 
@@ -1494,8 +1493,12 @@ int excel_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
     wbook_free(&book);
     free_sheet();
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
+    gretl_pop_c_numeric_locale();
+
+#ifdef WIN32
+    if (fdb != NULL) {
+	fclose(fdb);
+    }
 #endif
 
     return err;
