@@ -19,7 +19,6 @@
 
 #include "libgretl.h"
 #include "gretl_matrix.h"
-#include "gretl_matrix_private.h"
 #include "system.h"
 
 #define FDEBUG 0
@@ -266,7 +265,7 @@ static void fiml_form_uhat (fiml_system *fsys, const double **Z, int t1)
     }
 
 #if FDEBUG
-    gretl_matrix_print(fsys->uhat, "fiml uhat", NULL);
+    gretl_matrix_print(fsys->uhat, "fiml uhat");
 #endif
 }
 
@@ -292,7 +291,7 @@ fiml_form_sigma_and_psi (fiml_system *fsys, const double **Z, int t1)
     gretl_matrix_divide_by_scalar(fsys->sigma, fsys->n);
 
 #if FDEBUG
-    gretl_matrix_print(fsys->sigma, "fiml Sigma", NULL);
+    gretl_matrix_print(fsys->sigma, "fiml Sigma");
 #endif
 
     if (!err) {
@@ -301,7 +300,7 @@ fiml_form_sigma_and_psi (fiml_system *fsys, const double **Z, int t1)
     }
 
 #if FDEBUG
-    gretl_matrix_print(fsys->psi, "Sigma-inverse", NULL);
+    gretl_matrix_print(fsys->psi, "Sigma-inverse");
 #endif
 
     if (!err) {
@@ -312,7 +311,7 @@ fiml_form_sigma_and_psi (fiml_system *fsys, const double **Z, int t1)
     }
 
 #if FDEBUG
-    gretl_matrix_print(fsys->psi, "fiml Psi-transpose", NULL);
+    gretl_matrix_print(fsys->psi, "fiml Psi-transpose");
 #endif
 
     return err;
@@ -379,7 +378,7 @@ static void fiml_form_depvar (fiml_system *fsys)
     }
 
 #if FDEBUG > 1
-    gretl_matrix_print(fsys->arty, "fiml artificial Y", NULL);
+    gretl_matrix_print(fsys->arty, "fiml artificial Y");
 #endif
 }
 
@@ -452,7 +451,7 @@ fiml_form_indepvars (fiml_system *fsys, const double **Z, int t1)
     }
 
 #if FDEBUG > 1
-    gretl_matrix_print(fsys->artx, "fiml artificial X", NULL);
+    gretl_matrix_print(fsys->artx, "fiml artificial X");
 #endif
 }
 
@@ -468,12 +467,12 @@ static void fiml_uhat_init (fiml_system *fsys)
 
     for (i=0; i<fsys->g; i++) {
 	for (t=0; t<fsys->n; t++) {
-	    x = gretl_matrix_get(uhat, i, t);
+	    x = gretl_matrix_get(uhat, t, i);
 	    gretl_matrix_set(fsys->uhat, t, i, x);
 	}
     }
 
-    gretl_matrix_print(fsys->uhat, "uhat from 3SLS", NULL);
+    gretl_matrix_print(fsys->uhat, "uhat from 3SLS");
 }
 
 #endif
@@ -552,7 +551,7 @@ static void fiml_G_init (fiml_system *fsys, const DATAINFO *pdinfo)
 	printf(" %s", pdinfo->varname[enlist[i]]);
     }
     putchar('\n');
-    gretl_matrix_print(fsys->G, "fiml Gamma", NULL);
+    gretl_matrix_print(fsys->G, "fiml Gamma");
 #endif
 }
 
@@ -580,7 +579,7 @@ static void fiml_G_update (fiml_system *fsys)
     }
 
 #if FDEBUG
-    gretl_matrix_print(fsys->G, "fiml Gamma", NULL);
+    gretl_matrix_print(fsys->G, "fiml Gamma");
 #endif
 }
 
@@ -628,7 +627,7 @@ static void fiml_B_init (fiml_system *fsys, const DATAINFO *pdinfo)
 	printf(" %s", pdinfo->varname[exlist[i]]);
     }
     putchar('\n');
-    gretl_matrix_print(fsys->B, "fiml B", NULL);
+    gretl_matrix_print(fsys->B, "fiml B");
 #endif
 }
 
@@ -653,7 +652,7 @@ static void fiml_B_update (fiml_system *fsys)
     }
 
 #if FDEBUG
-    gretl_matrix_print(fsys->B, "fiml B", NULL);
+    gretl_matrix_print(fsys->B, "fiml B");
 #endif
 }
 
@@ -707,14 +706,14 @@ static int fiml_ll (fiml_system *fsys, const double **Z, int t1)
 
     tr = 0.0;
     for (i=0; i<fsys->g; i++) {
-	double epe, eit, ejt, sij;
+	double epe, eti, etj, sij;
 
 	for (j=0; j<fsys->g; j++) {
 	    epe = 0.0;
 	    for (t=0; t<fsys->n; t++) {
-		eit = gretl_matrix_get(fsys->uhat, t, i);
-		ejt = gretl_matrix_get(fsys->uhat, t, j);
-		epe += eit * ejt;
+		eti = gretl_matrix_get(fsys->uhat, t, i);
+		etj = gretl_matrix_get(fsys->uhat, t, j);
+		epe += eti * etj;
 	    }
 	    sij = gretl_matrix_get(fsys->Stmp, i, j);
 	    tr += sij * epe;
@@ -742,7 +741,7 @@ static int fiml_endog_rhs (fiml_system *fsys, const double **Z, int t1)
 	fputs("inversion of G failed\n", stderr);
     } else {
 #if FDEBUG
-	gretl_matrix_print(fsys->Gtmp, "G-inverse", NULL);
+	gretl_matrix_print(fsys->Gtmp, "G-inverse");
 #endif
 	gretl_matrix_multiply(fsys->WB1, fsys->Gtmp, fsys->WB2);
     } 
@@ -782,8 +781,8 @@ fiml_adjust_estimates (fiml_system *fsys, const double **Z, int t1,
     copy_estimates_to_btmp(fsys);
 
 #if FDEBUG
-    gretl_matrix_print(fsys->btmp, "parameter estimates", NULL);
-    gretl_matrix_print(fsys->artb, "estimated gradients", NULL);
+    gretl_matrix_print(fsys->btmp, "parameter estimates");
+    gretl_matrix_print(fsys->artb, "estimated gradients");
 #endif
 
     while (!improved && !err && step > minstep) {
@@ -809,7 +808,6 @@ fiml_adjust_estimates (fiml_system *fsys, const double **Z, int t1,
 	err = fiml_ll(fsys, Z, t1);
 	if (!err) {
 	    if (fsys->ll > llbak) {
-		*instep = step;
 		improved = 1;
 	    } else {
 		step /= 2.0;
@@ -817,6 +815,8 @@ fiml_adjust_estimates (fiml_system *fsys, const double **Z, int t1,
 	}
     }
 
+    *instep = step;
+
     return err;
 }
 
@@ -953,8 +953,13 @@ int fiml_driver (gretl_equation_system *sys, double ***pZ,
 	    err = gretl_matrix_restricted_ols(fsys->arty, fsys->artx, R, NULL,
 					      fsys->artb, NULL, NULL);
 	} else {
+#if 0
+	    err = gretl_matrix_svd_ols(fsys->arty, fsys->artx, fsys->artb, 
+				       NULL, NULL, NULL);
+#else
 	    err = gretl_matrix_ols(fsys->arty, fsys->artx, fsys->artb, 
 				   NULL, NULL, NULL);
+#endif
 	}
 
 	if (err) {
