@@ -51,13 +51,14 @@ static void wsheet_init (wsheet *sheet)
     sheet->varname = NULL;
     sheet->label = NULL;
     sheet->name = NULL;
+    sheet->flags = 0;
 }
 
 static void wsheet_free (wsheet *sheet)
 {
-    int i;
     int rows = sheet->maxrow + 1 - sheet->row_offset;
     int cols = sheet->maxcol + 1 - sheet->col_offset;
+    int i;
 
     for (i=0; i<cols; i++) {
 	if (sheet->varname != NULL) {
@@ -117,9 +118,9 @@ static void wsheet_print_info (wsheet *sheet)
 #endif
 }
 
-#define VTYPE_IS_NUMERIC(v) (v) == VALUE_BOOLEAN || \
-                            (v) == VALUE_INTEGER || \
-                            (v) == VALUE_FLOAT
+#define VTYPE_IS_NUMERIC(v) ((v) == VALUE_BOOLEAN || \
+                             (v) == VALUE_INTEGER || \
+                             (v) == VALUE_FLOAT)
 
 
 static int wsheet_allocate (wsheet *sheet, int cols, int rows)
@@ -188,7 +189,7 @@ static int wsheet_get_real_size (xmlNodePtr node, wsheet *sheet)
 	if (!xmlStrcmp(p->name, (UTF) "Cell")) {
 	    int i, j;
 
-	    tmp = xmlGetProp(p, (UTF) "Row");
+	    tmp = (char *) xmlGetProp(p, (UTF) "Row");
 	    if (tmp) {
 		i = atoi(tmp);
 		free(tmp);
@@ -196,7 +197,7 @@ static int wsheet_get_real_size (xmlNodePtr node, wsheet *sheet)
 		    sheet->maxrow = i;
 		}
 	    }
-	    tmp = xmlGetProp(p, (UTF) "Col");
+	    tmp = (char *) xmlGetProp(p, (UTF) "Col");
 	    if (tmp) {
 		j = atoi(tmp);
 		free(tmp);
@@ -214,6 +215,19 @@ static int wsheet_get_real_size (xmlNodePtr node, wsheet *sheet)
     return 0;
 }
 
+static void check_for_date_format (wsheet *sheet, const char *fmt)
+{
+#if 0
+    fprintf(stderr, "check_for_date_format: fmt = '%s'\n", fmt);
+#endif
+
+    if (strchr(fmt, '/') || 
+	(strstr(fmt, "mm") && !(strchr(fmt, ':'))) || 
+	strstr(fmt, "yy")) {
+	sheet->flags |= FIRST_COL_DATE_FORMAT;
+    }
+}
+
 static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 {
     xmlNodePtr p = node->xmlChildrenNode;
@@ -260,20 +274,21 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 	    x = NADBL;
 	    i = 0; t = 0;
 
-	    tmp = xmlGetProp(p, (UTF) "Col");
+	    tmp = (char *) xmlGetProp(p, (UTF) "Col");
 	    if (tmp) {
 		i = atoi(tmp);
 		i_real = i - colmin;
 		free(tmp);
 	    }
-	    tmp = xmlGetProp(p, (UTF) "Row");
+	    tmp = (char *) xmlGetProp(p, (UTF) "Row");
 	    if (tmp) {
 		t = atoi(tmp);
 		t_real = t - rowmin;
 		free(tmp);
 	    }
 	    if (i_real >= 0 && t_real >= 0) {
-		tmp = xmlGetProp(p, (UTF) "ValueType");
+		tmp = (char *) xmlGetProp(p, (UTF) "ValueType");
+
 		if (tmp) {
 		    vtype = atoi(tmp);
 		    free(tmp);
@@ -283,31 +298,41 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 			    i, t);
 		    err = 1;
 		}
+
 		/* check the top-left cell */
 		if (!err && i_real == 0 && t_real == 0) {
 		    if (VTYPE_IS_NUMERIC(vtype)) {
 			pputs(prn, _("Expected to find a variable name"));
 			err = 1;
 		    }
-		}
-		else if (!err && i_real >= 1 && t_real == 0 && 
-			 !(vtype == VALUE_STRING)) {
+		} else if (!err && i_real >= 1 && t_real == 0 && 
+			   !(vtype == VALUE_STRING)) {
 		    /* ought to be a varname here */
 		    pputs(prn, _("Expected to find a variable name"));
 		    err = 1;
 		}
-		if (!err && (tmp = xmlNodeGetContent(p))) {
+
+		if (!err && (tmp = (char *) xmlNodeGetContent(p))) {
 		    if (VTYPE_IS_NUMERIC(vtype) || vtype == VALUE_STRING) {
 			if (i_real == 0) {
 			    strncat(sheet->label[t_real], tmp, OBSLEN - 1);
 			}
 		    }
+
+		    if (i_real == 0 && t_real == 1 && VTYPE_IS_NUMERIC(vtype)) {
+			char *fmt = (char *) xmlGetProp(p, (UTF) "ValueFormat");
+
+			if (fmt) {
+			    check_for_date_format(sheet, fmt);
+			    free(fmt);
+			}
+		    }
+
 		    if (VTYPE_IS_NUMERIC(vtype)) {
 			x = atof(tmp);
 			sheet->Z[i_real][t_real] = x;
 			toprows[t_real] = leftcols[i_real] = 0;
-		    }
-		    else if (vtype == VALUE_STRING) {
+		    } else if (vtype == VALUE_STRING) {
 			if (t_real == 0) {
 			    strncat(sheet->varname[i_real], tmp, VNAMELEN - 1);
 			    if (i_real == 0 && !strcmp(tmp, "obs")) {
@@ -400,7 +425,7 @@ static int wsheet_get_data (const char *fname, wsheet *sheet, PRN *prn)
 		    while (snode != NULL && !err) {
 			if (!xmlStrcmp(snode->name, (UTF) "Name")) {
 			    sheetcount++;
-			    tmp = xmlNodeGetContent(snode);
+			    tmp = (char *) xmlNodeGetContent(snode);
 			    if (tmp) {
 				if (!strcmp(tmp, sheet->name) &&
 				    sheetcount == sheet->ID + 1) {
@@ -410,14 +435,14 @@ static int wsheet_get_data (const char *fname, wsheet *sheet, PRN *prn)
 			    }
 			} else if (got_sheet &&
 				 !xmlStrcmp(snode->name, (UTF) "MaxCol")) {
-			    tmp = xmlNodeGetContent(snode);
+			    tmp = (char *) xmlNodeGetContent(snode);
 			    if (tmp) {
 				sheet->maxcol = atoi(tmp);
 				free(tmp);
 			    }
 			} else if (got_sheet &&
 				 !xmlStrcmp(snode->name, (UTF) "MaxRow")) {
-			    tmp = xmlNodeGetContent(snode);
+			    tmp = (char *) xmlNodeGetContent(snode);
 			    if (tmp) {
 				sheet->maxrow = atoi(tmp);
 				free(tmp);
@@ -452,7 +477,6 @@ static int wbook_record_name (char *name, wbook *book)
     int ns = book->nsheets + 1;
 
     sheetnames = realloc(book->sheetnames, ns * sizeof *sheetnames);
-
     if (sheetnames == NULL) {
 	return 1;
     }
@@ -503,7 +527,7 @@ static int wbook_get_info (const char *fname, wbook *book, PRN *prn)
 	    sub = cur->xmlChildrenNode;
 	    while (sub != NULL && !err) {
 		if (!xmlStrcmp(sub->name, (UTF) "SheetName")) {
-		    tmp = xmlNodeGetContent(sub);
+		    tmp = (char *) xmlNodeGetContent(sub);
 		    if (tmp != NULL) {
 			if (wbook_record_name(tmp, book)) {
 			    err = 1;
@@ -554,62 +578,21 @@ static int wsheet_labels_complete (wsheet *sheet)
     return complete;
 }
 
-static int consistent_date_labels (wsheet *sheet)
-{
-    int t, rows = sheet->maxrow + 1 - sheet->row_offset;
-    int tstart = 1 + sheet->row_offset;
-    int pd = 0, pdbak = 0;
-    double x, xbak = 0.0;
-    char *test;
-
-    fputs("testing for consistent date labels\n", stderr);
-
-    for (t=tstart; t<rows; t++) {
-	test = sheet->label[t];
-	if (*test == '\0') {
-	    fprintf(stderr, " no: blank cell at row %d\n", t);
-	    return 0;
-	}
-	if (*test == '"' || *test == '\'') test++;
-	pd = label_is_date(test);
-	if (pd == 0) {
-	    fprintf(stderr, " no: label '%s' at row %d is not a date\n", 
-		    test, t);
-	    return 0;
-	}
-	x = atof(test);
-	if (t == tstart) {
-	    pdbak = pd;
-	} else { 
-	    if (pd != pdbak) {
-		fprintf(stderr, " no: got inconsistent data frequencies %d and %d\n",
-			pdbak, pd);
-		return 0;
-	    }		
-	    if (x <= xbak) {
-		return 0;
-	    }
-	}
-	xbak = x;
-    }
-
-    return pd;
-}
-
 static int rigorous_dates_check (wsheet *sheet, DATAINFO *pdinfo)
 {
     int t, rows = sheet->maxrow + 1 - sheet->row_offset;
     int startrow = 1 + sheet->row_offset;
     int n, nbak = 0, err = 0;
 
-    fputs("Doing rigorous dates check\n", stderr);
+    fprintf(stderr, "Doing rigorous dates check: pd = %d, stobs = '%s'\n", 
+	    pdinfo->pd, pdinfo->stobs);
 
     for (t=startrow; t<rows; t++) {
 	n = dateton(sheet->label[t], pdinfo);
 	if (t > startrow && n != nbak + 1) {
-	    fprintf(stderr, "problem: date[%d]='%s' but date[%d]='%s'\n",
-		    t - startrow + 1, sheet->label[t],
-		    t - startrow, sheet->label[t-1]);
+	    fprintf(stderr, "problem: date[%d]='%s' (%d) but date[%d]='%s' (%d)\n",
+		    t - startrow + 1, sheet->label[t], n,
+		    t - startrow, sheet->label[t-1], nbak);
 	    err = 1;
 	    break;
 	}
@@ -624,9 +607,10 @@ int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 {
     wbook book;
     wsheet sheet;
-    int err = 0, sheetnum = -1;
+    int sheetnum = -1;
     double **newZ = NULL;
     DATAINFO *newinfo;
+    int err = 0;
 
     newinfo = datainfo_new();
     if (newinfo == NULL) {
@@ -634,35 +618,36 @@ int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	return 1;
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
+    wsheet_init(&sheet);
+
+    gretl_push_c_numeric_locale();
 
     if (wbook_get_info(fname, &book, prn)) {
 	pputs(prn, _("Failed to get workbook info"));
-#ifdef ENABLE_NLS
-	setlocale(LC_NUMERIC, "");
-#endif
-	return 1;
-    } else {
-	wbook_print_info(&book);
-    }
+	err = 1;
+	goto getout;
+    } 
+
+    wbook_print_info(&book);
 
     if (book.nsheets == 0) {
 	pputs(prn, _("No worksheets found"));
+	err = 1;
+	goto getout;
     }
-    else if (book.nsheets > 1) {
+
+    if (book.nsheets > 1) {
 	wsheet_menu(&book, 1);
 	sheetnum = book.selected;
-    }
-    else {
+    } else {
 	wsheet_menu(&book, 0);
 	sheetnum = 0;
     }
 
-    if (book.selected == -1) err = -1;
-
-    wsheet_init(&sheet);
+    if (book.selected == -1) {
+	/* canceled */
+	err = -1;
+    }
 
     if (!err && sheetnum >= 0) {
 	fprintf(stderr, "Getting data...\n");
@@ -673,35 +658,46 @@ int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	    err = wsheet_get_data(fname, &sheet, prn);
 	    if (!err) {
 		wsheet_print_info(&sheet);
-	    }
+		book.flags |= sheet.flags;
+	    } 
 	}
-    } /* else ??? */
+    } 
 
-    wbook_free(&book);
-
-    if (!err) {
+    if (err) {
+	goto getout;
+    } else {
+	int nrows = sheet.maxrow + 1 - sheet.row_offset;
 	int i, j, t, i_sheet, label_strings = sheet.text_cols;
 	int time_series = 0;
 	int blank_cols = 0;
+	int pd = 0;
 
-	if (obs_column_heading(sheet.label[0])) {
-	    int pd = consistent_date_labels(&sheet);
-
-	    if (pd) {
-		time_series_setup(sheet.label[1], newinfo, pd,
-				  &sheet.text_cols,
-				  &time_series, &label_strings);
-		rigorous_dates_check(&sheet, newinfo);
-	    }
+	if (book_numeric_dates(book)) {
+	    pd = pd_from_numeric_dates(nrows, sheet.row_offset, 0, sheet.label,
+				       &book);
+	} else if (obs_column_heading(sheet.label[0])) {
+	    pd = consistent_date_labels(nrows, sheet.row_offset, 0, sheet.label);
 	}
 
+	if (pd) {
+	    time_series_setup(sheet.label[1], newinfo, pd,
+			      &sheet.text_cols, &time_series, 
+			      &label_strings, book.flags);
+	    if (!book_numeric_dates(book)) {
+		rigorous_dates_check(&sheet, newinfo);
+	    }
+	}	
+
 	newinfo->v = sheet.maxcol + 2 - sheet.col_offset - sheet.text_cols;
-	newinfo->n = sheet.maxrow - sheet.row_offset;
+	newinfo->n = sheet.maxrow - sheet.row_offset + book.totmiss;
 
 	fprintf(stderr, "newinfo->v = %d, newinfo->n = %d\n",
 		newinfo->v, newinfo->n);
 
-	start_new_Z(&newZ, newinfo, 0);
+	err = start_new_Z(&newZ, newinfo, 0);
+	if (err) {
+	    goto getout;
+	}
 
 	if (!time_series) {
 	    strcpy(newinfo->stobs, "1");
@@ -715,14 +711,22 @@ int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	j = 1;
+
 	for (i=1; i<newinfo->v; i++) {
 	    i_sheet = i - 1 + sheet.text_cols;
 	    if (*sheet.varname[i_sheet] == '\0') {
 		blank_cols++;
 	    } else {
+		int s = 1;
+
 		strcpy(newinfo->varname[j], sheet.varname[i_sheet]);
 		for (t=0; t<newinfo->n; t++) {
-		    newZ[j][t] = sheet.Z[i_sheet][t+1];
+		    if (book.missmask != NULL) {
+			while (book.missmask[t]) {
+			    newZ[j][t++] = NADBL;
+			}
+		    }
+		    newZ[j][t] = sheet.Z[i_sheet][s++];
 		}
 		j++;
 	    }
@@ -753,13 +757,18 @@ int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	} else {
 	    err = merge_data(pZ, pdinfo, newZ, newinfo, prn);
 	}
-    } 	    
+    } 
 
+ getout:
+
+    wbook_free(&book);
     wsheet_free(&sheet);
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
+
+    if (err) {
+	free(newinfo);
+    }
 
     return err;
 }
