@@ -19,6 +19,11 @@
 
 #ifdef EXCEL_IMPORTER
 
+# ifndef WIN32
+#  define BUILDING_PLUGIN
+#  include "../gui2/dialogs.h" /* for infobox */
+# endif
+
 static void set_all_missing (double **Z, DATAINFO *pdinfo)
 {
     int i, t;
@@ -30,22 +35,28 @@ static void set_all_missing (double **Z, DATAINFO *pdinfo)
     }
 }
 
-#endif
+#endif /* EXCEL_IMPORTER */
 
 static void time_series_setup (const char *s, DATAINFO *newinfo, int pd,
 			       int *text_cols, int *time_series, 
-			       int *label_strings)
+			       int *label_strings, BookFlag flags)
 {
-    if (*s == '"' || *s == '\'') s++;
+    if (flags & FIRST_COL_DATE_FORMAT) {
+	int d0 = atoi(s);
+
+	MS_excel_date_string(newinfo->stobs, d0, pd, flags & DATE_BASE_1904);
+    } else {
+	if (*s == '"' || *s == '\'') s++;
+	strcpy(newinfo->stobs, s);
+	colonize_obs(newinfo->stobs);
+    }
 
     newinfo->pd = pd;
     newinfo->structure = TIME_SERIES;
 
-    strcpy(newinfo->stobs, s);
-    colonize_obs(newinfo->stobs);
     fprintf(stderr, "stobs='%s'\n", newinfo->stobs);
-
     newinfo->sd0 = get_date_x(newinfo->pd, newinfo->stobs);
+    fprintf(stderr, "sd0=%g\n", newinfo->sd0);
 
     if (text_cols != NULL) {
 	*text_cols = 1;
@@ -67,11 +78,6 @@ static int label_is_date (char *str)
     int i, d, pd = 0;
     double dd, sub;
 
-    if (*str == '"' || *str == '\'') {
-	str++;
-	len--;
-    }
-
 #if 0
     fprintf(stderr, "label_is_date: looking at '%s'\n", str);
 #endif
@@ -83,17 +89,12 @@ static int label_is_date (char *str)
 	}
     }
 
-    if (len == 4 && sscanf(str, "%4d", &d) == 1 &&
-	d > 0 && d < 3000) {
+    if (len == 4 && sscanf(str, "%4d", &d) && d > 0 && d < 3000) {
 	pd = 1;
-    }
-    else if (len == 6 && sscanf(str, "%lf", &dd) == 1 &&
-	dd > 0 && dd < 3000) { 
+    } else if (len == 6 && sscanf(str, "%lf", &dd) && dd > 0 && dd < 3000) { 
 	sub = 10.0 * (dd - (int) dd);
 	if (sub >= .999 && sub <= 4.001) pd = 4;
-    }
-    else if (len == 7 && sscanf(str, "%lf", &dd) == 1 &&
-	dd > 0 && dd < 3000) {
+    } else if (len == 7 && sscanf(str, "%lf", &dd) && dd > 0 && dd < 3000) {
 	sub = 100.0 * (dd - (int) dd);
 	if (sub >= .9999 && sub <= 12.0001) pd = 12;
     }
@@ -101,6 +102,231 @@ static int label_is_date (char *str)
     return pd;
 }
 
+static int pd_from_dmult (double dm)
+{
+    int pd = 0;
+
+    if (dm > 250.0) {
+	pd = 1;
+    } else if (dm > 90.0) {
+	pd = 4;
+    } else if (dm > 29.0) {
+	pd = 12;
+    } else if (dm > 6.0) {
+	pd = 52;
+    } else if (dm > 1.4) {
+	pd = 5;
+    } else if (dm > 1.16) {
+	pd = 6;
+    } else if (dm > 0.9) {
+	pd = 7;
+    }
+
+    return pd;
+}
+
+#define dmax(f) ((f == 1)? 366 : (f == 4)? 92 : 31)
+
+static int 
+calendar_missing_obs (int diff, int pd, double d1, BookFlag flags)
+{
+    int mc = 0;
+
+    if (pd == 52) {
+	if (diff > 7) {
+	    mc = (diff / 7) - 1;
+	}
+    } else if (pd == 7) {
+	if (diff > 1) {
+	    mc = diff - 1;
+	}
+    } else if (pd == 1 || pd == 4 || pd == 12) {
+	if (diff > dmax(pd)) {
+	    double xmc = diff / (365.0 / pd);
+
+	    mc = floor(xmc - .5);
+	}
+    } else if ((pd == 5 || pd == 6) && diff > 1) {
+	char dstr[12];
+	int wday;
+
+	MS_excel_date_string(dstr, d1, 0, flags & DATE_BASE_1904);
+	wday = get_day_of_week(dstr);
+
+	if (wday == 1) {
+	    if (pd == 5) {
+		mc = diff - 3;
+	    } else {
+		mc = diff - 2;
+	    }
+	} else {
+	    mc = diff - 1;
+	}
+    }
+
+    return mc;
+}
+
+#ifdef EXCEL_IMPORTER
+# define cell_val(t,o) (rows[t].cells[o])
+#else
+# define cell_val(t,o) (labels[t])
+#endif
+
+static int pd_from_numeric_dates (int nrows, int row_offset, int col_offset,
+				  char **labels, wbook *book)
+{
+    char dstr[12];
+    int tstart = 1 + row_offset;
+    int ndays, nobs = nrows - tstart;
+    int mc, t, pd = 0;
+    double x1, x2, dmult, diff;
+    char *test;
+
+    fprintf(stderr, "check for consistent numeric dates in col %d (nobs = %d)\n", 
+	    col_offset, nobs);
+
+    /* look at first date */
+    test = cell_val(tstart, col_offset);
+    if (sscanf(test, "%lf", &x1)) {
+	MS_excel_date_string(dstr, (int) x1, 0, book->flags & DATE_BASE_1904);
+	fprintf(stderr, "numeric date on row %d = %g (%s)\n", tstart, 
+		x1, dstr);
+    } else {
+	fprintf(stderr, "failed to read starting\n");
+	return 0;
+    }
+
+    /* look at last date */
+    test = cell_val(nrows - 1, col_offset);
+    if (sscanf(test, "%lf", &x2)) {
+	MS_excel_date_string(dstr, (int) x2, 0, book->flags & DATE_BASE_1904);
+	fprintf(stderr, "numeric date on row %d = %g (%s)\n", nrows - 1, 
+		x2, dstr);
+    } else {
+	fprintf(stderr, "failed to read ending date\n");
+	return 0;
+    }
+
+    /* compare number of obs and calendar span of data */
+    ndays = (int) x2 - (int) x1 + 1;
+    dmult = (double) ndays / nobs;
+    fprintf(stderr, "Calendar interval = %d days\n", ndays);
+    fprintf(stderr, "Calendar days per observation = %g\n", dmult);
+    pd = pd_from_dmult(dmult);
+
+    if (pd > 0) {
+	fprintf(stderr, "provisional data frequency = %d\n", pd);
+    } else {
+	fputs("Can't make sense of this\n", stderr);
+	return 0;
+    }
+
+    book->totmiss = 0;
+
+    for (t=tstart; t<nrows; t++) {
+	test = cell_val(t, col_offset);
+
+	if (sscanf(test, "%lf", &x1) != 1) {
+	    fprintf(stderr, "Problem: blank cell at row %d\n", t + 1);
+	    return 0;
+	}
+
+	if (t > tstart) {
+	    diff = x1 - x2;
+	    mc = calendar_missing_obs((int) diff, pd, x1, book->flags);
+	    if (mc > 0) {
+		fprintf(stderr, "row %d: calendar gap = %g, %d values missing?\n", 
+			t, diff, mc);
+		book->totmiss += mc;
+	    }
+	}
+
+	x2 = x1;
+    }
+
+    if (book->totmiss > 0) {
+	int i, s = 0;
+
+	fprintf(stderr, "Total missing values = %d\n", book->totmiss);
+	book->missmask = calloc(nobs + book->totmiss, 1);
+
+	if (book->missmask == NULL) {
+	    fprintf(stderr, "Out of memory allocating missing obs mask\n");
+	    return 0;
+	}
+
+	for (t=tstart; t<nrows; t++) {
+	    test = cell_val(t, col_offset);
+	    sscanf(test, "%lf", &x1);
+	    if (t > tstart) {
+		mc = calendar_missing_obs((int) (x1 - x2), pd, x1, book->flags);
+		for (i=0; i<mc; i++) {
+		    book->missmask[s++] = 1;
+		}
+	    }
+	    x2 = x1;
+	    s++;
+	}
+    }
+
+    fprintf(stderr, "Setting data frequency = %d\n", pd);
+
+    return pd;
+}
+
+static int 
+consistent_date_labels (int nrows, int row_offset, int col_offset, char **labels)
+{
+    int t, tstart = 1 + row_offset;
+    int pd = 0, pdbak = 0;
+    double x, xbak = 0.0;
+    char *test;
+
+    fprintf(stderr, "testing for consistent date labels in col %d\n", 
+	    col_offset);
+
+    for (t=tstart; t<nrows; t++) {
+	test = cell_val(t, col_offset);
+
+	if (*test == '\0') {
+	    fprintf(stderr, " no: blank cell at row %d\n", t + 1);
+	    return 0;
+	} else if (*test == '"' || *test == '\'') {
+	    test++;
+	}
+
+	pd = label_is_date(test);
+
+	if (pd == 0) {
+	    fprintf(stderr, " no: label '%s' on row %d is not a valid date\n", 
+		    test, t + 1);
+	    return 0;
+	}
+
+	x = atof(test);
+
+	if (t > tstart) {
+	    if (pd != pdbak) {
+		fprintf(stderr, " no: got inconsistent data frequencies %d and %d\n",
+			pdbak, pd);
+		return 0;
+	    }
+	    if (x <= xbak) {
+		fprintf(stderr, " no: got %g <= %g\n", x, xbak);
+		return 0;
+	    }
+	}
+
+	pdbak = pd;
+	xbak = x;
+    }
+
+    fprintf(stderr, " yes: data frequency = %d\n", pd);
+
+    return pd;
+}
+
 static int obs_column_heading (const char *label)
 {
     int ret = 0;
@@ -158,6 +384,8 @@ static void wbook_free (wbook *book)
     }
     free(book->sheetnames);
     free(book->byte_offsets);
+    free(book->xf_list);
+    free(book->missmask);
 }
 
 static void wbook_init (wbook *book)
@@ -168,6 +396,11 @@ static void wbook_init (wbook *book)
     book->sheetnames = NULL;
     book->byte_offsets = NULL;
     book->selected = 0;
+    book->debug = 0;
+    book->flags = 0;
+    book->xf_list = NULL;
+    book->totmiss = 0;
+    book->missmask = NULL;
 }
 
 static const char *column_label (int col)
@@ -201,6 +434,34 @@ static void colspin_changed (GtkEditable *ed, GtkWidget *w)
     }
 }
 
+#ifdef EXCEL_IMPORTER
+
+static 
+void debug_callback (GtkWidget *w, wbook *book)
+{
+    static int done;
+
+    book->debug = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+    if (book->debug && !done) {
+	gchar *msg;
+
+# ifdef WIN32
+	make_debug_fname();
+	msg = g_strdup_printf(I_("Sending debugging output to %s"),
+			      debug_fname);
+	MessageBox(NULL, msg, "gretl", MB_OK | MB_ICONINFORMATION);
+# else
+	msg = g_strdup_printf(_("Sending debugging output to %s"),
+			      "stderr");
+	infobox(msg);
+# endif
+	g_free(msg);
+	done = 1;
+    }
+}
+
+#endif /* EXCEL_IMPORTER */
+
 #if GTK_MAJOR_VERSION >= 2
 
 static
@@ -317,6 +578,8 @@ static void wsheet_menu (wbook *book, int multisheet)
 
     gtk_window_set_title(GTK_WINDOW(w), _("gretl: spreadsheet import"));
 
+    g_signal_connect_after(G_OBJECT(w), "delete_event",
+			   G_CALLBACK(wsheet_menu_cancel), book);
     g_signal_connect(G_OBJECT(w), "destroy",  
 		     G_CALLBACK(gtk_main_quit), NULL);
 
@@ -366,6 +629,14 @@ static void wsheet_menu (wbook *book, int multisheet)
 	add_sheets_list(vbox, book);
     }
 
+#ifdef EXCEL_IMPORTER
+    tmp = gtk_check_button_new_with_label(_("Produce debugging output"));
+    g_signal_connect(G_OBJECT(tmp), "toggled", G_CALLBACK(debug_callback), 
+		     book);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), FALSE);
+    gtk_box_pack_start(GTK_BOX(vbox), tmp, TRUE, TRUE, 5);
+#endif
+
     hbox = gtk_hbox_new (TRUE, 5);
     gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
@@ -382,7 +653,7 @@ static void wsheet_menu (wbook *book, int multisheet)
     gtk_box_pack_start (GTK_BOX (hbox), 
                         tmp, TRUE, TRUE, 0);
     g_signal_connect (G_OBJECT (tmp), "clicked", 
-		      G_CALLBACK (wsheet_menu_cancel), book);
+		      G_CALLBACK(wsheet_menu_cancel), book);
     g_signal_connect_swapped (G_OBJECT (tmp), "clicked", 
 			      G_CALLBACK (gtk_widget_destroy), 
 			      G_OBJECT (w));
@@ -521,6 +792,15 @@ static void wsheet_menu (wbook *book, int multisheet)
 	gtk_container_add(GTK_CONTAINER(frame), list);
     }
 
+#ifdef EXCEL_IMPORTER
+    tmp = gtk_check_button_new_with_label(_("Produce debugging output"));
+    gtk_signal_connect(GTK_OBJECT(tmp), "toggled", 
+		       GTK_SIGNAL_FUNC(debug_callback), 
+		       book);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), FALSE);
+    gtk_box_pack_start(GTK_BOX(vbox), tmp, TRUE, TRUE, 5);
+#endif
+
     hbox = gtk_hbox_new (TRUE, 5);
     gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
 
