@@ -19,7 +19,6 @@
 
 #include "libgretl.h"
 #include "gretl_matrix.h"
-#include "gretl_matrix_private.h"
 
 #include "f2c.h"
 #include "clapack_double.h"
@@ -229,11 +228,8 @@ static int leverage_plot (const MODEL *pmod, gretl_matrix *S,
 	}
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
+    gretl_push_c_numeric_locale();
 
-    fputs("# leverage/influence plot\n", fp);
     fputs("set size 1.0,1.0\nset multiplot\nset size 1.0,0.48\n", fp);
     fputs("set xzeroaxis\n", fp);
     fputs("set nokey\n", fp); 
@@ -295,9 +291,7 @@ static int leverage_plot (const MODEL *pmod, gretl_matrix *S,
     fputs("e\n", fp);
     fputs("set nomultiplot\n", fp);
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
 
     fclose(fp);
 
@@ -499,6 +493,8 @@ gretl_matrix *model_leverage (const MODEL *pmod, double ***pZ,
 
     lp = 2.0 * n / m;
 
+    obs_marker_init(pdinfo);
+
     /* print the results */
     for (t=0; t<modn; t++) {
 	double h, s, d, f = NADBL;
