@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) by Ramu Ramanathan and Allin Cottrell
+ *   Copyright (c) by Allin Cottrell
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #include "clapack_double.h"
 #include "gretl_model.h"
 
-#undef PDEBUG
+#define PDEBUG 0
 
 enum vcv_ops {
     VCV_INIT,
@@ -34,7 +34,7 @@ enum vcv_ops {
 typedef struct diagnostics_t_ diagnostics_t;
 
 struct diagnostics_t_ {
-    int nunits;           /* total cross-section units */
+    int nunits;           /* total cross-sectional units */
     int effn;             /* effective (included) cross-section units */
     int T;                /* times-series length of panel */
     int effT;             /* effective times-series length */
@@ -49,7 +49,6 @@ struct diagnostics_t_ {
     double *sigma;        /* Hausman covariance matrix */
     double fe_var;        /* fixed-effects error variance */
     double gm_var;        /* group means error variance */
-    double theta;         /* GLS weighting coefficient */
     const MODEL *pooled;  /* reference model (pooled OLS) */
 };
 
@@ -76,7 +75,6 @@ varying_vars_list (const double **Z, const DATAINFO *pdinfo,
 #define panel_index(i,t) ((panidx.ts)? (i * panidx.T + t + panidx.offset) : \
                                        (t * panidx.n + i + panidx.offset))
 
-/* .................................................................. */
 
 static void 
 diagnostics_init (diagnostics_t *diag, const MODEL *pmod, gretlopt opt)
@@ -133,8 +131,7 @@ static int var_is_varying (const int *list, int v)
     return ret;
 }
 
-static int 
-haus_alloc (diagnostics_t *diag)
+static int hausman_allocate (diagnostics_t *diag)
 {
     int nbeta = diag->vlist[0] - 2;
     int nsigma = (nbeta * nbeta + nbeta) / 2;
@@ -156,7 +153,7 @@ haus_alloc (diagnostics_t *diag)
     }
 
 #if PDEBUG
-    fprintf(stderr, "haus_alloc: allocated %d terms for bdiff, "
+    fprintf(stderr, "hausman_allocate: alloc'd %d terms for bdiff, "
 	    "%d for sigma\n", nbeta, nsigma);
 #endif
 
@@ -181,25 +178,39 @@ static void print_panel_coeff (const MODEL *pmod,
    are subtracted */
 
 static DATAINFO *
-within_groups_dataset (const double **Z, double ***wZ, 
-		       diagnostics_t *diag)
+within_groups_dataset (const double **Z, double ***wZ, diagnostics_t *diag)
 {
     DATAINFO *winfo;
+    int wnobs = 0;
     int i, j, k;
     int t, bigt;
 
 #if PDEBUG
-    fprintf(stderr, "within_groups_dataset: nvar=%d, nobs=%d, *wZ=%p\n", 
+    fprintf(stderr, "within_groups: nvar=%d, pooled model nobs=%d, *wZ=%p\n", 
 	    diag->vlist[0], diag->pooled->nobs, (void *) *wZ);
 #endif
 
-    winfo = create_new_dataset(wZ, diag->vlist[0], diag->pooled->nobs, 0);
+    for (i=0; i<diag->nunits; i++) { 
+	if (diag->unit_obs[i] > 1) {
+	    /* we need more than one observation to compute any
+	       within group variation */
+	    wnobs += diag->unit_obs[i];
+	}
+    }
+
+    winfo = create_new_dataset(wZ, diag->vlist[0], wnobs, 0);
     if (winfo == NULL) {
 	return NULL;
     }
 
+#if PDEBUG
+    fprintf(stderr, "within_groups_dataset: now *wZ=%p (wnobs = %d)\n", 
+	    (void *) *wZ, wnobs);
+#endif
+
     k = 0;
     for (j=1; j<=diag->vlist[0]; j++) { 
+	int s = 0;
 
 	if (diag->vlist[j] == 0) {
 	    continue;
@@ -216,7 +227,14 @@ within_groups_dataset (const double **Z, double ***wZ,
 	    int Ti = diag->unit_obs[i];
 	    double xbar = 0.0;
 
-	    if (Ti == 0) {
+#if PDEBUG
+	    fprintf(stderr, "looking at x-sect unit %d\n", i);
+#endif
+
+	    if (Ti <= 1) {
+#if PDEBUG
+		fprintf(stderr, " skipping because Ti = %d\n", Ti);
+#endif
 		continue;
 	    }
 
@@ -226,20 +244,25 @@ within_groups_dataset (const double **Z, double ***wZ,
 		    xbar += Z[diag->vlist[j]][bigt];
 		}
 	    }
-
 	    xbar /= (double) Ti;
+
 #if PDEBUG > 1
 	    fprintf(stderr, "xbar for var %d, unit %d = %g\n", 
 		    diag->vlist[j], i, xbar);
 #endif
 	    for (t=0; t<diag->T; t++) {
+		if (s >= winfo->n) {
+		    fprintf(stderr, "*** Error: overflow of wZ at s = %d!\n", s);
+		    break;
+		}
 		bigt = panel_index(i, t);
 		if (!na(diag->pooled->uhat[bigt])) {
-		    (*wZ)[k][bigt] = Z[diag->vlist[j]][bigt] - xbar;
-		}
+		    (*wZ)[k][s] = Z[diag->vlist[j]][bigt] - xbar;
 #if PDEBUG > 1
-		fprintf(stderr, "Set wZ[%d][%d] = %g\n", k, bigt, (*wZ)[k][bigt]);
+		    fprintf(stderr, "Set wZ[%d][%d] = %g\n", k, s, (*wZ)[k][s]);
 #endif
+		    s++;
+		}
 	    }
 	}
     }
@@ -283,16 +306,20 @@ group_means_dataset (diagnostics_t *diag,
 	    if (Ti == 0) {
 		continue;
 	    }
+
 	    for (t=0; t<diag->T; t++) {
 		bigt = panel_index(i, t);
 		if (!na(diag->pooled->uhat[bigt])) {
 		    xx += Z[diag->pooled->list[j]][bigt];
 		}
 	    }
+
 	    (*gZ)[k][s] = xx / (double) Ti;
+
 #if PDEBUG > 1
-	    fprintf(stderr, "Set gZ[%d][%d] = %g\n", k, i, (*gZ)[k][i]);
+	    fprintf(stderr, "Set gZ[%d][%d] = %g\n", k, i, (*gZ)[k][s]);
 #endif
+
 	    s++;
 	}
 	k++;
@@ -311,12 +338,11 @@ group_means_variance (diagnostics_t *diag, double ***gZ, DATAINFO *ginfo)
     int i, j;
     int err = 0;
 
-    gmlist = malloc((diag->pooled->list[0] + 1) * sizeof *gmlist);
+    gmlist = gretl_list_new(diag->pooled->list[0]);
     if (gmlist == NULL) {
 	return E_ALLOC;
     }
 
-    gmlist[0] = diag->pooled->list[0];
     j = 1;
     for (i=1; i<=gmlist[0]; i++) { 
 	if (diag->pooled->list[i] == 0) {
@@ -332,6 +358,9 @@ group_means_variance (diagnostics_t *diag, double ***gZ, DATAINFO *ginfo)
 	diag->gm_var = gmmod.sigma * gmmod.sigma;
     } else {
 	err = gmmod.errcode;
+#if PDEBUG
+	fprintf(stderr, "error %d estimating group_means_variance\n", err);
+#endif
     }
 
     clear_model(&gmmod);
@@ -363,7 +392,7 @@ vcv_skip (const MODEL *pmod, int i, const diagnostics_t *diag, int op)
 
 /* fill out the covariance matrix for use with the Hausman test:
    the entries that get transcribed here are only those for
-   slopes with respect time-varying variables */
+   slopes with respect to time-varying variables */
 
 static void 
 vcv_slopes (diagnostics_t *diag, const MODEL *pmod, int op)
@@ -444,6 +473,14 @@ static int bXb (diagnostics_t *diag)
 	diag->H = 0.0;
 	for (i=0; i<diag->nbeta; i++) {
 	    diag->H += x[i] * diag->bdiff[i];
+#if PDEBUG
+	    fprintf(stderr, "added %g * %g to diag->H: now = %g\n",
+		    x[i], diag->bdiff[i], diag->H);
+#endif
+
+	}
+	if (diag->H < 0.0) {
+	    diag->H = NADBL;
 	}
     }
 
@@ -482,7 +519,7 @@ static void apply_panel_df_correction (MODEL *pmod, int ndf)
 #define MINOBS 1
 
 /* calculate the fixed effects regression, either using dummy variables
-   or by subtracting means from the data
+   or by subtracting the group means from the data
 */
 
 static MODEL
@@ -509,20 +546,27 @@ fixed_effects_model (diagnostics_t *diag, double ***pZ, DATAINFO *pdinfo,
     ndum--;
 
 #if PDEBUG
-    fprintf(stderr, "ndum = %d, dvlen = %d\n", ndum, dvlen);
+    fprintf(stderr, "ndum = %d, dvlen = diag->vlist[0] + ndum = %d+%d = %d\n", 
+	    ndum, diag->vlist[0], ndum, dvlen);
 #endif
 
     /* should we use a set of dummy variables, or subtract
        the group means? */
 
     if (ndum <= 20 || ndum < diag->vlist[0]) {
+#if PDEBUG
+	fprintf(stderr, " using dummy variables approach\n");
+#endif
 	*usedum = 1;
     } else {
+#if PDEBUG
+	fprintf(stderr, " subtracting group means\n");
+#endif
 	*usedum = 0;
     }
 
     if (*usedum == 0) {
-	double **wZ;
+	double **wZ = NULL;
 	DATAINFO *winfo;
 
 	felist = malloc((diag->vlist[0]) * sizeof *felist);
@@ -566,9 +610,7 @@ fixed_effects_model (diagnostics_t *diag, double ***pZ, DATAINFO *pdinfo,
 	gretl_print_destroy(prn);
 #endif
 
-	free_Z(wZ, winfo);
-	clear_datainfo(winfo, CLEAR_FULL);
-	free(winfo);
+	destroy_dataset(wZ, winfo);
 	    
     } else {
 	int j = 0;
@@ -769,13 +811,14 @@ static int random_effects (diagnostics_t *diag,
     double **reZ;
     DATAINFO *reinfo;
     MODEL remod;
+    double theta;
     int *relist;
     int re_n = diag->T * diag->effn;
     int i, j, k, t;
     int err = 0;
 
     /* regression list */
-    relist = malloc((diag->pooled->list[0] + 1) * sizeof *relist);
+    relist = gretl_list_new(diag->pooled->list[0]);
     if (relist == NULL) {
 	return E_ALLOC;
     }
@@ -792,28 +835,41 @@ static int random_effects (diagnostics_t *diag,
 	    diag->T, diag->effn, diag->T * diag->effn);
 #endif
 
-    relist[0] = diag->pooled->list[0];
+    /* Create transformed variables: original data minus theta
+       times the appropriate group mean.  Note: for unbalanced
+       panels, theta varies across the units.
+    */
 
-    /* create transformed variables: original data minus theta
-       times the appropriate group mean */
+    theta = 1.0 - sqrt(diag->fe_var / (diag->effT * diag->gm_var));
 
-    k = 1;
+    k = 0;
     for (j=1; j<=relist[0]; j++) {
 	const double *xj = Z[diag->pooled->list[j]];
-	const double *gm = gZ[k];
+	const double *gm = NULL;
+	double theta_i;
 	int bigt, rt, u = 0;
 
 	if (diag->pooled->list[j] == 0) {
 	    relist[j] = 0;
-	    continue;
+	} else {
+	    k++;
+	    relist[j] = k;
+	    gm = gZ[k];
 	}
 
-	relist[j] = k;
-
 	for (i=0; i<diag->nunits; i++) {
-	    if (diag->unit_obs[i] == 0) {
+	    int Ti = diag->unit_obs[i];
+
+	    if (Ti == 0) {
 		continue;
 	    }
+
+	    if (Ti != diag->effT) {
+		theta_i = 1.0 - sqrt(diag->fe_var / (Ti * diag->gm_var));
+	    } else {
+		theta_i = theta;
+	    }
+
 	    for (t=0; t<diag->T; t++) {
 		bigt = panel_index(i, t);
 		if (pdinfo->structure == STACKED_TIME_SERIES) {
@@ -821,19 +877,17 @@ static int random_effects (diagnostics_t *diag,
 		} else {
 		    rt = t * diag->effn + u;
 		}
-		if (na(diag->pooled->uhat[bigt])) {
+		if (relist[j] == 0) {
+		    /* the intercept */
+		    reZ[0][rt] -= theta_i;
+		} else if (na(diag->pooled->uhat[bigt])) {
 		    reZ[k][rt] = NADBL;
 		} else {
-		    reZ[k][rt] = xj[bigt] - diag->theta * gm[u];
+		    reZ[k][rt] = xj[bigt] - theta_i * gm[u];
 		}
 	    }
 	    u++;
 	}
-	k++;
-    }
-
-    for (t=0; t<re_n; t++) {
-	reZ[0][t] -= diag->theta;
     }
 
     remod = lsq(relist, &reZ, reinfo, OLS, OPT_A | OPT_Z, 0.0);
@@ -862,18 +916,12 @@ static int random_effects (diagnostics_t *diag,
     }
 
     clear_model(&remod);
-
-    free_Z(reZ, reinfo);
-    clear_datainfo(reinfo, CLEAR_FULL);
-    free(reinfo);
-
+    destroy_dataset(reZ, reinfo);
     free(relist);    
 
     return err;
 }
 
-/* .................................................................. */
-
 static void 
 unit_error_variances (double *uvar, const MODEL *pmod, const DATAINFO *pdinfo,
 		      int nunits, int T, const int *unit_obs)
@@ -898,15 +946,16 @@ unit_error_variances (double *uvar, const MODEL *pmod, const DATAINFO *pdinfo,
 static int 
 breusch_pagan_LM (diagnostics_t *diag, const DATAINFO *pdinfo, PRN *prn)
 {
-    double LM, eprime = 0.0;
-    int i, t;
+    double LM, A = 0.0;
+    int n = diag->pooled->nobs;
+    int i, t, M = 0;
 
     pputs(prn, _("\nMeans of pooled OLS residuals for cross-sectional "
 		 "units:\n\n"));
 
     for (i=0; i<diag->nunits; i++) {
 	int Ti = diag->unit_obs[i];
-	double u, ubar = 0.0;
+	double u, usum = 0.0;
 
 	if (Ti == 0) {
 	    continue;
@@ -915,18 +964,16 @@ breusch_pagan_LM (diagnostics_t *diag, const DATAINFO *pdinfo, PRN *prn)
 	for (t=0; t<diag->T; t++) {
 	    u = diag->pooled->uhat[panel_index(i, t)];
 	    if (!na(u)) {
-		ubar += u;
+		usum += u;
 	    }
 	}
-	ubar /= (double) Ti; 
-	pprintf(prn, _(" unit %2d: %13.5g\n"), i + 1, ubar);
-	eprime += ubar * ubar;
+
+	pprintf(prn, _(" unit %2d: %13.5g\n"), i + 1, usum / (double) Ti);
+	A += usum * usum;
+	M += Ti * Ti;
     }
 
-    /* FIXME unbalanced panels */
-
-    LM = (double) diag->pooled->nobs / (2.0 * (diag->effT - 1.0)) * 
-	pow((diag->effT * diag->effT * eprime / diag->pooled->ess) - 1.0, 2);
+    LM = (n * n /(2.0 * (M - n))) * pow((A / diag->pooled->ess) - 1.0, 2);
 
     pprintf(prn, _("\nBreusch-Pagan test statistic:\n"
 		   " LM = %g with p-value = prob(chi-square(1) > %g) = %g\n"), 
@@ -939,8 +986,6 @@ breusch_pagan_LM (diagnostics_t *diag, const DATAINFO *pdinfo, PRN *prn)
     return 0;
 }
 
-/* .................................................................. */
-
 static int do_hausman_test (diagnostics_t *diag, PRN *prn)
 {
 #if PDEBUG
@@ -1096,7 +1141,7 @@ diagnostics_setup (diagnostics_t *diag, const MODEL *pmod,
     return err;
 }
 
-/* .................................................................. */
+#define ALLOW_UNBAL 1
 
 int panel_diagnostics (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
 		       gretlopt opt, PRN *prn)
@@ -1146,12 +1191,21 @@ int panel_diagnostics (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 #endif
 
     /* can we do the Hausman test or not? */
-    if (!unbal && xdf > 0) {
-	err = haus_alloc(&diag);
+#ifdef ALLOW_UNBAL
+    if (xdf > 0) {
+	err = hausman_allocate(&diag);
 	if (err) {
 	    goto bailout;
 	}
     } 
+#else
+    if (!unbal && xdf > 0) {
+	err = hausman_allocate(&diag);
+	if (err) {
+	    goto bailout;
+	}
+    } 
+#endif
 
     if (!unbal) {
 	pprintf(prn, _("      Diagnostics: assuming a balanced panel with %d "
@@ -1165,11 +1219,13 @@ int panel_diagnostics (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	goto bailout;
     }
 
+#ifndef ALLOW_UNBAL
     if (unbal) {
 	pprintf(prn, "Omitting random effects model since "
 		"panel is unbalanced\n");
 	goto bailout;
     }
+#endif
 
     breusch_pagan_LM(&diag, pdinfo, prn);
 
@@ -1195,16 +1251,13 @@ int panel_diagnostics (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	} else {
 	    pprintf(prn, _("Residual variance for group means "
 			   "regression: %g\n\n"), diag.gm_var);    
-	    diag.theta = 1.0 - sqrt(diag.fe_var / (diag.effT * diag.gm_var));
 	    random_effects(&diag, (const double **) *pZ, pdinfo, 
 			   (const double **) gZ, prn);
 	    do_hausman_test(&diag, prn);
 	}
 
 	if (ginfo != NULL) {
-	    free_Z(gZ, ginfo);
-	    clear_datainfo(ginfo, CLEAR_FULL);
-	    free(ginfo);
+	    destroy_dataset(gZ, ginfo);
 	}
     }
 
@@ -1215,8 +1268,6 @@ int panel_diagnostics (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
-/* .................................................................. */
-
 static int 
 write_uvar_to_dataset (double *uvar, int nunits, int T,
 		       double **Z, DATAINFO *pdinfo)
@@ -1335,12 +1386,13 @@ ml_hetero_test (MODEL *pmod, double s2, const double *uvar,
     x2 = pmod->nobs * log(s2) - s2h;
     df--;
 
-    test = new_test_on_model(pmod, GRETL_TEST_GROUPWISE);
+    test = model_test_new(GRETL_TEST_GROUPWISE);
     if (test != NULL) {
 	model_test_set_teststat(test, GRETL_STAT_LR);
 	model_test_set_dfn(test, df);
 	model_test_set_value(test, x2);
 	model_test_set_pvalue(test, chisq(x2, df));
+	maybe_add_test_to_model(pmod, test);
     } else {
 	err = 1;
     }
@@ -1586,8 +1638,6 @@ MODEL panel_wls_by_unit (const int *list, double ***pZ, DATAINFO *pdinfo,
     return mdl;
 }
 
-/* .................................................................. */
-
 static void panel_copy_var (double **targZ, DATAINFO *targinfo, int targv,
 			    double *src, DATAINFO *srcinfo, int srcv,
 			    int order)
@@ -1658,8 +1708,7 @@ int panel_autocorr_test (MODEL *pmod, int order,
     if (order > pdinfo->pd - 1) return E_DF;
     if (pmod->ncoeff + order >= sn) return E_DF;
 
-    if (pdinfo->structure != STACKED_TIME_SERIES ||
-	!balanced_panel(pdinfo)) { 
+    if (pdinfo->structure != STACKED_TIME_SERIES || !balanced_panel(pdinfo)) { 
         return E_DATA;
     }
 
@@ -1758,9 +1807,7 @@ int panel_autocorr_test (MODEL *pmod, int order,
 		_("Chi-square"), order, trsq, chisq(trsq, order));
 
 	if (opt & OPT_S) {
-	    ModelTest *test;
-
-	    test = new_test_on_model(pmod, GRETL_TEST_AUTOCORR);
+	    ModelTest *test = model_test_new(GRETL_TEST_AUTOCORR);
 
 	    if (test != NULL) {
 		int dfd = aux.nobs - pmod->ncoeff - order;
@@ -1771,6 +1818,7 @@ int panel_autocorr_test (MODEL *pmod, int order,
 		model_test_set_dfd(test, dfd);
 		model_test_set_value(test, LMF);
 		model_test_set_pvalue(test, pval);
+		maybe_add_test_to_model(pmod, test);
 	    }	    
 	}
     }
@@ -1778,9 +1826,7 @@ int panel_autocorr_test (MODEL *pmod, int order,
     free(aclist);
     clear_model(&aux); 
 
-    free_Z(tmpZ, tmpinfo);
-    clear_datainfo(tmpinfo, CLEAR_FULL);
-    free(tmpinfo);
+    destroy_dataset(tmpZ, tmpinfo);
 
     return err;
 }
