@@ -19,7 +19,6 @@
 
 #include "libgretl.h"
 #include "gretl_matrix.h"
-#include "gretl_matrix_private.h"
 #include "system.h"
 #include "system_private.h"
 
@@ -31,51 +30,21 @@ extern int fiml_driver (gretl_equation_system *sys, double ***pZ,
 
 /* liml.c */
 extern int liml_driver (gretl_equation_system *sys, double ***pZ, 
-			DATAINFO *pdinfo, gretlopt opt, PRN *prn);
+			DATAINFO *pdinfo, PRN *prn);
 
 static void 
 print_system_vcv (const gretl_equation_system *sys, PRN *prn)
 {
-    const gretl_matrix *m = sys->sigma;
-    gretl_matrix *mcopy;
-    int jmax = 1;
-    char numstr[16];
-    double x, ldet = NADBL;
-    int i, j;
+    int dim = sys->sigma->rows;
+    int df = dim * (dim - 1) / 2;
+    double ldet;
 
-    pprintf(prn, "%s\n(%s)\n\n",
-	    _("Cross-equation VCV for residuals"),
-	    _("correlations above the diagonal"));
+    ldet = gretl_vcv_log_determinant(sys->sigma);
 
-    for (i=0; i<m->rows; i++) {
-	for (j=0; j<jmax; j++) {
-	    pprintf(prn, "%#10.5g ", gretl_matrix_get(m, i, j));
-	}
-	for (j=jmax; j<m->cols; j++) {
-	    x = gretl_matrix_get(m, i, i) * gretl_matrix_get(m, j, j);
-	    x = sqrt(x);
-	    x = gretl_matrix_get(m, i, j) / x;
-	    sprintf(numstr,"(%.3f)", x); 
-	    pprintf(prn, "%11s", numstr);
-	}
-	pputc(prn, '\n');
-	if (jmax < m->cols) {
-	    jmax++;
-	}
-    }
-
-    mcopy = gretl_matrix_copy(m);
-    if (mcopy != NULL) {
-	ldet = gretl_vcv_log_determinant(mcopy);
-	if (!na(ldet)) {
-	    pprintf(prn, "\n%s = %g\n", _("log determinant"), ldet);
-	}
-	gretl_matrix_free(mcopy);
-    }
+    print_contemp_covariance_matrix(sys->sigma, ldet, prn);
 
     if (sys->method == SYS_SUR && sys->iters > 0) {
 	if (!na(ldet) && sys->diag != 0.0) {
-	    int df = m->rows * (m->rows - 1) / 2;
 	    double lr = sys->n_obs * (sys->diag - ldet);
 
 	    pprintf(prn, "%s:\n", _("LR test for diagonal covariance matrix"));
@@ -86,8 +55,6 @@ print_system_vcv (const gretl_equation_system *sys, PRN *prn)
 	double lm = sys->diag;
 
 	if (lm > 0) {
-	    int df = m->rows * (m->rows - 1) / 2;
-	
 	    pprintf(prn, "%s:\n", _("Breusch-Pagan test for diagonal covariance matrix"));
 	    pprintf(prn, "  %s(%d) = %g %s %g\n", _("Chi-square"),
 		    df, lm, _("with p-value"), chisq(lm, df));
@@ -189,7 +156,7 @@ gls_sigma_from_uhat (gretl_equation_system *sys, gretl_matrix *sigma)
 	for (j=i; j<m; j++) {
 	    xx = 0.0;
 	    for (t=0; t<T; t++) {
-		xx += gretl_matrix_get(e, i, t) * gretl_matrix_get(e, j, t);
+		xx += gretl_matrix_get(e, t, i) * gretl_matrix_get(e, t, j);
 	    }
 	    if (geomean) {
 		xx /= system_vcv_denom(sys, i, j);
@@ -239,7 +206,7 @@ sys_resids (gretl_equation_system *sys, int eq, const double **Z)
 	pmod->yhat[t] = yh;
 	pmod->uhat[t] = Z[pmod->list[1]][t] - yh;
 	/* for cross-equation vcv */
-	gretl_matrix_set(sys->uhat, pmod->ID, t - pmod->t1, pmod->uhat[t]);
+	gretl_matrix_set(sys->uhat, t - pmod->t1, pmod->ID, pmod->uhat[t]);
 	pmod->ess += pmod->uhat[t] * pmod->uhat[t];
     }
 
@@ -332,16 +299,21 @@ calculate_sys_coefficients (gretl_equation_system *sys,
     }
 
 #if SDEBUG
-    gretl_matrix_print(y, "in calc_coeffs, betahat", NULL);
+    gretl_matrix_print(y, "in calc_coeffs, betahat");
 #endif
 
+#if 1
     err = gretl_invert_general_matrix(vcv);
+#else
+    /* very memory-intensive for big matrices */
+    err = gretl_SVD_invert_matrix(vcv);
+#endif
     if (err) {
 	return err;
     }
 
 #if SDEBUG
-    gretl_matrix_print(vcv, "in calc_coeffs, vcv", NULL);
+    gretl_matrix_print(vcv, "in calc_coeffs, vcv");
 #endif
 
     j0 = 0;
@@ -402,11 +374,10 @@ calculate_sys_coefficients (gretl_equation_system *sys,
     return err;
 }
 
-/* FIXME below: naming and labeling of added variables */
-
-static void add_results_to_dataset (gretl_equation_system *sys, 
-				    int i, int *pj,
-				    double **Z, DATAINFO *pdinfo)
+static void 
+add_system_results_to_dataset (gretl_equation_system *sys, 
+			       int i, int *pj,
+			       double **Z, DATAINFO *pdinfo)
 {
     const MODEL *pmod = sys->models[i];
     int t;
@@ -419,17 +390,7 @@ static void add_results_to_dataset (gretl_equation_system *sys,
 		Z[*pj][t] = pmod->uhat[t];
 	    }
 	}
-	sprintf(pdinfo->varname[*pj], "uhat_s%02d", i + 1);
-	if (sys->method == SYS_SUR) {
-	    sprintf(VARLABEL(pdinfo, *pj), _("SUR residual, equation %d"), 
-		    i + 1);
-	} else if (sys->method == SYS_3SLS) {
-	    sprintf(VARLABEL(pdinfo, *pj), _("3SLS residual, equation %d"), 
-		    i + 1);
-	} else {
-	    sprintf(VARLABEL(pdinfo, *pj), "system residual, equation %d", 
-		    i + 1);
-	}
+	make_system_data_info(sys, i + 1, pdinfo, *pj, GRETL_SYSTEM_SAVE_UHAT);
 	*pj += 1;
     }
 
@@ -440,18 +401,8 @@ static void add_results_to_dataset (gretl_equation_system *sys,
 	    } else {
 		Z[*pj][t] = pmod->yhat[t];
 	    }
-	}	
-	sprintf(pdinfo->varname[*pj], "yhat_s%02d", i + 1);
-	if (sys->method == SYS_SUR) {
-	    sprintf(VARLABEL(pdinfo, *pj), _("SUR fitted value, equation %d"), 
-		    i + 1);
-	} else if (sys->method == SYS_3SLS) {
-	    sprintf(VARLABEL(pdinfo, *pj), _("3SLS fitted value, equation %d"), 
-		    i + 1);
-	} else {
-	    sprintf(VARLABEL(pdinfo, *pj), "system fitted value, equation %d", 
-		    i + 1);
-	}	    
+	}
+	make_system_data_info(sys, i + 1, pdinfo, *pj, GRETL_SYSTEM_SAVE_YHAT);
 	*pj += 1;
     }
 }
@@ -521,7 +472,10 @@ print_system_overidentification_test (const gretl_equation_system *sys,
 	pputc(prn, '\n');
     } else if ((sys->method == SYS_3SLS || sys->method == SYS_SUR) && df > 0) {
 
-	if (na(sys->X2) || sys->X2 == 0.0) {
+	if (na(sys->X2) || sys->X2 <= 0.0) {
+	    pputs(prn, _("Warning: the Hansen-Sargan over-identification test "
+		  "failed.\nThis probably indicates that the estimation "
+		  "problem is ill-conditioned.\n"));
 	    return;
 	}
 
@@ -576,7 +530,7 @@ static int hansen_sargan_test (gretl_equation_system *sys,
 	for (j=i; j<nx; j++) {
 	    Wj = Z[exlist[j+1]] + sys->t1;
 	    x = 0.0;
-	    for (t=0; t<T; t++) {
+	    for (t=0; t<sys->n_obs; t++) {
 		x += Wi[t] * Wj[t];
 	    }
 	    gretl_matrix_set(WTW, i, j, x);
@@ -587,7 +541,10 @@ static int hansen_sargan_test (gretl_equation_system *sys,
     }
 
     err = gretl_invert_symmetric_matrix(WTW);
-    if (err) goto bailout;
+    if (err) {
+	sys->X2 = NADBL;
+	goto bailout;
+    }
 
     /* set up vectors of SUR or 3SLS residuals, transposed, times W:
        these are stacked in an m * nx matrix
@@ -597,7 +554,7 @@ static int hansen_sargan_test (gretl_equation_system *sys,
 	    Wj = Z[exlist[j+1]] + sys->t1;
 	    x = 0.0;
 	    for (t=0; t<T; t++) {
-		x += gretl_matrix_get(sys->uhat, i, t) * Wj[t];
+		x += gretl_matrix_get(sys->uhat, t, i) * Wj[t];
 	    }
 	    gretl_matrix_set(eW, i, j, x);
 	}
@@ -648,33 +605,14 @@ static int basic_system_allocate (gretl_equation_system *sys,
 				  gretl_matrix **X,
 				  gretl_matrix **y)
 {
-    MODEL **pmods;
     int m = sys->n_equations;
     int T = sys->n_obs;
     int ldx = mk + nr;
-    int i, j;
-
-    sys->models = NULL;
 
     /* allocate a model for each stochastic equation */
-    pmods = malloc(m * sizeof *pmods);
-    if (pmods == NULL) {
-	return E_ALLOC;
-    }
-    for (i=0; i<m; i++) {
-	pmods[i] = gretl_model_new();
-	if (pmods[i] == NULL) {
-	    for (j=0; j<i; j++) {
-		free_model(pmods[j]);
-	    }
-	    free(pmods);
-	    return E_ALLOC;
-	}
-    }
+    sys->models = gretl_model_array_new(m);
 
-    sys->models = pmods;
-
-    sys->uhat = gretl_matrix_alloc(m, T);
+    sys->uhat = gretl_matrix_alloc(T, m);
     if (sys->uhat == NULL) {
 	return E_ALLOC;
     }
@@ -758,10 +696,10 @@ save_and_print_results (gretl_equation_system *sys,
     if (sys->name != NULL) {
 	pprintf(prn, "%s, %s\n", _("Equation system"), sys->name);
 	pprintf(prn, "%s: %s\n", _("Estimator"), 
-		system_get_full_string(sys, opt));
+		system_get_full_string(sys));
     } else {
 	pprintf(prn, "%s, %s\n", _("Equation system"),
-		system_get_full_string(sys, opt));
+		system_get_full_string(sys));
     }
 
     if (sys->iters > 0) {
@@ -779,7 +717,7 @@ save_and_print_results (gretl_equation_system *sys,
     for (i=0; i<m; i++) {
 	printmodel(sys->models[i], pdinfo, OPT_NONE, prn);
 	if (!err) {
-	    add_results_to_dataset(sys, i, &j, *pZ, pdinfo);
+	    add_system_results_to_dataset(sys, i, &j, *pZ, pdinfo);
 	}
     }
 
@@ -801,7 +739,7 @@ double sur_ll (gretl_equation_system *sys)
     int m = sys->n_equations;
     int T = sys->n_obs;
     gretl_matrix *sigtmp;
-    double ldet, ll;
+    double ldet;
 
     sigtmp = gretl_matrix_alloc(m, m);
     if (sigtmp == NULL) return NADBL;
@@ -810,17 +748,15 @@ double sur_ll (gretl_equation_system *sys)
     ldet = gretl_vcv_log_determinant(sigtmp);
 
     if (na(ldet)) {
-	ll = NADBL;
+	sys->ll = NADBL;
     } else {
-	ll = -(m * T / 2.0) * (LN_2_PI + 1.0);
-	ll -= (T / 2.0) * ldet;
+	sys->ll = -(m * T / 2.0) * (LN_2_PI + 1.0);
+	sys->ll -= (T / 2.0) * ldet;
     }
 
-    sys->ll = ll;
-
     gretl_matrix_free(sigtmp);
 
-    return ll;
+    return sys->ll;
 }
 
 /* if we're estimating with a specified set of linear restrictions,
@@ -946,7 +882,7 @@ static void clean_up_models (gretl_equation_system *sys)
 	    sys->method == SYS_TSLS || sys->method == SYS_LIML) {
 	    tsls_free_data(sys->models[i]);
 	}
-	free_model(sys->models[i]);
+	gretl_model_free(sys->models[i]);
     }
 
     free(sys->models);
@@ -955,7 +891,8 @@ static void clean_up_models (gretl_equation_system *sys)
     sys->ess = ess;
 }
 
-/* general function that forms the basis for all system estimates */
+/* general function that forms the basis for all specific system
+   estimators */
 
 int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo, 
 		     gretlopt opt, PRN *prn)
@@ -982,7 +919,7 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 
     sys->iters = 0;
 
-    if (opt & OPT_T) {
+    if (sys->flags & GRETL_SYS_ITERATE) {
 	do_iteration = 1;
     }
     
@@ -1055,9 +992,9 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	    *pmod = lsq(list, pZ, pdinfo, OLS, OPT_A, 0.0);
 	} else if (method == SYS_3SLS || method == SYS_FIML || 
 		   method == SYS_TSLS) {
-	    *pmod = tsls_func(list, 0, pZ, pdinfo, OPT_S);
+	    *pmod = tsls_func(list, SYSTEM, pZ, pdinfo, OPT_NONE);
 	} else if (method == SYS_LIML) {
-	    *pmod = tsls_func(list, 0, pZ, pdinfo, OPT_N | OPT_S);
+	    *pmod = tsls_func(list, SYSTEM, pZ, pdinfo, OPT_N);
 	}
 
 	if (freeit) {
@@ -1082,7 +1019,7 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	}
 
 	for (t=0; t<T; t++) {
-	    gretl_matrix_set(sys->uhat, i, t, pmod->uhat[t + sys->t1]);
+	    gretl_matrix_set(sys->uhat, t, i, pmod->uhat[t + sys->t1]);
 	}
     }
 
@@ -1091,7 +1028,7 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     if (method == SYS_LIML) {
 	/* compute the minimum eigenvalues and generated the
 	   suitably transformed data matrices */
-	err = liml_driver(sys, pZ, pdinfo, opt, prn);
+	err = liml_driver(sys, pZ, pdinfo, prn);
 	if (err) goto cleanup;
     }
 
@@ -1104,7 +1041,7 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     gls_sigma_from_uhat(sys, sys->sigma);
 
 #if SDEBUG
-    gretl_matrix_print(sys->sigma, "gls_sigma_from_uhat", NULL);
+    gretl_matrix_print(sys->sigma, "gls_sigma_from_uhat");
 #endif
 
     /* simple single-equation method, no restrictions to test and 
@@ -1259,8 +1196,8 @@ int system_estimate (gretl_equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     }
 
 #if SDEBUG
-    gretl_matrix_print(X, "sys X", NULL);
-    gretl_matrix_print(y, "sys y", NULL);
+    gretl_matrix_print(X, "sys X");
+    gretl_matrix_print(y, "sys y");
 #endif    
 
     /* The estimates calculated below will be SUR, 3SLS or LIML,
