@@ -300,15 +300,7 @@ static int graph_series (double **Z, DATAINFO *pdinfo, int opt)
 	return E_FOPEN;
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
-
-    if (opt == TRAMO_SEATS) {
-	fputs("# TRAMO/SEATS tri-graph (no auto-parse)\n", fp);
-    } else {
-	fputs("# X-12-ARIMA tri-graph (no auto-parse)\n", fp);
-    }
+    gretl_push_c_numeric_locale();
 
     if (pdinfo->pd == 4) {
 	if ((pdinfo->t2 - pdinfo->t1) / 4 < 8) {
@@ -373,9 +365,7 @@ static int graph_series (double **Z, DATAINFO *pdinfo, int opt)
 
     fputs("set nomultiplot\n", fp);
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
 
     fclose(fp);
 
@@ -476,9 +466,7 @@ static int add_series_from_file (const char *fname, int code,
 	Z[v][t] = NADBL;
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
+    gretl_push_c_numeric_locale();
 
     if (opt == TRAMO_SEATS) {
 	int i = 0;
@@ -517,9 +505,7 @@ static int add_series_from_file (const char *fname, int code,
 	}
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
 
     fclose(fp);
 
@@ -567,9 +553,7 @@ static int write_tramo_file (const char *fname,
     fp = gretl_fopen(fname, "w");
     if (fp == NULL) return 1;
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif   
+    gretl_push_c_numeric_locale();
 
     x = date(pdinfo->t1, pdinfo->pd, pdinfo->sd0);
     startyr = (int) x;
@@ -595,9 +579,7 @@ static int write_tramo_file (const char *fname,
 	request->code = TRAMO_ONLY; /* not running SEATS */
     }
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
 
     fclose(fp);
 
@@ -615,11 +597,9 @@ static int write_spc_file (const char *fname,
     char *p, tmp[8];
 
     fp = gretl_fopen(fname, "w");
-    if (fp == NULL) return 1;    
+    if (fp == NULL) return 1;   
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif 
+    gretl_push_c_numeric_locale();
 
     x = date(pdinfo->t1, pdinfo->pd, pdinfo->sd0);
     startyr = (int) x;
@@ -662,9 +642,7 @@ static int write_spc_file (const char *fname,
 
     fputs("}\n", fp);
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
+    gretl_pop_c_numeric_locale();
 
     fclose(fp);
 
@@ -926,9 +904,7 @@ int write_tx_data (char *fname, int varnum,
 	}
     }
 
-    free_Z(tmpZ, tmpinfo);
-    clear_datainfo(tmpinfo, CLEAR_FULL);
-    free(tmpinfo);
+    destroy_dataset(tmpZ, tmpinfo);
 
     return err;
 }
