@@ -316,6 +316,7 @@ static double fpval (double *beta, double *cnorm, double *wght,
 	/* check to see if gamma(4) is needed */
 	sd4 = sqrt(ssrt / (np1 - 4) * xomx[15]);
 	ttest = fabs(gamma[3]) / sd4;
+
 	if (ttest > precrt) {
 	    d1 = stat;
 	    crfit = gamma[0] + gamma[1] * stat + gamma[2] * (d1 * d1) + 
@@ -493,7 +494,7 @@ static int cholx (double *a, int m, int n)
 	    } else {
 		if (a[i + i * m] <= 0.0) {
 		    err = i;
-		    goto L20;
+		    goto cholx_exit;
 		}
 	    }
 	    if (i == j) {
@@ -512,14 +513,14 @@ static int cholx (double *a, int m, int n)
 	    ooa = 1. / a[j + j * m];
 	    if (i >= j) {
 		t = 1.;
-		goto L12;
+		goto cholx_jump;
 	    }
 	    kl = j - 1;
 	    t = 0.;
 	    for (k = i; k <= kl; ++k) {
 		t -= a[i + k * m] * a[k + j * m];
 	    }
-L12:
+	cholx_jump:
 	    a[i + j * m] = t * ooa;
 	}
     }
@@ -535,7 +536,8 @@ L12:
 	}
     }
 
-L20:
+ cholx_exit:
+
     return err;
 }
 
@@ -619,86 +621,65 @@ static double ddnor (double ystar)
 
     x = -y * root2;
 
-    if (x > 0.) {
-	goto L1;
-    }
-    if (x < 0.) {
-	goto L2;
+    if (x == 0.0) {
+	return .5;
+    } else if (x < 0.0) {
+	x = -x;
+	isw = -1;
     }
 
-    return .5;
+    if (x > 4.0) {
+	x2 = x * x;
+	xm2 = 1.0 / x2;
+	xm4 = xm2 * xm2;
+	xm6 = xm4 * xm2;
+	xm8 = xm4 * xm4;
+	xm10 = xm6 * xm4;
+	top = p[0] + p[1] * xm2 + p[2] * xm4 + p[3] * xm6 + p[4] * xm8 + p[5] * 
+	    xm10;
+	bot = q[0] + q[1] * xm2 + q[2] * xm4 + q[3] * xm6 + q[4] * xm8 + xm10;
+	crap = orpi + top / (bot * x2);
+	erfc = exp(-x2) * crap / x;
 
- L2:
-    x = -x;
-    isw = -1;
+	if (isw == -1) {
+	    erfc = 2. - erfc;
+	}
+    } else if (x > 0.477) {
+	x2 = x * x;
+	x3 = x2 * x;
+	x4 = x2 * x2;
+	x5 = x3 * x2;
+	x6 = x3 * x3;
+	x7 = x3 * x4;
+	x8 = x4 * x4;
+	top = a[0] + a[1] * x + a[2] * x2 + a[3] * x3 + a[4] * 
+	    x4 + a[5] * x5 + a[6] * x6 + a[7] * x7 + a[8] * x8;
+	bot = b[0] + b[1] * x + b[2] * x2 + b[3] * x3 + 
+	    b[4] * x4 + b[5] * x5 + b[6] * x6 + b[7] * x7 + x8;
+	erfc = exp(-x2) * top / bot;
 
- L1:
-    if (x < .477) {
-	goto L10;
+	if (isw == -1) {
+	    erfc = 2. - erfc;
+	}
+    } else {
+	x2 = x * x;
+	x4 = x2 * x2;
+	x6 = x4 * x2;
+	x8 = x4 * x4;
+	top = c[0] + c[1] * x2 + c[2] * x4 + c[3] * x6 + c[4] * x8;
+	bot = d[0] + d[1] * x2 + d[2] * x4 + d[3] * x6 + x8;
+	erf = x * top / bot;
+
+	erf *= isw;
+	erfc = 1.0 - erf;
     }
-    if (x <= 4.) {
-	goto L20;
-    }
-
-    /* evaluate erfc for x.gt.4.0 */
-    x2 = x * x;
-    xm2 = 1. / x2;
-    xm4 = xm2 * xm2;
-    xm6 = xm4 * xm2;
-    xm8 = xm4 * xm4;
-    xm10 = xm6 * xm4;
-    top = p[0] + p[1] * xm2 + p[2] * xm4 + p[3] * xm6 + p[4] * xm8 + p[5] * 
-	xm10;
-    bot = q[0] + q[1] * xm2 + q[2] * xm4 + q[3] * xm6 + q[4] * xm8 + xm10;
-    crap = orpi + top / (bot * x2);
-    erfc = exp(-x2) * crap / x;
-
-    if (isw == -1) {
-	erfc = 2. - erfc;
-    }
-
-    return erfc * .5;
- L20:
-
-    /* evaluate erfc for .477.lt.x.le.4.0 */
-    x2 = x * x;
-    x3 = x2 * x;
-    x4 = x2 * x2;
-    x5 = x3 * x2;
-    x6 = x3 * x3;
-    x7 = x3 * x4;
-    x8 = x4 * x4;
-    top = a[0] + a[1] * x + a[2] * x2 + a[3] * x3 + a[4] * 
-	x4 + a[5] * x5 + a[6] * x6 + a[7] * x7 + a[8] * x8;
-    bot = b[0] + b[1] * x + b[2] * x2 + b[3] * x3 + 
-	b[4] * x4 + b[5] * x5 + b[6] * x6 + b[7] * x7 + x8;
-    erfc = exp(-x2) * top / bot;
-
-    if (isw == -1) {
-	erfc = 2. - erfc;
-    }
-
-    return erfc * .5;
- L10:
-
-    /* evaluate erf for x.lt..477 */
-    x2 = x * x;
-    x4 = x2 * x2;
-    x6 = x4 * x2;
-    x8 = x4 * x4;
-    top = c[0] + c[1] * x2 + c[2] * x4 + c[3] * x6 + c[4] * x8;
-    bot = d[0] + d[1] * x2 + d[2] * x4 + d[3] * x6 + x8;
-    erf = x * top / bot;
-
-    erf *= isw;
-    erfc = 1. - erf;
 
     return erfc * .5;
 }
 
 double mackinnon_pvalue (double tval, int n, int niv, int itv, char *path)
 {
-    double val;
+    double val = NADBL;
     int check;
 
 #ifdef URDEBUG
@@ -709,15 +690,10 @@ double mackinnon_pvalue (double tval, int n, int niv, int itv, char *path)
     fflush(fdb);
 #endif
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "C");
-#endif
-
+    gretl_push_c_numeric_locale();
     check = urcval(niv, itv, n, tval, &val, path);
+    gretl_pop_c_numeric_locale();
 
-#ifdef ENABLE_NLS
-    setlocale(LC_NUMERIC, "");
-#endif
 
 #ifdef URDEBUG
     fclose(fdb);
